@@ -1,14 +1,20 @@
-import type { InternationalizationAPI } from "@gnu-taler/web-util/browser";
-import { v1 as form_902_11e_v1 } from "./902_11e.js";
-import { v1 as form_902_12e_v1 } from "./902_12e.js";
-import { v1 as form_902_13e_v1 } from "./902_13e.js";
-import { v1 as form_902_15e_v1 } from "./902_15e.js";
-import { v1 as form_902_1e_v1 } from "./902_1e.js";
-import { v1 as form_902_4e_v1 } from "./902_4e.js";
-import { v1 as form_902_5e_v1 } from "./902_5e.js";
-import { v1 as form_902_9e_v1 } from "./902_9e.js";
+/*
+ This file is part of GNU Taler
+ (C) 2022-2024 Taler Systems S.A.
+
+ GNU Taler is free software; you can redistribute it and/or modify it under the
+ terms of the GNU General Public License as published by the Free Software
+ Foundation; either version 3, or (at your option) any later version.
+
+ GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
+ WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License along with
+ GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
+ */
+import type { FormMetadata, InternationalizationAPI } from "@gnu-taler/web-util/browser";
 import { v1 as simplest } from "./simplest.js";
-import { BaseForm, FormMetadata } from "./declaration.js";
 
 const languages = (i18n: InternationalizationAPI) => [
   {
@@ -122,52 +128,52 @@ const languages = (i18n: InternationalizationAPI) => [
 ];
 
 
-const forms: (i18n: InternationalizationAPI) => Array<FormMetadata<BaseForm>> = (i18n) => [
+export const preloadedForms: (i18n: InternationalizationAPI) => Array<FormMetadata> = (i18n) => [
   {
     label: i18n.str`Simple comment`,
-    id: "simple_comment",
+    id: "__simple_comment",
     version: 1,
-    impl: simplest(i18n),
-  }, {
-    label: i18n.str`Identification form`,
-    id: "902.1e",
-    version: 1,
-    impl: form_902_1e_v1(i18n),
-  }, {
-    label: i18n.str`Operational legal entity or partnership`,
-    id: "902.11e",
-    version: 1,
-    impl: form_902_11e_v1(i18n),
-  }, {
-    label: i18n.str`Foundations`,
-    id: "902.12e",
-    version: 1,
-    impl: form_902_12e_v1(i18n),
-  }, {
-    label: i18n.str`Declaration for trusts`,
-    id: "902.13e",
-    version: 1,
-    impl: form_902_13e_v1(i18n),
-  }, {
-    label: i18n.str`Information on life insurance policies`,
-    id: "902.15e",
-    version: 1,
-    impl: form_902_15e_v1(i18n),
-  }, {
-    label: i18n.str`Declaration of beneficial owner`,
-    id: "902.9e",
-    version: 1,
-    impl: form_902_9e_v1(i18n),
-  }, {
-    label: i18n.str`Customer profile`,
-    id: "902.5e",
-    version: 1,
-    impl: form_902_5e_v1(i18n),
-  }, {
-    label: i18n.str`Risk profile`,
-    id: "902.4e",
-    version: 1,
-    impl: form_902_4e_v1(i18n),
+    config: simplest(i18n),
+  // }, {
+  //   label: i18n.str`Identification form`,
+  //   id: "902.1e",
+  //   version: 1,
+  //   config: form_902_1e_v1(i18n),
+  // }, {
+  //   label: i18n.str`Operational legal entity or partnership`,
+  //   id: "902.11e",
+  //   version: 1,
+  //   config: form_902_11e_v1(i18n),
+  // }, {
+  //   label: i18n.str`Foundations`,
+  //   id: "902.12e",
+  //   version: 1,
+  //   config: form_902_12e_v1(i18n),
+  // }, {
+  //   label: i18n.str`Declaration for trusts`,
+  //   id: "902.13e",
+  //   version: 1,
+  //   config: form_902_13e_v1(i18n),
+  // }, {
+  //   label: i18n.str`Information on life insurance policies`,
+  //   id: "902.15e",
+  //   version: 1,
+  //   config: form_902_15e_v1(i18n),
+  // }, {
+  //   label: i18n.str`Declaration of beneficial owner`,
+  //   id: "902.9e",
+  //   version: 1,
+  //   config: form_902_9e_v1(i18n),
+  // }, {
+  //   label: i18n.str`Customer profile`,
+  //   id: "902.5e",
+  //   version: 1,
+  //   config: form_902_5e_v1(i18n),
+  // }, {
+  //   label: i18n.str`Risk profile`,
+  //   id: "902.4e",
+  //   version: 1,
+  //   config: form_902_4e_v1(i18n),
   },
 ];
 
@@ -199,4 +205,3 @@ const currencies = (i18n: InternationalizationAPI) => [
   },
 ];
 
-globalThis.amlExchangeBackoffice = { currencies, languages, forms }
