@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022 Taler Systems S.A.
+ (C) 2022-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -18,5 +18,8 @@ import { App } from "./App.js";
 import { h, render } from "preact";
 
 const app = document.getElementById("app");
-
-render(<App />, app as any);
+if (!app) {
+  console.error("could not found app element")
+} else {
+  render(<App />, app);
+}
