@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022 Taler Systems S.A.
+ (C) 2022-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -20,23 +20,22 @@
  */
 
 import * as tests from "@gnu-taler/web-util/testing";
-import {
-  CasesUI as TestedComponent,
-} from "./Cases.js";
-import { AmountString } from "@gnu-taler/taler-util";
-import { AmlExchangeBackend } from "../utils/types.js";
+import { CasesUI as TestedComponent } from "./Cases.js";
+import { AmountString, TalerExchangeApi } from "@gnu-taler/taler-util";
 
 export default {
   title: "cases",
 };
 
 export const OneRow = tests.createExample(TestedComponent, {
-  filter: AmlExchangeBackend.AmlState.normal,
+  filter: TalerExchangeApi.AmlState.normal,
   onChangeFilter: () => null,
-  records: [{
-    current_state: AmlExchangeBackend.AmlState.normal,
-    h_payto: "QWEQWEQWEQWE",
-    rowid: 1,
-    threshold: "USD:1" as AmountString
-  }]
+  records: [
+    {
+      current_state: TalerExchangeApi.AmlState.normal,
+      h_payto: "QWEQWEQWEQWE",
+      rowid: 1,
+      threshold: "USD:1" as AmountString,
+    },
+  ],
 });
