@@ -22,7 +22,7 @@ function main(): void {
   try {
     const container = document.getElementById("container");
     if (!container) {
-      throw Error("container not found, can't mount page contents");
+      throw new Error("container not found, can't mount page contents");
     }
     render(h(App, {}), container);
   } catch (e) {
