@@ -20,10 +20,12 @@
  */
 
 import { ReducerState } from "@gnu-taler/anastasis-core";
-import { createExample, reducerStatesExample } from "../../utils/index.js";
+import { reducerStatesExample } from "../../utils/index.js";
 import { SecretEditorScreen as TestedComponent } from "./SecretEditorScreen.js";
+import * as tests from "@gnu-taler/web-util/testing";
 
 export default {
+  title: "Secret editor",
   component: TestedComponent,
   args: {
     order: 7,
@@ -34,11 +36,15 @@ export default {
   },
 };
 
-export const WithSecretNamePreselected = createExample(TestedComponent, {
-  ...reducerStatesExample.secretEdition,
-  secret_name: "someSecretName",
-} as ReducerState);
+export const WithSecretNamePreselected = tests.createExample(
+  TestedComponent,
+  {},
+  {
+    ...reducerStatesExample.secretEdition,
+    secret_name: "someSecretName",
+  } as ReducerState,
+);
 
-export const WithoutName = createExample(TestedComponent, {
+export const WithoutName = tests.createExample(TestedComponent, {}, {
   ...reducerStatesExample.secretEdition,
 } as ReducerState);
