@@ -31,7 +31,7 @@ import {
   HttpStatusCode,
   TranslatedString,
   assertUnreachable,
-  createRFC8959AccessTokenEncoded
+  createRFC8959AccessTokenEncoded,
 } from "@gnu-taler/taler-util";
 import { useEffect } from "preact/hooks";
 import { useSessionState } from "./hooks/session.js";
@@ -108,10 +108,6 @@ function PublicRounting({
     }
   }, [location]);
 
-  if (location === undefined) {
-    return <Fragment />;
-  }
-
   async function doAutomaticLogin(username: string, password: string) {
     await handleError(async () => {
       const resp = await lib
@@ -122,7 +118,10 @@ function PublicRounting({
           refreshable: true,
         });
       if (resp.type === "ok") {
-        onLoggedUser(username, createRFC8959AccessTokenEncoded(resp.body.access_token));
+        onLoggedUser(
+          username,
+          createRFC8959AccessTokenEncoded(resp.body.access_token),
+        );
       } else {
         switch (resp.case) {
           case HttpStatusCode.Unauthorized:
@@ -149,6 +148,7 @@ function PublicRounting({
   }
 
   switch (location.name) {
+    case undefined:
     case "login": {
       return (
         <Fragment>
@@ -278,10 +278,6 @@ function PrivateRouting({
     }
   }, [location]);
 
-  if (location === undefined) {
-    return <Fragment />;
-  }
-
   switch (location.name) {
     case "operationDetails": {
       return (
@@ -395,9 +391,7 @@ function PrivateRouting({
           routeMyAccountDetails={privatePages.myAccountDetails}
           routeMyAccountPassword={privatePages.myAccountPassword}
           routeConversionConfig={privatePages.conversionConfig}
-          onCashout={() =>
-            navigateTo(privatePages.home.url({}))
-          }
+          onCashout={() => navigateTo(privatePages.home.url({}))}
           onAuthorizationRequired={() =>
             navigateTo(privatePages.solveSecondFactor.url({}))
           }
@@ -465,9 +459,7 @@ function PrivateRouting({
           routeMyAccountDetails={privatePages.myAccountDetails}
           routeMyAccountPassword={privatePages.myAccountPassword}
           routeConversionConfig={privatePages.conversionConfig}
-          onCashout={() =>
-            navigateTo(privatePages.home.url({}))
-          }
+          onCashout={() => navigateTo(privatePages.home.url({}))}
           onAuthorizationRequired={() =>
             navigateTo(privatePages.solveSecondFactor.url({}))
           }
@@ -475,6 +467,7 @@ function PrivateRouting({
         />
       );
     }
+    case undefined:
     case "home": {
       if (isAdmin) {
         return (
