@@ -16,6 +16,7 @@
 
 import {
   AbsoluteTime,
+  Amounts,
   HttpStatusCode,
   TalerErrorCode,
   TranslatedString,
@@ -26,6 +27,7 @@ import {
   Attention,
   LocalNotificationBanner,
   notifyInfo,
+  useBankCoreApiContext,
   useLocalNotification,
   useTalerWalletIntegrationAPI,
   useTranslationContext,
@@ -37,6 +39,7 @@ import { useBankState } from "../../hooks/bank-state.js";
 import { usePreferences } from "../../hooks/preferences.js";
 import { ShouldBeSameUser } from "../WithdrawalConfirmationQuestion.js";
 import { State } from "./index.js";
+import { RenderAmount, doAutoFocus } from "../PaytoWireTransferForm.js";
 
 export function InvalidPaytoView({ payto }: State.InvalidPayto) {
   return <div>Payto from server is not valid &quot;{payto}&quot;</div>;
@@ -53,6 +56,7 @@ export function NeedConfirmationView({
   onConfirm: doConfirm,
   routeHere,
   account,
+  details,
   id,
   onAuthorizationRequired,
 }: State.NeedConfirmation) {
@@ -60,6 +64,11 @@ export function NeedConfirmationView({
   const [settings] = usePreferences();
   const [notification, notify, errorHandler] = useLocalNotification();
   const [, updateBankState] = useBankState();
+  const { config } = useBankCoreApiContext();
+  const wireFee =
+    config.wire_transfer_fees === undefined
+      ? Amounts.zeroOfCurrency(config.currency)
+      : Amounts.parseOrThrow(config.wire_transfer_fees);
 
   async function onCancel() {
     errorHandler(async () => {
@@ -181,6 +190,164 @@ export function NeedConfirmationView({
                 e.preventDefault();
               }}
             >
+              <div class="px-4 mt-4">
+                <div class="w-full">
+                  <dl class="">
+                    {((): VNode => {
+                      if (!details.account.isKnown) {
+                        return (
+                          <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                            <dt class="text-sm font-medium leading-6 text-gray-900">
+                              <i18n.Translate>
+                                Payment provider's account
+                              </i18n.Translate>
+                            </dt>
+                            <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                              {details.account.targetPath}
+                            </dd>
+                          </div>
+                        );
+                      }
+                      switch (details.account.targetType) {
+                        case "iban": {
+                          const name = details.account.params["receiver-name"];
+                          return (
+                            <Fragment>
+                              <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                <dt class="text-sm font-medium leading-6 text-gray-900">
+                                  <i18n.Translate>
+                                    Payment provider's account number
+                                  </i18n.Translate>
+                                </dt>
+                                <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                  {details.account.iban}
+                                </dd>
+                              </div>
+                              {name && (
+                                <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                  <dt class="text-sm font-medium leading-6 text-gray-900">
+                                    <i18n.Translate>
+                                      Payment provider's name
+                                    </i18n.Translate>
+                                  </dt>
+                                  <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                    {name}
+                                  </dd>
+                                </div>
+                              )}
+                            </Fragment>
+                          );
+                        }
+                        case "x-taler-bank": {
+                          const name = details.account.params["receiver-name"];
+                          return (
+                            <Fragment>
+                              <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                <dt class="text-sm font-medium leading-6 text-gray-900">
+                                  <i18n.Translate>
+                                    Payment provider's account bank hostname
+                                  </i18n.Translate>
+                                </dt>
+                                <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                  {details.account.host}
+                                </dd>
+                              </div>
+                              <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                <dt class="text-sm font-medium leading-6 text-gray-900">
+                                  <i18n.Translate>
+                                    Payment provider's account id
+                                  </i18n.Translate>
+                                </dt>
+                                <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                  {details.account.account}
+                                </dd>
+                              </div>
+                              {name && (
+                                <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                  <dt class="text-sm font-medium leading-6 text-gray-900">
+                                    <i18n.Translate>
+                                      Payment provider's name
+                                    </i18n.Translate>
+                                  </dt>
+                                  <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                    {name}
+                                  </dd>
+                                </div>
+                              )}
+                            </Fragment>
+                          );
+                        }
+                        case "bitcoin": {
+                          const name = details.account.params["receiver-name"];
+                          return (
+                            <Fragment>
+                              <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                <dt class="text-sm font-medium leading-6 text-gray-900">
+                                  <i18n.Translate>
+                                    Payment provider's account address
+                                  </i18n.Translate>
+                                </dt>
+                                <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                  {details.account.address}
+                                </dd>
+                              </div>
+                              {name && (
+                                <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                  <dt class="text-sm font-medium leading-6 text-gray-900">
+                                    <i18n.Translate>
+                                      Payment provider's name
+                                    </i18n.Translate>
+                                  </dt>
+                                  <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                    {name}
+                                  </dd>
+                                </div>
+                              )}
+                            </Fragment>
+                          );
+                        }
+                        default: {
+                          assertUnreachable(details.account);
+                        }
+                      }
+                    })()}
+                    <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                      <dt class="text-sm font-medium leading-6 text-gray-900">
+                        <i18n.Translate>Amount</i18n.Translate>
+                      </dt>
+                      <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                        {details.amount !== undefined ? (
+                          <RenderAmount
+                            value={details.amount}
+                            spec={config.currency_specification}
+                          />
+                        ) : (
+                          <i18n.Translate>
+                            No amount specified yet.
+                          </i18n.Translate>
+                        )}
+                      </dd>
+                    </div>
+                    {Amounts.isZero(wireFee) ? undefined : (
+                      <Fragment>
+                        <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                          <dt class="text-sm font-medium leading-6 text-gray-900">
+                            <i18n.Translate>Cost</i18n.Translate>
+                          </dt>
+                          <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                            <RenderAmount
+                              value={wireFee}
+                              negative
+                              withColor
+                              spec={config.currency_specification}
+                            />
+                          </dd>
+                        </div>
+                      </Fragment>
+                    )}
+                  </dl>
+                </div>
+              </div>
               <div class="flex items-center justify-between gap-x-6 border-t border-gray-900/10 px-4 py-4 sm:px-8">
                 <button
                   type="button"
@@ -338,7 +505,11 @@ export function ConfirmedView({ routeClose }: State.Confirmed) {
   );
 }
 
-export function ReadyView({ uri, onAbort: doAbort }: State.Ready): VNode {
+export function ReadyView({
+  uri,
+  focus,
+  onAbort: doAbort,
+}: State.Ready): VNode {
   const { i18n } = useTranslationContext();
   const walletInegrationApi = useTalerWalletIntegrationAPI();
   const [notification, notify, errorHandler] = useLocalNotification();
@@ -387,60 +558,79 @@ export function ReadyView({ uri, onAbort: doAbort }: State.Ready): VNode {
     <Fragment>
       <LocalNotificationBanner notification={notification} />
 
-      <div class="flex justify-end mt-4">
-        <button
-          type="button"
-          name="cancel"
-          class="inline-flex items-center rounded-md bg-red-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-red-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-red-500"
-          onClick={onAbort}
-        >
-          <i18n.Translate>Cancel</i18n.Translate>
-        </button>
-      </div>
-
-      <div class="bg-white shadow sm:rounded-lg mt-4">
-        <div class="p-4">
+      <div class="bg-white shadow-xl sm:rounded-lg">
+        <div class="px-4 py-5 sm:p-6">
           <h3 class="text-base font-semibold leading-6 text-gray-900">
-            <i18n.Translate>On this device</i18n.Translate>
+            <i18n.Translate>
+              If you have a Taler wallet installed on this device
+            </i18n.Translate>
           </h3>
-          <div class="mt-2 sm:flex sm:items-start sm:justify-between">
-            <div class="max-w-xl text-sm text-gray-500">
-              <p>
-                <i18n.Translate>
-                  If you are using a web browser on desktop you can also
-                </i18n.Translate>
-              </p>
-            </div>
-            <div class="mt-5 sm:ml-6 sm:mt-0 sm:flex sm:flex-shrink-0 sm:items-center">
+          <div class="mt-4 mb-4 text-sm text-gray-500">
+            <p>
+              <i18n.Translate>
+                Your wallet will display the details of the transaction
+                including the fees (if applicable). If you do not yet have a
+                wallet, please follow the instructions
+              </i18n.Translate>{" "}
               <a
-                href={talerWithdrawUri}
-                name="start"
-                class="inline-flex items-center  disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
+                class="font-semibold text-indigo-600 hover:text-indigo-900"
+                name="wallet page"
+                href="https://taler.net/en/wallet.html"
               >
-                <i18n.Translate>Start</i18n.Translate>
+                <i18n.Translate>on this page</i18n.Translate>
               </a>
-            </div>
+              .
+            </p>
+          </div>
+          <div class="flex items-center justify-between gap-x-6  pt-2 mt-2 ">
+            <button
+              type="button"
+              name="cancel"
+              class="text-sm font-semibold leading-6 text-gray-900"
+              // class="inline-flex items-center rounded-md bg-white px-3 py-2 text-sm font-semibold text-black shadow-sm "
+              onClick={onAbort}
+            >
+              <i18n.Translate>Cancel</i18n.Translate>
+            </button>
+
+            <a
+              href={talerWithdrawUri}
+              name="withdraw"
+              class="inline-flex items-center  disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
+            >
+              <i18n.Translate>Withdraw</i18n.Translate>
+            </a>
           </div>
         </div>
       </div>
-      <div class="bg-white shadow sm:rounded-lg mt-2">
-        <div class="p-4">
+
+      <div class="bg-white shadow-xl sm:rounded-lg mt-8">
+        <div class="px-4 py-5 sm:p-6">
           <h3 class="text-base font-semibold leading-6 text-gray-900">
-            <i18n.Translate>On a mobile phone</i18n.Translate>
+            <i18n.Translate>
+              Or if you have the Taler wallet on another device
+            </i18n.Translate>
           </h3>
-          <div class="mt-2 sm:flex sm:items-start sm:justify-between">
-            <div class="max-w-xl text-sm text-gray-500">
-              <p>
-                <i18n.Translate>
-                  Scan the QR code with your mobile device.
-                </i18n.Translate>
-              </p>
-            </div>
+          <div class="mt-4 max-w-xl text-sm text-gray-500">
+            <i18n.Translate>
+              Scan the QR below to start the withdrawal.
+            </i18n.Translate>
           </div>
           <div class="mt-2 max-w-md ml-auto mr-auto">
             <QR text={talerWithdrawUri} />
           </div>
         </div>
+        <div class="flex items-center justify-center gap-x-6 border-t border-gray-900/10 px-4 py-4 sm:px-8">
+          <button
+            type="button"
+            // class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md  px-3 py-2 text-sm font-semibold text-black shadow-sm hover:bg-red-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-red-600"
+            class="text-sm font-semibold leading-6 text-gray-900"
+            // handler={onAbortHandler}
+            onClick={onAbort}
+          >
+            <i18n.Translate>Cancel</i18n.Translate>
+          </button>
+        </div>
       </div>
     </Fragment>
   );
