@@ -79,6 +79,7 @@ export function PaytoWireTransferForm({
   routeHere,
   onAuthorizationRequired,
   limit,
+  balance,
 }: Props): VNode {
   const [inputType, setInputType] = useState<"form" | "payto" | "qr">("form");
   const isRawPayto = inputType !== "form";
@@ -111,6 +112,16 @@ export function PaytoWireTransferForm({
       ? ("x-taler-bank" as const)
       : ("iban" as const);
 
+  const wireFee =
+    config.wire_transfer_fees === undefined
+      ? Amounts.zeroOfCurrency(config.currency)
+      : Amounts.parseOrThrow(config.wire_transfer_fees);
+
+  const limitWithFee =
+    Amounts.cmp(limit, wireFee) === 1
+      ? Amounts.sub(limit, wireFee).amount
+      : Amounts.zeroOfAmount(limit);
+
   const errorsWire = undefinedIfEmpty({
     account: !account
       ? i18n.str`Required`
@@ -124,7 +135,7 @@ export function PaytoWireTransferForm({
       ? i18n.str`Required`
       : !parsedAmount
         ? i18n.str`Not valid`
-        : validateAmount(parsedAmount, limit, i18n),
+        : validateAmount(parsedAmount, limitWithFee, i18n),
   });
 
   const parsed = !rawPaytoInput ? undefined : parsePaytoUri(rawPaytoInput);
@@ -134,7 +145,7 @@ export function PaytoWireTransferForm({
       ? i18n.str`Required`
       : !parsed
         ? i18n.str`Does not follow the pattern`
-        : validateRawPayto(parsed, limit, url.host, i18n, paytoType),
+        : validateRawPayto(parsed, limitWithFee, url.host, i18n, paytoType),
   });
 
   async function doSend() {
@@ -479,9 +490,9 @@ export function PaytoWireTransferForm({
           e.preventDefault();
         }}
       >
-        <div class="p-4 sm:p-8">
+        <div class="m-4">
           {!isRawPayto ? (
-            <div class="grid max-w-xs grid-cols-1 gap-x-6 gap-y-8 sm:grid-cols-6">
+            <div class="grid max-w-xs grid-cols-1 gap-x-6 gap-y-8 ">
               {(() => {
                 switch (paytoType) {
                   case "x-taler-bank": {
@@ -622,7 +633,45 @@ export function PaytoWireTransferForm({
               </div>
             </div>
           )}
+          {Amounts.cmp(limitWithFee, balance) > 0 ? (
+            <p class="mt-2 text-sm text-gray-900">
+              <i18n.Translate>
+                You can transfer{" "}
+                <RenderAmount
+                  value={limitWithFee}
+                  spec={config.currency_specification}
+                />
+              </i18n.Translate>
+            </p>
+          ) : undefined}
         </div>
+        {Amounts.isZero(wireFee) ? undefined : (
+          <div class="px-4 my-4">
+            <div class="grid max-w-2xl grid-cols-1 gap-x-6 gap-y-8 sm:grid-cols-6">
+              <div class="sm:col-span-6">
+                <dl class="mt-4 space-y-4">
+                  <Fragment>
+                    <div class="flex items-center justify-between ">
+                      <dt class="flex items-center text-sm text-gray-600">
+                        <span>
+                          <i18n.Translate>Cost</i18n.Translate>
+                        </span>
+                      </dt>
+                      <dd class="text-sm text-gray-900">
+                        <RenderAmount
+                          value={wireFee}
+                          negative
+                          withColor
+                          spec={config.currency_specification}
+                        />
+                      </dd>
+                    </div>
+                  </Fragment>
+                </dl>
+              </div>
+            </div>
+          </div>
+        )}
         <div class="flex items-center justify-between gap-x-6 border-t border-gray-900/10 px-4 py-4 sm:px-8">
           {routeCancel ? (
             <a
