@@ -76,11 +76,12 @@ export function QrCodeSection({
   return (
     <Fragment>
       <LocalNotificationBanner notification={notification} />
+
       <div class="bg-white shadow-xl sm:rounded-lg">
         <div class="px-4 py-5 sm:p-6">
           <h3 class="text-base font-semibold leading-6 text-gray-900">
             <i18n.Translate>
-              If you have a Taler wallet installed in this device
+              If you have a Taler wallet installed on this device
             </i18n.Translate>
           </h3>
           <div class="mt-4 mb-4 text-sm text-gray-500">
@@ -88,19 +89,19 @@ export function QrCodeSection({
               <i18n.Translate>
                 Your wallet will display the details of the transaction
                 including the fees (if applicable). If you do not yet have a
-                wallet, please follow the instructions on
-              </i18n.Translate>
+                wallet, please follow the instructions
+              </i18n.Translate>{" "}
               <a
-                class="font-semibold text-gray-500 hover:text-gray-400"
+                class="font-semibold text-indigo-600 hover:text-indigo-900"
                 name="wallet page"
                 href="https://taler.net/en/wallet.html"
               >
-                <i18n.Translate>this page</i18n.Translate>
+                <i18n.Translate>on this page</i18n.Translate>
               </a>
               .
             </p>
           </div>
-          <div class="flex items-center justify-between gap-x-6 border-t border-gray-900/10 pt-2 mt-2 ">
+          <div class="flex items-center justify-between gap-x-6  pt-2 mt-2 ">
             <Button
               type="button"
               name="cancel"
@@ -124,7 +125,7 @@ export function QrCodeSection({
         <div class="px-4 py-5 sm:p-6">
           <h3 class="text-base font-semibold leading-6 text-gray-900">
             <i18n.Translate>
-              Or if you have the Taler wallet in another device
+              Or if you have the Taler wallet on another device
             </i18n.Translate>
           </h3>
           <div class="mt-4 max-w-xl text-sm text-gray-500">
