@@ -612,60 +612,60 @@ function createFormValidator(
     const errors = undefinedIfEmpty<FormErrors<FormType>>({
       conv: undefinedIfEmpty<FormErrors<FormType["conv"]>>({
         cashin_min_amount: !state.conv.cashin_min_amount
-          ? i18n.str`required`
+          ? i18n.str`Required`
           : !cashin_min_amount
-            ? i18n.str`invalid`
+            ? i18n.str`Invalid`
             : undefined,
         cashin_tiny_amount: !state.conv.cashin_tiny_amount
-          ? i18n.str`required`
+          ? i18n.str`Required`
           : !cashin_tiny_amount
-            ? i18n.str`invalid`
+            ? i18n.str`Invalid`
             : undefined,
         cashin_fee: !state.conv.cashin_fee
-          ? i18n.str`required`
+          ? i18n.str`Required`
           : !cashin_fee
-            ? i18n.str`invalid`
+            ? i18n.str`Invalid`
             : undefined,
 
         cashout_min_amount: !state.conv.cashout_min_amount
-          ? i18n.str`required`
+          ? i18n.str`Required`
           : !cashout_min_amount
-            ? i18n.str`invalid`
+            ? i18n.str`Invalid`
             : undefined,
         cashout_tiny_amount: !state.conv.cashin_tiny_amount
-          ? i18n.str`required`
+          ? i18n.str`Required`
           : !cashout_tiny_amount
-            ? i18n.str`invalid`
+            ? i18n.str`Invalid`
             : undefined,
         cashout_fee: !state.conv.cashin_fee
-          ? i18n.str`required`
+          ? i18n.str`Required`
           : !cashout_fee
-            ? i18n.str`invalid`
+            ? i18n.str`Invalid`
             : undefined,
 
         cashin_rounding_mode: !state.conv.cashin_rounding_mode
-          ? i18n.str`required`
+          ? i18n.str`Required`
           : undefined,
         cashout_rounding_mode: !state.conv.cashout_rounding_mode
-          ? i18n.str`required`
+          ? i18n.str`Required`
           : undefined,
 
         cashin_ratio: !state.conv.cashin_ratio
-          ? i18n.str`required`
+          ? i18n.str`Required`
           : Number.isNaN(cashin_ratio)
-            ? i18n.str`invalid`
+            ? i18n.str`Invalid`
             : undefined,
         cashout_ratio: !state.conv.cashout_ratio
-          ? i18n.str`required`
+          ? i18n.str`Required`
           : Number.isNaN(cashout_ratio)
-            ? i18n.str`invalid`
+            ? i18n.str`Rnvalid`
             : undefined,
       }),
 
       amount: !state.amount
-        ? i18n.str`required`
+        ? i18n.str`Required`
         : !am
-          ? i18n.str`invalid`
+          ? i18n.str`Invalid`
           : undefined,
     });
 
