@@ -13,14 +13,27 @@
  You should have received a copy of the GNU General Public License along with
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
-import { VNode, h } from "preact";
+import { Attention, useTranslationContext } from "@gnu-taler/web-util/browser";
+import { Fragment, VNode, h } from "preact";
+import { useSessionState } from "../hooks/session.js";
 
-type Props = {
-  nonce: string;
-}
-export function CallengeCompleted({nonce}:Props):VNode {
+export function CallengeCompleted(): VNode {
+  const { state } = useSessionState();
+
+  const { i18n } = useTranslationContext();
   
-  return <div>
-    completed {nonce}
-  </div>
-}
\ No newline at end of file
+  return (
+    <div class="m-4">
+      <Attention
+        title={i18n.str`Challenge completed`}
+        type="success"
+      >
+        <i18n.Translate>
+          You will be redirected to <a href={state?.completedURL} class="break-all">&quot;
+          {state?.completedURL}
+          &quot;</a>
+        </i18n.Translate>
+      </Attention>
+    </div>
+  );
+}
