@@ -2730,16 +2730,24 @@ export class BridgeIDBTransaction
         if (BridgeIDBFactory.enableTracing) {
           console.log("beginning backend transaction to process operation");
         }
-        this._backendTransaction = await this._backend.beginTransaction(
+        const newBackendTx = await this._backend.beginTransaction(
           this._db._backendConnection,
           Array.from(this._scope),
           this.mode,
         );
         if (BridgeIDBFactory.enableTracing) {
           console.log(
-            `started backend transaction (${this._backendTransaction.transactionCookie})`,
+            `started backend transaction (${newBackendTx.transactionCookie})`,
           );
         }
+        if (this._aborted) {
+          // Probably there is a more elegant way to do this by aborting the
+          // beginTransaction call when the transaction was aborted.
+          // That would require changing the DB backend API.
+          this._backend.rollback(newBackendTx);
+        } else {
+          this._backendTransaction = newBackendTx;
+        }
       }
 
       if (!request._source) {
