@@ -31,7 +31,7 @@ export async function initTestIndexedDB(): Promise<void> {
   });
 
   idbFactory = new BridgeIDBFactory(backend);
-  backend.enableTracing = true;
+  backend.enableTracing = false;
   BridgeIDBFactory.enableTracing = false;
 }
 
