@@ -60,6 +60,9 @@ import TemplateQrPage from "./paths/instance/templates/qr/index.js";
 import TemplateUpdatePage from "./paths/instance/templates/update/index.js";
 import TemplateUsePage from "./paths/instance/templates/use/index.js";
 import TokenPage from "./paths/instance/token/index.js";
+import TokenFamilyCreatePage from "./paths/instance/tokenfamilies/create/index.js";
+import TokenFamilyListPage from "./paths/instance/tokenfamilies/list/index.js";
+import TokenFamilyUpdatePage from "./paths/instance/tokenfamilies/update/index.js";
 import TransferCreatePage from "./paths/instance/transfers/create/index.js";
 import TransferListPage from "./paths/instance/transfers/list/index.js";
 import InstanceUpdatePage, {
@@ -70,7 +73,6 @@ import WebhookCreatePage from "./paths/instance/webhooks/create/index.js";
 import WebhookListPage from "./paths/instance/webhooks/list/index.js";
 import WebhookUpdatePage from "./paths/instance/webhooks/update/index.js";
 import { LoginPage } from "./paths/login/index.js";
-import { NotFoundPage } from "./paths/notfound/index.js";
 import { Settings } from "./paths/settings/index.js";
 import { Notification } from "./utils/types.js";
 
@@ -91,10 +93,6 @@ export enum InstancePaths {
   order_new = "/order/new",
   order_details = "/order/:oid/details",
 
-  reserves_list = "/reserves",
-  reserves_details = "/reserves/:rid/details",
-  reserves_new = "/reserves/new",
-
   kyc = "/kyc",
 
   transfers_list = "/transfers",
@@ -106,6 +104,10 @@ export enum InstancePaths {
   templates_use = "/templates/:tid/use",
   templates_qr = "/templates/:tid/qr",
 
+  token_family_list = "/tokenfamilies",
+  token_family_update = "/tokenfamilies/:slug/update",
+  token_family_new = "/tokenfamilies/new",
+
   webhooks_list = "/webhooks",
   webhooks_update = "/webhooks/:tid/update",
   webhooks_new = "/webhooks/new",
@@ -117,9 +119,6 @@ export enum InstancePaths {
   interface = "/interface",
 }
 
-// eslint-disable-next-line @typescript-eslint/no-empty-function
-// const noop = () => { };
-
 export enum AdminPaths {
   list_instances = "/instances",
   new_instance = "/instance/new",
@@ -139,7 +138,6 @@ export const publicPages = {
 
 const history = createHashHistory();
 export function Routing(_p: Props): VNode {
-  // const { i18n } = useTranslationContext();
   const { state } = useSessionContext();
 
   type GlobalNotifState =
@@ -163,81 +161,10 @@ export function Routing(_p: Props): VNode {
     accounts !== undefined &&
     accounts.accounts.length < 1 &&
     (AbsoluteTime.isNever(preference.hideMissingAccountUntil) ||
-      AbsoluteTime.cmp(now, preference.hideMissingAccountUntil) > 1);
+      AbsoluteTime.cmp(now, preference.hideMissingAccountUntil) > 0);
 
   const shouldLogin = state.status === "loggedOut";
 
-  // function ServerErrorRedirectTo(to: InstancePaths | AdminPaths) {
-  //   return function ServerErrorRedirectToImpl(
-  //     error: HttpError<TalerErrorDetail>,
-  //   ) {
-  //     if (error.type === ErrorType.TIMEOUT) {
-  //       setGlobalNotification({
-  //         message: i18n.str`The request to the backend take too long and was cancelled`,
-  //         description: i18n.str`Diagnostic from ${error.info.url} is "${error.message}"`,
-  //         type: "ERROR",
-  //         to,
-  //       });
-  //     } else {
-  //       setGlobalNotification({
-  //         message: i18n.str`The backend reported a problem: HTTP status #${error.status}`,
-  //         description: i18n.str`Diagnostic from ${error.info.url} is '${error.message}'`,
-  //         details:
-  //           error.type === ErrorType.CLIENT || error.type === ErrorType.SERVER
-  //             ? error.payload.hint
-  //             : undefined,
-  //         type: "ERROR",
-  //         to,
-  //       });
-  //     }
-  //     return <Redirect to={to} />;
-  //   };
-  // }
-
-  // const LoginPageAccessDeniend = onUnauthorized
-  // const LoginPageAccessDenied = () => {
-  //   return (
-  //     <Fragment>
-  //       <NotificationCard
-  //         notification={{
-  //           message: i18n.str`Access denied`,
-  //           description: i18n.str`Session expired or password changed.`,
-  //           type: "ERROR",
-  //         }}
-  //       />
-  //       <LoginPage />
-  //     </Fragment>
-  //   );
-  // };
-
-  // function IfAdminCreateDefaultOr<T>(Next: FunctionComponent<unknown>) {
-  //   return function IfAdminCreateDefaultOrImpl(props?: T) {
-  //     if (state.isAdmin && state.instance === DEFAULT_ADMIN_USERNAME) {
-  //       return (
-  //         <Fragment>
-  //           <NotificationCard
-  //             notification={{
-  //               message: i18n.str`No 'default' instance configured yet.`,
-  //               description: i18n.str`Create a 'default' instance to begin using the merchant backoffice.`,
-  //               type: "INFO",
-  //             }}
-  //           />
-  //           <InstanceCreatePage
-  //             forceId={DEFAULT_ADMIN_USERNAME}
-  //             onConfirm={() => {
-  //               route(InstancePaths.bank_list);
-  //             }}
-  //           />
-  //         </Fragment>
-  //       );
-  //     }
-  //     if (props) {
-  //       return <Next {...props} />;
-  //     }
-  //     return <Next />;
-  //   };
-  // }
-
   if (shouldLogin) {
     return (
       <Fragment>
@@ -251,12 +178,24 @@ export function Routing(_p: Props): VNode {
     return (
       <Fragment>
         <Menu />
-        <BankAccountBanner />
-        <BankAccountCreatePage
-          onConfirm={() => {
-            route(InstancePaths.bank_list);
-          }}
-        />
+        <Router history={history}>
+          <Route path={InstancePaths.interface} component={Settings} />
+          <Route
+            default
+            component={() => {
+              return (
+                <Fragment>
+                  <BankAccountBanner />
+                  <BankAccountCreatePage
+                    onConfirm={() => {
+                      route(InstancePaths.bank_list);
+                    }}
+                  />
+                </Fragment>
+              );
+            }}
+          />
+        </Router>
       </Fragment>
     );
   }
@@ -267,21 +206,24 @@ export function Routing(_p: Props): VNode {
       <KycBanner />
       <NotificationCard notification={globalNotification} />
       {error && (
-        <NotificationCard
-          notification={{
-            message: "Internal error, please repot",
-            type: "ERROR",
-            description: (
-              <pre>
-                {
-                  (error instanceof Error
-                    ? error.stack
-                    : String(error)) as TranslatedString
-                }
-              </pre>
-            ),
-          }}
-        />
+        <Fragment>
+          <NotificationCard
+            notification={{
+              message: "Internal error, please report",
+              type: "ERROR",
+
+              description: (
+                <pre>
+                  {
+                    (error instanceof Error
+                      ? error.stack
+                      : String(error)) as TranslatedString
+                  }
+                </pre>
+              ),
+            }}
+          />
+        </Fragment>
       )}
 
       <Router
@@ -472,6 +414,39 @@ export function Routing(_p: Props): VNode {
             route(InstancePaths.transfers_list);
           }}
         />
+        {/* *
+         * Token family pages
+         */}
+        <Route
+          path={InstancePaths.token_family_list}
+          component={TokenFamilyListPage}
+          onCreate={() => {
+            route(InstancePaths.token_family_new);
+          }}
+          onSelect={(slug: string) => {
+            route(InstancePaths.token_family_update.replace(":slug", slug));
+          }}
+        />
+        <Route
+          path={InstancePaths.token_family_update}
+          component={TokenFamilyUpdatePage}
+          onConfirm={() => {
+            route(InstancePaths.token_family_list);
+          }}
+          onBack={() => {
+            route(InstancePaths.token_family_list);
+          }}
+        />
+        <Route
+          path={InstancePaths.token_family_new}
+          component={TokenFamilyCreatePage}
+          onConfirm={() => {
+            route(InstancePaths.token_family_list);
+          }}
+          onBack={() => {
+            route(InstancePaths.token_family_list);
+          }}
+        />
         {/**
          * Webhooks pages
          */}
@@ -601,7 +576,7 @@ export function Routing(_p: Props): VNode {
          * Example pages
          */}
         <Route path="/loading" component={Loading} />
-        <Route default component={NotFoundPage} />
+        <Route default component={Redirect} to={InstancePaths.order_list} />
       </Router>
     </Fragment>
   );
@@ -618,52 +593,12 @@ function AdminInstanceUpdatePage({
   id,
   ...rest
 }: { id: string } & InstanceUpdatePageProps): VNode {
-  // const { i18n } = useTranslationContext();
 
   return (
     <Fragment>
       <InstanceAdminUpdatePage
         {...rest}
         instanceId={id}
-        // onLoadError={(error: HttpError<TalerErrorDetail>) => {
-        //   const notif =
-        //     error.type === ErrorType.TIMEOUT
-        //       ? {
-        //         message: i18n.str`The request to the backend take too long and was cancelled`,
-        //         description: i18n.str`Diagnostic from ${error.info.url} is '${error.message}'`,
-        //         type: "ERROR" as const,
-        //       }
-        //       : {
-        //         message: i18n.str`The backend reported a problem: HTTP status #${error.status}`,
-        //         description: i18n.str`Diagnostic from ${error.info.url} is '${error.message}'`,
-        //         details:
-        //           error.type === ErrorType.CLIENT ||
-        //             error.type === ErrorType.SERVER
-        //             ? error.payload.hint
-        //             : undefined,
-        //         type: "ERROR" as const,
-        //       };
-        //   return (
-        //     <Fragment>
-        //       <NotificationCard notification={notif} />
-        //       <LoginPage />
-        //     </Fragment>
-        //   );
-        // }}
-        // onUnauthorized={() => {
-        //   return (
-        //     <Fragment>
-        //       <NotificationCard
-        //         notification={{
-        //           message: i18n.str`Access denied`,
-        //           description: i18n.str`The access token provided is invalid`,
-        //           type: "ERROR",
-        //         }}
-        //       />
-        //       <LoginPage />
-        //     </Fragment>
-        //   );
-        // }}
       />
     </Fragment>
   );
@@ -729,7 +664,7 @@ function KycBanner(): VNode {
     <NotificationCard
       notification={{
         type: "WARN",
-        message: "KYC verification needed",
+        message: i18n.str`KYC verification needed`,
         description: (
           <div>
             <p>
