@@ -50,7 +50,7 @@ export function InputArray<T>({
 
   const error = localError || formError;
 
-  const array: any[] = (value ? value! : []) as any;
+  const array: T[keyof T][] = (value ? value! : []);
   const [currentValue, setCurrentValue] = useState("");
   const { i18n } = useTranslationContext();
 
@@ -104,12 +104,12 @@ export function InputArray<T>({
                     return;
                   }
                   setLocalError(null);
-                  onChange([v, ...array] as any);
+                  onChange([v, ...array] as T[keyof T]);
                   setCurrentValue("");
                 }}
-                data-tooltip={i18n.str`add element to the list`}
+                data-tooltip={i18n.str`Add element to the list`}
               >
-                <i18n.Translate>add</i18n.Translate>
+                <i18n.Translate>Add</i18n.Translate>
               </button>
             </p>
           </div>
@@ -121,12 +121,12 @@ export function InputArray<T>({
                 class="tag is-medium is-info mb-0"
                 style={{ maxWidth: "90%" }}
               >
-                {v}
+                {String(v)}
               </span>
               <a
                 class="tag is-medium is-danger is-delete mb-0"
                 onClick={() => {
-                  onChange(array.filter((f) => f !== v) as any);
+                  onChange(array.filter((f) => f !== v) as T[keyof T]);
                   setCurrentValue(toStr(v));
                 }}
               />
