@@ -23,6 +23,7 @@ import { ComponentChildren, h, VNode } from "preact";
 import { InputWithAddon } from "./InputWithAddon.js";
 import { InputProps } from "./useField.js";
 import { AmountString } from "@gnu-taler/taler-util";
+import { useSessionContext } from "../../context/session.js";
 
 export interface Props<T> extends InputProps<T> {
   expand?: boolean;
@@ -43,7 +44,7 @@ export function InputCurrency<T>({
   children,
   side,
 }: Props<keyof T>): VNode {
-  const { config } = useMerchantApiContext();
+  const { config } = useSessionContext();
   return (
     <InputWithAddon<T>
       name={name}
