@@ -124,25 +124,25 @@ export function InputDate<T>({
           <span
             data-tooltip={
               withTimestampSupport
-                ? i18n.str`change value to unknown date`
-                : i18n.str`change value to empty`
+                ? i18n.str`Change value to unknown date`
+                : i18n.str`Change value to empty`
             }
           >
             <button
               class="button is-info mr-3"
               onClick={() => onChange(undefined as any)}
             >
-              <i18n.Translate>clear</i18n.Translate>
+              <i18n.Translate>Clear</i18n.Translate>
             </button>
           </span>
         )}
         {withTimestampSupport && (
-          <span data-tooltip={i18n.str`change value to never`}>
+          <span data-tooltip={i18n.str`Change value to never`}>
             <button
               class="button is-info"
               onClick={() => onChange({ t_s: "never" } as any)}
             >
-              <i18n.Translate>never</i18n.Translate>
+              <i18n.Translate>Never</i18n.Translate>
             </button>
           </span>
         )}
