@@ -92,7 +92,7 @@ export function InputImage<T>({
                       "",
                     ),
                   );
-                  return onChange(`data:${f[0].type};base64,${b64}` as any);
+                  return onChange(`data:${f[0].type};base64,${b64}` as T[keyof T]);
                 });
               }}
             />
@@ -102,7 +102,7 @@ export function InputImage<T>({
           {error && <p class="help is-danger">{error}</p>}
           {sizeError && (
             <p class="help is-danger">
-              <i18n.Translate>Image should be smaller than 1 MB</i18n.Translate>
+              <i18n.Translate>Image must be smaller than 1 MB</i18n.Translate>
             </p>
           )}
           {!value && (
