@@ -103,7 +103,7 @@ export function InputSearchOnList<T extends Entity>({
       <InputWithAddon<Search>
         name="name"
         label={label}
-        tooltip={i18n.str`enter description or id`}
+        tooltip={i18n.str`Enter description or id`}
         addonAfter={
           <span class="icon">
             <i class="mdi mdi-magnify" />
