@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -78,14 +78,16 @@ export function InputSelector<T>({
                 );
               })}
             </select>
-
-            {help}
           </p>
-            {required && (
-              <span class="icon has-text-danger is-right" style={{height: "2.5em"}}>
-                <i class="mdi mdi-alert" />
-              </span>
-            )}
+          <p class="help">{help}</p>
+          {required && (
+            <span
+              class="icon has-text-danger is-right"
+              style={{ height: "2.5em" }}
+            >
+              <i class="mdi mdi-alert" />
+            </span>
+          )}
           {error && <p class="help is-danger">{error}</p>}
         </div>
       </div>
