@@ -18,7 +18,7 @@
  *
  * @author Sebastian Javier Marchano (sebasjm)
  */
-import { h, VNode } from "preact";
+import { ComponentChildren, h, VNode } from "preact";
 import { InputProps, useField } from "./useField.js";
 
 interface Props<T> extends InputProps<T> {
@@ -26,11 +26,12 @@ interface Props<T> extends InputProps<T> {
   readonly?: boolean;
   expand?: boolean;
   threeState?: boolean;
+  side?: ComponentChildren;
   toBoolean?: (v?: any) => boolean | undefined;
   fromBoolean?: (s: boolean | undefined) => any;
 }
 
-const defaultToBoolean = (f?: any): boolean | undefined => f || "";
+const defaultToBoolean = (f?: any): boolean | undefined => f;
 const defaultFromBoolean = (v: boolean | undefined): any => v as any;
 
 export function InputToggle<T>({
@@ -41,6 +42,7 @@ export function InputToggle<T>({
   label,
   help,
   threeState,
+  side,
   expand,
   fromBoolean = defaultFromBoolean,
   toBoolean = defaultToBoolean,
@@ -56,7 +58,7 @@ export function InputToggle<T>({
   return (
     <div class="field is-horizontal">
       <div class="field-label is-normal">
-        <label class="label" >
+        <label class="label">
           {label}
           {tooltip && (
             <span class="icon has-tooltip-right" data-tooltip={tooltip}>
@@ -68,23 +70,37 @@ export function InputToggle<T>({
       <div class="field-body is-flex-grow-3">
         <div class="field">
           <p class={expand ? "control is-expanded" : "control"}>
+            {/* {String(toBoolean(value))} */}
             <label class="toggle" style={{ marginLeft: 4, marginTop: 0 }}>
               <input
                 type="checkbox"
-                class={toBoolean(value) === undefined ? "is-indeterminate" : "toggle-checkbox"}
+                class={"toggle-checkbox"}
                 checked={toBoolean(value)}
                 placeholder={placeholder}
+                ref={(d) => {
+                  if (d) {
+                    d.indeterminate =
+                      !!threeState && toBoolean(value) === undefined;
+                  }
+                }}
                 readonly={readonly}
                 name={String(name)}
                 disabled={readonly}
                 onChange={onCheckboxClick}
               />
-              <div class={`toggle-switch ${readonly ? "disabled" : ""}`} style={{ cursor: readonly ? "default" : undefined }}></div>
+
+              <div
+                class={`toggle-switch ${readonly ? "disabled" : ""} ${
+                  toBoolean(value) === undefined ? "no-dot" : ""
+                }`}
+                style={{ cursor: readonly ? "default" : undefined }}
+              ></div>
             </label>
-            {help}
+            <p>{help}</p>
           </p>
           {error && <p class="help is-danger">{error}</p>}
         </div>
+        {side}
       </div>
     </div>
   );
