@@ -19,15 +19,13 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import {
-  useMerchantApiContext,
-  useTranslationContext,
-} from "@gnu-taler/web-util/browser";
+import { TalerError } from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useSessionContext } from "../../context/session.js";
 import { useInstanceKYCDetails } from "../../hooks/instance.js";
 import { LangSelector } from "./LangSelector.js";
-import { TalerError } from "@gnu-taler/taler-util";
+import { usePreference } from "../../hooks/preference.js";
 
 // const GIT_HASH = typeof __GIT_HASH__ !== "undefined" ? __GIT_HASH__ : undefined;
 const VERSION = typeof __VERSION__ !== "undefined" ? __VERSION__ : undefined;
@@ -38,17 +36,17 @@ interface Props {
 
 export function Sidebar({ mobile }: Props): VNode {
   const { i18n } = useTranslationContext();
+  const { state, logOut, config } = useSessionContext();
   const kycStatus = useInstanceKYCDetails();
+  const [pref] = usePreference();
 
   const needKYC =
     kycStatus !== undefined &&
     !(kycStatus instanceof TalerError) &&
     kycStatus.type === "ok" &&
     !!kycStatus.body;
-  const { state, logOut } = useSessionContext();
   const isLoggedIn = state.status === "loggedIn";
   const hasToken = isLoggedIn && state.token !== undefined;
-  const { config, url: backendURL } = useMerchantApiContext();
 
   return (
     <aside
@@ -115,13 +113,25 @@ export function Sidebar({ mobile }: Props): VNode {
               <li>
                 <a href={"/templates"} class="has-icon">
                   <span class="icon">
-                    <i class="mdi mdi-newspaper" />
+                    <i class="mdi mdi-qrcode" />
                   </span>
                   <span class="menu-item-label">
                     <i18n.Translate>Templates</i18n.Translate>
                   </span>
                 </a>
               </li>
+              {pref.developerMode ? (
+                <li>
+                  <a href={"/tokenfamilies"} class="has-icon">
+                    <span class="icon">
+                      <i class="mdi mdi-key" />
+                    </span>
+                    <span class="menu-item-label">
+                      <i18n.Translate>Token Families</i18n.Translate>
+                    </span>
+                  </a>
+                </li>
+              ) : undefined}
               {needKYC && (
                 <li>
                   <a href={"/kyc"} class="has-icon">
@@ -160,7 +170,7 @@ export function Sidebar({ mobile }: Props): VNode {
               <li>
                 <a href={"/webhooks"} class="has-icon">
                   <span class="icon">
-                    <i class="mdi mdi-newspaper" />
+                    <i class="mdi mdi-webhook" />
                   </span>
                   <span class="menu-item-label">
                     <i18n.Translate>Webhooks</i18n.Translate>
@@ -195,10 +205,7 @@ export function Sidebar({ mobile }: Props): VNode {
         </p>
         <ul class="menu-list">
           <li>
-            <a
-              class="has-icon is-state-info is-hoverable"
-              href="/interface"
-            >
+            <a class="has-icon is-state-info is-hoverable" href="/interface">
               <span class="icon">
                 <i class="mdi mdi-newspaper" />
               </span>
@@ -212,9 +219,7 @@ export function Sidebar({ mobile }: Props): VNode {
               <span style={{ width: "3rem" }} class="icon">
                 <i class="mdi mdi-web" />
               </span>
-              <span class="menu-item-label">
-                {backendURL.hostname}
-              </span>
+              <span class="menu-item-label">{state.backendUrl.hostname}</span>
             </div>
           </li>
           <li>
