@@ -28,6 +28,12 @@ function getInstanceTitle(path: string, id: string): string {
   switch (path) {
     case InstancePaths.settings:
       return `${id}: Settings`;
+    case InstancePaths.bank_new:
+      return `${id}: New bank account`;
+    case InstancePaths.bank_list:
+      return `${id}: Bank accounts`;
+    case InstancePaths.bank_update:
+      return `${id}: Update bank Account`;
     case InstancePaths.order_list:
       return `${id}: Orders`;
     case InstancePaths.order_new:
@@ -38,10 +44,6 @@ function getInstanceTitle(path: string, id: string): string {
       return `${id}: New product`;
     case InstancePaths.inventory_update:
       return `${id}: Update product`;
-    case InstancePaths.reserves_new:
-      return `${id}: New reserve`;
-    case InstancePaths.reserves_list:
-      return `${id}: Reserves`;
     case InstancePaths.transfers_list:
       return `${id}: Transfers`;
     case InstancePaths.transfers_new:
@@ -53,11 +55,11 @@ function getInstanceTitle(path: string, id: string): string {
     case InstancePaths.webhooks_update:
       return `${id}: Update webhook`;
     case InstancePaths.otp_devices_list:
-      return `${id}: otp devices`;
+      return `${id}: OTP devices`;
     case InstancePaths.otp_devices_new:
-      return `${id}: New otp devices`;
+      return `${id}: New OTP device`;
     case InstancePaths.otp_devices_update:
-      return `${id}: Update otp devices`;
+      return `${id}: Update OTP device`;
     case InstancePaths.templates_new:
       return `${id}: New template`;
     case InstancePaths.templates_update:
@@ -68,6 +70,12 @@ function getInstanceTitle(path: string, id: string): string {
       return `${id}: Use template`;
     case InstancePaths.interface:
       return `${id}: Interface`;
+    case InstancePaths.token_family_list:
+      return `${id}: Token families`;
+    case InstancePaths.token_family_new:
+      return `${id}: New token family`;
+    case InstancePaths.token_family_update:
+      return `${id}: Update token family`;
     default:
       return "";
   }
