@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -16,24 +16,25 @@
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { h, VNode } from "preact";
 import { useState } from "preact/hooks";
-import { MerchantBackend, WithId } from "../../declaration.js";
 import { ProductMap } from "../../paths/instance/orders/create/CreatePage.js";
 import { FormErrors, FormProvider } from "../form/FormProvider.js";
 import { InputNumber } from "../form/InputNumber.js";
 import { InputSearchOnList } from "../form/InputSearchOnList.js";
+import { TalerMerchantApi } from "@gnu-taler/taler-util";
+import { WithId } from "../../declaration.js";
 
 type Form = {
-  product: MerchantBackend.Products.ProductDetail & WithId;
+  product: TalerMerchantApi.ProductDetail & WithId;
   quantity: number;
 };
 
 interface Props {
   currentProducts: ProductMap;
   onAddProduct: (
-    product: MerchantBackend.Products.ProductDetail & WithId,
+    product: TalerMerchantApi.ProductDetail & WithId,
     quantity: number,
   ) => void;
-  inventory: (MerchantBackend.Products.ProductDetail & WithId)[];
+  inventory: (TalerMerchantApi.ProductDetail & WithId)[];
 }
 
 export function InventoryProductForm({
@@ -109,7 +110,7 @@ export function InventoryProductForm({
               <InputNumber<Form>
                 name="quantity"
                 label={i18n.str`Quantity`}
-                tooltip={i18n.str`how many products will be added`}
+                tooltip={i18n.str`How many products will be added`}
               />
             )}
           </div>
