@@ -13,13 +13,12 @@
  You should have received a copy of the GNU General Public License along with
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
-import { AmountString, TalerMerchantApi } from "@gnu-taler/taler-util";
+import { AmountString, Amounts, TalerMerchantApi } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
-import { Fragment, h, VNode } from "preact";
+import { Fragment, VNode, h } from "preact";
 import { useCallback, useEffect, useState } from "preact/hooks";
-import * as yup from "yup";
 import { useListener } from "../../hooks/listener.js";
-import { NonInventoryProductSchema as schema } from "../../schemas/index.js";
+import { undefinedIfEmpty } from "../../utils/table.js";
 import { FormErrors, FormProvider } from "../form/FormProvider.js";
 import { Input } from "../form/Input.js";
 import { InputCurrency } from "../form/InputCurrency.js";
@@ -69,7 +68,7 @@ export function NonInventoryProductFrom({
       <div class="buttons">
         <button
           class="button is-success"
-          data-tooltip={i18n.str`describe and add a product that is not in the inventory list`}
+          data-tooltip={i18n.str`Describe and add a product that is not in the inventory list`}
           onClick={() => setShowCreateProduct(true)}
         >
           <i18n.Translate>Add custom product</i18n.Translate>
@@ -140,38 +139,39 @@ interface NonInventoryProduct {
 }
 
 export function ProductForm({ onSubscribe, initial }: ProductProps): VNode {
+  const { i18n } = useTranslationContext();
   const [value, valueHandler] = useState<Partial<NonInventoryProduct>>({
     taxes: [],
     ...initial,
   });
-  let errors: FormErrors<NonInventoryProduct> = {};
-  try {
-    schema.validateSync(value, { abortEarly: false });
-  } catch (err) {
-    if (err instanceof yup.ValidationError) {
-      const yupErrors = err.inner as yup.ValidationError[];
-      errors = yupErrors.reduce(
-        (prev, cur) =>
-          !cur.path ? prev : { ...prev, [cur.path]: cur.message },
-        {},
-      );
-    }
-  }
+  const errors = undefinedIfEmpty<FormErrors<NonInventoryProduct>>({
+    quantity:
+      value.quantity === undefined
+        ? i18n.str`Required`
+        : typeof value.quantity !== "number"
+          ? i18n.str`Must be a number`
+          : value.quantity < 1
+            ? i18n.str`Must be grater than 0`
+            : undefined,
+    description: !value.description ? i18n.str`Required` : undefined,
+    unit: !value.description ? i18n.str`Required` : undefined,
+    price: !value.price
+      ? i18n.str`Required`
+      : Amounts.parse(value.price) === undefined
+        ? i18n.str`Invalid`
+        : undefined,
+  });
 
   const submit = useCallback((): Entity | undefined => {
     return value as TalerMerchantApi.Product;
   }, [value]);
 
-  const hasErrors = Object.keys(errors).some(
-    (k) => (errors as any)[k] !== undefined,
-  );
+  const hasErrors = errors !== undefined;
 
   useEffect(() => {
     onSubscribe(hasErrors ? undefined : submit);
   }, [submit, hasErrors]);
 
-  const { i18n } = useTranslationContext();
-
   return (
     <div>
       <FormProvider<NonInventoryProduct>
@@ -183,29 +183,29 @@ export function ProductForm({ onSubscribe, initial }: ProductProps): VNode {
         <InputImage<NonInventoryProduct>
           name="image"
           label={i18n.str`Image`}
-          tooltip={i18n.str`photo of the product`}
+          tooltip={i18n.str`Photo of the product.`}
         />
         <Input<NonInventoryProduct>
           name="description"
           inputType="multiline"
           label={i18n.str`Description`}
-          tooltip={i18n.str`full product description`}
+          tooltip={i18n.str`Full product description.`}
         />
         <Input<NonInventoryProduct>
           name="unit"
           label={i18n.str`Unit`}
-          tooltip={i18n.str`name of the product unit`}
+          tooltip={i18n.str`Name of the product unit.`}
         />
         <InputCurrency<NonInventoryProduct>
           name="price"
           label={i18n.str`Price`}
-          tooltip={i18n.str`amount in the current currency`}
+          tooltip={i18n.str`Amount in the current currency.`}
         />
 
         <InputNumber<NonInventoryProduct>
           name="quantity"
           label={i18n.str`Quantity`}
-          tooltip={i18n.str`how many products will be added`}
+          tooltip={i18n.str`How many products will be added.`}
         />
 
         <InputTaxes<NonInventoryProduct> name="taxes" label={i18n.str`Taxes`} />
