@@ -19,17 +19,13 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { AmountString, TalerMerchantApi } from "@gnu-taler/taler-util";
+import { AmountString, Amounts, TalerMerchantApi } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { h } from "preact";
 import { useCallback, useEffect, useState } from "preact/hooks";
-import * as yup from "yup";
 import { useSessionContext } from "../../context/session.js";
-import {
-  ProductCreateSchema as createSchema,
-  ProductUpdateSchema as updateSchema,
-} from "../../schemas/index.js";
-import { FormErrors, FormProvider } from "../form/FormProvider.js";
+import { undefinedIfEmpty } from "../../utils/table.js";
+import { FormProvider } from "../form/FormProvider.js";
 import { Input } from "../form/Input.js";
 import { InputCurrency } from "../form/InputCurrency.js";
 import { InputImage } from "../form/InputImage.js";
@@ -47,6 +43,9 @@ interface Props {
 }
 
 export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
+  const { i18n } = useTranslationContext();
+  const { state } = useSessionContext();
+
   const [value, valueHandler] = useState<Partial<Entity & { stock: Stock }>>({
     address: {},
     description_i18n: {},
@@ -65,25 +64,30 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
             nextRestock: initial.next_restock,
           },
   });
-  let errors: FormErrors<Entity> = {};
 
-  try {
-    (alreadyExist ? updateSchema : createSchema).validateSync(value, {
-      abortEarly: false,
-    });
-  } catch (err) {
-    if (err instanceof yup.ValidationError) {
-      const yupErrors = err.inner as yup.ValidationError[];
-      errors = yupErrors.reduce(
-        (prev, cur) =>
-          !cur.path ? prev : { ...prev, [cur.path]: cur.message },
-        {},
-      );
-    }
+  const errors = undefinedIfEmpty({
+    product_id: !value.product_id ? i18n.str`Required` : undefined,
+    description: !value.description ? i18n.str`Required` : undefined,
+    unit: !value.unit ? i18n.str`Required` : undefined,
+    price: !value.price
+      ? i18n.str`Required`
+      : Amounts.parse(value.price) === undefined
+        ? i18n.str`Invalid amount`
+        : undefined,
+    minimum_age:
+      value.minimum_age === undefined
+        ? undefined
+        : value.minimum_age < 1
+          ? i18n.str`Must be greater than 0`
+          : undefined,
+  });
+
+  if (alreadyExist && errors) {
+    // on update, we remove some validations
+    delete errors.product_id;
   }
-  const hasErrors = Object.keys(errors).some(
-    (k) => (errors as Record<string, unknown>)[k] !== undefined,
-  );
+
+  const hasErrors = errors !== undefined;
 
   const submit = useCallback((): Entity | undefined => {
     const stock = value.stock;
@@ -114,8 +118,6 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
     onSubscribe(hasErrors ? undefined : submit);
   }, [submit, hasErrors]);
 
-  const { i18n } = useTranslationContext();
-  const { state } = useSessionContext();
   return (
     <div>
       <FormProvider<Entity>
@@ -129,47 +131,47 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
             name="product_id"
             addonBefore={new URL("product/", state.backendUrl.href).href}
             label={i18n.str`ID`}
-            tooltip={i18n.str`product identification to use in URLs (for internal use only)`}
+            tooltip={i18n.str`Product identification to use in URLs (for internal use only)`}
           />
         )}
         <InputImage<Entity>
           name="image"
           label={i18n.str`Image`}
-          tooltip={i18n.str`illustration of the product for customers`}
+          tooltip={i18n.str`Illustration of the product for customers`}
         />
         <Input<Entity>
           name="description"
           inputType="multiline"
           label={i18n.str`Description`}
-          tooltip={i18n.str`product description for customers`}
+          tooltip={i18n.str`Product description for customers`}
         />
         <InputNumber<Entity>
           name="minimum_age"
           label={i18n.str`Age restriction`}
-          tooltip={i18n.str`is this product restricted for customer below certain age?`}
-          help={i18n.str`minimum age of the customer`}
+          tooltip={i18n.str`Is this product restricted for customer below certain age?`}
+          help={i18n.str`Minimum age of the customer`}
         />
         <Input<Entity>
           name="unit"
           label={i18n.str`Unit name`}
-          tooltip={i18n.str`unit describing quantity of product sold (e.g. 2 kilograms, 5 liters, 3 items, 5 meters) for customers`}
-          help={i18n.str`example: kg, items or liters`}
+          tooltip={i18n.str`Unit describing quantity of product sold (e.g. 2 kilograms, 5 liters, 3 items, 5 meters) for customers`}
+          help={i18n.str`Example: kg, items or liters`}
         />
         <InputCurrency<Entity>
           name="price"
           label={i18n.str`Price per unit`}
-          tooltip={i18n.str`sale price for customers, including taxes, for above units of the product`}
+          tooltip={i18n.str`Sale price for customers, including taxes, for above units of the product`}
         />
         <InputStock
           name="stock"
           label={i18n.str`Stock`}
           alreadyExist={alreadyExist}
-          tooltip={i18n.str`inventory for products with finite supply (for internal use only)`}
+          tooltip={i18n.str`Inventory for products with finite supply (for internal use only)`}
         />
         <InputTaxes<Entity>
           name="taxes"
           label={i18n.str`Taxes`}
-          tooltip={i18n.str`taxes included in the product price, exposed to customers`}
+          tooltip={i18n.str`Taxes included in the product price, exposed to customers`}
         />
       </FormProvider>
     </div>
