@@ -13,21 +13,18 @@
  You should have received a copy of the GNU General Public License along with
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
-import {
-  useMerchantApiContext
-} from "@gnu-taler/web-util/browser";
-import { useState } from "preact/hooks";
 
 // FIX default import https://github.com/microsoft/TypeScript/issues/49189
-import { AccessToken, TalerHttpError, TalerMerchantManagementResultByMethod } from "@gnu-taler/taler-util";
+import {
+  AccessToken,
+  TalerHttpError,
+  TalerMerchantManagementResultByMethod,
+} from "@gnu-taler/taler-util";
 import _useSWR, { SWRHook, mutate } from "swr";
 import { useSessionContext } from "../context/session.js";
-import { PAGINATED_LIST_REQUEST } from "../utils/constants.js";
-import { buildPaginatedResult } from "./webhooks.js";
 const useSWR = _useSWR as unknown as SWRHook;
 
-export interface InstanceBankAccountFilter {
-}
+export interface InstanceBankAccountFilter {}
 
 export function revalidateInstanceBankAccounts() {
   return mutate(
@@ -37,13 +34,12 @@ export function revalidateInstanceBankAccounts() {
   );
 }
 export function useInstanceBankAccounts() {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useMerchantApiContext();
+  const { state, lib } = useSessionContext();
 
   // const [offset, setOffset] = useState<string | undefined>();
 
-  async function fetcher([token, bid]: [AccessToken, string]) {
-    return await instance.listBankAccounts(token, {
+  async function fetcher([token, _bid]: [AccessToken, string]) {
+    return await lib.instance.listBankAccounts(token, {
       // limit: PAGINATED_LIST_REQUEST,
       // offset: bid,
       // order: "dec",
@@ -53,35 +49,37 @@ export function useInstanceBankAccounts() {
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"listBankAccounts">,
     TalerHttpError
-  >([session.token, "offset", "listBankAccounts"], fetcher);
+  >([state.token, "offset", "listBankAccounts"], fetcher);
 
   if (error) return error;
   if (data === undefined) return undefined;
   if (data.type !== "ok") return data;
 
   // return buildPaginatedResult(data.body.accounts, offset, setOffset, (d) => d.h_wire)
+  const filtered = data.body.accounts.filter((a) => a.active);
+  data.body.accounts = filtered;
   return data;
 }
 
 export function revalidateBankAccountDetails() {
   return mutate(
-    (key) => Array.isArray(key) && key[key.length - 1] === "getBankAccountDetails",
+    (key) =>
+      Array.isArray(key) && key[key.length - 1] === "getBankAccountDetails",
     undefined,
     { revalidate: true },
   );
 }
 export function useBankAccountDetails(h_wire: string) {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useMerchantApiContext();
+  const { state, lib } = useSessionContext();
 
   async function fetcher([token, wireId]: [AccessToken, string]) {
-    return await instance.getBankAccountDetails(token, wireId);
+    return await lib.instance.getBankAccountDetails(token, wireId);
   }
 
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"getBankAccountDetails">,
     TalerHttpError
-  >([session.token, h_wire, "getBankAccountDetails"], fetcher);
+  >([state.token, h_wire, "getBankAccountDetails"], fetcher);
 
   if (data) return data;
   if (error) return error;
