@@ -36,13 +36,12 @@ export function revalidateInstanceProducts() {
   );
 }
 export function useInstanceProducts() {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useSessionContext();
+  const { state, lib } = useSessionContext();
 
   const [offset, setOffset] = useState<number | undefined>();
 
   async function fetcher([token, bid]: [AccessToken, number]) {
-    const list = await instance.listProducts(token, {
+    const list = await lib.instance.listProducts(token, {
       limit: PAGINATED_LIST_REQUEST,
       offset: bid === undefined ? undefined: String(bid),
       order: "dec",
@@ -52,7 +51,7 @@ export function useInstanceProducts() {
     }
     const all: Array<ProductWithId | undefined> = await Promise.all(
       list.body.products.map(async (c) => {
-        const r = await instance.getProductDetails(token, c.product_id);
+        const r = await lib.instance.getProductDetails(token, c.product_id);
         if (r.type === "fail") {
           return undefined;
         }
@@ -68,7 +67,7 @@ export function useInstanceProducts() {
     OperationOk<{ products: ProductWithId[] }> |
     TalerMerchantManagementErrorsByMethod<"listProducts">,
     TalerHttpError
-  >([session.token, offset, "listProductsWithId"], fetcher);
+  >([state.token, offset, "listProductsWithId"], fetcher);
 
   if (error) return error;
   if (data === undefined) return undefined;
@@ -85,17 +84,16 @@ export function revalidateProductDetails() {
   );
 }
 export function useProductDetails(productId: string) {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useSessionContext();
+  const { state, lib } = useSessionContext();
 
   async function fetcher([pid, token]: [string, AccessToken]) {
-    return await instance.getProductDetails(token, pid);
+    return await lib.instance.getProductDetails(token, pid);
   }
 
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"getProductDetails">,
     TalerHttpError
-  >([productId, session.token, "getProductDetails"], fetcher);
+  >([productId, state.token, "getProductDetails"], fetcher);
 
   if (data) return data;
   if (error) return error;
