@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -21,4 +21,8 @@ import "./scss/main.scss";
 
 const app = document.getElementById("app");
 
-render(<Application />, app as any);
+if (app) {
+  render(<Application />, app);
+} else {
+  console.error("HTML element with id 'app' not found.");
+}
