@@ -20,6 +20,7 @@
 import { h, VNode } from "preact";
 import { CreatedSuccessfully } from "../../../components/notifications/CreatedSuccessfully.js";
 import { Entity } from "./index.js";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 
 export function InstanceCreatedSuccessfully({
   entity,
@@ -28,11 +29,14 @@ export function InstanceCreatedSuccessfully({
   entity: Entity;
   onConfirm: () => void;
 }): VNode {
+  const { i18n } = useTranslationContext();
   return (
     <CreatedSuccessfully onConfirm={onConfirm}>
       <div class="field is-horizontal">
         <div class="field-label is-normal">
-          <label class="label">ID</label>
+          <label class="label">
+            <i18n.Translate>ID</i18n.Translate>
+          </label>
         </div>
         <div class="field-body is-flex-grow-3">
           <div class="field">
@@ -44,7 +48,9 @@ export function InstanceCreatedSuccessfully({
       </div>
       <div class="field is-horizontal">
         <div class="field-label is-normal">
-          <label class="label">Business Name</label>
+          <label class="label">
+            <i18n.Translate>Business Name</i18n.Translate>
+          </label>
         </div>
         <div class="field-body is-flex-grow-3">
           <div class="field">
@@ -56,7 +62,9 @@ export function InstanceCreatedSuccessfully({
       </div>
       <div class="field is-horizontal">
         <div class="field-label is-normal">
-          <label class="label">Access token</label>
+          <label class="label">
+            <i18n.Translate>Access token</i18n.Translate>
+          </label>
         </div>
         <div class="field-body is-flex-grow-3">
           <div class="field">
