@@ -27,10 +27,7 @@ import {
   TalerMerchantApi,
   TalerProtocolDuration,
 } from "@gnu-taler/taler-util";
-import {
-  useMerchantApiContext,
-  useTranslationContext,
-} from "@gnu-taler/web-util/browser";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { format, isFuture } from "date-fns";
 import { Fragment, VNode, h } from "preact";
 import { useEffect, useState } from "preact/hooks";
@@ -49,9 +46,11 @@ import { InputToggle } from "../../../../components/form/InputToggle.js";
 import { InventoryProductForm } from "../../../../components/product/InventoryProductForm.js";
 import { NonInventoryProductFrom } from "../../../../components/product/NonInventoryProductForm.js";
 import { ProductList } from "../../../../components/product/ProductList.js";
+import { useSessionContext } from "../../../../context/session.js";
 import { usePreference } from "../../../../hooks/preference.js";
 import { rate } from "../../../../utils/amount.js";
 import { undefinedIfEmpty } from "../../../../utils/table.js";
+import { WithId } from "../../../../declaration.js";
 
 interface Props {
   onCreate: (d: TalerMerchantApi.PostOrderRequest) => void;
@@ -134,11 +133,11 @@ export function CreatePage({
   instanceConfig,
   instanceInventory,
 }: Props): VNode {
-  const { config } = useMerchantApiContext();
+  const { config } = useSessionContext();
   const instance_default = with_defaults(instanceConfig, config.currency);
   const [value, valueHandler] = useState(instance_default);
   const zero = Amounts.zeroOfCurrency(config.currency);
-  const [settings, updateSettings] = usePreference();
+  const [pref, updatePrefs] = usePreference();
   const inventoryList = Object.values(value.inventoryProducts || {});
   const productList = Object.values(value.products || {});
 
@@ -148,15 +147,15 @@ export function CreatePage({
     ? undefined
     : Amounts.parse(value.pricing.order_price);
 
-  const errors: FormErrors<Entity> = {
+  const errors = undefinedIfEmpty<FormErrors<Entity>>({
     pricing: undefinedIfEmpty({
-      summary: !value.pricing?.summary ? i18n.str`required` : undefined,
+      summary: !value.pricing?.summary ? i18n.str`Required` : undefined,
       order_price: !value.pricing?.order_price
-        ? i18n.str`required`
+        ? i18n.str`Required`
         : !parsedPrice
-          ? i18n.str`not valid`
+          ? i18n.str`Invalid`
           : Amounts.isZero(parsedPrice)
-            ? i18n.str`must be greater than 0`
+            ? i18n.str`Must be greater than 0`
             : undefined,
     }),
     payments: undefinedIfEmpty({
@@ -167,58 +166,59 @@ export function CreatePage({
               value.payments.refund_deadline,
               value.payments.pay_deadline,
             ) === -1
-          ? i18n.str`refund deadline cannot be before pay deadline`
+          ? i18n.str`Refund deadline can't be before pay deadline`
           : value.payments.wire_transfer_deadline &&
               Duration.cmp(
                 value.payments.wire_transfer_deadline,
                 value.payments.refund_deadline,
               ) === -1
-            ? i18n.str`wire transfer deadline cannot be before refund deadline`
+            ? i18n.str`Wire transfer deadline can't be before refund deadline`
             : undefined,
       pay_deadline: !value.payments?.pay_deadline
-        ? i18n.str`required`
+        ? i18n.str`Required`
         : value.payments.wire_transfer_deadline &&
             Duration.cmp(
               value.payments.wire_transfer_deadline,
               value.payments.pay_deadline,
             ) === -1
-          ? i18n.str`wire transfer deadline cannot be before pay deadline`
+          ? i18n.str`Wire transfer deadline can't be before pay deadline`
           : undefined,
       wire_transfer_deadline: !value.payments?.wire_transfer_deadline
-        ? i18n.str`required`
+        ? i18n.str`Required`
         : undefined,
       auto_refund_deadline: !value.payments?.auto_refund_deadline
         ? undefined
         : !value.payments?.refund_deadline
-          ? i18n.str`should have a refund deadline`
+          ? i18n.str`Must have a refund deadline`
           : Duration.cmp(
                 value.payments.refund_deadline,
                 value.payments.auto_refund_deadline,
               ) == -1
-            ? i18n.str`auto refund cannot be after refund deadline`
+            ? i18n.str`Auto refund can't be after refund deadline`
             : undefined,
     }),
     shipping: undefinedIfEmpty({
       delivery_date: !value.shipping?.delivery_date
         ? undefined
         : !isFuture(value.shipping.delivery_date)
-          ? i18n.str`should be in the future`
+          ? i18n.str`Must be in the future`
           : undefined,
     }),
-  };
-  const hasErrors = Object.keys(errors).some(
-    (k) => (errors as any)[k] !== undefined,
-  );
+  });
+  const hasErrors = errors !== undefined;
 
   const submit = (): void => {
-    const order = value as any; //schema.cast(value);
+    const order = value;
+    const price = order.pricing?.order_price as AmountString | undefined;
+    const summary = order.pricing?.summary;
     if (!value.payments) return;
     if (!value.shipping) return;
+    if (!price || ! summary) return;
 
     const request: TalerMerchantApi.PostOrderRequest = {
       order: {
-        amount: order.pricing.order_price,
-        summary: order.pricing.summary,
+        amount: price,
+        summary: summary,
         products: productList,
         extra: undefinedIfEmpty(value.extra),
         pay_deadline: AbsoluteTime.toProtocolTimestamp(
@@ -339,7 +339,9 @@ export function CreatePage({
 
   const minAgeByProducts = inventoryList.reduce(
     (cur, prev) =>
-      !prev.product.minimum_age || cur > prev.product.minimum_age ? cur : prev.product.minimum_age,
+      !prev.product.minimum_age || cur > prev.product.minimum_age
+        ? cur
+        : prev.product.minimum_age,
     0,
   );
 
@@ -360,9 +362,9 @@ export function CreatePage({
         <div class="tabs is-toggle is-fullwidth is-small">
           <ul>
             <li
-              class={!settings.advanceOrderMode ? "is-active" : ""}
+              class={!pref.advanceOrderMode ? "is-active" : ""}
               onClick={() => {
-                updateSettings("advanceOrderMode", false);
+                updatePrefs("advanceOrderMode", false);
               }}
             >
               <a>
@@ -372,9 +374,9 @@ export function CreatePage({
               </a>
             </li>
             <li
-              class={settings.advanceOrderMode ? "is-active" : ""}
+              class={pref.advanceOrderMode ? "is-active" : ""}
               onClick={() => {
-                updateSettings("advanceOrderMode", true);
+                updatePrefs("advanceOrderMode", true);
               }}
             >
               <a>
@@ -395,8 +397,10 @@ export function CreatePage({
               alternative={
                 allProducts.length > 0 && (
                   <p>
-                    {allProducts.length} products with a total price of{" "}
-                    {totalAsString}.
+                    <i18n.Translate>
+                      {allProducts.length} products with a total price of{" "}
+                      {totalAsString}.
+                    </i18n.Translate>
                   </p>
                 )
               }
@@ -408,7 +412,7 @@ export function CreatePage({
                 inventory={instanceInventory}
               />
 
-              {settings.advanceOrderMode && (
+              {pref.advanceOrderMode && (
                 <NonInventoryProductFrom
                   productToEdit={editingProduct}
                   onAddProduct={(p) => {
@@ -450,7 +454,7 @@ export function CreatePage({
                     name="pricing.products_price"
                     label={i18n.str`Total price`}
                     readonly
-                    tooltip={i18n.str`total product price added up`}
+                    tooltip={i18n.str`Total product price added up`}
                   />
                   <InputCurrency
                     name="pricing.order_price"
@@ -470,7 +474,7 @@ export function CreatePage({
                 <InputCurrency
                   name="pricing.order_price"
                   label={i18n.str`Order price`}
-                  tooltip={i18n.str`final order price`}
+                  tooltip={i18n.str`Final order price`}
                 />
               )}
 
@@ -481,7 +485,7 @@ export function CreatePage({
                 tooltip={i18n.str`Title of the order to be shown to the customer`}
               />
 
-              {settings.advanceOrderMode && (
+              {pref.advanceOrderMode && (
                 <InputGroup
                   name="shipping"
                   label={i18n.str`Shipping and Fulfillment`}
@@ -496,7 +500,7 @@ export function CreatePage({
                     <InputGroup
                       name="shipping.delivery_location"
                       label={i18n.str`Location`}
-                      tooltip={i18n.str`address where the products will be delivered`}
+                      tooltip={i18n.str`Address where the products will be delivered`}
                     >
                       <InputLocation name="shipping.delivery_location" />
                     </InputGroup>
@@ -509,13 +513,13 @@ export function CreatePage({
                 </InputGroup>
               )}
 
-              {(settings.advanceOrderMode || requiresSomeTalerOptions) && (
+              {(pref.advanceOrderMode || requiresSomeTalerOptions) && (
                 <InputGroup
                   name="payments"
                   label={i18n.str`Taler payment options`}
                   tooltip={i18n.str`Override default Taler payment settings for this order`}
                 >
-                  {(settings.advanceOrderMode || noDefault_payDeadline) && (
+                  {(pref.advanceOrderMode || noDefault_payDeadline) && (
                     <InputDuration
                       name="payments.pay_deadline"
                       label={i18n.str`Payment time`}
@@ -541,13 +545,13 @@ export function CreatePage({
                               valueHandler(c);
                             }}
                           >
-                            <i18n.Translate>default</i18n.Translate>
+                            <i18n.Translate>Default</i18n.Translate>
                           </button>
                         </span>
                       }
                     />
                   )}
-                  {settings.advanceOrderMode && (
+                  {pref.advanceOrderMode && (
                     <InputDuration
                       name="payments.refund_deadline"
                       label={i18n.str`Refund time`}
@@ -574,13 +578,13 @@ export function CreatePage({
                               });
                             }}
                           >
-                            <i18n.Translate>default</i18n.Translate>
+                            <i18n.Translate>Default</i18n.Translate>
                           </button>
                         </span>
                       }
                     />
                   )}
-                  {(settings.advanceOrderMode || noDefault_wireDeadline) && (
+                  {(pref.advanceOrderMode || noDefault_wireDeadline) && (
                     <InputDuration
                       name="payments.wire_transfer_deadline"
                       label={i18n.str`Wire transfer time`}
@@ -608,13 +612,13 @@ export function CreatePage({
                               });
                             }}
                           >
-                            <i18n.Translate>default</i18n.Translate>
+                            <i18n.Translate>Default</i18n.Translate>
                           </button>
                         </span>
                       }
                     />
                   )}
-                  {settings.advanceOrderMode && (
+                  {pref.advanceOrderMode && (
                     <InputDuration
                       name="payments.auto_refund_deadline"
                       label={i18n.str`Auto-refund time`}
@@ -628,21 +632,21 @@ export function CreatePage({
                     />
                   )}
 
-                  {settings.advanceOrderMode && (
+                  {pref.advanceOrderMode && (
                     <InputCurrency
                       name="payments.max_fee"
                       label={i18n.str`Maximum fee`}
                       tooltip={i18n.str`Maximum fees the merchant is willing to cover for this order. Higher deposit fees must be covered in full by the consumer.`}
                     />
                   )}
-                  {settings.advanceOrderMode && (
+                  {pref.advanceOrderMode && (
                     <InputToggle
                       name="payments.createToken"
                       label={i18n.str`Create token`}
                       tooltip={i18n.str`If the order ID is easy to guess the token will prevent user to steal orders from others.`}
                     />
                   )}
-                  {settings.advanceOrderMode && (
+                  {pref.advanceOrderMode && (
                     <InputNumber
                       name="payments.minimum_age"
                       label={i18n.str`Minimum age required`}
@@ -657,7 +661,7 @@ export function CreatePage({
                 </InputGroup>
               )}
 
-              {settings.advanceOrderMode && (
+              {pref.advanceOrderMode && (
                 <InputGroup
                   name="extra"
                   label={i18n.str`Additional information`}
@@ -679,7 +683,6 @@ export function CreatePage({
                                 value.extra &&
                                 value.extra[key] !== undefined
                               ) {
-                                console.log(value.extra);
                                 delete value.extra[key];
                               }
                               valueHandler({
@@ -731,7 +734,7 @@ export function CreatePage({
                         e.preventDefault();
                       }}
                     >
-                      add
+                      <i18n.Translate>Add</i18n.Translate>
                     </button>
                   </div>
                 </InputGroup>
