@@ -18,10 +18,7 @@ import {
   TalerError,
   assertUnreachable,
 } from "@gnu-taler/taler-util";
-import {
-  useMerchantApiContext,
-  useTranslationContext,
-} from "@gnu-taler/web-util/browser";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
@@ -30,9 +27,9 @@ import { NotificationCard } from "../../../../components/menu/index.js";
 import { useSessionContext } from "../../../../context/session.js";
 import { useOrderDetails } from "../../../../hooks/order.js";
 import { Notification } from "../../../../utils/types.js";
+import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { DetailPage } from "./DetailPage.js";
-import { LoginPage } from "../../../login/index.js";
 
 export interface Props {
   oid: string;
@@ -42,8 +39,7 @@ export interface Props {
 export default function Update({ oid, onBack }: Props): VNode {
   const result = useOrderDetails(oid);
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
-  const { lib: api } = useMerchantApiContext();
-  const { state } = useSessionContext();
+  const { state, lib } = useSessionContext();
 
   const { i18n } = useTranslationContext();
 
@@ -65,7 +61,7 @@ export default function Update({ oid, onBack }: Props): VNode {
         );
       }
       case HttpStatusCode.Unauthorized: {
-        return <LoginPage />
+        return <LoginPage />;
       }
       default: {
         assertUnreachable(result);
@@ -84,19 +80,27 @@ export default function Update({ oid, onBack }: Props): VNode {
           if (state.status !== "loggedIn") {
             return;
           }
-          api.instance
+          lib.instance
             .addRefund(state.token, id, value)
-            .then(() =>
-              setNotif({
-                message: i18n.str`refund created successfully`,
-                type: "SUCCESS",
-              }),
-            )
+            .then((resp) => {
+              if (resp.type === "ok") {
+                setNotif({
+                  message: i18n.str`Refund created successfully`,
+                  type: "SUCCESS",
+                });
+              } else {
+                setNotif({
+                  message: i18n.str`Could not create the refund`,
+                  type: "ERROR",
+                  description: resp.detail.hint,
+                });
+              }
+            })
             .catch((error) =>
               setNotif({
-                message: i18n.str`could not create the refund`,
+                message: i18n.str`Could not create the refund`,
                 type: "ERROR",
-                description: error.message,
+                description: error instanceof Error ? error.message : String(error),
               }),
             );
         }}
