@@ -27,6 +27,7 @@ import { useState } from "preact/hooks";
 import { DatePicker } from "../../../../components/picker/DatePicker.js";
 import { dateFormatForSettings, usePreference } from "../../../../hooks/preference.js";
 import { CardTable } from "./Table.js";
+import { WithId } from "../../../../declaration.js";
 
 export interface ListPageProps {
   onShowAll: () => void;
@@ -79,7 +80,7 @@ export function ListPage({
   isWiredActive,
 }: ListPageProps): VNode {
   const { i18n } = useTranslationContext();
-  const dateTooltip = i18n.str`select date to show nearby orders`;
+  const dateTooltip = i18n.str`Select date to show nearby orders`;
   const [pickDate, setPickDate] = useState(false);
   const [settings] = usePreference();
 
@@ -92,7 +93,7 @@ export function ListPage({
               <li class={isNotPaidActive}>
                 <div
                   class="has-tooltip-right"
-                  data-tooltip={i18n.str`only show paid orders`}
+                  data-tooltip={i18n.str`Only show paid orders`}
                 >
                   <a onClick={onShowNotPaid}>
                     <i18n.Translate>New</i18n.Translate>
@@ -102,7 +103,7 @@ export function ListPage({
               <li class={isPaidActive}>
                 <div
                   class="has-tooltip-right"
-                  data-tooltip={i18n.str`only show paid orders`}
+                  data-tooltip={i18n.str`Only show paid orders`}
                 >
                   <a onClick={onShowPaid}>
                     <i18n.Translate>Paid</i18n.Translate>
@@ -112,7 +113,7 @@ export function ListPage({
               <li class={isRefundedActive}>
                 <div
                   class="has-tooltip-right"
-                  data-tooltip={i18n.str`only show orders with refunds`}
+                  data-tooltip={i18n.str`Only show orders with refunds`}
                 >
                   <a onClick={onShowRefunded}>
                     <i18n.Translate>Refunded</i18n.Translate>
@@ -122,7 +123,7 @@ export function ListPage({
               <li class={isNotWiredActive}>
                 <div
                   class="has-tooltip-left"
-                  data-tooltip={i18n.str`only show orders where customers paid, but wire payments from payment provider are still pending`}
+                  data-tooltip={i18n.str`Only show orders where customers paid, but wire payments from payment provider are still pending`}
                 >
                   <a onClick={onShowNotWired}>
                     <i18n.Translate>Not wired</i18n.Translate>
@@ -132,7 +133,7 @@ export function ListPage({
               <li class={isWiredActive}>
                 <div
                   class="has-tooltip-left"
-                  data-tooltip={i18n.str`only show orders where customers paid, but wire payments from payment provider are still pending`}
+                  data-tooltip={i18n.str`Only show orders where customers paid, but wire payments from payment provider are still pending`}
                 >
                   <a onClick={onShowWired}>
                     <i18n.Translate>Completed</i18n.Translate>
@@ -142,7 +143,7 @@ export function ListPage({
               <li class={isAllActive}>
                 <div
                   class="has-tooltip-right"
-                  data-tooltip={i18n.str`remove all filters`}
+                  data-tooltip={i18n.str`Remove all filters`}
                 >
                   <a onClick={onShowAll}>
                     <i18n.Translate>All</i18n.Translate>
@@ -160,7 +161,7 @@ export function ListPage({
                   <a class="button is-fullwidth" onClick={() => onSelectDate(undefined)}>
                     <span
                       class="icon"
-                      data-tooltip={i18n.str`clear date filter`}
+                      data-tooltip={i18n.str`Clear date filter`}
                     >
                       <i class="mdi mdi-close" />
                     </span>
@@ -174,7 +175,7 @@ export function ListPage({
                     type="text"
                     readonly
                     value={!jumpToDate || jumpToDate.t_ms === "never" ? "" : format(jumpToDate.t_ms, dateFormatForSettings(settings))}
-                    placeholder={i18n.str`date (${dateFormatForSettings(settings)})`}
+                    placeholder={i18n.str`Jump to date (${dateFormatForSettings(settings)})`}
                     onClick={() => {
                       setPickDate(true);
                     }}
