@@ -19,18 +19,19 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { randomRfc3548Base32Key, TalerMerchantApi } from "@gnu-taler/taler-util";
+import {
+  randomRfc3548Base32Key,
+  TalerMerchantApi,
+} from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, h, VNode } from "preact";
 import { useState } from "preact/hooks";
 import { AsyncButton } from "../../../../components/exception/AsyncButton.js";
-import {
-  FormErrors,
-  FormProvider,
-} from "../../../../components/form/FormProvider.js";
+import { FormProvider } from "../../../../components/form/FormProvider.js";
 import { Input } from "../../../../components/form/Input.js";
 import { InputSelector } from "../../../../components/form/InputSelector.js";
 import { InputWithAddon } from "../../../../components/form/InputWithAddon.js";
+import { WithId } from "../../../../declaration.js";
 
 type Entity = TalerMerchantApi.OtpDevicePatchDetails & WithId;
 
@@ -47,15 +48,8 @@ export function UpdatePage({ device, onUpdate, onBack }: Props): VNode {
   const [state, setState] = useState<Partial<Entity>>(device);
   const [showKey, setShowKey] = useState(false);
 
-  const errors: FormErrors<Entity> = {};
-
-  const hasErrors = Object.keys(errors).some(
-    (k) => (errors as any)[k] !== undefined,
-  );
-
   const submitForm = () => {
-    if (hasErrors) return Promise.reject();
-    return onUpdate(state as any);
+    return onUpdate(state as Entity);
   };
 
   return (
@@ -67,7 +61,8 @@ export function UpdatePage({ device, onUpdate, onBack }: Props): VNode {
               <div class="level-left">
                 <div class="level-item">
                   <span class="is-size-4">
-                    Device: <b>{device.id}</b>
+                    <i18n.Translate>Device:</i18n.Translate>
+                    <b>{device.id}</b>
                   </span>
                 </div>
               </div>
@@ -79,11 +74,7 @@ export function UpdatePage({ device, onUpdate, onBack }: Props): VNode {
         <section class="section is-main-section">
           <div class="columns">
             <div class="column is-four-fifths">
-              <FormProvider
-                object={state}
-                valueHandler={setState}
-                errors={errors}
-              >
+              <FormProvider object={state} valueHandler={setState}>
                 <Input<Entity>
                   name="otp_device_description"
                   label={i18n.str`Description`}
@@ -106,8 +97,8 @@ export function UpdatePage({ device, onUpdate, onBack }: Props): VNode {
                       inputType={showKey ? "text" : "password"}
                       help={
                         state.otp_key === undefined
-                          ? "Not modified"
-                          : "Be sure to be very hard to guess or use the random generator"
+                          ? i18n.str`Not modified`
+                          : i18n.str`Be sure to be very hard to guess or use the random generator`
                       }
                       tooltip={i18n.str`Your device need to have exactly the same value`}
                       fromStr={(v) => v.toUpperCase()}
@@ -131,25 +122,25 @@ export function UpdatePage({ device, onUpdate, onBack }: Props): VNode {
                       side={
                         state.otp_key === undefined ? (
                           <button
-                            onClick={(e) => {
+                            onClick={() => {
                               setState((s) => ({ ...s, otp_key: "" }));
                             }}
                             class="button"
                           >
-                            change key
+                            <i18n.Translate>Change key</i18n.Translate>
                           </button>
                         ) : (
                           <button
-                            data-tooltip={i18n.str`generate random secret key`}
+                            data-tooltip={i18n.str`Generate random secret key`}
                             class="button is-info mr-3"
-                            onClick={(e) => {
+                            onClick={() => {
                               setState((s) => ({
                                 ...s,
                                 otp_key: randomRfc3548Base32Key(),
                               }));
                             }}
                           >
-                            <i18n.Translate>random</i18n.Translate>
+                            <i18n.Translate>Random</i18n.Translate>
                           </button>
                         )
                       }
@@ -165,12 +156,8 @@ export function UpdatePage({ device, onUpdate, onBack }: Props): VNode {
                   </button>
                 )}
                 <AsyncButton
-                  disabled={hasErrors}
-                  data-tooltip={
-                    hasErrors
-                      ? i18n.str`Need to complete marked fields`
-                      : "confirm operation"
-                  }
+                  disabled={false}
+                  data-tooltip={i18n.str`Confirm operation`}
                   onClick={submitForm}
                 >
                   <i18n.Translate>Confirm</i18n.Translate>
