@@ -34,8 +34,7 @@ interface Props {
   onConfirm: () => void;
 }
 export default function CreateProduct({ onConfirm, onBack }: Props): VNode {
-  const { lib } = useSessionContext();
-  const { state } = useSessionContext();
+  const { state, lib } = useSessionContext();
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
   const { i18n } = useTranslationContext();
 
@@ -45,13 +44,28 @@ export default function CreateProduct({ onConfirm, onBack }: Props): VNode {
       <CreatePage
         onBack={onBack}
         onCreate={(request: TalerMerchantApi.ProductAddDetail) => {
-          return lib.instance.addProduct(state.token, request)
-            .then(() => onConfirm())
+          return lib.instance
+            .addProduct(state.token, request)
+            .then((resp) => {
+              if (resp.type === "ok") {
+                setNotif({
+                  message: i18n.str`Product created successfully`,
+                  type: "SUCCESS",
+                });
+                onConfirm();
+              } else {
+                setNotif({
+                  message: i18n.str`Could not create product`,
+                  type: "ERROR",
+                  description: resp.detail.hint,
+                });
+              }
+            })
             .catch((error) => {
               setNotif({
-                message: i18n.str`could not create product`,
+                message: i18n.str`Could not create product`,
                 type: "ERROR",
-                description: error.message,
+                description: error instanceof Error ? error.message : String(error),
               });
             });
         }}
