@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,7 +19,8 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { h, VNode, FunctionalComponent } from "preact";
+import { AmountString } from "@gnu-taler/taler-util";
+import { FunctionalComponent, h } from "preact";
 import { UpdatePage as TestedComponent } from "./UpdatePage.js";
 
 export default {
@@ -46,7 +47,7 @@ export const WithManagedStock = createExample(TestedComponent, {
     description: "description1",
     description_i18n: {} as any,
     image: "",
-    price: "TESTKUDOS:10",
+    price: "TESTKUDOS:10" as AmountString,
     taxes: [],
     total_lost: 10,
     total_sold: 5,
@@ -62,7 +63,7 @@ export const WithInfiniteStock = createExample(TestedComponent, {
     description: "description1",
     description_i18n: {} as any,
     image: "",
-    price: "TESTKUDOS:10",
+    price: "TESTKUDOS:10" as AmountString,
     taxes: [],
     total_lost: 10,
     total_sold: 5,
