@@ -21,7 +21,6 @@
 
 import { HttpStatusCode, TalerError, TalerMerchantApi, assertUnreachable } from "@gnu-taler/taler-util";
 import {
-  useMerchantApiContext,
   useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
@@ -32,9 +31,9 @@ import { NotificationCard } from "../../../../components/menu/index.js";
 import { useSessionContext } from "../../../../context/session.js";
 import { useProductDetails } from "../../../../hooks/product.js";
 import { Notification } from "../../../../utils/types.js";
+import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { UpdatePage } from "./UpdatePage.js";
-import { LoginPage } from "../../../login/index.js";
 
 export type Entity = TalerMerchantApi.ProductAddDetail;
 interface Props {
@@ -49,8 +48,7 @@ export default function UpdateProduct({
 }: Props): VNode {
   const result = useProductDetails(pid);
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
-  const { lib } = useMerchantApiContext();
-  const { state } = useSessionContext();
+  const { state, lib } = useSessionContext();
 
   const { i18n } = useTranslationContext();
 
@@ -80,12 +78,27 @@ export default function UpdateProduct({
         onBack={onBack}
         onUpdate={(data) => {
           return lib.instance.updateProduct(state.token, pid, data)
-            .then(onConfirm)
+            .then(resp => {
+              if (resp.type === "ok") {
+                setNotif({
+                  message: i18n.str`Product (ID: ${pid}) has been updated`,
+                  type: "SUCCESS",
+                });
+                onConfirm()
+              } else {
+                setNotif({
+                  message: i18n.str`Could not update product`,
+                  type: "ERROR",
+                  description: resp.detail.hint,
+                });
+
+              }
+            })
             .catch((error) => {
               setNotif({
-                message: i18n.str`could not create product`,
+                message: i18n.str`Could not update product`,
                 type: "ERROR",
-                description: error.message,
+                description: error instanceof Error ? error.message : String(error),
               });
             });
         }}
