@@ -63,7 +63,7 @@ export function CardTable({
         <div class="card-header-icon" aria-label="more options">
           <span
             class="has-tooltip-left"
-            data-tooltip={i18n.str`add new templates`}
+            data-tooltip={i18n.str`Add new templates`}
           >
             <button class="button is-info" type="button" onClick={onCreate}>
               <span class="icon is-small">
@@ -124,10 +124,10 @@ function Table({
       {onLoadMoreBefore && (
         <button
           class="button is-fullwidth"
-          data-tooltip={i18n.str`load more templates before the first one`}
+          data-tooltip={i18n.str`Load more templates before the first one`}
           onClick={onLoadMoreBefore}
         >
-          <i18n.Translate>load first page</i18n.Translate>
+          <i18n.Translate>Load first page</i18n.Translate>
         </button>
       )}
       <table class="table is-fullwidth is-striped is-hoverable is-fullwidth">
@@ -162,21 +162,21 @@ function Table({
                   <div class="buttons is-right">
                     <button
                       class="button is-danger is-small has-tooltip-left"
-                      data-tooltip={i18n.str`delete selected templates from the database`}
+                      data-tooltip={i18n.str`Delete selected templates from the database`}
                       onClick={() => onDelete(i)}
                     >
-                      Delete
+                      <i18n.Translate>Delete</i18n.Translate>
                     </button>
                     <button
                       class="button is-info is-small has-tooltip-left"
-                      data-tooltip={i18n.str`use template to create new order`}
+                      data-tooltip={i18n.str`Use template to create new order`}
                       onClick={() => onNewOrder(i)}
                     >
-                      Use template
+                      <i18n.Translate>Use template</i18n.Translate>
                     </button>
                     <button
                       class="button is-info is-small has-tooltip-left"
-                      data-tooltip={i18n.str`create qr code for the template`}
+                      data-tooltip={i18n.str`Create qr code for the template`}
                       onClick={() => onQR(i)}
                     >
                       QR
@@ -191,10 +191,10 @@ function Table({
       {onLoadMoreAfter && (
         <button
           class="button is-fullwidth"
-          data-tooltip={i18n.str`load more templates after the last one`}
+          data-tooltip={i18n.str`Load more templates after the last one`}
           onClick={onLoadMoreAfter}
         >
-          <i18n.Translate>load next page</i18n.Translate>
+          <i18n.Translate>Load next page</i18n.Translate>
         </button>
       )}
     </div>
