@@ -19,22 +19,16 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { TalerMerchantApi, stringifyPayTemplateUri } from "@gnu-taler/taler-util";
 import {
-  useMerchantApiContext,
-  useTranslationContext,
-} from "@gnu-taler/web-util/browser";
+  TalerMerchantApi,
+  stringifyPayTemplateUri,
+} from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
-import { useState } from "preact/hooks";
 import { QR } from "../../../../components/exception/QR.js";
-import {
-  FormErrors,
-  FormProvider,
-} from "../../../../components/form/FormProvider.js";
-import { Input } from "../../../../components/form/Input.js";
-import { InputCurrency } from "../../../../components/form/InputCurrency.js";
+import { useSessionContext } from "../../../../context/session.js";
 
-type Entity = TalerMerchantApi.UsingTemplateDetails;
+// type Entity = TalerMerchantApi.UsingTemplateDetails;
 
 interface Props {
   contract: TalerMerchantApi.TemplateContractDetails;
@@ -42,39 +36,17 @@ interface Props {
   onBack?: () => void;
 }
 
-export function QrPage({ contract, id: templateId, onBack }: Props): VNode {
+export function QrPage({ id: templateId, onBack }: Props): VNode {
   const { i18n } = useTranslationContext();
-  const { config, url: backendUrl } = useMerchantApiContext();
+  const { state } = useSessionContext();
 
-  // const [state, setState] = useState<Partial<Entity>>({
-  //   amount: contract.amount,
-  //   summary: contract.summary,
-  // });
-
-  // const errors: FormErrors<Entity> = {};
-
-  // const fixedAmount = !!contract.amount;
-  // const fixedSummary = !!contract.summary;
-
-  // const templateParams: Record<string, string> = {};
-  // if (!fixedAmount) {
-  //   if (state.amount) {
-  //     templateParams.amount = state.amount;
-  //   } else {
-  //     templateParams.amount = config.currency;
-  //   }
-  // }
-
-  // if (!fixedSummary) {
-  //   templateParams.summary = state.summary ?? "";
-  // }
-
-  const merchantBaseUrl = backendUrl.href;
+  const merchantBaseUrl = state.backendUrl.href;
 
   const payTemplateUri = stringifyPayTemplateUri({
     merchantBaseUrl,
     templateId,
-    templateParams: {},
+    // FIXME!
+    //templateParams: {},
   });
 
   return (
@@ -82,7 +54,9 @@ export function QrPage({ contract, id: templateId, onBack }: Props): VNode {
       <section id="printThis">
         <QR text={payTemplateUri} />
         <pre style={{ textAlign: "center" }}>
-          <a href={payTemplateUri}>{payTemplateUri}</a>
+          <a target="_blank" rel="noreferrer" href={payTemplateUri}>
+            {payTemplateUri}
+          </a>
         </pre>
       </section>
 
@@ -90,35 +64,6 @@ export function QrPage({ contract, id: templateId, onBack }: Props): VNode {
         <div class="columns">
           <div class="column" />
           <div class="column is-four-fifths">
-            {/* <p class="is-size-5 mt-5 mb-5">
-              <i18n.Translate>
-                Here you can specify a default value for fields that are not
-                fixed. Default values can be edited by the customer before the
-                payment.
-              </i18n.Translate>
-            </p> */}
-
-            <p></p>
-            {/* <FormProvider
-              object={state}
-              valueHandler={setState}
-              errors={errors}
-            >
-              <InputCurrency<Entity>
-                name="amount"
-                label={i18n.str`Amount`}
-                readonly
-                tooltip={i18n.str`Amount of the order`}
-              />
-              <Input<Entity>
-                name="summary"
-                inputType="multiline"
-                readonly
-                label={i18n.str`Summary`}
-                tooltip={i18n.str`Title of the order to be shown to the customer`}
-              />
-            </FormProvider> */}
-
             <div class="buttons is-right mt-5">
               {onBack && (
                 <button class="button" onClick={onBack}>
