@@ -28,8 +28,7 @@ import {
   TranslatedString,
 } from "@gnu-taler/taler-util";
 import {
-  useMerchantApiContext,
-  useTranslationContext,
+  useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
@@ -45,13 +44,15 @@ import { InputNumber } from "../../../../components/form/InputNumber.js";
 import { InputSelector } from "../../../../components/form/InputSelector.js";
 import { InputToggle } from "../../../../components/form/InputToggle.js";
 import { TextField } from "../../../../components/form/TextField.js";
+import { useSessionContext } from "../../../../context/session.js";
 import { useInstanceOtpDevices } from "../../../../hooks/otp.js";
+import { WithId } from "../../../../declaration.js";
 
 type Entity = {
   description?: string;
   otpId?: string | null;
   summary?: string;
-  amount?: AmountString;
+  amount?: string;
   minimum_age?: number;
   pay_duration?: Duration;
   summary_editable?: boolean;
@@ -67,7 +68,7 @@ interface Props {
 
 export function UpdatePage({ template, onUpdate, onBack }: Props): VNode {
   const { i18n } = useTranslationContext();
-  const { url: backendUrl, config } = useMerchantApiContext();
+  const { config, state: session } = useSessionContext();
 
   const [state, setState] = useState<Partial<Entity>>({
     description: template.template_description,
@@ -75,8 +76,8 @@ export function UpdatePage({ template, onUpdate, onBack }: Props): VNode {
     otpId: template.otp_id,
     pay_duration: template.template_contract.pay_duration
       ? Duration.fromTalerProtocolDuration(
-          template.template_contract.pay_duration,
-        )
+        template.template_contract.pay_duration,
+      )
       : undefined,
     summary:
       template.editable_defaults?.summary ?? template.template_contract.summary,
@@ -84,8 +85,8 @@ export function UpdatePage({ template, onUpdate, onBack }: Props): VNode {
       template.editable_defaults?.amount ??
       (template.template_contract.amount as AmountString | undefined),
     currency_editable: !!template.editable_defaults?.currency,
-    summary_editable: !!template.editable_defaults?.summary,
-    amount_editable: !!template.editable_defaults?.amount,
+    summary_editable: template.editable_defaults?.summary !== undefined,
+    amount_editable: template.editable_defaults?.amount !== undefined,
   });
 
   function updateState(up: (s: Partial<Entity>) => Partial<Entity>) {
@@ -114,24 +115,24 @@ export function UpdatePage({ template, onUpdate, onBack }: Props): VNode {
   const parsedPrice = !state.amount ? undefined : Amounts.parse(state.amount);
 
   const errors: FormErrors<Entity> = {
-    description: !state.description ? i18n.str`should not be empty` : undefined,
+    description: !state.description ? i18n.str`Required` : undefined,
     amount: !state.amount
-      ? undefined
+      ? state.amount_editable ? undefined : i18n.str`Required`
       : !parsedPrice
-        ? i18n.str`not valid`
+        ? i18n.str`Invalid`
         : Amounts.isZero(parsedPrice)
-          ? i18n.str`must be greater than 0`
+          ? state.amount_editable ? undefined : i18n.str`Must be greater than 0`
           : undefined,
     minimum_age:
       state.minimum_age && state.minimum_age < 0
-        ? i18n.str`should be greater that 0`
+        ? i18n.str`Must be greater that 0`
         : undefined,
     pay_duration: !state.pay_duration
-      ? i18n.str`can't be empty`
+      ? i18n.str`Required`
       : state.pay_duration.d_ms === "forever"
         ? undefined
         : state.pay_duration.d_ms < 1000 // less than one second
-          ? i18n.str`to short`
+          ? i18n.str`Too short`
           : undefined,
   };
 
@@ -141,23 +142,28 @@ export function UpdatePage({ template, onUpdate, onBack }: Props): VNode {
     (k) => (errors as Record<string, unknown>)[k] !== undefined,
   );
 
+  const zero = Amounts.stringify(Amounts.zeroOfCurrency(config.currency))
+
   const submitForm = () => {
     if (hasErrors) return Promise.reject();
+    const contract_amount = state.amount_editable ? undefined : state.amount as AmountString
+    const contract_summary = state.summary_editable ? undefined : state.summary
+    const template_contract: TalerMerchantApi.TemplateContractDetails = {
+      minimum_age: state.minimum_age!,
+      pay_duration: Duration.toTalerProtocolDuration(state.pay_duration!),
+      amount: contract_amount,
+      summary: contract_summary,
+      currency:
+        cList.length > 1 && state.currency_editable
+          ? undefined
+          : config.currency,
+    }
     return onUpdate({
       template_description: state.description!,
-      template_contract: {
-        minimum_age: state.minimum_age!,
-        pay_duration: Duration.toTalerProtocolDuration(state.pay_duration!),
-        amount: state.amount_editable ? undefined : state.amount,
-        summary: state.summary_editable ? undefined : state.summary,
-        currency:
-          cList.length > 1 && state.currency_editable
-            ? undefined
-            : config.currency,
-      },
+      template_contract,
       editable_defaults: {
-        amount: !state.amount_editable ? undefined : state.amount,
-        summary: !state.summary_editable ? undefined : state.summary,
+        amount: !state.amount_editable ? undefined : (state.amount ?? zero),
+        summary: !state.summary_editable ? undefined : (state.summary ?? ""),
         currency:
           cList.length === 1 || !state.currency_editable
             ? undefined
@@ -176,7 +182,7 @@ export function UpdatePage({ template, onUpdate, onBack }: Props): VNode {
               <div class="level-left">
                 <div class="level-item">
                   <span class="is-size-4">
-                    {new URL(`templates/${template.id}`, backendUrl.href).href}
+                    {new URL(`templates/${template.id}`, session.backendUrl.href).href}
                   </span>
                 </div>
               </div>
@@ -230,7 +236,7 @@ export function UpdatePage({ template, onUpdate, onBack }: Props): VNode {
                     />
                     <TextField name="sc" label={i18n.str`Supported currencies`}>
                       <i18n.Translate>
-                        supported currencies: {cList.join(", ")}
+                        Supported currencies: {cList.join(", ")}
                       </i18n.Translate>
                     </TextField>
                   </Fragment>
@@ -288,7 +294,7 @@ export function UpdatePage({ template, onUpdate, onBack }: Props): VNode {
                   data-tooltip={
                     hasErrors
                       ? i18n.str`Need to complete marked fields`
-                      : "confirm operation"
+                      : i18n.str`Confirm operation`
                   }
                   onClick={submitForm}
                 >
