@@ -19,24 +19,25 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { HttpStatusCode, TalerError, TalerMerchantApi, assertUnreachable } from "@gnu-taler/taler-util";
 import {
-  useMerchantApiContext,
-  useTranslationContext
-} from "@gnu-taler/web-util/browser";
+  HttpStatusCode,
+  TalerError,
+  TalerMerchantApi,
+  assertUnreachable,
+} from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
 import { Loading } from "../../../../components/exception/loading.js";
 import { NotificationCard } from "../../../../components/menu/index.js";
 import { useSessionContext } from "../../../../context/session.js";
-import {
-  useTemplateDetails,
-} from "../../../../hooks/templates.js";
+import { useTemplateDetails } from "../../../../hooks/templates.js";
 import { Notification } from "../../../../utils/types.js";
+import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { UpdatePage } from "./UpdatePage.js";
-import { LoginPage } from "../../../login/index.js";
+import { WithId } from "../../../../declaration.js";
 
 export type Entity = TalerMerchantApi.TemplatePatchDetails & WithId;
 
@@ -50,27 +51,26 @@ export default function UpdateTemplate({
   onConfirm,
   onBack,
 }: Props): VNode {
-  const { lib } = useMerchantApiContext();
-  const { state } = useSessionContext();
+  const { state, lib } = useSessionContext();
   const result = useTemplateDetails(tid);
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
 
   const { i18n } = useTranslationContext();
 
-  if (!result) return <Loading />
+  if (!result) return <Loading />;
   if (result instanceof TalerError) {
-    return <ErrorLoadingMerchant error={result} />
+    return <ErrorLoadingMerchant error={result} />;
   }
   if (result.type === "fail") {
-    switch(result.case) {
+    switch (result.case) {
       case HttpStatusCode.NotFound: {
-        return <NotFoundPageOrAdminCreate />
+        return <NotFoundPageOrAdminCreate />;
       }
       case HttpStatusCode.Unauthorized: {
-        return <LoginPage />
+        return <LoginPage />;
       }
       default: {
-        assertUnreachable(result)
+        assertUnreachable(result);
       }
     }
   }
@@ -79,16 +79,31 @@ export default function UpdateTemplate({
     <Fragment>
       <NotificationCard notification={notif} />
       <UpdatePage
-        template={{...result.body, id: tid}}
+        template={{ ...result.body, id: tid }}
         onBack={onBack}
         onUpdate={(data) => {
-          return lib.instance.updateTemplate(state.token, tid, data)
-            .then(onConfirm)
+          return lib.instance
+            .updateTemplate(state.token, tid, data)
+            .then((resp) => {
+              if (resp.type === "ok") {
+                setNotif({
+                  message: i18n.str`Template (ID: ${tid}) has been updated`,
+                  type: "SUCCESS",
+                });
+                onConfirm();
+              } else {
+                setNotif({
+                  message: i18n.str`Could not update template`,
+                  type: "ERROR",
+                  description: resp.detail.hint,
+                });
+              }
+            })
             .catch((error) => {
               setNotif({
-                message: i18n.str`could not update template`,
+                message: i18n.str`Could not update template`,
                 type: "ERROR",
-                description: error.message,
+                description: error instanceof Error ? error.message : String(error),
               });
             });
         }}
