@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,14 +19,15 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
+import { TalerMerchantApi } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { format } from "date-fns";
 import { h, VNode } from "preact";
 import { StateUpdater, useState } from "preact/hooks";
-import { MerchantBackend, WithId } from "../../../../declaration.js";
-import { datetimeFormatForSettings, useSettings } from "../../../../hooks/useSettings.js";
+import { datetimeFormatForSettings, usePreference } from "../../../../hooks/preference.js";
+import { WithId } from "../../../../declaration.js";
 
-type Entity = MerchantBackend.Transfers.TransferDetails & WithId;
+type Entity = TalerMerchantApi.TransferDetails & WithId;
 
 interface Props {
   transfers: Entity[];
@@ -34,8 +35,6 @@ interface Props {
   onCreate: () => void;
   accounts: string[];
   onLoadMoreBefore?: () => void;
-  hasMoreBefore?: boolean;
-  hasMoreAfter?: boolean;
   onLoadMoreAfter?: () => void;
 }
 
@@ -45,8 +44,6 @@ export function CardTable({
   onDelete,
   onLoadMoreAfter,
   onLoadMoreBefore,
-  hasMoreAfter,
-  hasMoreBefore,
 }: Props): VNode {
   const [rowSelection, rowSelectionHandler] = useState<string[]>([]);
 
@@ -64,7 +61,7 @@ export function CardTable({
         <div class="card-header-icon" aria-label="more options">
           <span
             class="has-tooltip-left"
-            data-tooltip={i18n.str`add new transfer`}
+            data-tooltip={i18n.str`Add new transfer`}
           >
             <button class="button is-info" type="button" onClick={onCreate}>
               <span class="icon is-small">
@@ -85,8 +82,6 @@ export function CardTable({
                 rowSelectionHandler={rowSelectionHandler}
                 onLoadMoreAfter={onLoadMoreAfter}
                 onLoadMoreBefore={onLoadMoreBefore}
-                hasMoreAfter={hasMoreAfter}
-                hasMoreBefore={hasMoreBefore}
               />
             ) : (
               <EmptyTable />
@@ -103,35 +98,26 @@ interface TableProps {
   onDelete: (id: Entity) => void;
   rowSelectionHandler: StateUpdater<string[]>;
   onLoadMoreBefore?: () => void;
-  hasMoreBefore?: boolean;
-  hasMoreAfter?: boolean;
   onLoadMoreAfter?: () => void;
 }
 
-function toggleSelected<T>(id: T): (prev: T[]) => T[] {
-  return (prev: T[]): T[] =>
-    prev.indexOf(id) == -1 ? [...prev, id] : prev.filter((e) => e != id);
-}
-
 function Table({
   instances,
   onLoadMoreAfter,
   onDelete,
   onLoadMoreBefore,
-  hasMoreAfter,
-  hasMoreBefore,
 }: TableProps): VNode {
   const { i18n } = useTranslationContext();
-  const [settings] = useSettings();
+  const [settings] = usePreference();
   return (
     <div class="table-container">
-      {hasMoreBefore && (
+      {onLoadMoreBefore && (
         <button
           class="button is-fullwidth"
-          data-tooltip={i18n.str`load more transfers before the first one`}
+          data-tooltip={i18n.str`Load more transfers before the first one`}
           onClick={onLoadMoreBefore}
         >
-          <i18n.Translate>load newer transfers</i18n.Translate>
+          <i18n.Translate>Load first page</i18n.Translate>
         </button>
       )}
       <table class="table is-fullwidth is-striped is-hoverable is-fullwidth">
@@ -143,12 +129,6 @@ function Table({
             <th>
               <i18n.Translate>Credit</i18n.Translate>
             </th>
-            <th>
-              <i18n.Translate>Address</i18n.Translate>
-            </th>
-            <th>
-              <i18n.Translate>Exchange URL</i18n.Translate>
-            </th>
             <th>
               <i18n.Translate>Confirmed</i18n.Translate>
             </th>
@@ -165,10 +145,8 @@ function Table({
           {instances.map((i) => {
             return (
               <tr key={i.id}>
-                <td>{i.id}</td>
+                <td title={i.wtid}>{i.wtid.substring(0,16)}...</td>
                 <td>{i.credit_amount}</td>
-                <td>{i.payto_uri}</td>
-                <td>{i.exchange_url}</td>
                 <td>{i.confirmed ? i18n.str`yes` : i18n.str`no`}</td>
                 <td>{i.verified ? i18n.str`yes` : i18n.str`no`}</td>
                 <td>
@@ -182,13 +160,13 @@ function Table({
                     : i18n.str`unknown`}
                 </td>
                 <td>
-                  {i.verified === undefined ? (
+                  {i.verified !== true ? (
                     <button
                       class="button is-danger is-small has-tooltip-left"
-                      data-tooltip={i18n.str`delete selected transfer from the database`}
+                      data-tooltip={i18n.str`Delete selected transfer from the database`}
                       onClick={() => onDelete(i)}
                     >
-                      Delete
+                      <i18n.Translate>Delete</i18n.Translate>
                     </button>
                   ) : undefined}
                 </td>
@@ -197,13 +175,13 @@ function Table({
           })}
         </tbody>
       </table>
-      {hasMoreAfter && (
+      {onLoadMoreAfter && (
         <button
           class="button is-fullwidth"
-          data-tooltip={i18n.str`load more transfer after the last one`}
+          data-tooltip={i18n.str`Load more transfers after the last one`}
           onClick={onLoadMoreAfter}
         >
-          <i18n.Translate>load older transfers</i18n.Translate>
+          <i18n.Translate>Load next page</i18n.Translate>
         </button>
       )}
     </div>
@@ -216,7 +194,7 @@ function EmptyTable(): VNode {
     <div class="content has-text-grey has-text-centered">
       <p>
         <span class="icon is-large">
-          <i class="mdi mdi-emoticon-sad mdi-48px" />
+          <i class="mdi mdi-magnify mdi-48px" />
         </span>
       </p>
       <p>
