@@ -37,7 +37,6 @@ export type Entity = Omit<Omit<TalerMerchantApi.InstanceReconfigurationMessage,
   default_wire_transfer_delay: Duration,
 };
 
-//TalerMerchantApi.InstanceAuthConfigurationMessage
 interface Props {
   onUpdate: (d: TalerMerchantApi.InstanceReconfigurationMessage) => void;
   selected: TalerMerchantApi.QueryInstancesResponse;
@@ -69,40 +68,38 @@ export function UpdatePage({
 
   const { i18n } = useTranslationContext();
 
-  const errors: FormErrors<Entity> = {
-    name: !value.name ? i18n.str`required` : undefined,
+  const errors = undefinedIfEmpty<FormErrors<Entity>>({
+    name: !value.name ? i18n.str`Required` : undefined,
     user_type: !value.user_type
-      ? i18n.str`required`
+      ? i18n.str`Required`
       : value.user_type !== "business" && value.user_type !== "individual"
-        ? i18n.str`should be business or individual`
+        ? i18n.str`Must be business or individual`
         : undefined,
     default_pay_delay: !value.default_pay_delay
-      ? i18n.str`required`
+      ? i18n.str`Required`
       : !!value.default_wire_transfer_delay &&
         value.default_wire_transfer_delay.d_ms !== "forever" &&
         value.default_pay_delay.d_ms !== "forever" &&
         value.default_pay_delay.d_ms > value.default_wire_transfer_delay.d_ms ?
-        i18n.str`pay delay can't be greater than wire transfer delay` : undefined,
+        i18n.str`Pay delay can't be greater than wire transfer delay` : undefined,
     default_wire_transfer_delay: !value.default_wire_transfer_delay
-      ? i18n.str`required`
+      ? i18n.str`Required`
       : undefined,
     address: undefinedIfEmpty({
       address_lines:
         value.address?.address_lines && value.address?.address_lines.length > 7
-          ? i18n.str`max 7 lines`
+          ? i18n.str`Max 7 lines`
           : undefined,
     }),
     jurisdiction: undefinedIfEmpty({
       address_lines:
         value.address?.address_lines && value.address?.address_lines.length > 7
-          ? i18n.str`max 7 lines`
+          ? i18n.str`Max 7 lines`
           : undefined,
     }),
-  };
+  });
 
-  const hasErrors = Object.keys(errors).some(
-    (k) => (errors as any)[k] !== undefined,
-  );
+  const hasErrors = errors !== undefined;
 
   const submit = async (): Promise<void> => {
     const { default_pay_delay, default_wire_transfer_delay, ...rest } = value as Required<Entity>;
@@ -111,7 +108,7 @@ export function UpdatePage({
       default_wire_transfer_delay: Duration.toTalerProtocolDuration(default_wire_transfer_delay),
       ...rest,
     }
-    await onUpdate(result);
+    onUpdate(result);
   };
   // const [active, setActive] = useState(false);
 
@@ -159,7 +156,7 @@ export function UpdatePage({
                 data-tooltip={
                   hasErrors
                     ? i18n.str`Need to complete marked fields`
-                    : "confirm operation"
+                    : i18n.str`Confirm operation`
                 }
                 disabled={hasErrors}
               >
