@@ -30,6 +30,7 @@ import {
 import { Input } from "../../../../components/form/Input.js";
 import { InputSelector } from "../../../../components/form/InputSelector.js";
 import { TalerMerchantApi } from "@gnu-taler/taler-util";
+import { undefinedIfEmpty } from "../../../../utils/table.js";
 
 type Entity = TalerMerchantApi.WebhookAddDetails;
 
@@ -45,26 +46,26 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
 
   const [state, setState] = useState<Partial<Entity>>({});
 
-  const errors: FormErrors<Entity> = {
-    webhook_id: !state.webhook_id ? i18n.str`required` : undefined,
-    event_type: !state.event_type ? i18n.str`required`
-      : state.event_type !== "pay" && state.event_type !== "refund" ? i18n.str`it should be "pay" or "refund"`
+  const errors = undefinedIfEmpty<FormErrors<Entity>>({
+    webhook_id: !state.webhook_id ? i18n.str`Required` : undefined,
+    event_type: !state.event_type
+      ? i18n.str`Required`
+      : state.event_type !== "pay" && state.event_type !== "refund"
+        ? i18n.str`Must be "pay" or "refund"`
         : undefined,
     http_method: !state.http_method
-      ? i18n.str`required`
+      ? i18n.str`Required`
       : !validMethod.includes(state.http_method)
-        ? i18n.str`should be one of '${validMethod.join(", ")}'`
+        ? i18n.str`Must be one of '${validMethod.join(", ")}'`
         : undefined,
-    url: !state.url ? i18n.str`required` : undefined,
-  };
+    url: !state.url ? i18n.str`Required` : undefined,
+  });
 
-  const hasErrors = Object.keys(errors).some(
-    (k) => (errors as any)[k] !== undefined,
-  );
+  const hasErrors = errors !== undefined;
 
   const submitForm = () => {
     if (hasErrors) return Promise.reject();
-    return onCreate(state as any);
+    return onCreate(state as TalerMerchantApi.WebhookAddDetails);
   };
 
   return (
@@ -88,8 +89,8 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                 label={i18n.str`Event`}
                 values={[
                   i18n.str`Choose one...`,
-                  i18n.str`pay`,
-                  i18n.str`refund`,
+                  i18n.str`Pay`,
+                  i18n.str`Refund`,
                 ]}
                 tooltip={i18n.str`The event of the webhook: why the webhook is used`}
               />
@@ -114,28 +115,79 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
               />
 
               <p>
-                The text below support <a target="_blank" rel="noreferrer" href="https://mustache.github.io/mustache.5.html">mustache</a> template engine. Any string
-                between <pre style={{ display: "inline", padding: 0 }}>&#123;&#123;</pre> and <pre style={{ display: "inline", padding: 0 }}>&#125;&#125;</pre> will
-                be replaced with replaced with the value of the corresponding variable.
+                {/* prettier will add some nodes which we don't want because of i18n */}
+                {/* prettier-ignore */}
+                <i18n.Translate>
+                  The text below support <a
+                    target="_blank"
+                    rel="noreferrer"
+                    href="https://mustache.github.io/mustache.5.html"
+                  >
+                    mustache
+                  </a> template engine. Any string between <pre style={{ display: "inline", padding: 0 }}>
+                    &#123;&#123;
+                  </pre> and <pre style={{ display: "inline", padding: 0 }}>
+                    &#125;&#125;
+                  </pre> will be replaced with replaced with the value of the
+                  corresponding variable.
+                </i18n.Translate>
               </p>
               <p>
-                For example <pre style={{ display: "inline", padding: 0 }}>&#123;&#123;contract_terms.amount&#125;&#125;</pre> will be replaced
-                with the the order's price
+                {/* prettier will add some nodes which we don't want because of i18n */}
+                {/* prettier-ignore */}
+                <i18n.Translate>
+                  For example <pre style={{ display: "inline", padding: 0 }}>
+                    &#123;&#123;contract_terms.amount&#125;&#125;
+                  </pre> will be replaced with the the order's price
+                </i18n.Translate>
               </p>
               <p>
-                The short list of variables are:
+                <i18n.Translate>
+                  The short list of variables are:
+                </i18n.Translate>
               </p>
               <div class="menu">
-
-                <ul class="menu-list" style={{ listStyleType: "disc", marginLeft: 20 }}>
-                  <li><b>contract_terms.summary:</b> order's description </li>
-                  <li><b>contract_terms.amount:</b> order's price </li>
-                  <li><b>order_id:</b> order's unique identification </li>
-                  {state.event_type === "refund" && <Fragment>
-                    <li><b>refund_amout:</b> the amount that was being refunded</li>
-                    <li><b>reason:</b> the reason entered by the merchant staff for granting the refund</li>
-                    <li><b>timestamp:</b> time of the refund in nanoseconds since 1970</li>
-                  </Fragment>}
+                <ul
+                  class="menu-list"
+                  style={{ listStyleType: "disc", marginLeft: 20 }}
+                >
+                  <li>
+                    <b>contract_terms.summary:</b>{" "}
+                    <i18n.Translate>order's description</i18n.Translate>
+                  </li>
+                  <li>
+                    <b>contract_terms.amount:</b>{" "}
+                    <i18n.Translate>order's price</i18n.Translate>
+                  </li>
+                  <li>
+                    <b>order_id:</b>{" "}
+                    <i18n.Translate>
+                      order's unique identification
+                    </i18n.Translate>
+                  </li>
+                  {state.event_type === "refund" && (
+                    <Fragment>
+                      <li>
+                        <b>refund_amout:</b>{" "}
+                        <i18n.Translate>
+                          the amount that was being refunded
+                        </i18n.Translate>
+                      </li>
+                      <li>
+                        <b>reason:</b>{" "}
+                        <i18n.Translate>
+                          the reason entered by the merchant staff for granting
+                          the refund
+                        </i18n.Translate>
+                      </li>
+                      <li>
+                        <b>timestamp:</b>{" "}
+                        <i18n.Translate>
+                          time of the refund in nanoseconds since 1970
+                        </i18n.Translate>
+                      </li>
+                    </Fragment>
+                  )}
                 </ul>
               </div>
               {/* <Input<Entity>
@@ -163,7 +215,7 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                 data-tooltip={
                   hasErrors
                     ? i18n.str`Need to complete marked fields`
-                    : "confirm operation"
+                    : i18n.str`Confirm operation`
                 }
                 onClick={submitForm}
               >
