@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -23,7 +23,7 @@ export interface KeyValue {
 export interface Notification {
   message: string;
   description?: string | VNode;
-  details?: string | VNode;
+  details?: string | VNode | string;
   type: MessageType;
 }
 
