@@ -85,7 +85,7 @@ export async function runBench3(configJson: any): Promise<void> {
     // my assumption is that the in-memory db file gets too large
     if (i % restartWallet == 0) {
       if (Object.keys(wallet).length !== 0) {
-        wallet.stop();
+        await wallet.client.call(WalletApiOperation.Shutdown, {});
         console.log("wallet DB stats", j2s(getDbStats!()));
       }
 
@@ -115,9 +115,7 @@ export async function runBench3(configJson: any): Promise<void> {
       exchangeBaseUrl: b3conf.exchange,
     });
 
-    await wallet.runTaskLoop({
-      stopWhenDone: true,
-    });
+    await wallet.client.call(WalletApiOperation.TestingWaitTasksDone, {});
 
     logger.info(
       `Finished withdrawal amount=${withdrawAmount} time=${Date.now() - start}`,
@@ -135,15 +133,13 @@ export async function runBench3(configJson: any): Promise<void> {
         depositPaytoUri: payto,
       });
 
-      await wallet.runTaskLoop({
-        stopWhenDone: true,
-      });
+      await wallet.client.call(WalletApiOperation.TestingWaitTasksDone, {});
 
       logger.info(`Finished deposit amount=10 time=${Date.now() - start}`);
     }
   }
 
-  wallet.stop();
+  await wallet.client.call(WalletApiOperation.Shutdown, {});
   console.log("wallet DB stats", j2s(getDbStats!()));
 }
 
