@@ -36,9 +36,12 @@ import {
   MerchantInstanceConfig,
   PartialMerchantInstanceConfig,
   PaytoString,
+  TalerCoreBankHttpClient,
   TalerCorebankApiClient,
   TalerError,
+  TalerExchangeHttpClient,
   TalerMerchantApi,
+  TalerMerchantManagementHttpClient,
   WalletNotification,
   createEddsaKeyPair,
   eddsaGetPublic,
@@ -736,9 +739,20 @@ export class FakebankService
       "bank",
     );
     await this.pingUntilAvailable();
-    const bankClient = new TalerCorebankApiClient(this.corebankApiBaseUrl);
-    for (const acc of this.accounts) {
-      await bankClient.registerAccount(acc.accountName, acc.accountPassword);
+    // Check version
+    {
+      const bankClient = new TalerCoreBankHttpClient(this.corebankApiBaseUrl);
+      // This would fail/throw if the version doesn't match.
+      const resp = await bankClient.getConfig();
+      this.globalTestState.assertTrue(resp.type === "ok");
+    }
+    // Register bank accounts
+    {
+      // FIXME: This is using the old bank client!
+      const bankClient = new TalerCorebankApiClient(this.corebankApiBaseUrl);
+      for (const acc of this.accounts) {
+        await bankClient.registerAccount(acc.accountName, acc.accountPassword);
+      }
     }
   }
 
@@ -805,6 +819,7 @@ export class LibeufinBankService
       "registration_bonus",
       `${bc.currency}:100`,
     );
+    config.setString("libeufin-bank", "ALLOW_REGISTRATION", "yes");
     const cfgFilename = testDir + "/bank.conf";
     config.writeTo(cfgFilename, { excludeDefaults: true });
 
@@ -890,9 +905,20 @@ export class LibeufinBankService
       "libeufin-bank-httpd",
     );
     await this.pingUntilAvailable();
-    const bankClient = new TalerCorebankApiClient(this.corebankApiBaseUrl);
-    for (const acc of this.accounts) {
-      await bankClient.registerAccount(acc.accountName, acc.accountPassword);
+    // Check version
+    {
+      const bankClient = new TalerCoreBankHttpClient(this.corebankApiBaseUrl);
+      // This would fail/throw if the version doesn't match.
+      const resp = await bankClient.getConfig();
+      this.globalTestState.assertTrue(resp.type === "ok");
+    }
+    // Register accounts
+    {
+      // FIXME: This still uses the old-style client.
+      const bankClient = new TalerCorebankApiClient(this.corebankApiBaseUrl);
+      for (const acc of this.accounts) {
+        await bankClient.registerAccount(acc.accountName, acc.accountPassword);
+      }
     }
   }
 
@@ -1468,6 +1494,26 @@ export class ExchangeService implements ExchangeServiceInterface {
     await sh(this.globalState, "rm-secmod-keys", `rm ${eddsaKeydir}/*`);
   }
 
+  /**
+   * Generate a new master public key for the exchange.
+   */
+  async regenerateMasterPub(): Promise<void> {
+    const cfg = Configuration.load(this.configFilename);
+    const masterPrivFile = cfg
+      .getPath("exchange-offline", "master_priv_file")
+      .required();
+    fs.unlinkSync(masterPrivFile);
+    const exchangeMasterKey = createEddsaKeyPair();
+    fs.writeFileSync(masterPrivFile, Buffer.from(exchangeMasterKey.eddsaPriv));
+    cfg.setString(
+      "exchange",
+      "master_public_key",
+      encodeCrock(exchangeMasterKey.eddsaPub),
+    );
+
+    cfg.writeTo(this.configFilename, { excludeDefaults: true });
+  }
+
   async purgeDatabase(): Promise<void> {
     await sh(
       this.globalState,
@@ -1556,6 +1602,13 @@ export class ExchangeService implements ExchangeServiceInterface {
 
     await this.pingUntilAvailable();
 
+    {
+      const exchangeClient = new TalerExchangeHttpClient(this.baseUrl);
+      // Would throw on incompatible version.
+      const configResp = await exchangeClient.getConfig();
+      this.globalState.assertTrue(configResp.type === "ok");
+    }
+
     const skipKeyup = opts.skipKeyup ?? false;
 
     if (!skipKeyup) {
@@ -1701,6 +1754,15 @@ export class MerchantService implements MerchantServiceInterface {
       ],
       `merchant-${this.merchantConfig.name}`,
     );
+
+    await this.pingUntilAvailable();
+    {
+      const merchantClient = new TalerMerchantManagementHttpClient(
+        this.makeInstanceBaseUrl(),
+      );
+      const configResp = await merchantClient.getConfig();
+      this.globalState.assertTrue(configResp.type === "ok");
+    }
   }
 
   static async create(
@@ -2255,10 +2317,6 @@ export function generateRandomTestIban(salt: string | null = null): string {
   return `DE${check_digits}${bban}`;
 }
 
-export function getWireMethodForTest(): string {
-  return "x-taler-bank";
-}
-
 /**
  * Generate a payto address, whose authority depends
  * on whether the banking is served by euFin or Pybank.
