@@ -116,6 +116,10 @@ export interface EnvOptions {
 
   mixedAgeRestriction?: boolean;
 
+  skipWireFeeCreation?: boolean;
+
+  walletTestObservability?: boolean;
+
   additionalExchangeConfig?(e: ExchangeService): void;
   additionalMerchantConfig?(m: MerchantService): void;
   additionalBankConfig?(b: BankService): void;
@@ -466,11 +470,12 @@ export async function createSimpleTestkudosEnvironmentV3(
     bank.corebankApiBaseUrl,
   ).href;
 
-  const exchangeBankAccount = {
+  const exchangeBankAccount: HarnessExchangeBankAccount = {
     wireGatewayApiBaseUrl,
     accountName: exchangeBankUsername,
     accountPassword: exchangeBankPassword,
     accountPaytoUri: exchangePaytoUri,
+    skipWireFeeCreation: opts.skipWireFeeCreation === true,
   };
 
   await exchange.addBankAccount("1", exchangeBankAccount);
@@ -553,9 +558,12 @@ export async function createSimpleTestkudosEnvironmentV3(
     ),
   });
 
-  const { walletClient, walletService } = await createWalletDaemonWithClient(
-    t,
-    { name: "wallet", persistent: true },
+  const { walletClient, walletService } = await createWalletDaemonWithClient(t,
+    {
+      name: "wallet",
+      persistent: true,
+      emitObservabilityEvents: !!opts.walletTestObservability,
+    },
   );
 
   console.log("setup done!");
@@ -577,6 +585,7 @@ export interface CreateWalletArgs {
   persistent?: boolean;
   overrideDbPath?: string;
   config?: PartialWalletRunConfig;
+  emitObservabilityEvents?: boolean;
 }
 
 export async function createWalletDaemonWithClient(
@@ -614,7 +623,7 @@ export async function createWalletDaemonWithClient(
   const defaultRunConfig = {
     testing: {
       skipDefaults: true,
-      emitObservabilityEvents: !!process.env["TALER_TEST_OBSERVABILITY"],
+      emitObservabilityEvents: !!process.env["TALER_TEST_OBSERVABILITY"] || !!args.emitObservabilityEvents,
     },
   } satisfies PartialWalletRunConfig;
   await walletClient.client.call(WalletApiOperation.InitWallet, {
