@@ -19,6 +19,7 @@
  */
 import {
   AccessToken,
+  AmountJson,
   AmountString,
   Amounts,
   BalancesResponse,
@@ -302,8 +303,6 @@ advancedCli
     const { walletClient, walletService, bank, exchange, merchant } =
       await createSimpleTestkudosEnvironmentV2(t);
     await walletClient.call(WalletApiOperation.RunIntegrationTestV2, {
-      amountToSpend: "TESTKUDOS:5" as AmountString,
-      amountToWithdraw: "TESTKUDOS:10" as AmountString,
       corebankApiBaseUrl: bank.corebankApiBaseUrl,
       exchangeBaseUrl: exchange.baseUrl,
       merchantBaseUrl: merchant.makeInstanceBaseUrl(),
@@ -761,7 +760,7 @@ deploymentCli
         "admin",
         bankAdminPassword,
         {
-          scope: "write",
+          scope: "readwrite",
           duration: {
             d_us: 1000 * 1000 * 10, //10 secs
           },
@@ -782,6 +781,7 @@ deploymentCli
      */
     let accountPayto: PaytoString;
     {
+      logger.info(`token: ${j2s(bankAdminToken)}`);
       const resp = await bank.createAccount(bankAdminToken, {
         name: name,
         password: password,
@@ -799,6 +799,7 @@ deploymentCli
         logger.error(
           `unable to provision bank account, HTTP response status ${resp.case}`,
         );
+        logger.error(j2s(resp));
         process.exit(2);
       }
       logger.info(`account ${id} successfully provisioned`);
@@ -1042,7 +1043,10 @@ deploymentCli
   .action(async (args) => {
     const httpLib = createPlatformHttpLib({});
     const baseUrl = args.provisionMerchantInstance.merchantApiBaseUrl;
-    const api = new TalerMerchantManagementHttpClient(baseUrl, httpLib);
+    const managementApi = new TalerMerchantManagementHttpClient(
+      baseUrl,
+      httpLib,
+    );
     const managementToken = createRFC8959AccessTokenEncoded(
       args.provisionMerchantInstance.managementToken,
     );
@@ -1059,7 +1063,7 @@ deploymentCli
     const bankPassword = args.provisionMerchantInstance.bankPassword;
     const accountPayto = args.provisionMerchantInstance.payto as PaytoString;
 
-    const createResp = await api.createInstance(managementToken, {
+    const createResp = await managementApi.createInstance(managementToken, {
       address: {},
       auth: {
         method: "token",
@@ -1086,18 +1090,28 @@ deploymentCli
       process.exit(2);
     }
 
-    const createAccountResp = await api.addBankAccount(instanceTokenEnc, {
-      payto_uri: accountPayto,
-      credit_facade_url: bankURL,
-      credit_facade_credentials:
-        bankUser && bankPassword
-          ? {
-              type: "basic",
-              username: bankUser,
-              password: bankPassword,
-            }
-          : undefined,
-    });
+    const instanceUrl = managementApi.getSubInstanceAPI(instanceId).href;
+
+    const instanceApi = new TalerMerchantInstanceHttpClient(
+      instanceUrl,
+      httpLib,
+    );
+
+    const createAccountResp = await instanceApi.addBankAccount(
+      instanceTokenEnc,
+      {
+        payto_uri: accountPayto,
+        credit_facade_url: bankURL,
+        credit_facade_credentials:
+          bankUser && bankPassword
+            ? {
+                type: "basic",
+                username: bankUser,
+                password: bankPassword,
+              }
+            : undefined,
+      },
+    );
     if (createAccountResp.type != "ok") {
       console.error(
         `unable to configure bank account for instance ${instanceId}, status ${createAccountResp.case}`,
@@ -1141,9 +1155,27 @@ deploymentCli
     logger.error(
       `unable to provision bank account, HTTP response status ${resp.case}`,
     );
+    logger.error(j2s(resp));
     process.exit(2);
   });
 
+function computeFee(args: {
+  currency: string;
+  spec: string;
+  coinMin: AmountJson;
+  coinMax: AmountJson;
+}): string {
+  if (args.spec === "none") {
+    return `${args.currency}:0`;
+  }
+
+  if (args.spec === "const") {
+    return Amounts.stringify(args.coinMin);
+  }
+
+  throw Error(`unsupported fee spec (${args.spec})`);
+}
+
 deploymentCli
   .subcommand("coincfg", "gen-coin-config", {
     help: "Generate a coin/denomination configuration for the exchange.",
@@ -1154,40 +1186,89 @@ deploymentCli
   .requiredOption("maxAmount", ["--max-amount"], clk.STRING, {
     help: "Largest denomination",
   })
+  .maybeOption("feeWithdraw", ["--fee-deposit"], clk.STRING)
+  .maybeOption("feeDeposit", ["--fee-deposit"], clk.STRING)
+  .maybeOption("feeRefresh", ["--fee-refresh"], clk.STRING)
+  .maybeOption("feeRefund", ["--fee-refund"], clk.STRING)
+  .maybeOption("fees", ["--fees"], clk.STRING)
   .flag("noFees", ["--no-fees"])
   .action(async (args) => {
+    let feespecWithdraw;
+    let feespecDeposit;
+    let feespecRefund;
+    let feespecRefresh;
+
+    if (args.coincfg.noFees) {
+      feespecDeposit = "none";
+      feespecWithdraw = "none";
+      feespecRefund = "none";
+      feespecRefresh = "none";
+    } else if (args.coincfg.fees) {
+      feespecWithdraw = args.coincfg.fees;
+      feespecDeposit = args.coincfg.fees;
+      feespecRefund = args.coincfg.fees;
+      feespecRefresh = args.coincfg.fees;
+    } else {
+      // Default: Only deposit fees
+      feespecWithdraw = args.coincfg.feeWithdraw ?? "none";
+      feespecDeposit = args.coincfg.feeDeposit ?? "const";
+      feespecRefund = args.coincfg.feeRefund ?? "none";
+      feespecRefresh = args.coincfg.feeRefresh ?? "none";
+    }
+
     let out = "";
 
     const stamp = Math.floor(new Date().getTime() / 1000);
 
-    const min = Amounts.parseOrThrow(args.coincfg.minAmount);
-    const max = Amounts.parseOrThrow(args.coincfg.maxAmount);
-    if (min.currency != max.currency) {
+    const coinMin = Amounts.parseOrThrow(args.coincfg.minAmount);
+    const coinMax = Amounts.parseOrThrow(args.coincfg.maxAmount);
+    if (coinMin.currency != coinMax.currency) {
       console.error("currency mismatch");
       process.exit(1);
     }
-    const currency = min.currency;
-    let x = min;
+    const currency = coinMin.currency;
+    let x = coinMin;
     let n = 1;
 
     out += "# Coin configuration for the exchange.\n";
     out += '# Should be placed in "/etc/taler/conf.d/exchange-coins.conf".\n';
     out += "\n";
 
-    while (Amounts.cmp(x, max) < 0) {
+    while (Amounts.cmp(x, coinMax) < 0) {
+      const feeWithdraw = computeFee({
+        currency,
+        coinMax,
+        coinMin,
+        spec: feespecWithdraw,
+      });
+      const feeDeposit = computeFee({
+        currency,
+        coinMax,
+        coinMin,
+        spec: feespecDeposit,
+      });
+      const feeRefresh = computeFee({
+        currency,
+        coinMax,
+        coinMin,
+        spec: feespecRefresh,
+      });
+      const feeRefund = computeFee({
+        currency,
+        coinMax,
+        coinMin,
+        spec: feespecRefund,
+      });
+
       out += `[COIN-${currency}-n${n}-t${stamp}]\n`;
       out += `VALUE = ${Amounts.stringify(x)}\n`;
       out += `DURATION_WITHDRAW = 7 days\n`;
       out += `DURATION_SPEND = 2 years\n`;
       out += `DURATION_LEGAL = 6 years\n`;
-      out += `FEE_WITHDRAW = ${currency}:0\n`;
-      if (args.coincfg.noFees) {
-        out += `FEE_DEPOSIT = ${currency}:0\n`;
-      } else {
-        out += `FEE_DEPOSIT = ${Amounts.stringify(min)}\n`;
-      }
-      out += `FEE_REFRESH = ${currency}:0\n`;
-      out += `FEE_REFUND = ${currency}:0\n`;
+      out += `FEE_WITHDRAW = ${feeWithdraw}\n`;
+      out += `FEE_DEPOSIT = ${feeDeposit}\n`;
+      out += `FEE_REFRESH = ${feeRefresh}\n`;
+      out += `FEE_REFUND = ${feeRefund}\n`;
       out += `RSA_KEYSIZE = 2048\n`;
       out += `CIPHER = RSA\n`;
       out += "\n";
