@@ -17,13 +17,14 @@
 /**
  * Imports.
  */
-import { Duration, j2s } from "@gnu-taler/taler-util";
+import { Duration, TalerCorebankApiClient, j2s } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { defaultCoinConfig } from "../harness/denomStructures.js";
 import {
+  BankService,
   ExchangeService,
-  FakebankService,
   GlobalTestState,
+  HarnessExchangeBankAccount,
   MerchantService,
   generateRandomPayto,
   setupDb,
@@ -31,6 +32,7 @@ import {
 import {
   createWalletDaemonWithClient,
   withdrawViaBankV2,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 /**
@@ -44,7 +46,7 @@ export async function runCurrencyScopeTest(t: GlobalTestState) {
     nameSuffix: "exchange2",
   });
 
-  const bank = await FakebankService.create(t, {
+  const bank = await BankService.create(t, {
     allowRegistrations: true,
     currency: "TESTKUDOS",
     database: dbDefault.connStr,
@@ -72,17 +74,25 @@ export async function runCurrencyScopeTest(t: GlobalTestState) {
     database: dbDefault.connStr,
   });
 
-  const exchangeOneBankAccount = await bank.createExchangeAccount(
-    "myexchange",
-    "x",
-  );
-  await exchangeOne.addBankAccount("1", exchangeOneBankAccount);
+  let exchangeOneBankAccount: HarnessExchangeBankAccount = {
+    wireGatewayApiBaseUrl: new URL(
+      "accounts/myexchange/taler-wire-gateway/",
+      bank.corebankApiBaseUrl,
+    ).href,
+    accountName: "myexchange",
+    accountPassword: "x",
+    accountPaytoUri: generateRandomPayto("myexchange"),
+  };
 
-  const exchangeTwoBankAccount = await bank.createExchangeAccount(
-    "myexchange2",
-    "x",
-  );
-  await exchangeTwo.addBankAccount("1", exchangeTwoBankAccount);
+  let exchangeTwoBankAccount: HarnessExchangeBankAccount = {
+    wireGatewayApiBaseUrl: new URL(
+      "accounts/myexchange2/taler-wire-gateway/",
+      bank.corebankApiBaseUrl,
+    ).href,
+    accountName: "myexchange2",
+    accountPassword: "x",
+    accountPaytoUri: generateRandomPayto("myexchange2"),
+  };
 
   bank.setSuggestedExchange(
     exchangeOne,
@@ -93,6 +103,31 @@ export async function runCurrencyScopeTest(t: GlobalTestState) {
 
   await bank.pingUntilAvailable();
 
+  const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
+    auth: {
+      username: "admin",
+      password: "adminpw",
+    },
+  });
+
+  await bankClient.registerAccountExtended({
+    name: exchangeOneBankAccount.accountName,
+    username: exchangeOneBankAccount.accountName,
+    password: exchangeOneBankAccount.accountPassword,
+    is_taler_exchange: true,
+    payto_uri: exchangeOneBankAccount.accountPaytoUri,
+  });
+  await exchangeOne.addBankAccount("1", exchangeOneBankAccount);
+
+  await bankClient.registerAccountExtended({
+    name: exchangeTwoBankAccount.accountName,
+    username: exchangeTwoBankAccount.accountName,
+    password: exchangeTwoBankAccount.accountPassword,
+    is_taler_exchange: true,
+    payto_uri: exchangeTwoBankAccount.accountPaytoUri,
+  });
+  await exchangeTwo.addBankAccount("1", exchangeTwoBankAccount);
+
   // Set up the first exchange
 
   exchangeOne.addOfferedCoins(defaultCoinConfig);
@@ -139,16 +174,16 @@ export async function runCurrencyScopeTest(t: GlobalTestState) {
 
   // Withdraw digital cash into the wallet.
 
-  const w1 = await withdrawViaBankV2(t, {
+  const w1 = await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange: exchangeOne,
     amount: "TESTKUDOS:6",
   });
 
-  const w2 = await withdrawViaBankV2(t, {
+  const w2 = await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange: exchangeTwo,
     amount: "TESTKUDOS:6",
   });
