@@ -27,8 +27,8 @@ import {
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { GlobalTestState, generateRandomPayto } from "../harness/harness.js";
 import {
-  createSimpleTestkudosEnvironmentV2,
-  withdrawViaBankV2,
+  createSimpleTestkudosEnvironmentV3,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 /**
@@ -37,14 +37,14 @@ import {
 export async function runDepositTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bank, exchange } =
-    await createSimpleTestkudosEnvironmentV2(t);
+  const { walletClient, bankClient, exchange } =
+    await createSimpleTestkudosEnvironmentV3(t);
 
   // Withdraw digital cash into the wallet.
 
-  const withdrawalResult = await withdrawViaBankV2(t, {
+  const withdrawalResult = await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
