@@ -40,7 +40,7 @@ import {
   checkReserve,
   downloadExchangeInfo,
   findDenomOrThrow,
-  topupReserveWithDemobank,
+  topupReserveWithBank,
   withdrawCoin,
 } from "@gnu-taler/taler-wallet-core/dbless";
 import { GlobalTestState, harnessHttpLib } from "../harness/harness.js";
@@ -76,7 +76,7 @@ export async function runExchangePurseTest(t: GlobalTestState) {
       method: "GET",
     });
 
-    await topupReserveWithDemobank({
+    await topupReserveWithBank({
       amount: "TESTKUDOS:10" as AmountString,
       http,
       reservePub: reserveKeyPair.pub,
