@@ -137,7 +137,9 @@ export async function runPaymentAbortTest(t: GlobalTestState) {
   // Can't have succeeded yet, but network error results in "pending" state.
   t.assertDeepEqual(confirmPayResp.type, ConfirmPayResultType.Pending);
 
-  const txns = await walletClient.call(WalletApiOperation.GetTransactions, {});
+  const txns = await walletClient.call(WalletApiOperation.GetTransactions, {
+    sort: "stable-ascending",
+  });
   console.log(j2s(txns));
 
   await walletClient.call(WalletApiOperation.AbortTransaction, {
@@ -146,7 +148,9 @@ export async function runPaymentAbortTest(t: GlobalTestState) {
 
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
 
-  const txns2 = await walletClient.call(WalletApiOperation.GetTransactions, {});
+  const txns2 = await walletClient.call(WalletApiOperation.GetTransactions, {
+    sort: "stable-ascending",
+  });
   console.log(j2s(txns2));
 
   const txTypes = txns2.transactions.map((x) => x.type);
