@@ -22,16 +22,16 @@ import {
   ConfirmPayResultType,
   Duration,
   MerchantApiClient,
-  MerchantContractTerms,
   PreparePayResultType,
+  TalerMerchantApi,
   j2s,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { GlobalTestState } from "../harness/harness.js";
 import {
   applyTimeTravelV2,
-  createSimpleTestkudosEnvironmentV2,
-  withdrawViaBankV2,
+  createSimpleTestkudosEnvironmentV3,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 /**
@@ -49,14 +49,14 @@ import {
 export async function runPaymentExpiredTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bank, exchange, merchant } =
-    await createSimpleTestkudosEnvironmentV2(t);
+  const { walletClient, bankClient, exchange, merchant } =
+    await createSimpleTestkudosEnvironmentV3(t);
 
   // Withdraw digital cash into the wallet.
 
-  await withdrawViaBankV2(t, {
+  await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
@@ -64,7 +64,7 @@ export async function runPaymentExpiredTest(t: GlobalTestState) {
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
 
   // Order that can only be paid within five minutes.
-  const order: Partial<MerchantContractTerms> = {
+  const order: TalerMerchantApi.Order = {
     summary: "Buy me!",
     amount: "TESTKUDOS:5",
     fulfillment_url: "taler://fulfillment-success/thx",
