@@ -21,7 +21,11 @@
 /**
  * Imports.
  */
-import { ConfirmPayResultType, MerchantApiClient } from "@gnu-taler/taler-util";
+import {
+  ConfirmPayResultType,
+  MerchantApiClient,
+  TalerCorebankApiClient,
+} from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { defaultCoinConfig } from "../harness/denomStructures.js";
 import {
@@ -39,7 +43,7 @@ import {
 } from "../harness/harness.js";
 import {
   createWalletDaemonWithClient,
-  withdrawViaBankV2,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 /**
@@ -64,10 +68,20 @@ export async function runPaymentFaultTest(t: GlobalTestState) {
     database: db.connStr,
   });
 
-  const exchangeBankAccount = await bank.createExchangeAccount(
-    "myexchange",
-    "x",
-  );
+  let receiverName = "Exchange";
+  let exchangeBankUsername = "exchange";
+  let exchangeBankPassword = "mypw";
+  let exchangePaytoUri = generateRandomPayto(exchangeBankUsername);
+
+  await exchange.addBankAccount("1", {
+    accountName: exchangeBankUsername,
+    accountPassword: exchangeBankPassword,
+    wireGatewayApiBaseUrl: new URL(
+      "accounts/exchange/taler-wire-gateway/",
+      bank.baseUrl,
+    ).href,
+    accountPaytoUri: exchangePaytoUri,
+  });
 
   const faultyExchange = new FaultInjectedExchangeService(t, exchange, 8091);
   // Base URL must contain port that the proxy is listening on.
@@ -75,16 +89,27 @@ export async function runPaymentFaultTest(t: GlobalTestState) {
     config.setString("exchange", "base_url", "http://localhost:8091/");
   });
 
-  bank.setSuggestedExchange(
-    faultyExchange,
-    exchangeBankAccount.accountPaytoUri,
-  );
+  bank.setSuggestedExchange(faultyExchange, exchangePaytoUri);
 
   await bank.start();
 
   await bank.pingUntilAvailable();
 
-  await exchange.addBankAccount("1", exchangeBankAccount);
+  const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
+    auth: {
+      username: "admin",
+      password: "adminpw",
+    },
+  });
+
+  await bankClient.registerAccountExtended({
+    name: receiverName,
+    password: exchangeBankPassword,
+    username: exchangeBankUsername,
+    is_taler_exchange: true,
+    payto_uri: exchangePaytoUri,
+  });
+
   exchange.addOfferedCoins(defaultCoinConfig);
 
   await exchange.start();
@@ -128,9 +153,9 @@ export async function runPaymentFaultTest(t: GlobalTestState) {
 
   await walletClient.call(WalletApiOperation.GetBalances, {});
 
-  const wres = await withdrawViaBankV2(t, {
+  const wres = await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange: faultyExchange,
     amount: "TESTKUDOS:20",
   });
