@@ -28,8 +28,8 @@ import {
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { GlobalTestState } from "../harness/harness.js";
 import {
-  createSimpleTestkudosEnvironmentV2,
-  withdrawViaBankV2,
+  createSimpleTestkudosEnvironmentV3,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 export async function runRefundTest(t: GlobalTestState) {
@@ -37,18 +37,18 @@ export async function runRefundTest(t: GlobalTestState) {
 
   const {
     walletClient: wallet,
-    bank,
+    bankClient,
     exchange,
     merchant,
-  } = await createSimpleTestkudosEnvironmentV2(t);
+  } = await createSimpleTestkudosEnvironmentV3(t);
 
   const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
 
   // Withdraw digital cash into the wallet.
 
-  const withdrawalRes = await withdrawViaBankV2(t, {
+  const withdrawalRes = await withdrawViaBankV3(t, {
     walletClient: wallet,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
