@@ -21,9 +21,9 @@ import { j2s } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { GlobalTestState } from "../harness/harness.js";
 import {
-  createSimpleTestkudosEnvironmentV2,
+  createSimpleTestkudosEnvironmentV3,
   createWalletDaemonWithClient,
-  withdrawViaBankV2,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 import { SyncService } from "../harness/sync.js";
 
@@ -33,8 +33,8 @@ import { SyncService } from "../harness/sync.js";
 export async function runWalletBackupBasicTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { commonDb, merchant, walletClient, bank, exchange } =
-    await createSimpleTestkudosEnvironmentV2(t);
+  const { commonDb, merchant, walletClient, bankClient, exchange } =
+    await createSimpleTestkudosEnvironmentV3(t);
 
   const sync = await SyncService.create(t, {
     currency: "TESTKUDOS",
@@ -83,9 +83,9 @@ export async function runWalletBackupBasicTest(t: GlobalTestState) {
     );
   }
 
-  await withdrawViaBankV2(t, {
+  await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:10",
   });
@@ -99,9 +99,9 @@ export async function runWalletBackupBasicTest(t: GlobalTestState) {
     console.log(bi);
   }
 
-  await withdrawViaBankV2(t, {
+  await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:5",
   });
@@ -158,9 +158,9 @@ export async function runWalletBackupBasicTest(t: GlobalTestState) {
 
     t.assertAmountEquals(bal1.balances[0].available, "TESTKUDOS:14.1");
 
-    await withdrawViaBankV2(t, {
+    await withdrawViaBankV3(t, {
       walletClient: walletClient2,
-      bank,
+      bankClient,
       exchange,
       amount: "TESTKUDOS:10",
     });
