@@ -23,12 +23,13 @@ import {
   MerchantApiClient,
   MerchantContractTerms,
   PreparePayResultType,
+  TalerMerchantApi,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { GlobalTestState } from "../harness/harness.js";
 import {
-  createSimpleTestkudosEnvironmentV2,
-  withdrawViaBankV2,
+  createSimpleTestkudosEnvironmentV3,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 /**
@@ -42,8 +43,8 @@ import {
 export async function runWalletBalanceTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { merchant, walletClient, exchange, bank } =
-    await createSimpleTestkudosEnvironmentV2(t);
+  const { merchant, walletClient, exchange, bankClient } =
+    await createSimpleTestkudosEnvironmentV3(t);
 
   await merchant.addInstanceWithWireAccount({
     id: "myinst",
@@ -60,9 +61,9 @@ export async function runWalletBalanceTest(t: GlobalTestState) {
 
   // Withdraw digital cash into the wallet.
 
-  const wres = await withdrawViaBankV2(t, {
+  const wres = await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
@@ -71,7 +72,7 @@ export async function runWalletBalanceTest(t: GlobalTestState) {
 
   console.log("withdrawal finished");
 
-  const order: Partial<MerchantContractTerms> = {
+  const order: TalerMerchantApi.Order = {
     summary: "Buy me!",
     amount: "TESTKUDOS:5",
     fulfillment_url: "taler://fulfillment-success/thx",
