@@ -20,7 +20,6 @@
 import {
   AbsoluteTime,
   AmountString,
-  Amounts,
   Duration,
   Logger,
   TalerBankConversionApi,
@@ -37,6 +36,7 @@ import {
   BankService,
   ExchangeService,
   GlobalTestState,
+  HarnessExchangeBankAccount,
   MerchantService,
   generateRandomPayto,
   setupDb,
@@ -102,8 +102,8 @@ async function runTestfakeConversionService(): Promise<TestfakeConversionService
             cashout_ratio: "1",
             cashout_rounding_mode: "zero",
             cashout_tiny_amount: "A:1" as AmountString,
-          }
-        } satisfies TalerBankConversionApi.IntegrationConfig),
+          },
+        } satisfies TalerBankConversionApi.TalerConversionInfoConfig),
       );
     } else if (path === "/cashin-rate") {
       res.writeHead(200, { "Content-Type": "application/json" });
@@ -157,17 +157,40 @@ export async function runWithdrawalConversionTest(t: GlobalTestState) {
     database: db.connStr,
   });
 
-  const exchangeBankAccount = await bank.createExchangeAccount(
-    "myexchange",
-    "x",
-  );
-  exchangeBankAccount.conversionUrl = "http://localhost:8071/";
+  let exchangeBankAccount: HarnessExchangeBankAccount = {
+    wireGatewayApiBaseUrl: new URL(
+      "accounts/myexchange/taler-wire-gateway/",
+      bank.corebankApiBaseUrl,
+    ).href,
+    accountName: "myexchange",
+    accountPassword: "x",
+    accountPaytoUri: generateRandomPayto("myexchange"),
+    conversionUrl: "http://localhost:8071/",
+  };
+
   await exchange.addBankAccount("1", exchangeBankAccount);
 
   await bank.start();
 
   await bank.pingUntilAvailable();
 
+  const bankClientAuth = {
+    username: "admin",
+    password: "adminpw",
+  };
+
+  const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
+    auth: bankClientAuth,
+  });
+
+  await bankClient.registerAccountExtended({
+    name: exchangeBankAccount.accountName,
+    username: exchangeBankAccount.accountName,
+    password: exchangeBankAccount.accountPassword,
+    is_taler_exchange: true,
+    payto_uri: exchangeBankAccount.accountPaytoUri,
+  });
+
   exchange.addOfferedCoins(defaultCoinConfig);
 
   await exchange.start();
@@ -195,20 +218,15 @@ export async function runWithdrawalConversionTest(t: GlobalTestState) {
     ),
   });
 
-  const { walletClient, walletService } = await createWalletDaemonWithClient(
-    t,
-    { name: "wallet" },
-  );
+  const { walletClient } = await createWalletDaemonWithClient(t, {
+    name: "wallet",
+  });
 
   await runTestfakeConversionService();
 
   // Create a withdrawal operation
 
-  const bankAccessApiClient = new TalerCorebankApiClient(
-    bank.corebankApiBaseUrl,
-  );
-
-  const user = await bankAccessApiClient.createRandomBankUser();
+  const user = await bankClient.createRandomBankUser();
 
   await walletClient.call(WalletApiOperation.AddExchange, {
     exchangeBaseUrl: exchange.baseUrl,
@@ -278,10 +296,7 @@ export async function runWithdrawalConversionTest(t: GlobalTestState) {
   const wireGatewayApiClient = new WireGatewayApiClient(
     exchangeBankAccount.wireGatewayApiBaseUrl,
     {
-      auth: {
-        username: exchangeBankAccount.accountName,
-        password: exchangeBankAccount.accountPassword,
-      },
+      auth: bankClientAuth,
     },
   );
 
