@@ -42,12 +42,14 @@ import { runDepositTest } from "./test-deposit.js";
 import { runExchangeDepositTest } from "./test-exchange-deposit.js";
 import { runExchangeManagementFaultTest } from "./test-exchange-management-fault.js";
 import { runExchangeManagementTest } from "./test-exchange-management.js";
+import { runExchangeMasterPubChangeTest } from "./test-exchange-master-pub-change.js";
 import { runExchangePurseTest } from "./test-exchange-purse.js";
 import { runExchangeTimetravelTest } from "./test-exchange-timetravel.js";
 import { runFeeRegressionTest } from "./test-fee-regression.js";
 import { runForcedSelectionTest } from "./test-forced-selection.js";
 import { runKycTest } from "./test-kyc.js";
 import { runLibeufinBankTest } from "./test-libeufin-bank.js";
+import { runMerchantCategoriesTest } from "./test-merchant-categories.js";
 import { runMerchantExchangeConfusionTest } from "./test-merchant-exchange-confusion.js";
 import { runMerchantInstancesDeleteTest } from "./test-merchant-instances-delete.js";
 import { runMerchantInstancesUrlsTest } from "./test-merchant-instances-urls.js";
@@ -106,6 +108,7 @@ import { runWalletDevExperimentsTest } from "./test-wallet-dev-experiments.js";
 import { runWalletExchangeUpdateTest } from "./test-wallet-exchange-update.js";
 import { runWalletGenDbTest } from "./test-wallet-gendb.js";
 import { runWalletInsufficientBalanceTest } from "./test-wallet-insufficient-balance.js";
+import { runWalletNetworkAvailabilityTest } from "./test-wallet-network-availability.js";
 import { runWalletNotificationsTest } from "./test-wallet-notifications.js";
 import { runWalletObservabilityTest } from "./test-wallet-observability.js";
 import { runWalletRefreshErrorsTest } from "./test-wallet-refresh-errors.js";
@@ -116,11 +119,13 @@ import { runWithdrawalAbortBankTest } from "./test-withdrawal-abort-bank.js";
 import { runWithdrawalAmountTest } from "./test-withdrawal-amount.js";
 import { runWithdrawalBankIntegratedTest } from "./test-withdrawal-bank-integrated.js";
 import { runWithdrawalConversionTest } from "./test-withdrawal-conversion.js";
+import { runWithdrawalExternalTest } from "./test-withdrawal-external.js";
 import { runWithdrawalFakebankTest } from "./test-withdrawal-fakebank.js";
 import { runWithdrawalFeesTest } from "./test-withdrawal-fees.js";
 import { runWithdrawalFlexTest } from "./test-withdrawal-flex.js";
 import { runWithdrawalHandoverTest } from "./test-withdrawal-handover.js";
 import { runWithdrawalHugeTest } from "./test-withdrawal-huge.js";
+import { runWithdrawalIdempotentTest } from "./test-withdrawal-idempotent.js";
 import { runWithdrawalManualTest } from "./test-withdrawal-manual.js";
 
 /**
@@ -229,11 +234,16 @@ const allTests: TestMainFunction[] = [
   runWalletBlockedPayPeerPullTest,
   runWalletExchangeUpdateTest,
   runWalletRefreshErrorsTest,
+  runWalletNetworkAvailabilityTest,
   runPeerPullLargeTest,
   runPeerPushLargeTest,
   runWithdrawalHandoverTest,
   runWithdrawalAmountTest,
   runWithdrawalFlexTest,
+  runExchangeMasterPubChangeTest,
+  runMerchantCategoriesTest,
+  runWithdrawalExternalTest,
+  runWithdrawalIdempotentTest,
 ];
 
 export interface TestRunSpec {
