@@ -15,12 +15,6 @@
  */
 
 import { codecForAny } from "./codec.js";
-import {
-  TalerMerchantApi,
-  codecForMerchantConfig,
-  codecForMerchantOrderPrivateStatusResponse,
-  codecForPostOrderResponse,
-} from "./http-client/types.js";
 import { HttpStatusCode } from "./http-status-codes.js";
 import {
   createPlatformHttpLib,
@@ -28,7 +22,6 @@ import {
   readSuccessResponseJsonOrThrow,
   readTalerErrorResponse,
 } from "./http.js";
-import { FacadeCredentials } from "./libeufin-api-types.js";
 import { LibtoolVersion } from "./libtool-version.js";
 import { Logger } from "./logging.js";
 import {
@@ -41,8 +34,16 @@ import {
   opSuccessFromHttp,
   opUnknownFailure,
 } from "./operation.js";
-import { AmountString } from "./taler-types.js";
 import { TalerProtocolDuration } from "./time.js";
+import { AmountString } from "./types-taler-common.js";
+import {
+  OtpDeviceAddDetails,
+  codecForTalerMerchantConfigResponse,
+  codecForMerchantOrderPrivateStatusResponse,
+  codecForPostOrderResponse,
+} from "./types-taler-merchant.js";
+
+import * as TalerMerchantApi from "./types-taler-merchant.js";
 
 const logger = new Logger("MerchantApiClient.ts");
 
@@ -87,21 +88,6 @@ export interface DeleteTippingReserveArgs {
   purge?: boolean;
 }
 
-interface MerchantBankAccount {
-  // The payto:// URI where the wallet will send coins.
-  payto_uri: string;
-
-  // Optional base URL for a facade where the
-  // merchant backend can see incoming wire
-  // transfers to reconcile its accounting
-  // with that of the exchange. Used by
-  // taler-merchant-wirewatch.
-  credit_facade_url?: string;
-
-  // Credentials for accessing the credit facade.
-  credit_facade_credentials?: FacadeCredentials;
-}
-
 export interface MerchantInstanceConfig {
   auth: MerchantAuthConfiguration;
   id: string;
@@ -119,23 +105,6 @@ export interface PrivateOrderStatusQuery {
   sessionId?: string;
 }
 
-export interface OtpDeviceAddDetails {
-  // Device ID to use.
-  otp_device_id: string;
-
-  // Human-readable description for the device.
-  otp_device_description: string;
-
-  // A base64-encoded key
-  otp_key: string;
-
-  // Algorithm for computing the POS confirmation.
-  otp_algorithm: number;
-
-  // Counter for counter-based OTP devices.
-  otp_ctr?: number;
-}
-
 /**
  * Client for the GNU Taler merchant backend.
  */
@@ -325,14 +294,14 @@ export class MerchantApiClient {
    * https://docs.taler.net/core/api-merchant.html#get--config
    *
    */
-  async getConfig(): Promise<OperationOk<TalerMerchantApi.VersionResponse>> {
+  async getConfig(): Promise<OperationOk<TalerMerchantApi.TalerMerchantConfigResponse>> {
     const url = new URL(`config`, this.baseUrl);
     const resp = await this.httpClient.fetch(url.href, {
       method: "GET",
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccessFromHttp(resp, codecForMerchantConfig());
+        return opSuccessFromHttp(resp, codecForTalerMerchantConfigResponse());
       default:
         return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
