@@ -23,7 +23,7 @@
  */
 import { codecForAmountString } from "./amounts.js";
 import { Codec, buildCodecForObject } from "./codec.js";
-import { AmountString } from "./taler-types.js";
+import { AmountString } from "./types-taler-common.js";
 
 /**
  * Status of a reserve.
