@@ -31,7 +31,7 @@ import {
   renderContext,
 } from "./codec.js";
 import { CurrencySpecification } from "./index.js";
-import { AmountString } from "./taler-types.js";
+import { AmountString } from "./types-taler-common.js";
 
 /**
  * Number of fractional units that one value unit represents.
@@ -76,6 +76,48 @@ export interface AmountJson {
   readonly currency: string;
 }
 
+/**
+ * Immutable amount.
+ */
+export class Amount {
+  static from(a: AmountLike): Amount {
+    return new Amount(Amounts.parseOrThrow(a), 0);
+  }
+
+  static zeroOfCurrency(currency: string): Amount {
+    return new Amount(Amounts.zeroOfCurrency(currency), 0);
+  }
+
+  add(...a: AmountLike[]): Amount {
+    if (this.saturated) {
+      return this;
+    }
+    const r = Amounts.add(this.val, ...a);
+    return new Amount(r.amount, r.saturated ? 1 : 0);
+  }
+
+  mult(n: number): Amount {
+    if (this.saturated) {
+      return this;
+    }
+    const r = Amounts.mult(this, n);
+    return new Amount(r.amount, r.saturated ? 1 : 0);
+  }
+
+  toJson(): AmountJson {
+    return { ...this.val };
+  }
+
+  toString(): AmountString {
+    return Amounts.stringify(this.val);
+  }
+
+  private constructor(
+    private val: AmountJson,
+    private saturated: number,
+  ) {}
+}
+
 export const codecForAmountJson = (): Codec<AmountJson> =>
   buildCodecForObject<AmountJson>()
     .property("currency", codecForString())
@@ -118,7 +160,7 @@ export interface Result {
 /**
  * Type for things that are treated like amounts.
  */
-export type AmountLike = string | AmountString | AmountJson;
+export type AmountLike = string | AmountString | AmountJson | Amount;
 
 export interface DivmodResult {
   quotient: number;
@@ -162,6 +204,9 @@ export class Amounts {
     if (typeof amt === "string") {
       return Amounts.parseOrThrow(amt);
     }
+    if (amt instanceof Amount) {
+      return amt.toJson();
+    }
     return amt;
   }
 
@@ -406,6 +451,9 @@ export class Amounts {
    * throw if the input is not a valid amount.
    */
   static parseOrThrow(s: AmountLike): AmountJson {
+    if (s instanceof Amount) {
+      return s.toJson();
+    }
     if (typeof s === "object") {
       if (typeof s.currency !== "string") {
         throw Error("invalid amount object");
