@@ -107,6 +107,10 @@ export interface DetailsMap {
     requestUrl: string;
     requestMethod: string;
     httpStatusCode: number;
+    /**
+     * Original response which is malformed
+     */
+    response?: string;
     validationError?: string;
     /**
      * Content type of the response, usually only specified if not the
@@ -166,6 +170,14 @@ export interface DetailsMap {
   [TalerErrorCode.WALLET_DB_UNAVAILABLE]: {
     innerError: TalerErrorDetail | undefined;
   };
+  [TalerErrorCode.WALLET_EXCHANGE_TOS_NOT_ACCEPTED]: {
+    exchangeBaseUrl: string;
+    tosStatus: string;
+    currentEtag: string | undefined;
+  };
+  [TalerErrorCode.WALLET_EXCHANGE_ENTRY_UPDATE_CONFLICT]: {
+    detail?: string;
+  };
 }
 
 type ErrBody<Y> = Y extends keyof DetailsMap ? DetailsMap[Y] : empty;
@@ -176,7 +188,7 @@ export function makeErrorDetail<C extends TalerErrorCode>(
   hint?: string,
 ): TalerErrorDetail {
   if (!hint && !(detail as any).hint) {
-    hint = getDefaultHint(code);
+    hint = getDefaultTalerErrorHint(code);
   }
   const when = AbsoluteTime.now();
   return { code, when, hint, ...detail };
@@ -198,7 +210,7 @@ export function summarizeTalerErrorDetail(ed: TalerErrorDetail): string {
   return `Error (${ed.code}/${errName})`;
 }
 
-function getDefaultHint(code: number): string {
+export function getDefaultTalerErrorHint(code: number): string {
   const errName = TalerErrorCode[code];
   if (errName) {
     return `Error (${errName})`;
@@ -235,6 +247,21 @@ type TalerHttpErrorsDetails = {
 export type TalerHttpError =
   TalerHttpErrorsDetails[keyof TalerHttpErrorsDetails];
 
+/**
+ * Construct typed error details.
+ * Fills in the hint with a default based on the error code name.
+ */
+export function makeTalerErrorDetail<C extends TalerErrorCode>(
+  code: C,
+  errBody: ErrBody<C>,
+  hint?: string,
+): TalerErrorDetail {
+  if (!hint) {
+    hint = getDefaultTalerErrorHint(code);
+  }
+  return { code, hint, ...errBody };
+}
+
 export class TalerError<T = any> extends Error {
   errorDetail: TalerErrorDetail & T;
   cause: Error | undefined;
@@ -252,7 +279,7 @@ export class TalerError<T = any> extends Error {
     cause?: Error,
   ): TalerError {
     if (!hint) {
-      hint = getDefaultHint(code);
+      hint = getDefaultTalerErrorHint(code);
     }
     const when = AbsoluteTime.now();
     return new TalerError<unknown>({ code, when, hint, ...detail }, cause);
