@@ -16,15 +16,18 @@
 
 import {
   AbsoluteTime,
+  AccessToken,
   HttpStatusCode,
   LibtoolVersion,
   LongPollParams,
   OperationAlternative,
   OperationFail,
   OperationOk,
+  PaginationParams,
+  TalerError,
   TalerErrorCode,
-  codecForChallenge,
-  codecForTanTransmission,
+  UserAndToken,
+  codecForTalerCommonConfigResponse,
   opKnownAlternativeFailure,
   opKnownHttpFailure,
   opKnownTalerFailure,
@@ -32,6 +35,7 @@ import {
 import {
   HttpRequestLibrary,
   createPlatformHttpLib,
+  readSuccessResponseJsonOrThrow,
   readTalerErrorResponse,
 } from "@gnu-taler/taler-util/http";
 import {
@@ -42,12 +46,8 @@ import {
   opSuccessFromHttp,
   opUnknownFailure,
 } from "../operation.js";
+import { WithdrawalOperationStatus } from "../types-taler-bank-integration.js";
 import {
-  AccessToken,
-  PaginationParams,
-  TalerCorebankApi,
-  UserAndToken,
-  WithdrawalOperationStatus,
   codecForAccountData,
   codecForBankAccountCreateWithdrawalResponse,
   codecForBankAccountTransactionInfo,
@@ -55,6 +55,7 @@ import {
   codecForCashoutPending,
   codecForCashoutStatusResponse,
   codecForCashouts,
+  codecForChallenge,
   codecForCoreBankConfig,
   codecForCreateTransactionResponse,
   codecForGlobalCashouts,
@@ -62,8 +63,9 @@ import {
   codecForMonitorResponse,
   codecForPublicAccountsResponse,
   codecForRegisterAccountResponse,
+  codecForTanTransmission,
   codecForWithdrawalPublicInfo,
-} from "./types.js";
+} from "../types-taler-corebank.js";
 import {
   CacheEvictor,
   IdempotencyRetry,
@@ -73,6 +75,8 @@ import {
   nullEvictor,
 } from "./utils.js";
 
+import * as TalerCorebankApi from "../types-taler-corebank.js";
+
 export type TalerCoreBankResultByMethod<
   prop extends keyof TalerCoreBankHttpClient,
 > = ResultByMethod<TalerCoreBankHttpClient, prop>;
@@ -91,6 +95,7 @@ export enum TalerCoreBankCacheEviction {
   CREATE_WITHDRAWAL,
   CREATE_CASHOUT,
 }
+
 /**
  * Protocol version spoken with the core bank.
  *
@@ -123,14 +128,47 @@ export class TalerCoreBankHttpClient {
    * https://docs.taler.net/core/api-corebank.html#config
    *
    */
-  async getConfig() {
+  async getConfig(): Promise<
+    OperationFail<HttpStatusCode.NotFound> | OperationOk<TalerCorebankApi.TalerCorebankConfigResponse>
+  > {
     const url = new URL(`config`, this.baseUrl);
     const resp = await this.httpLib.fetch(url.href, {
       method: "GET",
     });
     switch (resp.status) {
-      case HttpStatusCode.Ok:
-        return opSuccessFromHttp(resp, codecForCoreBankConfig());
+      case HttpStatusCode.Ok: {
+        const minBody = await readSuccessResponseJsonOrThrow(
+          resp,
+          codecForTalerCommonConfigResponse(),
+        );
+        // FIXME: Re-enable the check once fakebank and libeufin-bank return the name.
+        // const expectedName = "taler-corebank";
+        // if (minBody.name !== expectedName) {
+        //   throw TalerError.fromUncheckedDetail({
+        //     code: TalerErrorCode.GENERIC_UNEXPECTED_REQUEST_ERROR,
+        //     requestUrl: resp.requestUrl,
+        //     httpStatusCode: resp.status,
+        //     detail: `Unexpected server component name (got ${minBody.name}, expected ${expectedName}})`,
+        //   });
+        // }
+        if (!this.isCompatible(minBody.version)) {
+          throw TalerError.fromUncheckedDetail({
+            code: TalerErrorCode.GENERIC_CLIENT_UNSUPPORTED_PROTOCOL_VERSION,
+            requestUrl: resp.requestUrl,
+            httpStatusCode: resp.status,
+            detail: `Unsupported protocol version, client supports ${this.PROTOCOL_VERSION}, server supports ${minBody.version}`,
+          });
+        }
+        // Now that we've checked the basic body, re-parse the full response.
+        const body = await readSuccessResponseJsonOrThrow(
+          resp,
+          codecForCoreBankConfig(),
+        );
+        return {
+          type: "ok",
+          body,
+        };
+      }
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
