@@ -30,12 +30,11 @@ import {
   opSuccessFromHttp,
   opUnknownFailure,
 } from "../operation.js";
+import { LongPollParams, PaginationParams } from "../types-taler-common.js";
 import {
-  LongPollParams,
-  PaginationParams,
   codecForRevenueConfig,
   codecForRevenueIncomingHistory,
-} from "./types.js";
+} from "../types-taler-revenue.js";
 import { addLongPollingParam, addPaginationParams } from "./utils.js";
 
 export type TalerBankRevenueResultByMethod<
@@ -120,7 +119,10 @@ export class TalerRevenueHttpClient {
         return opSuccessFromHttp(resp, codecForRevenueIncomingHistory());
       // FIXME: missing in docs
       case HttpStatusCode.NoContent:
-        return opFixedSuccess({incoming_transactions: [], credit_account: "" });
+        return opFixedSuccess({
+          incoming_transactions: [],
+          credit_account: "",
+        });
       case HttpStatusCode.BadRequest:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Unauthorized:
