@@ -1,10 +1,16 @@
-import { HttpRequestLibrary, readTalerErrorResponse } from "../http-common.js";
+import {
+  HttpRequestLibrary,
+  readSuccessResponseJsonOrThrow,
+  readTalerErrorResponse,
+} from "../http-common.js";
 import { HttpStatusCode } from "../http-status-codes.js";
 import { createPlatformHttpLib } from "../http.js";
 import { LibtoolVersion } from "../libtool-version.js";
 import { hash } from "../nacl-fast.js";
 import {
   FailCasesByMethod,
+  OperationFail,
+  OperationOk,
   ResultByMethod,
   opEmptySuccess,
   opFixedSuccess,
@@ -27,14 +33,20 @@ import {
   OfficerAccount,
   PaginationParams,
   SigningKey,
-  TalerExchangeApi,
+  codecForTalerCommonConfigResponse,
+} from "../types-taler-common.js";
+import {
   codecForAmlDecisionDetails,
   codecForAmlRecords,
   codecForExchangeConfig,
   codecForExchangeKeys,
-} from "./types.js";
+} from "../types-taler-exchange.js";
 import { CacheEvictor, addPaginationParams, nullEvictor } from "./utils.js";
 
+import { TalerError } from "../errors.js";
+import { TalerErrorCode } from "../taler-error-codes.js";
+import * as TalerExchangeApi from "../types-taler-exchange.js";
+
 export type TalerExchangeResultByMethod<
   prop extends keyof TalerExchangeHttpClient,
 > = ResultByMethod<TalerExchangeHttpClient, prop>;
@@ -46,7 +58,6 @@ export enum TalerExchangeCacheEviction {
   CREATE_DESCISION,
 }
 
-
 /**
  */
 export class TalerExchangeHttpClient {
@@ -92,14 +103,47 @@ export class TalerExchangeHttpClient {
    * https://docs.taler.net/core/api-exchange.html#get--config
    *
    */
-  async getConfig() {
+  async getConfig(): Promise<
+    | OperationFail<HttpStatusCode.NotFound>
+    | OperationOk<TalerExchangeApi.ExchangeVersionResponse>
+  > {
     const url = new URL(`config`, this.baseUrl);
     const resp = await this.httpLib.fetch(url.href, {
       method: "GET",
     });
     switch (resp.status) {
-      case HttpStatusCode.Ok:
-        return opSuccessFromHttp(resp, codecForExchangeConfig());
+      case HttpStatusCode.Ok: {
+        const minBody = await readSuccessResponseJsonOrThrow(
+          resp,
+          codecForTalerCommonConfigResponse(),
+        );
+        const expectedName = "taler-exchange";
+        if (minBody.name !== expectedName) {
+          throw TalerError.fromUncheckedDetail({
+            code: TalerErrorCode.GENERIC_UNEXPECTED_REQUEST_ERROR,
+            requestUrl: resp.requestUrl,
+            httpStatusCode: resp.status,
+            detail: `Unexpected server component name (got ${minBody.name}, expected ${expectedName}})`,
+          });
+        }
+        if (!this.isCompatible(minBody.version)) {
+          throw TalerError.fromUncheckedDetail({
+            code: TalerErrorCode.GENERIC_CLIENT_UNSUPPORTED_PROTOCOL_VERSION,
+            requestUrl: resp.requestUrl,
+            httpStatusCode: resp.status,
+            detail: `Unsupported protocol version, client supports ${this.PROTOCOL_VERSION}, server supports ${minBody.version}`,
+          });
+        }
+        // Now that we've checked the basic body, re-parse the full response.
+        const body = await readSuccessResponseJsonOrThrow(
+          resp,
+          codecForExchangeConfig(),
+        );
+        return {
+          type: "ok",
+          body,
+        };
+      }
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
