@@ -19,18 +19,22 @@ import {
   FailCasesByMethod,
   HttpStatusCode,
   LibtoolVersion,
+  OperationFail,
+  OperationOk,
   PaginationParams,
   ResultByMethod,
+  TalerError,
+  TalerErrorCode,
   TalerMerchantApi,
+  TalerMerchantConfigResponse,
   codecForAbortResponse,
   codecForAccountAddResponse,
   codecForAccountKycRedirects,
   codecForAccountsSummaryResponse,
-  codecForBankAccountEntry,
+  codecForBankAccountDetail,
   codecForClaimResponse,
   codecForInstancesResponse,
   codecForInventorySummaryResponse,
-  codecForMerchantConfig,
   codecForMerchantOrderPrivateStatusResponse,
   codecForMerchantPosProductDetail,
   codecForMerchantRefundResponse,
@@ -46,6 +50,8 @@ import {
   codecForStatusGoto,
   codecForStatusPaid,
   codecForStatusStatusUnpaid,
+  codecForTalerCommonConfigResponse,
+  codecForTalerMerchantConfigResponse,
   codecForTansferList,
   codecForTemplateDetails,
   codecForTemplateSummaryResponse,
@@ -63,6 +69,7 @@ import {
   HttpRequestLibrary,
   HttpResponse,
   createPlatformHttpLib,
+  readSuccessResponseJsonOrThrow,
   readTalerErrorResponse,
 } from "@gnu-taler/taler-util/http";
 import { opSuccessFromHttp, opUnknownFailure } from "../operation.js";
@@ -108,11 +115,13 @@ export enum TalerMerchantInstanceCacheEviction {
   DELETE_TOKENFAMILY,
   LAST,
 }
+
 export enum TalerMerchantManagementCacheEviction {
   CREATE_INSTANCE = TalerMerchantInstanceCacheEviction.LAST + 1,
   UPDATE_INSTANCE,
   DELETE_INSTANCE,
 }
+
 /**
  * Protocol version spoken with the core bank.
  *
@@ -144,17 +153,48 @@ export class TalerMerchantInstanceHttpClient {
 
   /**
    * https://docs.taler.net/core/api-merchant.html#get--config
-   *
    */
-  async getConfig() {
+  async getConfig(): Promise<
+    | OperationFail<HttpStatusCode.NotFound>
+    | OperationOk<TalerMerchantConfigResponse>
+  > {
     const url = new URL(`config`, this.baseUrl);
-
     const resp = await this.httpLib.fetch(url.href, {
       method: "GET",
     });
     switch (resp.status) {
-      case HttpStatusCode.Ok:
-        return opSuccessFromHttp(resp, codecForMerchantConfig());
+      case HttpStatusCode.Ok: {
+        const minBody = await readSuccessResponseJsonOrThrow(
+          resp,
+          codecForTalerCommonConfigResponse(),
+        );
+        const expectedName = "taler-merchant";
+        if (minBody.name !== expectedName) {
+          throw TalerError.fromUncheckedDetail({
+            code: TalerErrorCode.GENERIC_UNEXPECTED_REQUEST_ERROR,
+            requestUrl: resp.requestUrl,
+            httpStatusCode: resp.status,
+            detail: `Unexpected server component name (got ${minBody.name}, expected ${expectedName}})`,
+          });
+        }
+        if (!this.isCompatible(minBody.version)) {
+          throw TalerError.fromUncheckedDetail({
+            code: TalerErrorCode.GENERIC_CLIENT_UNSUPPORTED_PROTOCOL_VERSION,
+            requestUrl: resp.requestUrl,
+            httpStatusCode: resp.status,
+            detail: `Unsupported protocol version, client supports ${this.PROTOCOL_VERSION}, server supports ${minBody.version}`,
+          });
+        }
+        // Now that we've checked the basic body, re-parse the full response.
+        const body = await readSuccessResponseJsonOrThrow(
+          resp,
+          codecForTalerMerchantConfigResponse(),
+        );
+        return {
+          type: "ok",
+          body,
+        };
+      }
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
@@ -707,7 +747,7 @@ export class TalerMerchantInstanceHttpClient {
 
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccessFromHttp(resp, codecForBankAccountEntry());
+        return opSuccessFromHttp(resp, codecForBankAccountDetail());
       case HttpStatusCode.Unauthorized: // FIXME: missing in docs
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.NotFound:
@@ -882,9 +922,8 @@ export class TalerMerchantInstanceHttpClient {
       default:
         return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
-
   }
-  
+
   /**
    * https://docs.taler.net/core/api-merchant.html#get-[-instances-$INSTANCE]-private-products-$PRODUCT_ID
    */
