@@ -9,7 +9,6 @@ export * from "./ReserveStatus.js";
 export * from "./ReserveTransaction.js";
 export { TaskThrottler } from "./TaskThrottler.js";
 export * from "./amounts.js";
-export * from "./backup-types.js";
 export * from "./bank-api-client.js";
 export * from "./base64.js";
 export * from "./bitcoin.js";
@@ -28,14 +27,12 @@ export * from "./http-client/challenger.js";
 export * from "./http-client/exchange.js";
 export * from "./http-client/merchant.js";
 export * from "./http-client/officer-account.js";
-export * from "./http-client/types.js";
 export { CacheEvictor } from "./http-client/utils.js";
 export * from "./http-status-codes.js";
 export * from "./i18n.js";
 export * from "./iban.js";
 export * from "./invariants.js";
 export * from "./kdf.js";
-export * from "./libeufin-api-types.js";
 export * from "./libtool-version.js";
 export * from "./logging.js";
 export {
@@ -50,13 +47,30 @@ export * from "./observability.js";
 export * from "./operation.js";
 export * from "./payto.js";
 export * from "./promises.js";
+export * from "./qr.js";
 export * from "./rfc3548.js";
 export * from "./taler-crypto.js";
-export * from "./taler-types.js";
 export * from "./taleruri.js";
 export * from "./time.js";
 export * from "./timer.js";
 export * from "./transaction-test-data.js";
-export * from "./transactions-types.js";
 export * from "./url.js";
-export * from "./wallet-types.js";
+
+export * from "./types-taler-bank-conversion.js";
+export * from "./types-taler-bank-integration.js";
+export * from "./types-taler-common.js";
+export * from "./types-taler-corebank.js";
+export * from "./types-taler-exchange.js";
+export * from "./types-taler-merchant.js";
+export * from "./types-taler-sync.js";
+export * from "./types-taler-wallet-transactions.js";
+export * from "./types-taler-wallet.js";
+
+export * as TalerBankConversionApi from "./types-taler-bank-conversion.js";
+export * as TalerBankIntegrationApi from "./types-taler-bank-integration.js";
+export * as ChallengerApi from "./types-taler-challenger.js";
+export * as TalerCorebankApi from "./types-taler-corebank.js";
+export * as TalerExchangeApi from "./types-taler-exchange.js";
+export * as TalerMerchantApi from "./types-taler-merchant.js";
+export * as TalerRevenueApi from "./types-taler-revenue.js";
+export * as TalerWireGatewayApi from "./types-taler-wire-gateway.js";
