@@ -146,7 +146,10 @@ export function opKnownTalerFailure<T extends TalerErrorCode>(
   return { type: "fail", case: s, detail };
 }
 
-export function opUnknownFailure(resp: HttpResponse, error: TalerErrorDetail): never {
+export function opUnknownFailure(
+  resp: HttpResponse,
+  error: TalerErrorDetail,
+): never {
   throw TalerError.fromDetail(
     TalerErrorCode.WALLET_UNEXPECTED_REQUEST_ERROR,
     {
@@ -179,15 +182,51 @@ export function narrowOpSuccessOrThrow<Body, ErrorEnum>(
   }
 }
 
+export async function succeedOrThrow<R, E>(
+  promise: Promise<OperationResult<R, E>>,
+): Promise<R> {
+  const resp = await promise;
+  if (isOperationOk(resp)) {
+    return resp.body;
+  }
+
+  if (isOperationFail(resp)) {
+    throw TalerError.fromUncheckedDetail({ ...resp, case: resp.case } as any);
+  }
+  throw TalerError.fromException(resp);
+}
+
+export async function failOrThrow<E>(
+  s: E,
+  promise: Promise<OperationResult<unknown, E>>,
+): Promise<TalerErrorDetail | undefined> {
+  const resp = await promise;
+  if (isOperationOk(resp)) {
+    throw TalerError.fromException(
+      new Error(`request succeed but failure "${s}" was expected`),
+    );
+  }
+  if (isOperationFail(resp) && resp.case === s) {
+    return resp.detail;
+  }
+  throw TalerError.fromException(
+    new Error(
+      `request failed with "${JSON.stringify(
+        resp,
+      )}" but case "${s}" was expected`,
+    ),
+  );
+}
+
 export type ResultByMethod<
   TT extends object,
   p extends keyof TT,
 > = TT[p] extends (...args: any[]) => infer Ret
   ? Ret extends Promise<infer Result>
-  ? Result extends OperationResult<any, any>
-  ? Result
-  : never
-  : never //api always use Promises
+    ? Result extends OperationResult<any, any>
+      ? Result
+      : never
+    : never //api always use Promises
   : never; //error cases just for functions
 
 export type FailCasesByMethod<TT extends object, p extends keyof TT> = Exclude<
@@ -195,4 +234,4 @@ export type FailCasesByMethod<TT extends object, p extends keyof TT> = Exclude<
   OperationOk<any>
 >;
 
-export type RedirectResult = { redirectURL: URL }
+export type RedirectResult = { redirectURL: URL };
