@@ -15,7 +15,22 @@
  */
 
 import { generateFakeSegwitAddress } from "./bitcoin.js";
-import { Codec, Context, DecodingError, renderContext } from "./codec.js";
+import {
+  Codec,
+  Context,
+  DecodingError,
+  buildCodecForObject,
+  codecForStringURL,
+  renderContext,
+} from "./codec.js";
+import {
+  AccessToken,
+  bytesToString,
+  codecForAccessToken,
+  codecOptional,
+  hashTruncate32,
+  stringToBytes,
+} from "./index.js";
 import { URLSearchParams } from "./url.js";
 
 export type PaytoUri =
@@ -24,8 +39,9 @@ export type PaytoUri =
   | PaytoUriTalerBank
   | PaytoUriBitcoin;
 
-declare const __payto_str: unique symbol;
-export type PaytoString = string & { [__payto_str]: true };
+// declare const __payto_str: unique symbol;
+// export type PaytoString = string & { [__payto_str]: true };
+export type PaytoString = string;
 
 export function codecForPaytoString(): Codec<PaytoString> {
   return {
@@ -144,7 +160,9 @@ export function buildPayto(
 }
 
 /**
- * Add query parameters to a payto URI
+ * Add query parameters to a payto URI.
+ *
+ * Existing parameters are preserved.
  */
 export function addPaytoQueryParams(
   s: string,
@@ -152,14 +170,35 @@ export function addPaytoQueryParams(
 ): string {
   const [acct, search] = s.slice(paytoPfx.length).split("?");
   const searchParams = new URLSearchParams(search || "");
-  const keys = Object.keys(params);
-  if (keys.length === 0) {
+  for (const [paramKey, paramValue] of Object.entries(params)) {
+    searchParams.set(paramKey, paramValue);
+  }
+  const paramList = [...searchParams.entries()];
+  if (paramList.length === 0) {
     return paytoPfx + acct;
   }
-  for (const k of keys) {
-    searchParams.set(k, params[k]);
-  }
-  return paytoPfx + acct + "?" + searchParams.toString();
+  return paytoPfx + acct + "?" + createSearchParams(paramList);
+}
+
+/**
+ * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/encodeURIComponent#encoding_for_rfc3986
+ */
+function encodeRFC3986URIComponent(str: string): string {
+  return encodeURIComponent(str).replace(
+    /[!'()*]/g,
+    (c) => `%${c.charCodeAt(0).toString(16).toUpperCase()}`,
+  );
+}
+const rfc3986 = encodeRFC3986URIComponent;
+
+/**
+ *
+ * https://www.rfc-editor.org/rfc/rfc3986
+ */
+function createSearchParams(paramList: [string, string][]): string {
+  return paramList
+    .map(([key, value]) => `${rfc3986(key)}=${rfc3986(value)}`)
+    .join("&");
 }
 
 /**
@@ -171,12 +210,15 @@ export function addPaytoQueryParams(
 export function stringifyPaytoUri(p: PaytoUri): PaytoString {
   const url = new URL(`${paytoPfx}${p.targetType}/${p.targetPath}`);
   const paramList = !p.params ? [] : Object.entries(p.params);
-  paramList.forEach(([key, value]) => {
-    url.searchParams.set(key, value);
-  });
+  url.search = createSearchParams(paramList);
   return url.href as PaytoString;
 }
 
+export function hashPaytoUri(p: PaytoUri): string {
+  const paytoUri = stringifyPaytoUri(p);
+  return bytesToString(hashTruncate32(stringToBytes(paytoUri + "\0")));
+}
+
 /**
  * Parse a valid payto:// uri into a PaytoUri object
  * RFC 8905
@@ -205,7 +247,8 @@ export function parsePaytoUri(s: string): PaytoUri | undefined {
   const searchParams = new URLSearchParams(search || "");
 
   searchParams.forEach((v, k) => {
-    params[k] = v;
+    // URLSearchParams already decodes uri components
+    params[k] = v; //decodeURIComponent(v);
   });
 
   if (targetType === "x-taler-bank") {
@@ -291,3 +334,22 @@ export function talerPaytoFromExchangeReserve(
 
   return `payto://${proto}/${url.host}${url.pathname}${reservePub}`;
 }
+
+/**
+ * The account letter is all the information
+ * the merchant backend requires from the
+ * bank account to check transfer.
+ *
+ */
+export type AccountLetter = {
+  accountURI: PaytoString;
+  infoURL: string;
+  accountToken?: AccessToken;
+};
+
+export const codecForAccountLetter = (): Codec<AccountLetter> =>
+  buildCodecForObject<AccountLetter>()
+    .property("infoURL", codecForStringURL(true))
+    .property("accountURI", codecForPaytoString())
+    .property("accountToken", codecOptional(codecForAccessToken()))
+    .build("AccountLetter");
