@@ -17,7 +17,10 @@ export interface QjsHttpOptions {
 }
 
 export interface QjsOsLib {
-  fetchHttp(url: string, options?: QjsHttpOptions): Promise<QjsHttpResp>;
+  fetchHttp(url: string, options?: QjsHttpOptions): {
+    promise: Promise<QjsHttpResp>,
+    cancelFn: () => number,
+  };
   postMessageToHost(s: string): void;
   setMessageFromHostHandler(h: (s: string) => void): void;
   rename(oldPath: string, newPath: string): number;
