@@ -21,23 +21,21 @@
 /**
  * Imports.
  */
-import * as nacl from "./nacl-fast.js";
-import { hmacSha256, hmacSha512 } from "./kdf.js";
 import bigint from "big-integer";
-import * as argon2 from "./argon2.js";
-import {
-  CoinEnvelope,
-  CoinPublicKeyString,
-  DenominationPubKey,
-  DenomKeyType,
-  HashCodeString,
-} from "./taler-types.js";
-import { Logger } from "./logging.js";
-import { secretbox } from "./nacl-fast.js";
 import * as fflate from "fflate";
-import { canonicalJson } from "./helpers.js";
-import { TalerProtocolDuration, TalerProtocolTimestamp } from "./time.js";
 import { AmountLike, Amounts } from "./amounts.js";
+import * as argon2 from "./argon2.js";
+import { canonicalJson } from "./helpers.js";
+import { hmacSha256, hmacSha512 } from "./kdf.js";
+import { Logger } from "./logging.js";
+import * as nacl from "./nacl-fast.js";
+import { secretbox } from "./nacl-fast.js";
+import { TalerProtocolDuration, TalerProtocolTimestamp } from "./time.js";
+import {
+  CoinPublicKeyString,
+  HashCodeString,
+} from "./types-taler-common.js";
+import { CoinEnvelope, DenomKeyType, DenominationPubKey } from "./types-taler-exchange.js";
 
 export type Flavor<T, FlavorT extends string> = T & {
   _flavor?: `taler.${FlavorT}`;
@@ -974,6 +972,7 @@ export function hashWire(paytoUri: string, salt: string): string {
 export enum TalerSignaturePurpose {
   MERCHANT_TRACK_TRANSACTION = 1103,
   WALLET_RESERVE_WITHDRAW = 1200,
+  WALLET_RESERVE_HISTORY = 1208,
   WALLET_COIN_DEPOSIT = 1201,
   GLOBAL_FEES = 1022,
   MASTER_DENOMINATION_KEY_VALIDITY = 1025,
