@@ -424,6 +424,10 @@ export class URLSearchParamsImpl {
     return output;
   }
 
+  entries() {
+    return [...this._list.map(x => [x[0], x[1]])];
+  }
+
   forEach(
     callbackfn: (
       value: string,
