@@ -46,7 +46,6 @@ import {
   buildCodecForUnion,
   bytesToString,
   canonicalJson,
-  canonicalizeBaseUrl,
   checkDbInvariant,
   checkLogicInvariant,
   codecForBoolean,
@@ -183,7 +182,7 @@ async function runBackupCycleForProvider(
   args: BackupForProviderArgs,
 ): Promise<TaskRunResult> {
   const provider = await wex.db.runReadOnlyTx(
-    ["backupProviders"],
+    { storeNames: ["backupProviders"] },
     async (tx) => {
       return tx.backupProviders.get(args.backupProviderBaseUrl);
     },
@@ -232,10 +231,10 @@ async function runBackupCycleForProvider(
     headers: {
       "content-type": "application/octet-stream",
       "sync-signature": syncSigResp.sig,
-      "if-none-match": newHash,
+      "if-none-match": JSON.stringify(newHash),
       ...(provider.lastBackupHash
         ? {
-            "if-match": provider.lastBackupHash,
+            "if-match": JSON.stringify(provider.lastBackupHash),
           }
         : {}),
     },
@@ -244,20 +243,23 @@ async function runBackupCycleForProvider(
   logger.trace(`sync response status: ${resp.status}`);
 
   if (resp.status === HttpStatusCode.NotModified) {
-    await wex.db.runReadWriteTx(["backupProviders"], async (tx) => {
-      const prov = await tx.backupProviders.get(provider.baseUrl);
-      if (!prov) {
-        return;
-      }
-      prov.lastBackupCycleTimestamp = timestampPreciseToDb(
-        TalerPreciseTimestamp.now(),
-      );
-      prov.state = {
-        tag: BackupProviderStateTag.Ready,
-        nextBackupTimestamp: timestampPreciseToDb(getNextBackupTimestamp()),
-      };
-      await tx.backupProviders.put(prov);
-    });
+    await wex.db.runReadWriteTx(
+      { storeNames: ["backupProviders"] },
+      async (tx) => {
+        const prov = await tx.backupProviders.get(provider.baseUrl);
+        if (!prov) {
+          return;
+        }
+        prov.lastBackupCycleTimestamp = timestampPreciseToDb(
+          TalerPreciseTimestamp.now(),
+        );
+        prov.state = {
+          tag: BackupProviderStateTag.Ready,
+          nextBackupTimestamp: timestampPreciseToDb(getNextBackupTimestamp()),
+        };
+        await tx.backupProviders.put(prov);
+      },
+    );
 
     removeAttentionRequest(wex, {
       entityId: provider.baseUrl,
@@ -290,18 +292,21 @@ async function runBackupCycleForProvider(
     if (res === undefined) {
       //claimed
 
-      await wex.db.runReadWriteTx(["backupProviders"], async (tx) => {
-        const prov = await tx.backupProviders.get(provider.baseUrl);
-        if (!prov) {
-          logger.warn("backup provider not found anymore");
-          return;
-        }
-        prov.shouldRetryFreshProposal = true;
-        prov.state = {
-          tag: BackupProviderStateTag.Retrying,
-        };
-        await tx.backupProviders.put(prov);
-      });
+      await wex.db.runReadWriteTx(
+        { storeNames: ["backupProviders"] },
+        async (tx) => {
+          const prov = await tx.backupProviders.get(provider.baseUrl);
+          if (!prov) {
+            logger.warn("backup provider not found anymore");
+            return;
+          }
+          prov.shouldRetryFreshProposal = true;
+          prov.state = {
+            tag: BackupProviderStateTag.Retrying,
+          };
+          await tx.backupProviders.put(prov);
+        },
+      );
 
       throw Error("not implemented");
       // return {
@@ -310,21 +315,24 @@ async function runBackupCycleForProvider(
     }
     const result = res;
 
-    await wex.db.runReadWriteTx(["backupProviders"], async (tx) => {
-      const prov = await tx.backupProviders.get(provider.baseUrl);
-      if (!prov) {
-        logger.warn("backup provider not found anymore");
-        return;
-      }
-      // const opId = TaskIdentifiers.forBackup(prov);
-      // await scheduleRetryInTx(ws, tx, opId);
-      prov.currentPaymentProposalId = result.proposalId;
-      prov.shouldRetryFreshProposal = false;
-      prov.state = {
-        tag: BackupProviderStateTag.Retrying,
-      };
-      await tx.backupProviders.put(prov);
-    });
+    await wex.db.runReadWriteTx(
+      { storeNames: ["backupProviders"] },
+      async (tx) => {
+        const prov = await tx.backupProviders.get(provider.baseUrl);
+        if (!prov) {
+          logger.warn("backup provider not found anymore");
+          return;
+        }
+        // const opId = TaskIdentifiers.forBackup(prov);
+        // await scheduleRetryInTx(ws, tx, opId);
+        prov.currentPaymentProposalId = result.proposalId;
+        prov.shouldRetryFreshProposal = false;
+        prov.state = {
+          tag: BackupProviderStateTag.Retrying,
+        };
+        await tx.backupProviders.put(prov);
+      },
+    );
 
     addAttentionRequest(
       wex,
@@ -343,21 +351,24 @@ async function runBackupCycleForProvider(
   }
 
   if (resp.status === HttpStatusCode.NoContent) {
-    await wex.db.runReadWriteTx(["backupProviders"], async (tx) => {
-      const prov = await tx.backupProviders.get(provider.baseUrl);
-      if (!prov) {
-        return;
-      }
-      prov.lastBackupHash = encodeCrock(currentBackupHash);
-      prov.lastBackupCycleTimestamp = timestampPreciseToDb(
-        TalerPreciseTimestamp.now(),
-      );
-      prov.state = {
-        tag: BackupProviderStateTag.Ready,
-        nextBackupTimestamp: timestampPreciseToDb(getNextBackupTimestamp()),
-      };
-      await tx.backupProviders.put(prov);
-    });
+    await wex.db.runReadWriteTx(
+      { storeNames: ["backupProviders"] },
+      async (tx) => {
+        const prov = await tx.backupProviders.get(provider.baseUrl);
+        if (!prov) {
+          return;
+        }
+        prov.lastBackupHash = encodeCrock(currentBackupHash);
+        prov.lastBackupCycleTimestamp = timestampPreciseToDb(
+          TalerPreciseTimestamp.now(),
+        );
+        prov.state = {
+          tag: BackupProviderStateTag.Ready,
+          nextBackupTimestamp: timestampPreciseToDb(getNextBackupTimestamp()),
+        };
+        await tx.backupProviders.put(prov);
+      },
+    );
 
     removeAttentionRequest(wex, {
       entityId: provider.baseUrl,
@@ -376,22 +387,25 @@ async function runBackupCycleForProvider(
     // const blob = await decryptBackup(backupConfig, backupEnc);
     // FIXME: Re-implement backup import with merging
     // await importBackup(ws, blob, cryptoData);
-    await wex.db.runReadWriteTx(["backupProviders"], async (tx) => {
-      const prov = await tx.backupProviders.get(provider.baseUrl);
-      if (!prov) {
-        logger.warn("backup provider not found anymore");
-        return;
-      }
-      prov.lastBackupHash = encodeCrock(hash(backupEnc));
-      // FIXME: Allocate error code for this situation?
-      // FIXME: Add operation retry record!
-      const opId = TaskIdentifiers.forBackup(prov);
-      //await scheduleRetryInTx(ws, tx, opId);
-      prov.state = {
-        tag: BackupProviderStateTag.Retrying,
-      };
-      await tx.backupProviders.put(prov);
-    });
+    await wex.db.runReadWriteTx(
+      { storeNames: ["backupProviders"] },
+      async (tx) => {
+        const prov = await tx.backupProviders.get(provider.baseUrl);
+        if (!prov) {
+          logger.warn("backup provider not found anymore");
+          return;
+        }
+        prov.lastBackupHash = encodeCrock(hash(backupEnc));
+        // FIXME: Allocate error code for this situation?
+        // FIXME: Add operation retry record!
+        const opId = TaskIdentifiers.forBackup(prov);
+        //await scheduleRetryInTx(ws, tx, opId);
+        prov.state = {
+          tag: BackupProviderStateTag.Retrying,
+        };
+        await tx.backupProviders.put(prov);
+      },
+    );
     logger.info("processed existing backup");
     // Now upload our own, merged backup.
     return await runBackupCycleForProvider(wex, args);
@@ -413,8 +427,12 @@ export async function processBackupForProvider(
   wex: WalletExecutionContext,
   backupProviderBaseUrl: string,
 ): Promise<TaskRunResult> {
+  if (!wex.ws.networkAvailable) {
+    return TaskRunResult.networkRequired();
+  }
+
   const provider = await wex.db.runReadOnlyTx(
-    ["backupProviders"],
+    { storeNames: ["backupProviders"] },
     async (tx) => {
       return await tx.backupProviders.get(backupProviderBaseUrl);
     },
@@ -444,9 +462,12 @@ export async function removeBackupProvider(
   wex: WalletExecutionContext,
   req: RemoveBackupProviderRequest,
 ): Promise<void> {
-  await wex.db.runReadWriteTx(["backupProviders"], async (tx) => {
-    await tx.backupProviders.delete(req.provider);
-  });
+  await wex.db.runReadWriteTx(
+    { storeNames: ["backupProviders"] },
+    async (tx) => {
+      await tx.backupProviders.delete(req.provider);
+    },
+  );
 }
 
 export interface RunBackupCycleRequest {
@@ -473,7 +494,7 @@ export async function runBackupCycle(
   req: RunBackupCycleRequest,
 ): Promise<void> {
   const providers = await wex.db.runReadOnlyTx(
-    ["backupProviders"],
+    { storeNames: ["backupProviders"] },
     async (tx) => {
       if (req.providers) {
         const rs = await Promise.all(
@@ -552,57 +573,65 @@ export async function addBackupProvider(
 ): Promise<AddBackupProviderResponse> {
   logger.info(`adding backup provider ${j2s(req)}`);
   await provideBackupState(wex);
-  const canonUrl = canonicalizeBaseUrl(req.backupProviderBaseUrl);
-  await wex.db.runReadWriteTx(["backupProviders"], async (tx) => {
-    const oldProv = await tx.backupProviders.get(canonUrl);
-    if (oldProv) {
-      logger.info("old backup provider found");
-      if (req.activate) {
-        oldProv.state = {
-          tag: BackupProviderStateTag.Ready,
-          nextBackupTimestamp: timestampPreciseToDb(
-            TalerPreciseTimestamp.now(),
-          ),
-        };
-        logger.info("setting existing backup provider to active");
-        await tx.backupProviders.put(oldProv);
+  const canonUrl = req.backupProviderBaseUrl;
+  await wex.db.runReadWriteTx(
+    { storeNames: ["backupProviders"] },
+    async (tx) => {
+      const oldProv = await tx.backupProviders.get(canonUrl);
+      if (oldProv) {
+        logger.info("old backup provider found");
+        if (req.activate) {
+          oldProv.state = {
+            tag: BackupProviderStateTag.Ready,
+            nextBackupTimestamp: timestampPreciseToDb(
+              TalerPreciseTimestamp.now(),
+            ),
+          };
+          logger.info("setting existing backup provider to active");
+          await tx.backupProviders.put(oldProv);
+        }
+        return;
       }
-      return;
-    }
-  });
+    },
+  );
   const termsUrl = new URL("config", canonUrl);
   const resp = await wex.http.fetch(termsUrl.href);
   const terms = await readSuccessResponseJsonOrThrow(
     resp,
     codecForSyncTermsOfServiceResponse(),
   );
-  await wex.db.runReadWriteTx(["backupProviders"], async (tx) => {
-    let state: BackupProviderState;
-    //FIXME: what is the difference provisional and ready?
-    if (req.activate) {
-      state = {
-        tag: BackupProviderStateTag.Ready,
-        nextBackupTimestamp: timestampPreciseToDb(TalerPreciseTimestamp.now()),
-      };
-    } else {
-      state = {
-        tag: BackupProviderStateTag.Provisional,
-      };
-    }
-    await tx.backupProviders.put({
-      state,
-      name: req.name,
-      terms: {
-        annualFee: terms.annual_fee,
-        storageLimitInMegabytes: terms.storage_limit_in_megabytes,
-        supportedProtocolVersion: terms.version,
-      },
-      shouldRetryFreshProposal: false,
-      paymentProposalIds: [],
-      baseUrl: canonUrl,
-      uids: [encodeCrock(getRandomBytes(32))],
-    });
-  });
+  await wex.db.runReadWriteTx(
+    { storeNames: ["backupProviders"] },
+    async (tx) => {
+      let state: BackupProviderState;
+      //FIXME: what is the difference provisional and ready?
+      if (req.activate) {
+        state = {
+          tag: BackupProviderStateTag.Ready,
+          nextBackupTimestamp: timestampPreciseToDb(
+            TalerPreciseTimestamp.now(),
+          ),
+        };
+      } else {
+        state = {
+          tag: BackupProviderStateTag.Provisional,
+        };
+      }
+      await tx.backupProviders.put({
+        state,
+        name: req.name,
+        terms: {
+          annualFee: terms.annual_fee,
+          storageLimitInMegabytes: terms.storage_limit_in_megabytes,
+          supportedProtocolVersion: terms.version,
+        },
+        shouldRetryFreshProposal: false,
+        paymentProposalIds: [],
+        baseUrl: canonUrl,
+        uids: [encodeCrock(getRandomBytes(32))],
+      });
+    },
+  );
 
   return await runFirstBackupCycleForProvider(wex, {
     backupProviderBaseUrl: canonUrl,
@@ -706,7 +735,7 @@ export async function getBackupInfo(
 ): Promise<BackupInfo> {
   const backupConfig = await provideBackupState(wex);
   const providerRecords = await wex.db.runReadOnlyTx(
-    ["backupProviders", "operationRetries"],
+    { storeNames: ["backupProviders", "operationRetries"] },
     async (tx) => {
       return await tx.backupProviders.iter().mapAsync(async (bp) => {
         const opId = TaskIdentifiers.forBackup(bp);
@@ -752,7 +781,7 @@ export async function getBackupRecovery(
 ): Promise<BackupRecovery> {
   const bs = await provideBackupState(wex);
   const providers = await wex.db.runReadOnlyTx(
-    ["backupProviders"],
+    { storeNames: ["backupProviders"] },
     async (tx) => {
       return await tx.backupProviders.iter().toArray();
     },
@@ -774,48 +803,52 @@ async function backupRecoveryTheirs(
   wex: WalletExecutionContext,
   br: BackupRecovery,
 ) {
-  await wex.db.runReadWriteTx(["backupProviders", "config"], async (tx) => {
-    let backupStateEntry: ConfigRecord | undefined = await tx.config.get(
-      ConfigRecordKey.WalletBackupState,
-    );
-    checkDbInvariant(!!backupStateEntry);
-    checkDbInvariant(
-      backupStateEntry.key === ConfigRecordKey.WalletBackupState,
-    );
-    backupStateEntry.value.lastBackupNonce = undefined;
-    backupStateEntry.value.lastBackupTimestamp = undefined;
-    backupStateEntry.value.lastBackupCheckTimestamp = undefined;
-    backupStateEntry.value.lastBackupPlainHash = undefined;
-    backupStateEntry.value.walletRootPriv = br.walletRootPriv;
-    backupStateEntry.value.walletRootPub = encodeCrock(
-      eddsaGetPublic(decodeCrock(br.walletRootPriv)),
-    );
-    await tx.config.put(backupStateEntry);
-    for (const prov of br.providers) {
-      const existingProv = await tx.backupProviders.get(prov.url);
-      if (!existingProv) {
-        await tx.backupProviders.put({
-          baseUrl: prov.url,
-          name: prov.name,
-          paymentProposalIds: [],
-          shouldRetryFreshProposal: false,
-          state: {
-            tag: BackupProviderStateTag.Ready,
-            nextBackupTimestamp: timestampPreciseToDb(
-              TalerPreciseTimestamp.now(),
-            ),
-          },
-          uids: [encodeCrock(getRandomBytes(32))],
-        });
+  await wex.db.runReadWriteTx(
+    { storeNames: ["backupProviders", "config"] },
+    async (tx) => {
+      let backupStateEntry: ConfigRecord | undefined = await tx.config.get(
+        ConfigRecordKey.WalletBackupState,
+      );
+      checkDbInvariant(!!backupStateEntry, `no backup entry`);
+      checkDbInvariant(
+        backupStateEntry.key === ConfigRecordKey.WalletBackupState,
+        `backup entry inconsistent`,
+      );
+      backupStateEntry.value.lastBackupNonce = undefined;
+      backupStateEntry.value.lastBackupTimestamp = undefined;
+      backupStateEntry.value.lastBackupCheckTimestamp = undefined;
+      backupStateEntry.value.lastBackupPlainHash = undefined;
+      backupStateEntry.value.walletRootPriv = br.walletRootPriv;
+      backupStateEntry.value.walletRootPub = encodeCrock(
+        eddsaGetPublic(decodeCrock(br.walletRootPriv)),
+      );
+      await tx.config.put(backupStateEntry);
+      for (const prov of br.providers) {
+        const existingProv = await tx.backupProviders.get(prov.url);
+        if (!existingProv) {
+          await tx.backupProviders.put({
+            baseUrl: prov.url,
+            name: prov.name,
+            paymentProposalIds: [],
+            shouldRetryFreshProposal: false,
+            state: {
+              tag: BackupProviderStateTag.Ready,
+              nextBackupTimestamp: timestampPreciseToDb(
+                TalerPreciseTimestamp.now(),
+              ),
+            },
+            uids: [encodeCrock(getRandomBytes(32))],
+          });
+        }
       }
-    }
-    const providers = await tx.backupProviders.iter().toArray();
-    for (const prov of providers) {
-      prov.lastBackupCycleTimestamp = undefined;
-      prov.lastBackupHash = undefined;
-      await tx.backupProviders.put(prov);
-    }
-  });
+      const providers = await tx.backupProviders.iter().toArray();
+      for (const prov of providers) {
+        prov.lastBackupCycleTimestamp = undefined;
+        prov.lastBackupHash = undefined;
+        await tx.backupProviders.put(prov);
+      }
+    },
+  );
 }
 
 async function backupRecoveryOurs(
@@ -831,7 +864,7 @@ export async function loadBackupRecovery(
 ): Promise<void> {
   const bs = await provideBackupState(wex);
   const providers = await wex.db.runReadOnlyTx(
-    ["backupProviders"],
+    { storeNames: ["backupProviders"] },
     async (tx) => {
       return await tx.backupProviders.iter().toArray();
     },
@@ -879,13 +912,16 @@ export async function provideBackupState(
   wex: WalletExecutionContext,
 ): Promise<WalletBackupConfState> {
   const bs: ConfigRecord | undefined = await wex.db.runReadOnlyTx(
-    ["config"],
+    { storeNames: ["config"] },
     async (tx) => {
       return await tx.config.get(ConfigRecordKey.WalletBackupState);
     },
   );
   if (bs) {
-    checkDbInvariant(bs.key === ConfigRecordKey.WalletBackupState);
+    checkDbInvariant(
+      bs.key === ConfigRecordKey.WalletBackupState,
+      `backup entry inconsistent`,
+    );
     return bs.value;
   }
   // We need to generate the key outside of the transaction
@@ -895,7 +931,7 @@ export async function provideBackupState(
   // FIXME: device ID should be configured when wallet is initialized
   // and be based on hostname
   const deviceId = `wallet-core-${encodeCrock(d)}`;
-  return await wex.db.runReadWriteTx(["config"], async (tx) => {
+  return await wex.db.runReadWriteTx({ storeNames: ["config"] }, async (tx) => {
     let backupStateEntry: ConfigRecord | undefined = await tx.config.get(
       ConfigRecordKey.WalletBackupState,
     );
@@ -913,6 +949,7 @@ export async function provideBackupState(
     }
     checkDbInvariant(
       backupStateEntry.key === ConfigRecordKey.WalletBackupState,
+      `backup entry inconsistent`,
     );
     return backupStateEntry.value;
   });
@@ -924,7 +961,10 @@ export async function getWalletBackupState(
 ): Promise<WalletBackupConfState> {
   const bs = await tx.config.get(ConfigRecordKey.WalletBackupState);
   checkDbInvariant(!!bs, "wallet backup state should be in DB");
-  checkDbInvariant(bs.key === ConfigRecordKey.WalletBackupState);
+  checkDbInvariant(
+    bs.key === ConfigRecordKey.WalletBackupState,
+    `backup entry inconsistent`,
+  );
   return bs.value;
 }
 
@@ -933,8 +973,8 @@ export async function setWalletDeviceId(
   deviceId: string,
 ): Promise<void> {
   await provideBackupState(wex);
-  await wex.db.runReadWriteTx(["config"], async (tx) => {
-    let backupStateEntry: ConfigRecord | undefined = await tx.config.get(
+  await wex.db.runReadWriteTx({ storeNames: ["config"] }, async (tx) => {
+    const backupStateEntry: ConfigRecord | undefined = await tx.config.get(
       ConfigRecordKey.WalletBackupState,
     );
     if (
