@@ -30,11 +30,15 @@ import {
   ExchangeTosStatus,
   ExchangeUpdateStatus,
   Logger,
+  ObservabilityEventType,
   RefreshReason,
+  TalerError,
+  TalerErrorCode,
   TalerErrorDetail,
   TalerPreciseTimestamp,
   TalerProtocolTimestamp,
   TombstoneIdStr,
+  Transaction,
   TransactionIdStr,
   WalletNotification,
   assertUnreachable,
@@ -44,6 +48,7 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   BackupProviderRecord,
+  CoinHistoryRecord,
   CoinRecord,
   DbPreciseTimestamp,
   DepositGroupRecord,
@@ -57,11 +62,12 @@ import {
   PurchaseRecord,
   RecoupGroupRecord,
   RefreshGroupRecord,
-  RewardRecord,
+  WalletDbAllStoresReadOnlyTransaction,
   WalletDbReadWriteTransaction,
   WithdrawalGroupRecord,
   timestampPreciseToDb,
 } from "./db.js";
+import { ReadyExchangeSummary } from "./exchanges.js";
 import { createRefreshGroup } from "./refresh.js";
 import { WalletExecutionContext, getDenomInfo } from "./wallet.js";
 
@@ -72,9 +78,9 @@ export interface CoinsSpendInfo {
   contributions: AmountJson[];
   refreshReason: RefreshReason;
   /**
-   * Identifier for what the coin has been spent for.
+   * Transaction for which the coin is spent.
    */
-  allocationId: TransactionIdStr;
+  transactionId: TransactionIdStr;
 }
 
 export async function makeCoinsVisible(
@@ -121,7 +127,10 @@ export async function makeCoinAvailable(
     coinRecord.exchangeBaseUrl,
     coinRecord.denomPubHash,
   ]);
-  checkDbInvariant(!!denom, `denomination of a coin is missing hash: ${coinRecord.denomPubHash}`);
+  checkDbInvariant(
+    !!denom,
+    `denomination of a coin is missing hash: ${coinRecord.denomPubHash}`,
+  );
   const ageRestriction = coinRecord.maxAge;
   let car = await tx.coinAvailability.get([
     coinRecord.exchangeBaseUrl,
@@ -144,11 +153,16 @@ export async function makeCoinAvailable(
   await tx.coinAvailability.put(car);
 }
 
+/**
+ * Spend coins. Marks the coins as used, adds a coin history items
+ * and creates refresh group.
+ */
 export async function spendCoins(
   wex: WalletExecutionContext,
   tx: WalletDbReadWriteTransaction<
     [
       "coins",
+      "coinHistory",
       "coinAvailability",
       "refreshGroups",
       "refreshSessions",
@@ -175,36 +189,21 @@ export async function spendCoins(
       coin.exchangeBaseUrl,
       coin.denomPubHash,
     );
-    checkDbInvariant(!!denom, `denomination of a coin is missing hash: ${coin.denomPubHash}`);
+    checkDbInvariant(
+      !!denom,
+      `denomination of a coin is missing hash: ${coin.denomPubHash}`,
+    );
     const coinAvailability = await tx.coinAvailability.get([
       coin.exchangeBaseUrl,
       coin.denomPubHash,
       coin.maxAge,
     ]);
-    checkDbInvariant(!!coinAvailability, `age denom info is missing for ${coin.maxAge}`);
+    checkDbInvariant(
+      !!coinAvailability,
+      `age denom info is missing for ${coin.maxAge}`,
+    );
     const contrib = csi.contributions[i];
-    if (coin.status !== CoinStatus.Fresh) {
-      const alloc = coin.spendAllocation;
-      if (!alloc) {
-        continue;
-      }
-      if (alloc.id !== csi.allocationId) {
-        // FIXME: assign error code
-        logger.info("conflicting coin allocation ID");
-        logger.info(`old ID: ${alloc.id}, new ID: ${csi.allocationId}`);
-        throw Error("conflicting coin allocation (id)");
-      }
-      if (0 !== Amounts.cmp(alloc.amount, contrib)) {
-        // FIXME: assign error code
-        throw Error("conflicting coin allocation (contrib)");
-      }
-      continue;
-    }
     coin.status = CoinStatus.Dormant;
-    coin.spendAllocation = {
-      id: csi.allocationId,
-      amount: Amounts.stringify(contrib),
-    };
     const remaining = Amounts.sub(denom.value, contrib);
     if (remaining.saturated) {
       throw Error("not enough remaining balance on coin for payment");
@@ -226,6 +225,21 @@ export async function spendCoins(
         coinAvailability.visibleCoinCount--;
       }
     }
+    let histEntry: CoinHistoryRecord | undefined = await tx.coinHistory.get(
+      coin.coinPub,
+    );
+    if (!histEntry) {
+      histEntry = {
+        coinPub: coin.coinPub,
+        history: [],
+      };
+    }
+    histEntry.history.push({
+      type: "spend",
+      transactionId: csi.transactionId,
+      amount: Amounts.stringify(contrib),
+    });
+    await tx.coinHistory.put(histEntry);
     await tx.coins.put(coin);
     await tx.coinAvailability.put(coinAvailability);
   }
@@ -236,7 +250,7 @@ export async function spendCoins(
     Amounts.currencyOf(csi.contributions[0]),
     refreshCoinPubs,
     csi.refreshReason,
-    csi.allocationId,
+    csi.transactionId,
   );
 }
 
@@ -257,6 +271,9 @@ export enum TombstoneTag {
 export function getExchangeTosStatusFromRecord(
   exchange: ExchangeEntryRecord,
 ): ExchangeTosStatus {
+  if (exchange.tosCurrentEtag == null) {
+    return ExchangeTosStatus.MissingTos;
+  }
   if (!exchange.tosAcceptedEtag) {
     return ExchangeTosStatus.Proposed;
   }
@@ -282,6 +299,8 @@ export function getExchangeUpdateStatusFromRecord(
       return ExchangeUpdateStatus.ReadyUpdate;
     case ExchangeEntryDbUpdateStatus.Suspended:
       return ExchangeUpdateStatus.Suspended;
+    case ExchangeEntryDbUpdateStatus.OutdatedUpdate:
+      return ExchangeUpdateStatus.OutdatedUpdate;
     default:
       assertUnreachable(r.updateStatus);
   }
@@ -364,6 +383,7 @@ export enum TaskRunResultType {
   Error = "error",
   LongpollReturnedPending = "longpoll-returned-pending",
   ScheduleLater = "schedule-later",
+  NetworkRequired = "network-required",
 }
 
 export type TaskRunResult =
@@ -372,7 +392,8 @@ export type TaskRunResult =
   | TaskRunBackoffResult
   | TaskRunProgressResult
   | TaskRunLongpollReturnedPendingResult
-  | TaskRunScheduleLaterResult;
+  | TaskRunScheduleLaterResult
+  | TaskRunNetworkRequiredResult;
 
 export namespace TaskRunResult {
   /**
@@ -419,6 +440,15 @@ export namespace TaskRunResult {
       type: TaskRunResultType.LongpollReturnedPending,
     };
   }
+  /**
+   * Network connection is required to complete the task.
+   * When network is back, the transaction will be retried.
+   */
+  export function networkRequired(): TaskRunResult {
+    return {
+      type: TaskRunResultType.NetworkRequired,
+    };
+  }
 }
 
 export interface TaskRunFinishedResult {
@@ -442,6 +472,10 @@ export interface TaskRunLongpollReturnedPendingResult {
   type: TaskRunResultType.LongpollReturnedPending;
 }
 
+export interface TaskRunNetworkRequiredResult {
+  type: TaskRunResultType.NetworkRequired;
+}
+
 export interface TaskRunErrorResult {
   type: TaskRunResultType.Error;
   errorDetail: TalerErrorDetail;
@@ -681,9 +715,6 @@ export namespace TaskIdentifiers {
       exchBaseUrl,
     )}` as TaskIdStr;
   }
-  export function forTipPickup(tipRecord: RewardRecord): TaskIdStr {
-    return `${PendingTaskType.RewardPickup}:${tipRecord.walletRewardId}` as TaskIdStr;
-  }
   export function forRefresh(
     refreshGroupRecord: RefreshGroupRecord,
   ): TaskIdStr {
@@ -766,6 +797,9 @@ export interface TransactionContext {
   resumeTransaction(): Promise<void>;
   failTransaction(): Promise<void>;
   deleteTransaction(): Promise<void>;
+  lookupFullTransaction(
+    tx: WalletDbAllStoresReadOnlyTransaction,
+  ): Promise<Transaction | undefined>;
 }
 
 declare const __taskIdStr: unique symbol;
@@ -798,7 +832,7 @@ export async function genericWaitForState(
       flag.raise();
     }
   });
-  const unregisterOnCancelled = wex.cancellationToken.onCancelled(() => {
+  const unregisterOnCancelled = wex.cancellationToken.onCancelled((reason) => {
     cancelNotif();
     flag.raise();
   });
@@ -821,3 +855,71 @@ export async function genericWaitForState(
     throw e;
   }
 }
+
+export function requireExchangeTosAcceptedOrThrow(
+  exchange: ReadyExchangeSummary,
+): void {
+  switch (exchange.tosStatus) {
+    case ExchangeTosStatus.Accepted:
+    case ExchangeTosStatus.MissingTos:
+      break;
+    default:
+      throw TalerError.fromDetail(
+        TalerErrorCode.WALLET_EXCHANGE_TOS_NOT_ACCEPTED,
+        {
+          exchangeBaseUrl: exchange.exchangeBaseUrl,
+          currentEtag: exchange.tosCurrentEtag,
+          tosStatus: exchange.tosStatus,
+        },
+      );
+  }
+}
+
+/**
+ * Run a request, but cancel the wallet execution context as soon as the client
+ * submits another request of the same type.
+ */
+export async function runWithClientCancellation<R, T>(
+  wex: WalletExecutionContext,
+  operation: string,
+  clientCancellationId: string | undefined,
+  handler: () => Promise<T>,
+): Promise<T> {
+  const clientCancelKey = clientCancellationId
+    ? `ccid:${operation}:${clientCancellationId}`
+    : undefined;
+  const cts = wex.cts;
+  if (clientCancelKey && cts) {
+    const prevCts = wex.ws.clientCancellationMap.get(clientCancelKey);
+    if (prevCts) {
+      wex.oc.observe({
+        type: ObservabilityEventType.Message,
+        contents: `Cancelling previous key ${clientCancelKey}`,
+      });
+      prevCts.cancel(
+        `cancelled by subsequent request with same cancellation ID`,
+      );
+    } else {
+      wex.oc.observe({
+        type: ObservabilityEventType.Message,
+        contents: `No previous key ${clientCancelKey}`,
+      });
+    }
+    wex.oc.observe({
+      type: ObservabilityEventType.Message,
+      contents: `Setting clientCancelKey ${clientCancelKey} to ${cts}`,
+    });
+    wex.ws.clientCancellationMap.set(clientCancelKey, cts);
+  }
+  try {
+    return await handler();
+  } finally {
+    wex.oc.observe({
+      type: ObservabilityEventType.Message,
+      contents: `Deleting clientCancelKey ${clientCancelKey} to ${cts}`,
+    });
+    if (clientCancelKey && wex.cts && !wex.cts.token.isCancelled) {
+      wex.ws.clientCancellationMap.delete(clientCancelKey);
+    }
+  }
+}
