@@ -214,6 +214,10 @@ export interface TalerCryptoInterface {
 
   signPurseCreation(req: SignPurseCreationRequest): Promise<EddsaSigningResult>;
 
+  signReserveHistoryReq(
+    req: SignReserveHistoryReqRequest,
+  ): Promise<SignReserveHistoryReqResponse>;
+
   signPurseDeposits(
     req: SignPurseDepositsRequest,
   ): Promise<SignPurseDepositsResponse>;
@@ -438,6 +442,11 @@ export const nullCrypto: TalerCryptoInterface = {
   ): Promise<SignCoinHistoryResponse> {
     throw new Error("Function not implemented.");
   },
+  signReserveHistoryReq: function (
+    req: SignReserveHistoryReqRequest,
+  ): Promise<SignReserveHistoryReqResponse> {
+    throw new Error("Function not implemented.");
+  },
 };
 
 export type WithArg<X> = X extends (req: infer T) => infer R
@@ -475,6 +484,15 @@ export interface SignPurseCreationRequest {
   minAge: number;
 }
 
+export interface SignReserveHistoryReqRequest {
+  reservePriv: string;
+  startOffset: number;
+}
+
+export interface SignReserveHistoryReqResponse {
+  sig: string;
+}
+
 export interface SpendCoinDetails {
   coinPub: string;
   coinPriv: string;
@@ -1468,15 +1486,12 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
     const hExchangeBaseUrl = hash(stringToBytes(req.exchangeBaseUrl + "\0"));
     const deposits: PurseDeposit[] = [];
     for (const c of req.coins) {
-      let haveAch: boolean;
       let maybeAch: Uint8Array;
       if (c.ageCommitmentProof) {
-        haveAch = true;
         maybeAch = decodeCrock(
           AgeRestriction.hashCommitment(c.ageCommitmentProof.commitment),
         );
       } else {
-        haveAch = false;
         maybeAch = new Uint8Array(32);
       }
       const sigBlob = buildSigPS(TalerSignaturePurpose.WALLET_PURSE_DEPOSIT)
@@ -1733,6 +1748,23 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       sig: sigResp.sig,
     };
   },
+  async signReserveHistoryReq(
+    tci: TalerCryptoInterfaceR,
+    req: SignReserveHistoryReqRequest,
+  ): Promise<SignReserveHistoryReqResponse> {
+    const reserveHistoryBlob = buildSigPS(
+      TalerSignaturePurpose.WALLET_RESERVE_HISTORY,
+    )
+      .put(bufferForUint64(req.startOffset))
+      .build();
+    const sigResp = await tci.eddsaSign(tci, {
+      msg: encodeCrock(reserveHistoryBlob),
+      priv: req.reservePriv,
+    });
+    return {
+      sig: sigResp.sig,
+    };
+  },
 };
 
 export interface EddsaSignRequest {
