@@ -279,6 +279,16 @@ export interface SignDeletePurseResponse {
   sig: EddsaSignatureString;
 }
 
+export interface SignCoinHistoryRequest {
+  coinPub: string;
+  coinPriv: string;
+  startOffset: number;
+}
+
+export interface SignCoinHistoryResponse {
+  sig: EddsaSignatureString;
+}
+
 export interface SignReservePurseCreateRequest {
   mergeTimestamp: TalerProtocolTimestamp;
 
