@@ -34,11 +34,13 @@ import {
   Amounts,
   AttentionInfo,
   BackupProviderTerms,
+  CancellationToken,
   Codec,
   CoinEnvelope,
   CoinPublicKeyString,
   CoinRefreshRequest,
   CoinStatus,
+  CurrencySpecification,
   DenomLossEventType,
   DenomSelectionState,
   DenominationInfo,
@@ -50,16 +52,18 @@ import {
   HashCodeString,
   Logger,
   RefreshReason,
+  ScopeInfo,
   TalerErrorDetail,
   TalerPreciseTimestamp,
   TalerProtocolDuration,
   TalerProtocolTimestamp,
-  Transaction,
   TransactionIdStr,
   UnblindedSignature,
   WireInfo,
   WithdrawalExchangeAccountDetails,
   codecForAny,
+  j2s,
+  stringifyScopeInfo,
 } from "@gnu-taler/taler-util";
 import { DbRetryInfo, TaskIdentifiers } from "./common.js";
 import {
@@ -150,7 +154,7 @@ export const CURRENT_DB_CONFIG_KEY = "currentMainDbName";
  * backwards-compatible way or object stores and indices
  * are added.
  */
-export const WALLET_DB_MINOR_VERSION = 10;
+export const WALLET_DB_MINOR_VERSION = 13;
 
 declare const symDbProtocolTimestamp: unique symbol;
 
@@ -247,6 +251,9 @@ export function timestampOptionalAbsoluteFromDb(
  * 0x0103_nnnn: aborting
  * 0x0110_nnnn: suspended
  * 0x0113_nnnn: suspended-aborting
+ * a=2: finalizing
+ * 0x0200_nnnn: finalizing
+ * 0x0210_nnnn: suspended-finalizing
  * a=5: final
  * 0x0500_nnnn: done
  * 0x0501_nnnn: failed
@@ -259,12 +266,12 @@ export function timestampOptionalAbsoluteFromDb(
 /**
  * First possible operation status in the active range (inclusive).
  */
-export const OPERATION_STATUS_ACTIVE_FIRST = 0x0100_0000;
+export const OPERATION_STATUS_NONFINAL_FIRST = 0x0100_0000;
 
 /**
  * LAST possible operation status in the active range (inclusive).
  */
-export const OPERATION_STATUS_ACTIVE_LAST = 0x0113_ffff;
+export const OPERATION_STATUS_NONFINAL_LAST = 0x0210_ffff;
 
 /**
  * Status of a withdrawal.
@@ -296,6 +303,11 @@ export enum WithdrawalGroupStatus {
   PendingReady = 0x0100_0004,
   SuspendedReady = 0x0110_0004,
 
+  /**
+   * Proposed to the user, has can choose to accept/refuse.
+   */
+  DialogProposed = 0x0101_0000,
+
   /**
    * We are telling the bank that we don't want to complete
    * the withdrawal!
@@ -337,6 +349,21 @@ export enum WithdrawalGroupStatus {
   AbortedExchange = 0x0503_0001,
 
   AbortedBank = 0x0503_0002,
+
+  /**
+   * User didn't refused the withdrawal.
+   */
+  AbortedUserRefused = 0x0503_0003,
+
+  /**
+   * Another wallet confirmed the withdrawal
+   * (by POSTing the reserve pub to the bank)
+   * before we had the chance.
+   *
+   * In this situation, we'll let the other wallet continue
+   * and give up ourselves.
+   */
+  AbortedOtherWallet = 0x0503_0004,
 }
 
 /**
@@ -355,7 +382,7 @@ export interface ReserveBankInfo {
   /**
    * Exchange payto URI that the bank will use to fund the reserve.
    */
-  exchangePaytoUri: string;
+  exchangePaytoUri?: string;
 
   /**
    * Time when the information about this reserve was posted to the bank.
@@ -372,6 +399,12 @@ export interface ReserveBankInfo {
    * Set to undefined if not confirmed yet.
    */
   timestampBankConfirmed: DbPreciseTimestamp | undefined;
+
+  wireTypes: string[] | undefined;
+
+  currency: string | undefined;
+
+  externalConfirmation?: boolean;
 }
 
 /**
@@ -609,6 +642,7 @@ export enum ExchangeEntryDbUpdateStatus {
   // Reserved 5 for backwards compatibility.
   Ready = 6,
   ReadyUpdate = 7,
+  OutdatedUpdate = 8,
 }
 
 /**
@@ -649,6 +683,8 @@ export interface ExchangeEntryRecord {
 
   updateStatus: ExchangeEntryDbUpdateStatus;
 
+  unavailableReason?: TalerErrorDetail;
+
   /**
    * If set to true, the next update to the exchange
    * status will request /keys with no-cache headers set.
@@ -674,6 +710,14 @@ export interface ExchangeEntryRecord {
    */
   nextUpdateStamp: DbPreciseTimestamp;
 
+  /**
+   * The number of times we tried to contact the exchange,
+   * the exchange returned a result, but it is conflicting with the
+   * existing exchange entry.
+   *
+   * We keep the retry counter here instead of using the task retries,
+   * as the task succeeded, the exchange is just not useable.
+   */
   updateRetryCounter?: number;
 
   lastKeysEtag: string | undefined;
@@ -865,15 +909,6 @@ export interface CoinRecord {
    */
   visible?: number;
 
-  /**
-   * Information about what the coin has been allocated for.
-   *
-   * Used for:
-   * - Diagnostics
-   * - Idempotency of applying a coin selection (e.g. after re-selection)
-   */
-  spendAllocation: CoinAllocation | undefined;
-
   /**
    * Maximum age of purchases that can be made with this coin.
    *
@@ -885,100 +920,50 @@ export interface CoinRecord {
 }
 
 /**
- * Coin allocation, i.e. what a coin has been used for.
+ * History item for a coin.
+ *
+ * DB-specific format,
  */
-export interface CoinAllocation {
-  /**
-   * ID of the allocation, should be the ID of the transaction that
-   */
-  id: TransactionIdStr;
-  amount: AmountString;
-}
+export type DbWalletCoinHistoryItem =
+  | {
+      type: "withdraw";
+      transactionId: TransactionIdStr;
+    }
+  | {
+      type: "spend";
+      transactionId: TransactionIdStr;
+      amount: AmountString;
+    }
+  | {
+      type: "refresh";
+      transactionId: TransactionIdStr;
+      amount: AmountString;
+    }
+  | {
+      type: "recoup";
+      transactionId: TransactionIdStr;
+      amount: AmountString;
+    }
+  | {
+      type: "refund";
+      transactionId: TransactionIdStr;
+      amount: AmountString;
+    };
 
 /**
- * Status of a reward we got from a merchant.
+ * History event for a coin from the wallet's perspective.
+ *
+ * The history might reference transactions that were already deleted from the wallet.
  */
-export interface RewardRecord {
+export interface CoinHistoryRecord {
+  coinPub: string;
   /**
-   * Has the user accepted the tip?  Only after the tip has been accepted coins
-   * withdrawn from the tip may be used.
-   */
-  acceptedTimestamp: DbPreciseTimestamp | undefined;
-
-  /**
-   * The tipped amount.
-   */
-  rewardAmountRaw: AmountString;
-
-  /**
-   * Effect on the balance (including fees etc).
-   */
-  rewardAmountEffective: AmountString;
-
-  /**
-   * Timestamp, the tip can't be picked up anymore after this deadline.
-   */
-  rewardExpiration: DbProtocolTimestamp;
-
-  /**
-   * The exchange that will sign our coins, chosen by the merchant.
-   */
-  exchangeBaseUrl: string;
-
-  /**
-   * Base URL of the merchant that is giving us the tip.
-   */
-  merchantBaseUrl: string;
-
-  /**
-   * Denomination selection made by the wallet for picking up
-   * this tip.
+   * History items for the coin.
    *
-   * FIXME: Put this into some DenomSelectionCacheRecord instead of
-   * storing it here!
+   * We store this as an array in the object store, as the coin history
+   * is pretty much always very small.
    */
-  denomsSel: DenomSelectionState;
-
-  denomSelUid: string;
-
-  /**
-   * Tip ID chosen by the wallet.
-   */
-  walletRewardId: string;
-
-  /**
-   * Secret seed used to derive planchets for this tip.
-   */
-  secretSeed: string;
-
-  /**
-   * The merchant's identifier for this reward.
-   */
-  merchantRewardId: string;
-
-  createdTimestamp: DbPreciseTimestamp;
-
-  /**
-   * The url to be redirected after the tip is accepted.
-   */
-  next_url: string | undefined;
-
-  /**
-   * Timestamp for when the wallet finished picking up the tip
-   * from the merchant.
-   */
-  pickedUpTimestamp: DbPreciseTimestamp | undefined;
-
-  status: RewardRecordStatus;
-}
-
-export enum RewardRecordStatus {
-  PendingPickup = 0x0100_0000,
-  SuspendedPickup = 0x0110_0000,
-  DialogAccept = 0x0101_0000,
-  Done = 0x0500_0000,
-  Aborted = 0x0500_0000,
-  Failed = 0x0501_000,
+  history: DbWalletCoinHistoryItem[];
 }
 
 export enum RefreshCoinStatus {
@@ -1155,10 +1140,15 @@ export enum PurchaseStatus {
 
   /**
    * Query for refund (until auto-refund deadline is reached).
+   *
+   * Legacy state for compatibility.
    */
   PendingQueryingAutoRefund = 0x0100_0004,
   SuspendedQueryingAutoRefund = 0x0110_0004,
 
+  FinalizingQueryingAutoRefund = 0x0200_0001,
+  SuspendedFinalizingQueryingAutoRefund = 0x0210_0001,
+
   PendingAcceptRefund = 0x0100_0005,
   SuspendedPendingAcceptRefund = 0x0110_0005,
 
@@ -1173,11 +1163,6 @@ export enum PurchaseStatus {
    */
   DialogShared = 0x0101_0001,
 
-  /**
-   * The user has rejected the proposal.
-   */
-  AbortedProposalRefused = 0x0503_0000,
-
   /**
    * Downloading or processing the proposal has failed permanently.
    */
@@ -1201,13 +1186,18 @@ export enum PurchaseStatus {
   DoneRepurchaseDetected = 0x0500_0001,
 
   /**
-   * The payment has been aborted.
+   * The user has rejected the proposal.
    */
-  AbortedIncompletePayment = 0x0503_0000,
+  AbortedProposalRefused = 0x0503_0000,
 
   AbortedRefunded = 0x0503_0001,
 
   AbortedOrderDeleted = 0x0503_0002,
+
+  /**
+   * The payment has been aborted.
+   */
+  AbortedIncompletePayment = 0x0503_0003,
 }
 
 /**
@@ -1416,6 +1406,7 @@ export interface WgInfoBankIntegrated {
    * a Taler-integrated bank.
    */
   bankInfo: ReserveBankInfo;
+
   /**
    * Info about withdrawal accounts, possibly including currency conversion.
    */
@@ -1507,7 +1498,7 @@ export interface WithdrawalGroupRecord {
    * The exchange base URL that we're withdrawing from.
    * (Redundantly stored, as the reserve record also has this info.)
    */
-  exchangeBaseUrl: string;
+  exchangeBaseUrl?: string;
 
   /**
    * When was the withdrawal operation started started?
@@ -1541,7 +1532,7 @@ export interface WithdrawalGroupRecord {
   /**
    * Amount that was sent by the user to fund the reserve.
    */
-  instructedAmount: AmountString;
+  instructedAmount?: AmountString;
 
   /**
    * Amount that was observed when querying the reserve that
@@ -1558,7 +1549,7 @@ export interface WithdrawalGroupRecord {
    * (Initial amount confirmed by the user, might differ with denomSel
    * on reselection.)
    */
-  rawWithdrawalAmount: AmountString;
+  rawWithdrawalAmount?: AmountString;
 
   /**
    * Amount that will be added to the balance when the withdrawal succeeds.
@@ -1566,12 +1557,12 @@ export interface WithdrawalGroupRecord {
    * (Initial amount confirmed by the user, might differ with denomSel
    * on reselection.)
    */
-  effectiveWithdrawalAmount: AmountString;
+  effectiveWithdrawalAmount?: AmountString;
 
   /**
    * Denominations selected for withdrawal.
    */
-  denomsSel: DenomSelectionState;
+  denomsSel?: DenomSelectionState;
 
   /**
    * UID of the denomination selection.
@@ -1953,7 +1944,7 @@ export enum PeerPullPaymentCreditStatus {
   SuspendedCreatePurse = 0x0110_0000,
   SuspendedReady = 0x0110_0001,
   SuspendedMergeKycRequired = 0x0110_0002,
-  SuspendedWithdrawing = 0x0110_0000,
+  SuspendedWithdrawing = 0x0110_0003,
 
   SuspendedAbortingDeletePurse = 0x0113_0000,
 
@@ -2352,13 +2343,26 @@ export interface GlobalCurrencyExchangeRecord {
 }
 
 /**
- * Primary key: transactionItem.transactionId
+ * Metadata for a transaction.
+ * This object store is effectively a materialzed view of transactions gathered
+ * from various other object stores.
+ *
+ * Primary key: transactionId
  */
-export interface TransactionRecord {
+export interface TransactionMetaRecord {
   /**
-   * Transaction item returned to the client.
+   * Transaction identifier.
+   * Also determines the type of the transaction.
    */
-  transactionItem: Transaction;
+  transactionId: string;
+
+  timestamp: DbPreciseTimestamp;
+
+  /**
+   * Status of the transaction, matches the status enum of the
+   * transaction of the type determined by the transaction ID.
+   */
+  status: number;
 
   /**
    * Exchanges involved in the transaction.
@@ -2391,6 +2395,23 @@ export interface DenomLossEventRecord {
   exchangeBaseUrl: string;
 }
 
+export interface CurrencyInfoRecord {
+  /**
+   * Stringified scope info.
+   */
+  scopeInfoStr: string;
+
+  /**
+   * Currency specification.
+   */
+  currencySpec: CurrencySpecification;
+
+  /**
+   * How did the currency info get set?
+   */
+  source: "exchange" | "user" | "preset";
+}
+
 /**
  * Schema definition for the IndexedDB
  * wallet database.
@@ -2410,21 +2431,33 @@ export const WalletStoresV1 = {
       }),
     },
   }),
-  transactions: describeStoreV2({
-    recordCodec: passthroughCodec<TransactionRecord>(),
-    storeName: "transactions",
-    keyPath: "transactionItem.transactionId",
-    versionAdded: 7,
+  transactionsMeta: describeStoreV2({
+    recordCodec: passthroughCodec<TransactionMetaRecord>(),
+    storeName: "transactionsMeta",
+    keyPath: "transactionId",
+    versionAdded: 13,
     indexes: {
       byCurrency: describeIndex("byCurrency", "currency", {
-        versionAdded: 7,
+        versionAdded: 13,
       }),
       byExchange: describeIndex("byExchange", "exchanges", {
-        versionAdded: 7,
+        versionAdded: 13,
         multiEntry: true,
       }),
+      byTimestamp: describeIndex("byTimestamp", "timestamp", {
+        versionAdded: 13,
+      }),
+      byStatus: describeIndex("byStatus", "status", {
+        versionAdded: 13,
+      }),
     },
   }),
+  currencyInfo: describeStoreV2({
+    recordCodec: passthroughCodec<CurrencyInfoRecord>(),
+    storeName: "currencyInfo",
+    keyPath: "scopeInfoStr",
+    versionAdded: 12,
+  }),
   globalCurrencyAuditors: describeStoreV2({
     recordCodec: passthroughCodec<GlobalCurrencyAuditorRecord>(),
     storeName: "globalCurrencyAuditors",
@@ -2475,6 +2508,12 @@ export const WalletStoresV1 = {
       }),
     },
   ),
+  coinHistory: describeStoreV2({
+    storeName: "coinHistory",
+    recordCodec: passthroughCodec<CoinHistoryRecord>(),
+    keyPath: "coinPub",
+    versionAdded: 11,
+  }),
   coins: describeStore(
     "coins",
     describeContents<CoinRecord>({
@@ -2607,9 +2646,10 @@ export const WalletStoresV1 = {
       ]),
     },
   ),
+  // Just a tombstone at this point.
   rewards: describeStore(
     "rewards",
-    describeContents<RewardRecord>({ keyPath: "walletRewardId" }),
+    describeContents<any>({ keyPath: "walletRewardId" }),
     {
       byMerchantTipIdAndBaseUrl: describeIndex("byMerchantRewardIdAndBaseUrl", [
         "merchantRewardId",
@@ -2656,7 +2696,9 @@ export const WalletStoresV1 = {
     describeContents<BankWithdrawUriRecord>({
       keyPath: "talerWithdrawUri",
     }),
-    {},
+    {
+      byGroup: describeIndex("byGroup", "withdrawalGroupId"),
+    },
   ),
   backupProviders: describeStore(
     "backupProviders",
@@ -2815,6 +2857,22 @@ export const WalletStoresV1 = {
     }),
     {},
   ),
+  // Obsolete store, not used anymore
+  _obsolete_transactions: describeStoreV2({
+    recordCodec: passthroughCodec<unknown>(),
+    storeName: "transactions",
+    keyPath: "transactionItem.transactionId",
+    versionAdded: 7,
+    indexes: {
+      byCurrency: describeIndex("byCurrency", "currency", {
+        versionAdded: 7,
+      }),
+      byExchange: describeIndex("byExchange", "exchanges", {
+        versionAdded: 7,
+        multiEntry: true,
+      }),
+    },
+  }),
 };
 
 export type WalletDbStoresArr = Array<StoreNames<typeof WalletStoresV1>>;
@@ -2915,6 +2973,8 @@ export interface DbDump {
   };
 }
 
+const logger = new Logger("db.ts");
+
 export async function exportSingleDb(
   idb: IDBFactory,
   dbName: string,
@@ -3056,8 +3116,6 @@ export interface FixupDescription {
  */
 export const walletDbFixups: FixupDescription[] = [];
 
-const logger = new Logger("db.ts");
-
 export async function applyFixups(
   db: DbAccess<typeof WalletStoresV1>,
 ): Promise<void> {
@@ -3206,6 +3264,7 @@ export function clearDatabase(db: IDBDatabase): Promise<void> {
   for (let i = 0; i < db.objectStoreNames.length; i++) {
     stores.push(db.objectStoreNames[i]);
   }
+  logger.info(`clearing object stores: ${j2s(stores)}`);
   const tx = db.transaction(stores, "readwrite");
   for (const store of stores) {
     tx.objectStore(store).clear();
@@ -3269,7 +3328,12 @@ export async function openStoredBackupsDatabase(
     onStoredBackupsDbUpgradeNeeded,
   );
 
-  const handle = new DbAccessImpl(backupsDbHandle, StoredBackupStores);
+  const handle = new DbAccessImpl(
+    backupsDbHandle,
+    StoredBackupStores,
+    {},
+    CancellationToken.CONTINUE,
+  );
   return handle;
 }
 
@@ -3283,7 +3347,7 @@ export async function openStoredBackupsDatabase(
 export async function openTalerDatabase(
   idbFactory: IDBFactory,
   onVersionChange: () => void,
-): Promise<DbAccess<typeof WalletStoresV1>> {
+): Promise<IDBDatabase> {
   const metaDbHandle = await openDatabase(
     idbFactory,
     TALER_WALLET_META_DB_NAME,
@@ -3292,9 +3356,14 @@ export async function openTalerDatabase(
     onMetaDbUpgradeNeeded,
   );
 
-  const metaDb = new DbAccessImpl(metaDbHandle, walletMetadataStore);
+  const metaDb = new DbAccessImpl(
+    metaDbHandle,
+    walletMetadataStore,
+    {},
+    CancellationToken.CONTINUE,
+  );
   let currentMainVersion: string | undefined;
-  await metaDb.runReadWriteTx(["metaConfig"], async (tx) => {
+  await metaDb.runReadWriteTx({ storeNames: ["metaConfig"] }, async (tx) => {
     const dbVersionRecord = await tx.metaConfig.get(CURRENT_DB_CONFIG_KEY);
     if (!dbVersionRecord) {
       currentMainVersion = TALER_WALLET_MAIN_DB_NAME;
@@ -3319,12 +3388,15 @@ export async function openTalerDatabase(
       case "taler-wallet-main-v9":
         // We consider this a pre-release
         // development version, no migration is done.
-        await metaDb.runReadWriteTx(["metaConfig"], async (tx) => {
-          await tx.metaConfig.put({
-            key: CURRENT_DB_CONFIG_KEY,
-            value: TALER_WALLET_MAIN_DB_NAME,
-          });
-        });
+        await metaDb.runReadWriteTx(
+          { storeNames: ["metaConfig"] },
+          async (tx) => {
+            await tx.metaConfig.put({
+              key: CURRENT_DB_CONFIG_KEY,
+              value: TALER_WALLET_MAIN_DB_NAME,
+            });
+          },
+        );
         break;
       default:
         throw Error(
@@ -3341,11 +3413,15 @@ export async function openTalerDatabase(
     onTalerDbUpgradeNeeded,
   );
 
-  const handle = new DbAccessImpl(mainDbHandle, WalletStoresV1);
+  const mainDbAccess = new DbAccessImpl(
+    mainDbHandle,
+    WalletStoresV1,
+    {},
+    CancellationToken.CONTINUE,
+  );
+  await applyFixups(mainDbAccess);
 
-  await applyFixups(handle);
-
-  return handle;
+  return mainDbHandle;
 }
 
 export async function deleteTalerDatabase(
@@ -3357,3 +3433,75 @@ export async function deleteTalerDatabase(
     req.onsuccess = () => resolve();
   });
 }
+
+/**
+ * High-level helpers to access the database.
+ * Eventually all access to the database should
+ * go through helpers in this namespace.
+ */
+export namespace WalletDbHelpers {
+  export interface GetCurrencyInfoDbResult {
+    /**
+     * Currency specification.
+     */
+    currencySpec: CurrencySpecification;
+
+    /**
+     * How did the currency info get set?
+     */
+    source: "exchange" | "user" | "preset";
+  }
+
+  export interface StoreCurrencyInfoDbRequest {
+    scopeInfo: ScopeInfo;
+    currencySpec: CurrencySpecification;
+    source: "exchange" | "user" | "preset";
+  }
+
+  export async function getCurrencyInfo(
+    tx: WalletDbReadOnlyTransaction<["currencyInfo"]>,
+    scopeInfo: ScopeInfo,
+  ): Promise<GetCurrencyInfoDbResult | undefined> {
+    const s = stringifyScopeInfo(scopeInfo);
+    const res = await tx.currencyInfo.get(s);
+    if (!res) {
+      return undefined;
+    }
+    return {
+      currencySpec: res.currencySpec,
+      source: res.source,
+    };
+  }
+
+  /**
+   * Store currency info for a scope.
+   *
+   * Overrides existing currency infos.
+   */
+  export async function upsertCurrencyInfo(
+    tx: WalletDbReadWriteTransaction<["currencyInfo"]>,
+    req: StoreCurrencyInfoDbRequest,
+  ): Promise<void> {
+    await tx.currencyInfo.put({
+      scopeInfoStr: stringifyScopeInfo(req.scopeInfo),
+      currencySpec: req.currencySpec,
+      source: req.source,
+    });
+  }
+
+  export async function insertCurrencyInfoUnlessExists(
+    tx: WalletDbReadWriteTransaction<["currencyInfo"]>,
+    req: StoreCurrencyInfoDbRequest,
+  ): Promise<void> {
+    const scopeInfoStr = stringifyScopeInfo(req.scopeInfo);
+    const oldRec = await tx.currencyInfo.get(scopeInfoStr);
+    if (oldRec) {
+      return;
+    }
+    await tx.currencyInfo.put({
+      scopeInfoStr: stringifyScopeInfo(req.scopeInfo),
+      currencySpec: req.currencySpec,
+      source: req.source,
+    });
+  }
+}
