@@ -31,6 +31,7 @@ import {
   CreateDepositGroupRequest,
   CreateDepositGroupResponse,
   DepositGroupFees,
+  DepositTransactionTrackingState,
   Duration,
   ExchangeBatchDepositRequest,
   ExchangeHandle,
@@ -48,6 +49,7 @@ import {
   TalerPreciseTimestamp,
   TalerProtocolTimestamp,
   TrackTransaction,
+  Transaction,
   TransactionAction,
   TransactionIdStr,
   TransactionMajorState,
@@ -72,7 +74,7 @@ import {
   stringToBytes,
 } from "@gnu-taler/taler-util";
 import { readSuccessResponseJsonOrThrow } from "@gnu-taler/taler-util/http";
-import { selectPayCoins } from "./coinSelection.js";
+import { selectPayCoins, selectPayCoinsInTx } from "./coinSelection.js";
 import {
   PendingTaskType,
   TaskIdStr,
@@ -80,6 +82,7 @@ import {
   TombstoneTag,
   TransactionContext,
   constructTaskIdentifier,
+  runWithClientCancellation,
   spendCoins,
 } from "./common.js";
 import {
@@ -90,7 +93,10 @@ import {
   DepositTrackingInfo,
   KycPendingInfo,
   RefreshOperationStatus,
+  WalletDbAllStoresReadOnlyTransaction,
+  timestampPreciseFromDb,
   timestampPreciseToDb,
+  timestampProtocolFromDb,
   timestampProtocolToDb,
 } from "./db.js";
 import { getExchangeWireDetailsInTx } from "./exchanges.js";
@@ -106,6 +112,7 @@ import {
 } from "./refresh.js";
 import {
   constructTransactionIdentifier,
+  isUnsuccessfulTransaction,
   notifyTransition,
   parseTransactionIdentifier,
 } from "./transactions.js";
@@ -134,6 +141,78 @@ export class DepositTransactionContext implements TransactionContext {
     });
   }
 
+  /**
+   * Get the full transaction details for the transaction.
+   *
+   * Returns undefined if the transaction is in a state where we do not have a
+   * transaction item (e.g. if it was deleted).
+   */
+  async lookupFullTransaction(
+    tx: WalletDbAllStoresReadOnlyTransaction,
+  ): Promise<Transaction | undefined> {
+    const dg = await tx.depositGroups.get(this.depositGroupId);
+    if (!dg) {
+      return undefined;
+    }
+    const ort = await tx.operationRetries.get(this.taskId);
+
+    let deposited = true;
+    if (dg.statusPerCoin) {
+      for (const d of dg.statusPerCoin) {
+        if (d == DepositElementStatus.DepositPending) {
+          deposited = false;
+        }
+      }
+    } else {
+      deposited = false;
+    }
+
+    const trackingState: DepositTransactionTrackingState[] = [];
+
+    for (const ts of Object.values(dg.trackingState ?? {})) {
+      trackingState.push({
+        amountRaw: ts.amountRaw,
+        timestampExecuted: timestampProtocolFromDb(ts.timestampExecuted),
+        wireFee: ts.wireFee,
+        wireTransferId: ts.wireTransferId,
+      });
+    }
+
+    let wireTransferProgress = 0;
+    if (dg.statusPerCoin) {
+      wireTransferProgress =
+        (100 *
+          dg.statusPerCoin.reduce(
+            (prev, cur) => prev + (cur === DepositElementStatus.Wired ? 1 : 0),
+            0,
+          )) /
+        dg.statusPerCoin.length;
+    }
+
+    const txState = computeDepositTransactionStatus(dg);
+    return {
+      type: TransactionType.Deposit,
+      txState,
+      txActions: computeDepositTransactionActions(dg),
+      amountRaw: Amounts.stringify(dg.counterpartyEffectiveDepositAmount),
+      amountEffective: isUnsuccessfulTransaction(txState)
+        ? Amounts.stringify(Amounts.zeroOfAmount(dg.totalPayCost))
+        : Amounts.stringify(dg.totalPayCost),
+      timestamp: timestampPreciseFromDb(dg.timestampCreated),
+      targetPaytoUri: dg.wire.payto_uri,
+      wireTransferDeadline: timestampProtocolFromDb(dg.wireTransferDeadline),
+      transactionId: constructTransactionIdentifier({
+        tag: TransactionType.Deposit,
+        depositGroupId: dg.depositGroupId,
+      }),
+      wireTransferProgress,
+      depositGroupId: dg.depositGroupId,
+      trackingState,
+      deposited,
+      ...(ort?.lastError ? { error: ort.lastError } : {}),
+    };
+  }
+
   async deleteTransaction(): Promise<void> {
     const depositGroupId = this.depositGroupId;
     const ws = this.wex;
@@ -387,11 +466,19 @@ export function computeDepositTransactionActions(
     case DepositOperationStatus.Finished:
       return [TransactionAction.Delete];
     case DepositOperationStatus.PendingDeposit:
-      return [TransactionAction.Suspend, TransactionAction.Abort];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Suspend,
+        TransactionAction.Abort,
+      ];
     case DepositOperationStatus.SuspendedDeposit:
       return [TransactionAction.Resume];
     case DepositOperationStatus.Aborting:
-      return [TransactionAction.Fail, TransactionAction.Suspend];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Fail,
+        TransactionAction.Suspend,
+      ];
     case DepositOperationStatus.Aborted:
       return [TransactionAction.Delete];
     case DepositOperationStatus.Failed:
@@ -399,9 +486,17 @@ export function computeDepositTransactionActions(
     case DepositOperationStatus.SuspendedAborting:
       return [TransactionAction.Resume, TransactionAction.Fail];
     case DepositOperationStatus.PendingKyc:
-      return [TransactionAction.Suspend, TransactionAction.Fail];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Suspend,
+        TransactionAction.Fail,
+      ];
     case DepositOperationStatus.PendingTrack:
-      return [TransactionAction.Suspend, TransactionAction.Abort];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Suspend,
+        TransactionAction.Abort,
+      ];
     case DepositOperationStatus.SuspendedKyc:
       return [TransactionAction.Resume, TransactionAction.Fail];
     case DepositOperationStatus.SuspendedTrack:
@@ -502,12 +597,13 @@ async function refundDepositGroup(
   const res = await wex.db.runReadWriteTx(
     {
       storeNames: [
+        "coinAvailability",
+        "coinHistory",
+        "coins",
+        "denominations",
         "depositGroups",
         "refreshGroups",
         "refreshSessions",
-        "coins",
-        "denominations",
-        "coinAvailability",
       ],
     },
     async (tx) => {
@@ -652,12 +748,20 @@ async function processDepositGroupPendingKyc(
     `kyc-check/${kycInfo.requirementRow}/${kycInfo.paytoHash}/${userType}`,
     kycInfo.exchangeBaseUrl,
   );
-  url.searchParams.set("timeout_ms", "10000");
-  logger.info(`kyc url ${url.href}`);
-  const kycStatusRes = await wex.http.fetch(url.href, {
-    method: "GET",
-    cancellationToken: wex.cancellationToken,
-  });
+
+  const kycStatusRes = await wex.ws.runLongpollQueueing(
+    wex,
+    url.hostname,
+    async (timeoutMs) => {
+      url.searchParams.set("timeout_ms", `${timeoutMs}`);
+      logger.info(`kyc url ${url.href}`);
+      return await wex.http.fetch(url.href, {
+        method: "GET",
+        cancellationToken: wex.cancellationToken,
+      });
+    },
+  );
+
   if (
     kycStatusRes.status === HttpStatusCode.Ok ||
     //FIXME: NoContent is not expected https://docs.taler.net/core/api-exchange.html#post--purses-$PURSE_PUB-merge
@@ -963,43 +1067,17 @@ async function processDepositGroupPendingDeposit(
 
   if (!depositGroup.payCoinSelection) {
     logger.info("missing coin selection for deposit group, selecting now");
-    // FIXME: Consider doing the coin selection inside the txn
-    const payCoinSel = await selectPayCoins(wex, {
-      restrictExchanges: {
-        auditors: [],
-        exchanges: contractData.allowedExchanges,
-      },
-      restrictWireMethod: contractData.wireMethod,
-      contractTermsAmount: Amounts.parseOrThrow(contractData.amount),
-      depositFeeLimit: Amounts.parseOrThrow(contractData.maxDepositFee),
-      prevPayCoins: [],
-    });
-
-    switch (payCoinSel.type) {
-      case "success":
-        logger.info("coin selection success");
-        break;
-      case "failure":
-        logger.info("coin selection failure");
-        throw TalerError.fromDetail(
-          TalerErrorCode.WALLET_DEPOSIT_GROUP_INSUFFICIENT_BALANCE,
-          {
-            insufficientBalanceDetails: payCoinSel.insufficientBalanceDetails,
-          },
-        );
-      case "prospective":
-        logger.info("coin selection prospective");
-        throw Error("insufficient balance (waiting on pending refresh)");
-      default:
-        assertUnreachable(payCoinSel);
-    }
 
     const transitionDone = await wex.db.runReadWriteTx(
       {
         storeNames: [
+          "contractTerms",
+          "exchanges",
+          "exchangeDetails",
           "depositGroups",
           "coins",
           "coinAvailability",
+          "coinHistory",
           "refreshGroups",
           "refreshSessions",
           "denominations",
@@ -1013,6 +1091,45 @@ async function processDepositGroupPendingDeposit(
         if (dg.statusPerCoin) {
           return false;
         }
+
+        const contractTermsRec = tx.contractTerms.get(
+          depositGroup.contractTermsHash,
+        );
+        if (!contractTermsRec) {
+          throw Error("contract terms for deposit not found in database");
+        }
+
+        const payCoinSel = await selectPayCoinsInTx(wex, tx, {
+          restrictExchanges: {
+            auditors: [],
+            exchanges: contractData.allowedExchanges,
+          },
+          restrictWireMethod: contractData.wireMethod,
+          contractTermsAmount: Amounts.parseOrThrow(contractData.amount),
+          depositFeeLimit: Amounts.parseOrThrow(contractData.maxDepositFee),
+          prevPayCoins: [],
+        });
+
+        switch (payCoinSel.type) {
+          case "success":
+            logger.info("coin selection success");
+            break;
+          case "failure":
+            logger.info("coin selection failure");
+            throw TalerError.fromDetail(
+              TalerErrorCode.WALLET_DEPOSIT_GROUP_INSUFFICIENT_BALANCE,
+              {
+                insufficientBalanceDetails:
+                  payCoinSel.insufficientBalanceDetails,
+              },
+            );
+          case "prospective":
+            logger.info("coin selection prospective");
+            throw Error("insufficient balance (waiting on pending refresh)");
+          default:
+            assertUnreachable(payCoinSel);
+        }
+
         dg.payCoinSelection = {
           coinContributions: payCoinSel.coinSel.coins.map(
             (x) => x.contribution,
@@ -1025,7 +1142,7 @@ async function processDepositGroupPendingDeposit(
         );
         await tx.depositGroups.put(dg);
         await spendCoins(wex, tx, {
-          allocationId: transactionId,
+          transactionId,
           coinPubs: dg.payCoinSelection.coinPubs,
           contributions: dg.payCoinSelection.coinContributions.map((x) =>
             Amounts.parseOrThrow(x),
@@ -1152,6 +1269,10 @@ export async function processDepositGroup(
   wex: WalletExecutionContext,
   depositGroupId: string,
 ): Promise<TaskRunResult> {
+  if (!wex.ws.networkAvailable) {
+    return TaskRunResult.networkRequired();
+  }
+
   const depositGroup = await wex.db.runReadOnlyTx(
     { storeNames: ["depositGroups"] },
     async (tx) => {
@@ -1248,11 +1369,17 @@ async function trackDeposit(
     wireHash,
   });
   url.searchParams.set("merchant_sig", sigResp.sig);
-  url.searchParams.set("timeout_ms", "30000");
-  const httpResp = await wex.http.fetch(url.href, {
-    method: "GET",
-    cancellationToken: wex.cancellationToken,
-  });
+  const httpResp = await wex.ws.runLongpollQueueing(
+    wex,
+    url.hostname,
+    async (timeoutMs) => {
+      url.searchParams.set("timeout_ms", `${timeoutMs}`);
+      return await wex.http.fetch(url.href, {
+        method: "GET",
+        cancellationToken: wex.cancellationToken,
+      });
+    },
+  );
   logger.trace(`deposits response status: ${httpResp.status}`);
   switch (httpResp.status) {
     case HttpStatusCode.Accepted: {
@@ -1284,6 +1411,22 @@ async function trackDeposit(
 export async function checkDepositGroup(
   wex: WalletExecutionContext,
   req: PrepareDepositRequest,
+): Promise<PrepareDepositResponse> {
+  return await runWithClientCancellation(
+    wex,
+    "checkDepositGroup",
+    req.clientCancellationId,
+    () => internalCheckDepositGroup(wex, req),
+  );
+}
+
+/**
+ * Check if creating a deposit group is possible and calculate
+ * the associated fees.
+ */
+export async function internalCheckDepositGroup(
+  wex: WalletExecutionContext,
+  req: PrepareDepositRequest,
 ): Promise<PrepareDepositResponse> {
   const p = parsePaytoUri(req.depositPaytoUri);
   if (!p) {
@@ -1590,20 +1733,21 @@ export async function createDepositGroup(
   const newTxState = await wex.db.runReadWriteTx(
     {
       storeNames: [
-        "depositGroups",
+        "coinAvailability",
+        "coinHistory",
         "coins",
-        "recoupGroups",
+        "contractTerms",
         "denominations",
+        "depositGroups",
+        "recoupGroups",
         "refreshGroups",
         "refreshSessions",
-        "coinAvailability",
-        "contractTerms",
       ],
     },
     async (tx) => {
       if (depositGroup.payCoinSelection) {
         await spendCoins(wex, tx, {
-          allocationId: transactionId,
+          transactionId,
           coinPubs: depositGroup.payCoinSelection.coinPubs,
           contributions: depositGroup.payCoinSelection.coinContributions.map(
             (x) => Amounts.parseOrThrow(x),
