@@ -79,23 +79,26 @@ export async function applyDevExperiment(
     }
     case "insert-pending-refresh": {
       const refreshGroupId = encodeCrock(getRandomBytes(32));
-      await wex.db.runReadWriteTx(["refreshGroups"], async (tx) => {
-        const newRg: RefreshGroupRecord = {
-          currency: "TESTKUDOS",
-          expectedOutputPerCoin: [],
-          inputPerCoin: [],
-          oldCoinPubs: [],
-          operationStatus: RefreshOperationStatus.Pending,
-          reason: RefreshReason.Manual,
-          refreshGroupId,
-          statusPerCoin: [],
-          timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
-          timestampFinished: undefined,
-          originatingTransactionId: undefined,
-          infoPerExchange: {},
-        };
-        await tx.refreshGroups.put(newRg);
-      });
+      await wex.db.runReadWriteTx(
+        { storeNames: ["refreshGroups"] },
+        async (tx) => {
+          const newRg: RefreshGroupRecord = {
+            currency: "TESTKUDOS",
+            expectedOutputPerCoin: [],
+            inputPerCoin: [],
+            oldCoinPubs: [],
+            operationStatus: RefreshOperationStatus.Pending,
+            reason: RefreshReason.Manual,
+            refreshGroupId,
+            statusPerCoin: [],
+            timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
+            timestampFinished: undefined,
+            originatingTransactionId: undefined,
+            infoPerExchange: {},
+          };
+          await tx.refreshGroups.put(newRg);
+        },
+      );
       wex.taskScheduler.startShepherdTask(
         constructTaskIdentifier({
           tag: PendingTaskType.Refresh,
@@ -105,23 +108,26 @@ export async function applyDevExperiment(
       return;
     }
     case "insert-denom-loss": {
-      await wex.db.runReadWriteTx(["denomLossEvents"], async (tx) => {
-        const eventId = encodeCrock(getRandomBytes(32));
-        const newRg: DenomLossEventRecord = {
-          amount: "TESTKUDOS:42",
-          currency: "TESTKUDOS",
-          exchangeBaseUrl: "https://exchange.test.taler.net/",
-          denomLossEventId: eventId,
-          denomPubHashes: [
-            encodeCrock(getRandomBytes(64)),
-            encodeCrock(getRandomBytes(64)),
-          ],
-          eventType: DenomLossEventType.DenomExpired,
-          status: DenomLossStatus.Done,
-          timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
-        };
-        await tx.denomLossEvents.put(newRg);
-      });
+      await wex.db.runReadWriteTx(
+        { storeNames: ["denomLossEvents"] },
+        async (tx) => {
+          const eventId = encodeCrock(getRandomBytes(32));
+          const newRg: DenomLossEventRecord = {
+            amount: "TESTKUDOS:42",
+            currency: "TESTKUDOS",
+            exchangeBaseUrl: "https://exchange.test.taler.net/",
+            denomLossEventId: eventId,
+            denomPubHashes: [
+              encodeCrock(getRandomBytes(64)),
+              encodeCrock(getRandomBytes(64)),
+            ],
+            eventType: DenomLossEventType.DenomExpired,
+            status: DenomLossStatus.Done,
+            timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
+          };
+          await tx.denomLossEvents.put(newRg);
+        },
+      );
       return;
     }
   }
