@@ -18,43 +18,29 @@
  * Module entry point for the wallet when used as a node module.
  */
 
-// Util functionality
-export * from "./util/promiseUtils.js";
-export * from "./util/query.js";
-
-export * from "./versions.js";
-
-export * from "./db.js";
-
-// Crypto and crypto workers
-// export * from "./crypto/workers/nodeThreadWorker.js";
-export type { CryptoWorker } from "./crypto/workers/cryptoWorkerInterface.js";
-export {
-  CryptoWorkerFactory,
-  CryptoDispatcher,
-} from "./crypto/workers/crypto-dispatcher.js";
-
-export * from "./pending-types.js";
-
-export { InternalWalletState } from "./internal-wallet-state.js";
-export * from "./wallet-api-types.js";
-export * from "./wallet.js";
-
-export * from "./operations/backup/index.js";
-
-export * from "./operations/exchanges.js";
-
-export * from "./operations/withdraw.js";
-export * from "./operations/refresh.js";
-
-export * from "./dbless.js";
-
-export * from "./crypto/cryptoTypes.js";
 export * from "./crypto/cryptoImplementation.js";
-
-export * from "./util/timer.js";
-export * from "./util/denominations.js";
-
+export * from "./crypto/cryptoTypes.js";
+export {
+  CryptoDispatcher,
+  CryptoWorkerFactory,
+} from "./crypto/workers/crypto-dispatcher.js";
+export type { CryptoWorker } from "./crypto/workers/cryptoWorkerInterface.js";
 export { SynchronousCryptoWorkerFactoryPlain } from "./crypto/workers/synchronousWorkerFactoryPlain.js";
 export * from "./host-common.js";
 export * from "./host.js";
+export * from "./versions.js";
+export * from "./wallet-api-types.js";
+export * from "./wallet.js";
+
+export { parseTransactionIdentifier } from "./transactions.js";
+
+export { createPairTimeline } from "./denominations.js";
+
+// FIXME: Should these really be exported?!
+export {
+  WalletStoresV1,
+  deleteTalerDatabase,
+  exportDb,
+  importDb,
+} from "./db.js";
+export { DbAccess } from "./query.js";
