@@ -31,7 +31,11 @@ import {
   codecOptional,
 } from "@gnu-taler/taler-util";
 import { SpendCoinDetails } from "./crypto/cryptoImplementation.js";
-import { DbPeerPushPaymentCoinSelection, ReserveRecord } from "./db.js";
+import {
+  DbPeerPushPaymentCoinSelection,
+  ReserveRecord,
+  WalletDbReadOnlyTransaction,
+} from "./db.js";
 import { getTotalRefreshCost } from "./refresh.js";
 import { WalletExecutionContext, getDenomInfo } from "./wallet.js";
 
@@ -43,68 +47,79 @@ export async function queryCoinInfosForSelection(
   csel: DbPeerPushPaymentCoinSelection,
 ): Promise<SpendCoinDetails[]> {
   let infos: SpendCoinDetails[] = [];
-  await wex.db.runReadOnlyTx(["coins", "denominations"], async (tx) => {
-    for (let i = 0; i < csel.coinPubs.length; i++) {
-      const coin = await tx.coins.get(csel.coinPubs[i]);
-      if (!coin) {
-        throw Error("coin not found anymore");
+  await wex.db.runReadOnlyTx(
+    { storeNames: ["coins", "denominations"] },
+    async (tx) => {
+      for (let i = 0; i < csel.coinPubs.length; i++) {
+        const coin = await tx.coins.get(csel.coinPubs[i]);
+        if (!coin) {
+          throw Error("coin not found anymore");
+        }
+        const denom = await getDenomInfo(
+          wex,
+          tx,
+          coin.exchangeBaseUrl,
+          coin.denomPubHash,
+        );
+        if (!denom) {
+          throw Error("denom for coin not found anymore");
+        }
+        infos.push({
+          coinPriv: coin.coinPriv,
+          coinPub: coin.coinPub,
+          denomPubHash: coin.denomPubHash,
+          denomSig: coin.denomSig,
+          ageCommitmentProof: coin.ageCommitmentProof,
+          contribution: csel.contributions[i],
+        });
       }
-      const denom = await getDenomInfo(
-        wex,
-        tx,
-        coin.exchangeBaseUrl,
-        coin.denomPubHash,
-      );
-      if (!denom) {
-        throw Error("denom for coin not found anymore");
-      }
-      infos.push({
-        coinPriv: coin.coinPriv,
-        coinPub: coin.coinPub,
-        denomPubHash: coin.denomPubHash,
-        denomSig: coin.denomSig,
-        ageCommitmentProof: coin.ageCommitmentProof,
-        contribution: csel.contributions[i],
-      });
-    }
-  });
+    },
+  );
   return infos;
 }
 
+export async function getTotalPeerPaymentCostInTx(
+  wex: WalletExecutionContext,
+  tx: WalletDbReadOnlyTransaction<["coins", "denominations"]>,
+  pcs: SelectedProspectiveCoin[],
+): Promise<AmountJson> {
+  const costs: AmountJson[] = [];
+  for (let i = 0; i < pcs.length; i++) {
+    const denomInfo = await getDenomInfo(
+      wex,
+      tx,
+      pcs[i].exchangeBaseUrl,
+      pcs[i].denomPubHash,
+    );
+    if (!denomInfo) {
+      throw Error(
+        "can't calculate payment cost, denomination for coin not found",
+      );
+    }
+    const amountLeft = Amounts.sub(denomInfo.value, pcs[i].contribution).amount;
+    const refreshCost = await getTotalRefreshCost(
+      wex,
+      tx,
+      denomInfo,
+      amountLeft,
+    );
+    costs.push(Amounts.parseOrThrow(pcs[i].contribution));
+    costs.push(refreshCost);
+  }
+  const zero = Amounts.zeroOfAmount(pcs[0].contribution);
+  return Amounts.sum([zero, ...costs]).amount;
+}
+
 export async function getTotalPeerPaymentCost(
   wex: WalletExecutionContext,
   pcs: SelectedProspectiveCoin[],
 ): Promise<AmountJson> {
-  return wex.db.runReadOnlyTx(["coins", "denominations"], async (tx) => {
-    const costs: AmountJson[] = [];
-    for (let i = 0; i < pcs.length; i++) {
-      const denomInfo = await getDenomInfo(
-        wex,
-        tx,
-        pcs[i].exchangeBaseUrl,
-        pcs[i].denomPubHash,
-      );
-      if (!denomInfo) {
-        throw Error(
-          "can't calculate payment cost, denomination for coin not found",
-        );
-      }
-      const amountLeft = Amounts.sub(
-        denomInfo.value,
-        pcs[i].contribution,
-      ).amount;
-      const refreshCost = await getTotalRefreshCost(
-        wex,
-        tx,
-        denomInfo,
-        amountLeft,
-      );
-      costs.push(Amounts.parseOrThrow(pcs[i].contribution));
-      costs.push(refreshCost);
-    }
-    const zero = Amounts.zeroOfAmount(pcs[0].contribution);
-    return Amounts.sum([zero, ...costs]).amount;
-  });
+  return wex.db.runReadOnlyTx(
+    { storeNames: ["coins", "denominations"] },
+    async (tx) => {
+      return getTotalPeerPaymentCostInTx(wex, tx, pcs);
+    },
+  );
 }
 
 interface ExchangePurseStatus {
@@ -131,13 +146,16 @@ export async function getMergeReserveInfo(
   const newReservePair = await wex.cryptoApi.createEddsaKeypair({});
 
   const mergeReserveRecord: ReserveRecord = await wex.db.runReadWriteTx(
-    ["exchanges", "reserves"],
+    { storeNames: ["exchanges", "reserves"] },
     async (tx) => {
       const ex = await tx.exchanges.get(req.exchangeBaseUrl);
-      checkDbInvariant(!!ex);
+      checkDbInvariant(!!ex, `no exchange record for ${req.exchangeBaseUrl}`);
       if (ex.currentMergeReserveRowId != null) {
         const reserve = await tx.reserves.get(ex.currentMergeReserveRowId);
-        checkDbInvariant(!!reserve);
+        checkDbInvariant(
+          !!reserve,
+          `reserver ${ex.currentMergeReserveRowId} missing in db`,
+        );
         return reserve;
       }
       const reserve: ReserveRecord = {
@@ -145,7 +163,10 @@ export async function getMergeReserveInfo(
         reservePub: newReservePair.pub,
       };
       const insertResp = await tx.reserves.put(reserve);
-      checkDbInvariant(typeof insertResp.key === "number");
+      checkDbInvariant(
+        typeof insertResp.key === "number",
+        `reserve key is not a number`,
+      );
       reserve.rowId = insertResp.key;
       ex.currentMergeReserveRowId = reserve.rowId;
       await tx.exchanges.put(ex);
