@@ -31,6 +31,7 @@ import {
   TalerPreciseTimestamp,
   TalerProtocolTimestamp,
   TalerUriAction,
+  Transaction,
   TransactionAction,
   TransactionIdStr,
   TransactionMajorState,
@@ -47,6 +48,7 @@ import {
   getRandomBytes,
   j2s,
   makeErrorDetail,
+  stringifyPayPullUri,
   stringifyTalerUri,
   talerPaytoFromExchangeReserve,
 } from "@gnu-taler/taler-util";
@@ -54,17 +56,23 @@ import { readSuccessResponseJsonOrThrow } from "@gnu-taler/taler-util/http";
 import {
   PendingTaskType,
   TaskIdStr,
+  TaskIdentifiers,
   TaskRunResult,
   TaskRunResultType,
   TombstoneTag,
   TransactionContext,
   constructTaskIdentifier,
+  requireExchangeTosAcceptedOrThrow,
+  runWithClientCancellation,
 } from "./common.js";
 import {
   KycPendingInfo,
   KycUserType,
+  OperationRetryRecord,
   PeerPullCreditRecord,
   PeerPullPaymentCreditStatus,
+  WalletDbAllStoresReadOnlyTransaction,
+  WithdrawalGroupRecord,
   WithdrawalGroupStatus,
   WithdrawalRecordType,
   timestampOptionalPreciseFromDb,
@@ -78,6 +86,7 @@ import {
 } from "./pay-peer-common.js";
 import {
   constructTransactionIdentifier,
+  isUnsuccessfulTransaction,
   notifyTransition,
 } from "./transactions.js";
 import { WalletExecutionContext } from "./wallet.js";
@@ -107,10 +116,125 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
     });
   }
 
+  /**
+   * Get the full transaction details for the transaction.
+   *
+   * Returns undefined if the transaction is in a state where we do not have a
+   * transaction item (e.g. if it was deleted).
+   */
+  async lookupFullTransaction(
+    tx: WalletDbAllStoresReadOnlyTransaction,
+  ): Promise<Transaction | undefined> {
+    const pullCredit = await tx.peerPullCredit.get(this.pursePub);
+    if (!pullCredit) {
+      return undefined;
+    }
+    const ct = await tx.contractTerms.get(pullCredit.contractTermsHash);
+    checkDbInvariant(!!ct, `no contract terms for p2p push ${this.pursePub}`);
+
+    const peerContractTerms = ct.contractTermsRaw;
+
+    let wsr: WithdrawalGroupRecord | undefined = undefined;
+    let wsrOrt: OperationRetryRecord | undefined = undefined;
+    if (pullCredit.withdrawalGroupId) {
+      wsr = await tx.withdrawalGroups.get(pullCredit.withdrawalGroupId);
+      if (wsr) {
+        const withdrawalOpId = TaskIdentifiers.forWithdrawal(wsr);
+        wsrOrt = await tx.operationRetries.get(withdrawalOpId);
+      }
+    }
+    const pullCreditOpId =
+      TaskIdentifiers.forPeerPullPaymentInitiation(pullCredit);
+    let pullCreditOrt = await tx.operationRetries.get(pullCreditOpId);
+
+    if (wsr) {
+      if (wsr.wgInfo.withdrawalType !== WithdrawalRecordType.PeerPullCredit) {
+        throw Error(`Unexpected withdrawalType: ${wsr.wgInfo.withdrawalType}`);
+      }
+      /**
+       * FIXME: this should be handled in the withdrawal process.
+       * PeerPull withdrawal fails until reserve have funds but it is not
+       * an error from the user perspective.
+       */
+      const silentWithdrawalErrorForInvoice =
+        wsrOrt?.lastError &&
+        wsrOrt.lastError.code ===
+          TalerErrorCode.WALLET_WITHDRAWAL_GROUP_INCOMPLETE &&
+        Object.values(wsrOrt.lastError.errorsPerCoin ?? {}).every((e) => {
+          return (
+            e.code === TalerErrorCode.WALLET_UNEXPECTED_REQUEST_ERROR &&
+            e.httpStatusCode === 409
+          );
+        });
+      const txState = computePeerPullCreditTransactionState(pullCredit);
+      checkDbInvariant(wsr.instructedAmount !== undefined, "wg uninitialized");
+      checkDbInvariant(wsr.denomsSel !== undefined, "wg uninitialized");
+      checkDbInvariant(wsr.exchangeBaseUrl !== undefined, "wg uninitialized");
+      return {
+        type: TransactionType.PeerPullCredit,
+        txState,
+        txActions: computePeerPullCreditTransactionActions(pullCredit),
+        amountEffective: isUnsuccessfulTransaction(txState)
+          ? Amounts.stringify(Amounts.zeroOfAmount(wsr.instructedAmount))
+          : Amounts.stringify(wsr.denomsSel.totalCoinValue),
+        amountRaw: Amounts.stringify(wsr.instructedAmount),
+        exchangeBaseUrl: wsr.exchangeBaseUrl,
+        timestamp: timestampPreciseFromDb(pullCredit.mergeTimestamp),
+        info: {
+          expiration: peerContractTerms.purse_expiration,
+          summary: peerContractTerms.summary,
+        },
+        talerUri: stringifyPayPullUri({
+          exchangeBaseUrl: wsr.exchangeBaseUrl,
+          contractPriv: wsr.wgInfo.contractPriv,
+        }),
+        transactionId: constructTransactionIdentifier({
+          tag: TransactionType.PeerPullCredit,
+          pursePub: pullCredit.pursePub,
+        }),
+        kycUrl: pullCredit.kycUrl,
+        ...(wsrOrt?.lastError
+          ? {
+              error: silentWithdrawalErrorForInvoice
+                ? undefined
+                : wsrOrt.lastError,
+            }
+          : {}),
+      };
+    }
+
+    const txState = computePeerPullCreditTransactionState(pullCredit);
+    return {
+      type: TransactionType.PeerPullCredit,
+      txState,
+      txActions: computePeerPullCreditTransactionActions(pullCredit),
+      amountEffective: isUnsuccessfulTransaction(txState)
+        ? Amounts.stringify(Amounts.zeroOfAmount(peerContractTerms.amount))
+        : Amounts.stringify(pullCredit.estimatedAmountEffective),
+      amountRaw: Amounts.stringify(peerContractTerms.amount),
+      exchangeBaseUrl: pullCredit.exchangeBaseUrl,
+      timestamp: timestampPreciseFromDb(pullCredit.mergeTimestamp),
+      info: {
+        expiration: peerContractTerms.purse_expiration,
+        summary: peerContractTerms.summary,
+      },
+      talerUri: stringifyPayPullUri({
+        exchangeBaseUrl: pullCredit.exchangeBaseUrl,
+        contractPriv: pullCredit.contractPriv,
+      }),
+      transactionId: constructTransactionIdentifier({
+        tag: TransactionType.PeerPullCredit,
+        pursePub: pullCredit.pursePub,
+      }),
+      kycUrl: pullCredit.kycUrl,
+      ...(pullCreditOrt?.lastError ? { error: pullCreditOrt.lastError } : {}),
+    };
+  }
+
   async deleteTransaction(): Promise<void> {
     const { wex: ws, pursePub } = this;
     await ws.db.runReadWriteTx(
-      ["withdrawalGroups", "peerPullCredit", "tombstones"],
+      { storeNames: ["withdrawalGroups", "peerPullCredit", "tombstones"] },
       async (tx) => {
         const pullIni = await tx.peerPullCredit.get(pursePub);
         if (!pullIni) {
@@ -140,7 +264,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
   async suspendTransaction(): Promise<void> {
     const { wex, pursePub, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPullCredit"],
+      { storeNames: ["peerPullCredit"] },
       async (tx) => {
         const pullCreditRec = await tx.peerPullCredit.get(pursePub);
         if (!pullCreditRec) {
@@ -200,7 +324,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
   async failTransaction(): Promise<void> {
     const { wex, pursePub, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPullCredit"],
+      { storeNames: ["peerPullCredit"] },
       async (tx) => {
         const pullCreditRec = await tx.peerPullCredit.get(pursePub);
         if (!pullCreditRec) {
@@ -251,7 +375,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
   async resumeTransaction(): Promise<void> {
     const { wex, pursePub, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPullCredit"],
+      { storeNames: ["peerPullCredit"] },
       async (tx) => {
         const pullCreditRec = await tx.peerPullCredit.get(pursePub);
         if (!pullCreditRec) {
@@ -310,7 +434,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
   async abortTransaction(): Promise<void> {
     const { wex, pursePub, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPullCredit"],
+      { storeNames: ["peerPullCredit"] },
       async (tx) => {
         const pullCreditRec = await tx.peerPullCredit.get(pursePub);
         if (!pullCreditRec) {
@@ -373,10 +497,16 @@ async function queryPurseForPeerPullCredit(
   );
   purseDepositUrl.searchParams.set("timeout_ms", "30000");
   logger.info(`querying purse status via ${purseDepositUrl.href}`);
-  const resp = await wex.http.fetch(purseDepositUrl.href, {
-    timeout: { d_ms: 60000 },
-    cancellationToken: wex.cancellationToken,
-  });
+  const resp = await wex.ws.runLongpollQueueing(
+    wex,
+    purseDepositUrl.hostname,
+    async () => {
+      return await wex.http.fetch(purseDepositUrl.href, {
+        timeout: { d_ms: 60000 },
+        cancellationToken: wex.cancellationToken,
+      });
+    },
+  );
   const transactionId = constructTransactionIdentifier({
     tag: TransactionType.PeerPullCredit,
     pursePub: pullIni.pursePub,
@@ -388,7 +518,7 @@ async function queryPurseForPeerPullCredit(
     case HttpStatusCode.Gone: {
       // Exchange says that purse doesn't exist anymore => expired!
       const transitionInfo = await wex.db.runReadWriteTx(
-        ["peerPullCredit"],
+        { storeNames: ["peerPullCredit"] },
         async (tx) => {
           const finPi = await tx.peerPullCredit.get(pullIni.pursePub);
           if (!finPi) {
@@ -426,9 +556,12 @@ async function queryPurseForPeerPullCredit(
     return TaskRunResult.backoff();
   }
 
-  const reserve = await wex.db.runReadOnlyTx(["reserves"], async (tx) => {
-    return await tx.reserves.get(pullIni.mergeReserveRowId);
-  });
+  const reserve = await wex.db.runReadOnlyTx(
+    { storeNames: ["reserves"] },
+    async (tx) => {
+      return await tx.reserves.get(pullIni.mergeReserveRowId);
+    },
+  );
 
   if (!reserve) {
     throw Error("reserve for peer pull credit not found in wallet DB");
@@ -449,7 +582,7 @@ async function queryPurseForPeerPullCredit(
     },
   });
   const transitionInfo = await wex.db.runReadWriteTx(
-    ["peerPullCredit"],
+    { storeNames: ["peerPullCredit"] },
     async (tx) => {
       const finPi = await tx.peerPullCredit.get(pullIni.pursePub);
       if (!finPi) {
@@ -484,12 +617,18 @@ async function longpollKycStatus(
     `kyc-check/${kycInfo.requirementRow}/${kycInfo.paytoHash}/${userType}`,
     exchangeUrl,
   );
-  url.searchParams.set("timeout_ms", "10000");
-  logger.info(`kyc url ${url.href}`);
-  const kycStatusRes = await wex.http.fetch(url.href, {
-    method: "GET",
-    cancellationToken: wex.cancellationToken,
-  });
+  const kycStatusRes = await wex.ws.runLongpollQueueing(
+    wex,
+    url.hostname,
+    async (timeoutMs) => {
+      url.searchParams.set("timeout_ms", `${timeoutMs}`);
+      logger.info(`kyc url ${url.href}`);
+      return await wex.http.fetch(url.href, {
+        method: "GET",
+        cancellationToken: wex.cancellationToken,
+      });
+    },
+  );
   if (
     kycStatusRes.status === HttpStatusCode.Ok ||
     // FIXME: NoContent is not expected https://docs.taler.net/core/api-exchange.html#post--purses-$PURSE_PUB-merge
@@ -497,7 +636,7 @@ async function longpollKycStatus(
     kycStatusRes.status === HttpStatusCode.NoContent
   ) {
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPullCredit"],
+      { storeNames: ["peerPullCredit"] },
       async (tx) => {
         const peerIni = await tx.peerPullCredit.get(pursePub);
         if (!peerIni) {
@@ -548,13 +687,15 @@ async function processPeerPullCreditAbortingDeletePurse(
   logger.info(`deleted purse with response status ${resp.status}`);
 
   const transitionInfo = await wex.db.runReadWriteTx(
-    [
-      "peerPullCredit",
-      "refreshGroups",
-      "denominations",
-      "coinAvailability",
-      "coins",
-    ],
+    {
+      storeNames: [
+        "peerPullCredit",
+        "refreshGroups",
+        "denominations",
+        "coinAvailability",
+        "coins",
+      ],
+    },
     async (tx) => {
       const ppiRec = await tx.peerPullCredit.get(pursePub);
       if (!ppiRec) {
@@ -593,7 +734,7 @@ async function handlePeerPullCreditWithdrawing(
   const wgId = pullIni.withdrawalGroupId;
   let finished: boolean = false;
   const transitionInfo = await wex.db.runReadWriteTx(
-    ["peerPullCredit", "withdrawalGroups"],
+    { storeNames: ["peerPullCredit", "withdrawalGroups"] },
     async (tx) => {
       const ppi = await tx.peerPullCredit.get(pullIni.pursePub);
       if (!ppi) {
@@ -640,16 +781,19 @@ async function handlePeerPullCreditCreatePurse(
 ): Promise<TaskRunResult> {
   const purseFee = Amounts.stringify(Amounts.zeroOfAmount(pullIni.amount));
   const pursePub = pullIni.pursePub;
-  const mergeReserve = await wex.db.runReadOnlyTx(["reserves"], async (tx) => {
-    return tx.reserves.get(pullIni.mergeReserveRowId);
-  });
+  const mergeReserve = await wex.db.runReadOnlyTx(
+    { storeNames: ["reserves"] },
+    async (tx) => {
+      return tx.reserves.get(pullIni.mergeReserveRowId);
+    },
+  );
 
   if (!mergeReserve) {
     throw Error("merge reserve for peer pull payment not found in database");
   }
 
   const contractTermsRecord = await wex.db.runReadOnlyTx(
-    ["contractTerms"],
+    { storeNames: ["contractTerms"] },
     async (tx) => {
       return tx.contractTerms.get(pullIni.contractTermsHash);
     },
@@ -737,7 +881,7 @@ async function handlePeerPullCreditCreatePurse(
   });
 
   const transitionInfo = await wex.db.runReadWriteTx(
-    ["peerPullCredit"],
+    { storeNames: ["peerPullCredit"] },
     async (tx) => {
       const pi2 = await tx.peerPullCredit.get(pursePub);
       if (!pi2) {
@@ -758,9 +902,16 @@ export async function processPeerPullCredit(
   wex: WalletExecutionContext,
   pursePub: string,
 ): Promise<TaskRunResult> {
-  const pullIni = await wex.db.runReadOnlyTx(["peerPullCredit"], async (tx) => {
-    return tx.peerPullCredit.get(pursePub);
-  });
+  if (!wex.ws.networkAvailable) {
+    return TaskRunResult.networkRequired();
+  }
+
+  const pullIni = await wex.db.runReadOnlyTx(
+    { storeNames: ["peerPullCredit"] },
+    async (tx) => {
+      return tx.peerPullCredit.get(pursePub);
+    },
+  );
   if (!pullIni) {
     throw Error("peer pull payment initiation not found in database");
   }
@@ -847,7 +998,7 @@ async function processPeerPullCreditKycRequired(
     const kycStatus = await kycStatusRes.json();
     logger.info(`kyc status: ${j2s(kycStatus)}`);
     const { transitionInfo, result } = await wex.db.runReadWriteTx(
-      ["peerPullCredit"],
+      { storeNames: ["peerPullCredit"] },
       async (tx) => {
         const peerInc = await tx.peerPullCredit.get(pursePub);
         if (!peerInc) {
@@ -892,7 +1043,22 @@ async function processPeerPullCreditKycRequired(
 /**
  * Check fees and available exchanges for a peer push payment initiation.
  */
-export async function checkPeerPullPaymentInitiation(
+export async function checkPeerPullCredit(
+  wex: WalletExecutionContext,
+  req: CheckPeerPullCreditRequest,
+): Promise<CheckPeerPullCreditResponse> {
+  return runWithClientCancellation(
+    wex,
+    "checkPeerPullCredit",
+    req.clientCancellationId,
+    async () => internalCheckPeerPullCredit(wex, req),
+  );
+}
+
+/**
+ * Check fees and available exchanges for a peer push payment initiation.
+ */
+export async function internalCheckPeerPullCredit(
   wex: WalletExecutionContext,
   req: CheckPeerPullCreditRequest,
 ): Promise<CheckPeerPullCreditResponse> {
@@ -922,6 +1088,11 @@ export async function checkPeerPullPaymentInitiation(
     Amounts.parseOrThrow(req.amount),
     undefined,
   );
+  if (wi.selectedDenoms.selectedDenoms.length === 0) {
+    throw Error(
+      `unable to check pull payment from ${exchangeUrl}, can't select denominations for instructed amount (${req.amount}`,
+    );
+  }
 
   logger.trace(`got withdrawal info`);
 
@@ -947,42 +1118,45 @@ async function getPreferredExchangeForCurrency(
 ): Promise<string | undefined> {
   // Find an exchange with the matching currency.
   // Prefer exchanges with the most recent withdrawal.
-  const url = await wex.db.runReadOnlyTx(["exchanges"], async (tx) => {
-    const exchanges = await tx.exchanges.iter().toArray();
-    let candidate = undefined;
-    for (const e of exchanges) {
-      if (e.detailsPointer?.currency !== currency) {
-        continue;
-      }
-      if (!candidate) {
-        candidate = e;
-        continue;
-      }
-      if (candidate.lastWithdrawal && !e.lastWithdrawal) {
-        continue;
-      }
-      const exchangeLastWithdrawal = timestampOptionalPreciseFromDb(
-        e.lastWithdrawal,
-      );
-      const candidateLastWithdrawal = timestampOptionalPreciseFromDb(
-        candidate.lastWithdrawal,
-      );
-      if (exchangeLastWithdrawal && candidateLastWithdrawal) {
-        if (
-          AbsoluteTime.cmp(
-            AbsoluteTime.fromPreciseTimestamp(exchangeLastWithdrawal),
-            AbsoluteTime.fromPreciseTimestamp(candidateLastWithdrawal),
-          ) > 0
-        ) {
+  const url = await wex.db.runReadOnlyTx(
+    { storeNames: ["exchanges"] },
+    async (tx) => {
+      const exchanges = await tx.exchanges.iter().toArray();
+      let candidate = undefined;
+      for (const e of exchanges) {
+        if (e.detailsPointer?.currency !== currency) {
+          continue;
+        }
+        if (!candidate) {
           candidate = e;
+          continue;
+        }
+        if (candidate.lastWithdrawal && !e.lastWithdrawal) {
+          continue;
+        }
+        const exchangeLastWithdrawal = timestampOptionalPreciseFromDb(
+          e.lastWithdrawal,
+        );
+        const candidateLastWithdrawal = timestampOptionalPreciseFromDb(
+          candidate.lastWithdrawal,
+        );
+        if (exchangeLastWithdrawal && candidateLastWithdrawal) {
+          if (
+            AbsoluteTime.cmp(
+              AbsoluteTime.fromPreciseTimestamp(exchangeLastWithdrawal),
+              AbsoluteTime.fromPreciseTimestamp(candidateLastWithdrawal),
+            ) > 0
+          ) {
+            candidate = e;
+          }
         }
       }
-    }
-    if (candidate) {
-      return candidate.baseUrl;
-    }
-    return undefined;
-  });
+      if (candidate) {
+        return candidate.baseUrl;
+      }
+      return undefined;
+    },
+  );
   return url;
 }
 
@@ -1007,7 +1181,8 @@ export async function initiatePeerPullPayment(
 
   const exchangeBaseUrl = maybeExchangeBaseUrl;
 
-  await fetchFreshExchange(wex, exchangeBaseUrl);
+  const exchange = await fetchFreshExchange(wex, exchangeBaseUrl);
+  requireExchangeTosAcceptedOrThrow(exchange);
 
   const mergeReserveInfo = await getMergeReserveInfo(wex, {
     exchangeBaseUrl: exchangeBaseUrl,
@@ -1025,7 +1200,10 @@ export async function initiatePeerPullPayment(
   const withdrawalGroupId = encodeCrock(getRandomBytes(32));
 
   const mergeReserveRowId = mergeReserveInfo.rowId;
-  checkDbInvariant(!!mergeReserveRowId);
+  checkDbInvariant(
+    !!mergeReserveRowId,
+    `merge reserve for ${exchangeBaseUrl} without rowid`,
+  );
 
   const contractEncNonce = encodeCrock(getRandomBytes(24));
 
@@ -1035,11 +1213,16 @@ export async function initiatePeerPullPayment(
     Amounts.parseOrThrow(req.partialContractTerms.amount),
     undefined,
   );
+  if (wi.selectedDenoms.selectedDenoms.length === 0) {
+    throw Error(
+      `unable to initiate pull payment from ${exchangeBaseUrl}, can't select denominations for instructed amount (${req.partialContractTerms.amount}`,
+    );
+  }
 
   const mergeTimestamp = TalerPreciseTimestamp.now();
 
   const transitionInfo = await wex.db.runReadWriteTx(
-    ["peerPullCredit", "contractTerms"],
+    { storeNames: ["peerPullCredit", "contractTerms"] },
     async (tx) => {
       const ppi: PeerPullCreditRecord = {
         amount: req.partialContractTerms.amount,
@@ -1170,15 +1353,31 @@ export function computePeerPullCreditTransactionActions(
 ): TransactionAction[] {
   switch (pullCreditRecord.status) {
     case PeerPullPaymentCreditStatus.PendingCreatePurse:
-      return [TransactionAction.Abort, TransactionAction.Suspend];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Abort,
+        TransactionAction.Suspend,
+      ];
     case PeerPullPaymentCreditStatus.PendingMergeKycRequired:
-      return [TransactionAction.Abort, TransactionAction.Suspend];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Abort,
+        TransactionAction.Suspend,
+      ];
     case PeerPullPaymentCreditStatus.PendingReady:
-      return [TransactionAction.Abort, TransactionAction.Suspend];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Abort,
+        TransactionAction.Suspend,
+      ];
     case PeerPullPaymentCreditStatus.Done:
       return [TransactionAction.Delete];
     case PeerPullPaymentCreditStatus.PendingWithdrawing:
-      return [TransactionAction.Abort, TransactionAction.Suspend];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Abort,
+        TransactionAction.Suspend,
+      ];
     case PeerPullPaymentCreditStatus.SuspendedCreatePurse:
       return [TransactionAction.Resume, TransactionAction.Abort];
     case PeerPullPaymentCreditStatus.SuspendedReady:
@@ -1190,7 +1389,11 @@ export function computePeerPullCreditTransactionActions(
     case PeerPullPaymentCreditStatus.Aborted:
       return [TransactionAction.Delete];
     case PeerPullPaymentCreditStatus.AbortingDeletePurse:
-      return [TransactionAction.Suspend, TransactionAction.Fail];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Suspend,
+        TransactionAction.Fail,
+      ];
     case PeerPullPaymentCreditStatus.Failed:
       return [TransactionAction.Delete];
     case PeerPullPaymentCreditStatus.Expired:
