@@ -33,6 +33,7 @@ import {
   HttpStatusCode,
   Logger,
   NotificationType,
+  ObservabilityEventType,
   PeerContractTerms,
   PreparePeerPullDebitRequest,
   PreparePeerPullDebitResponse,
@@ -42,6 +43,7 @@ import {
   TalerErrorCode,
   TalerPreciseTimestamp,
   TalerProtocolViolationError,
+  Transaction,
   TransactionAction,
   TransactionIdStr,
   TransactionMajorState,
@@ -49,6 +51,7 @@ import {
   TransactionState,
   TransactionType,
   assertUnreachable,
+  checkDbInvariant,
   checkLogicInvariant,
   codecForAny,
   codecForExchangeGetContractResponse,
@@ -80,7 +83,9 @@ import {
   PeerPullDebitRecordStatus,
   PeerPullPaymentIncomingRecord,
   RefreshOperationStatus,
+  WalletDbAllStoresReadOnlyTransaction,
   WalletStoresV1,
+  timestampPreciseFromDb,
   timestampPreciseToDb,
 } from "./db.js";
 import {
@@ -92,6 +97,7 @@ import { DbReadWriteTransaction, StoreNames } from "./query.js";
 import { createRefreshGroup } from "./refresh.js";
 import {
   constructTransactionIdentifier,
+  isUnsuccessfulTransaction,
   notifyTransition,
   parseTransactionIdentifier,
 } from "./transactions.js";
@@ -121,17 +127,62 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
     this.peerPullDebitId = peerPullDebitId;
   }
 
+  /**
+   * Get the full transaction details for the transaction.
+   *
+   * Returns undefined if the transaction is in a state where we do not have a
+   * transaction item (e.g. if it was deleted).
+   */
+  async lookupFullTransaction(
+    tx: WalletDbAllStoresReadOnlyTransaction,
+  ): Promise<Transaction | undefined> {
+    const pi = await tx.peerPullDebit.get(this.peerPullDebitId);
+    if (!pi) {
+      return undefined;
+    }
+    const ort = await tx.operationRetries.get(this.taskId);
+    const txState = computePeerPullDebitTransactionState(pi);
+    const ctRec = await tx.contractTerms.get(pi.contractTermsHash);
+    checkDbInvariant(!!ctRec, `no contract terms for ${this.transactionId}`);
+    const contractTerms = ctRec.contractTermsRaw;
+    return {
+      type: TransactionType.PeerPullDebit,
+      txState,
+      txActions: computePeerPullDebitTransactionActions(pi),
+      amountEffective: isUnsuccessfulTransaction(txState)
+        ? Amounts.stringify(Amounts.zeroOfAmount(pi.amount))
+        : pi.coinSel?.totalCost
+          ? pi.coinSel?.totalCost
+          : Amounts.stringify(pi.amount),
+      amountRaw: Amounts.stringify(pi.amount),
+      exchangeBaseUrl: pi.exchangeBaseUrl,
+      info: {
+        expiration: contractTerms.purse_expiration,
+        summary: contractTerms.summary,
+      },
+      timestamp: timestampPreciseFromDb(pi.timestampCreated),
+      transactionId: constructTransactionIdentifier({
+        tag: TransactionType.PeerPullDebit,
+        peerPullDebitId: pi.peerPullDebitId,
+      }),
+      ...(ort?.lastError ? { error: ort.lastError } : {}),
+    };
+  }
+
   async deleteTransaction(): Promise<void> {
     const transactionId = this.transactionId;
     const ws = this.wex;
     const peerPullDebitId = this.peerPullDebitId;
-    await ws.db.runReadWriteTx(["peerPullDebit", "tombstones"], async (tx) => {
-      const debit = await tx.peerPullDebit.get(peerPullDebitId);
-      if (debit) {
-        await tx.peerPullDebit.delete(peerPullDebitId);
-        await tx.tombstones.put({ id: transactionId });
-      }
-    });
+    await ws.db.runReadWriteTx(
+      { storeNames: ["peerPullDebit", "tombstones"] },
+      async (tx) => {
+        const debit = await tx.peerPullDebit.get(peerPullDebitId);
+        if (debit) {
+          await tx.peerPullDebit.delete(peerPullDebitId);
+          await tx.tombstones.put({ id: transactionId });
+        }
+      },
+    );
   }
 
   async suspendTransaction(): Promise<void> {
@@ -140,7 +191,7 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
     const wex = this.wex;
     const peerPullDebitId = this.peerPullDebitId;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPullDebit"],
+      { storeNames: ["peerPullDebit"] },
       async (tx) => {
         const pullDebitRec = await tx.peerPullDebit.get(peerPullDebitId);
         if (!pullDebitRec) {
@@ -233,11 +284,12 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
       {
         extraStores: [
           "coinAvailability",
+          "coinAvailability",
+          "coinHistory",
+          "coins",
           "denominations",
           "refreshGroups",
           "refreshSessions",
-          "coins",
-          "coinAvailability",
         ],
       },
       async (pi, tx) => {
@@ -304,7 +356,7 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
     const wex = this.wex;
     const extraStores = opts.extraStores ?? [];
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPullDebit", ...extraStores],
+      { storeNames: ["peerPullDebit", ...extraStores] },
       async (tx) => {
         const pi = await tx.peerPullDebit.get(this.peerPullDebitId);
         if (!pi) {
@@ -397,7 +449,7 @@ async function handlePurseCreationConflict(
     coinSelRes.result.coins,
   );
 
-  await ws.db.runReadWriteTx(["peerPullDebit"], async (tx) => {
+  await ws.db.runReadWriteTx({ storeNames: ["peerPullDebit"] }, async (tx) => {
     const myPpi = await tx.peerPullDebit.get(peerPullInc.peerPullDebitId);
     if (!myPpi) {
       return;
@@ -425,6 +477,11 @@ async function processPeerPullDebitPendingDeposit(
   wex: WalletExecutionContext,
   peerPullInc: PeerPullPaymentIncomingRecord,
 ): Promise<TaskRunResult> {
+  const ctx = new PeerPullDebitTransactionContext(
+    wex,
+    peerPullInc.peerPullDebitId,
+  );
+
   const pursePub = peerPullInc.pursePub;
 
   const coinSel = peerPullInc.coinSel;
@@ -464,15 +521,18 @@ async function processPeerPullDebitPendingDeposit(
     // FIXME: Missing notification here!
 
     const transitionDone = await wex.db.runReadWriteTx(
-      [
-        "exchanges",
-        "coins",
-        "denominations",
-        "refreshGroups",
-        "refreshSessions",
-        "peerPullDebit",
-        "coinAvailability",
-      ],
+      {
+        storeNames: [
+          "coinAvailability",
+          "coinHistory",
+          "coins",
+          "denominations",
+          "exchanges",
+          "peerPullDebit",
+          "refreshGroups",
+          "refreshSessions",
+        ],
+      },
       async (tx) => {
         const pi = await tx.peerPullDebit.get(peerPullDebitId);
         if (!pi) {
@@ -485,11 +545,7 @@ async function processPeerPullDebitPendingDeposit(
           return false;
         }
         await spendCoins(wex, tx, {
-          // allocationId: `txn:peer-pull-debit:${req.peerPullDebitId}`,
-          allocationId: constructTransactionIdentifier({
-            tag: TransactionType.PeerPullDebit,
-            peerPullDebitId,
-          }),
+          transactionId: ctx.transactionId,
           coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
           contributions: coinSelRes.result.coins.map((x) =>
             Amounts.parseOrThrow(x.contribution),
@@ -512,70 +568,82 @@ async function processPeerPullDebitPendingDeposit(
     }
   }
 
-  const coins = await queryCoinInfosForSelection(wex, coinSel);
-
-  const depositSigsResp = await wex.cryptoApi.signPurseDeposits({
-    exchangeBaseUrl: peerPullInc.exchangeBaseUrl,
-    pursePub: peerPullInc.pursePub,
-    coins,
-  });
-
   const purseDepositUrl = new URL(
     `purses/${pursePub}/deposit`,
     peerPullInc.exchangeBaseUrl,
   );
 
-  const depositPayload: ExchangePurseDeposits = {
-    deposits: depositSigsResp.deposits,
-  };
+  // FIXME: We could skip batches that we've already submitted.
 
-  if (logger.shouldLogTrace()) {
-    logger.trace(`purse deposit payload: ${j2s(depositPayload)}`);
+  const coins = await queryCoinInfosForSelection(wex, coinSel);
+
+  const maxBatchSize = 100;
+
+  for (let i = 0; i < coins.length; i += maxBatchSize) {
+    const batchSize = Math.min(maxBatchSize, coins.length - i);
+
+    wex.oc.observe({
+      type: ObservabilityEventType.Message,
+      contents: `Depositing batch at ${i}/${coins.length} of size ${batchSize}`,
+    });
+
+    const batchCoins = coins.slice(i, i + batchSize);
+    const depositSigsResp = await wex.cryptoApi.signPurseDeposits({
+      exchangeBaseUrl: peerPullInc.exchangeBaseUrl,
+      pursePub: peerPullInc.pursePub,
+      coins: batchCoins,
+    });
+
+    const depositPayload: ExchangePurseDeposits = {
+      deposits: depositSigsResp.deposits,
+    };
+
+    if (logger.shouldLogTrace()) {
+      logger.trace(`purse deposit payload: ${j2s(depositPayload)}`);
+    }
+
+    const httpResp = await wex.http.fetch(purseDepositUrl.href, {
+      method: "POST",
+      body: depositPayload,
+      cancellationToken: wex.cancellationToken,
+    });
+
+    switch (httpResp.status) {
+      case HttpStatusCode.Ok: {
+        const resp = await readSuccessResponseJsonOrThrow(
+          httpResp,
+          codecForAny(),
+        );
+        logger.trace(`purse deposit response: ${j2s(resp)}`);
+        continue;
+      }
+      case HttpStatusCode.Gone: {
+        await ctx.abortTransaction();
+        return TaskRunResult.backoff();
+      }
+      case HttpStatusCode.Conflict: {
+        return handlePurseCreationConflict(ctx, peerPullInc, httpResp);
+      }
+      default: {
+        const errResp = await readTalerErrorResponse(httpResp);
+        return {
+          type: TaskRunResultType.Error,
+          errorDetail: errResp,
+        };
+      }
+    }
   }
 
-  const httpResp = await wex.http.fetch(purseDepositUrl.href, {
-    method: "POST",
-    body: depositPayload,
-    cancellationToken: wex.cancellationToken,
+  // All batches succeeded, we can transition!
+
+  await ctx.transition(async (r) => {
+    if (r.status !== PeerPullDebitRecordStatus.PendingDeposit) {
+      return TransitionResultType.Stay;
+    }
+    r.status = PeerPullDebitRecordStatus.Done;
+    return TransitionResultType.Transition;
   });
-
-  const ctx = new PeerPullDebitTransactionContext(
-    wex,
-    peerPullInc.peerPullDebitId,
-  );
-
-  switch (httpResp.status) {
-    case HttpStatusCode.Ok: {
-      const resp = await readSuccessResponseJsonOrThrow(
-        httpResp,
-        codecForAny(),
-      );
-      logger.trace(`purse deposit response: ${j2s(resp)}`);
-
-      await ctx.transition(async (r) => {
-        if (r.status !== PeerPullDebitRecordStatus.PendingDeposit) {
-          return TransitionResultType.Stay;
-        }
-        r.status = PeerPullDebitRecordStatus.Done;
-        return TransitionResultType.Transition;
-      });
-      return TaskRunResult.finished();
-    }
-    case HttpStatusCode.Gone: {
-      await ctx.abortTransaction();
-      return TaskRunResult.backoff();
-    }
-    case HttpStatusCode.Conflict: {
-      return handlePurseCreationConflict(ctx, peerPullInc, httpResp);
-    }
-    default: {
-      const errResp = await readTalerErrorResponse(httpResp);
-      return {
-        type: TaskRunResultType.Error,
-        errorDetail: errResp,
-      };
-    }
-  }
+  return TaskRunResult.finished();
 }
 
 async function processPeerPullDebitAbortingRefresh(
@@ -590,7 +658,7 @@ async function processPeerPullDebitAbortingRefresh(
     peerPullDebitId,
   });
   const transitionInfo = await wex.db.runReadWriteTx(
-    ["peerPullDebit", "refreshGroups"],
+    { storeNames: ["peerPullDebit", "refreshGroups"] },
     async (tx) => {
       const refreshGroup = await tx.refreshGroups.get(abortRefreshGroupId);
       let newOpState: PeerPullDebitRecordStatus | undefined;
@@ -631,8 +699,12 @@ export async function processPeerPullDebit(
   wex: WalletExecutionContext,
   peerPullDebitId: string,
 ): Promise<TaskRunResult> {
+  if (!wex.ws.networkAvailable) {
+    return TaskRunResult.networkRequired();
+  }
+
   const peerPullInc = await wex.db.runReadOnlyTx(
-    ["peerPullDebit"],
+    { storeNames: ["peerPullDebit"] },
     async (tx) => {
       return tx.peerPullDebit.get(peerPullDebitId);
     },
@@ -662,7 +734,7 @@ export async function confirmPeerPullDebit(
   peerPullDebitId = parsedTx.peerPullDebitId;
 
   const peerPullInc = await wex.db.runReadOnlyTx(
-    ["peerPullDebit"],
+    { storeNames: ["peerPullDebit"] },
     async (tx) => {
       return tx.peerPullDebit.get(peerPullDebitId);
     },
@@ -674,6 +746,9 @@ export async function confirmPeerPullDebit(
     );
   }
 
+  const ctx = new PeerPullDebitTransactionContext(wex, peerPullDebitId);
+  const transactionId = ctx.transactionId;
+
   const instructedAmount = Amounts.parseOrThrow(peerPullInc.amount);
 
   const coinSelRes = await selectPeerCoins(wex, {
@@ -708,15 +783,18 @@ export async function confirmPeerPullDebit(
   // FIXME: Missing notification here!
 
   await wex.db.runReadWriteTx(
-    [
-      "exchanges",
-      "coins",
-      "denominations",
-      "refreshGroups",
-      "refreshSessions",
-      "peerPullDebit",
-      "coinAvailability",
-    ],
+    {
+      storeNames: [
+        "coinAvailability",
+        "coinHistory",
+        "coins",
+        "denominations",
+        "exchanges",
+        "peerPullDebit",
+        "refreshGroups",
+        "refreshSessions",
+      ],
+    },
     async (tx) => {
       const pi = await tx.peerPullDebit.get(peerPullDebitId);
       if (!pi) {
@@ -727,11 +805,7 @@ export async function confirmPeerPullDebit(
       }
       if (coinSelRes.type == "success") {
         await spendCoins(wex, tx, {
-          // allocationId: `txn:peer-pull-debit:${req.peerPullDebitId}`,
-          allocationId: constructTransactionIdentifier({
-            tag: TransactionType.PeerPullDebit,
-            peerPullDebitId,
-          }),
+          transactionId,
           coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
           contributions: coinSelRes.result.coins.map((x) =>
             Amounts.parseOrThrow(x.contribution),
@@ -749,10 +823,6 @@ export async function confirmPeerPullDebit(
     },
   );
 
-  const ctx = new PeerPullDebitTransactionContext(wex, peerPullDebitId);
-
-  const transactionId = ctx.transactionId;
-
   wex.ws.notify({
     type: NotificationType.BalanceChange,
     hintTransactionId: transactionId,
@@ -780,7 +850,7 @@ export async function preparePeerPullDebit(
   }
 
   const existing = await wex.db.runReadOnlyTx(
-    ["peerPullDebit", "contractTerms"],
+    { storeNames: ["peerPullDebit", "contractTerms"] },
     async (tx) => {
       const peerPullDebitRecord =
         await tx.peerPullDebit.indexes.byExchangeAndContractPriv.get([
@@ -893,7 +963,7 @@ export async function preparePeerPullDebit(
   const totalAmount = await getTotalPeerPaymentCost(wex, coins);
 
   await wex.db.runReadWriteTx(
-    ["peerPullDebit", "contractTerms"],
+    { storeNames: ["peerPullDebit", "contractTerms"] },
     async (tx) => {
       await tx.contractTerms.put({
         h: contractTermsHash,
@@ -975,7 +1045,7 @@ export function computePeerPullDebitTransactionActions(
 ): TransactionAction[] {
   switch (pullDebitRecord.status) {
     case PeerPullDebitRecordStatus.DialogProposed:
-      return [];
+      return [TransactionAction.Retry, TransactionAction.Delete];
     case PeerPullDebitRecordStatus.PendingDeposit:
       return [TransactionAction.Abort, TransactionAction.Suspend];
     case PeerPullDebitRecordStatus.Done:
