@@ -15,6 +15,7 @@
  */
 
 import {
+  AbsoluteTime,
   AcceptPeerPushPaymentResponse,
   Amounts,
   ConfirmPeerPushCreditRequest,
@@ -28,7 +29,7 @@ import {
   PreparePeerPushCreditResponse,
   TalerErrorCode,
   TalerPreciseTimestamp,
-  TalerProtocolTimestamp,
+  Transaction,
   TransactionAction,
   TransactionIdStr,
   TransactionMajorState,
@@ -56,19 +57,25 @@ import { readSuccessResponseJsonOrThrow } from "@gnu-taler/taler-util/http";
 import {
   PendingTaskType,
   TaskIdStr,
+  TaskIdentifiers,
   TaskRunResult,
   TaskRunResultType,
   TombstoneTag,
   TransactionContext,
   constructTaskIdentifier,
+  requireExchangeTosAcceptedOrThrow,
 } from "./common.js";
 import {
   KycPendingInfo,
   KycUserType,
+  OperationRetryRecord,
   PeerPushCreditStatus,
   PeerPushPaymentIncomingRecord,
+  WalletDbAllStoresReadOnlyTransaction,
+  WithdrawalGroupRecord,
   WithdrawalGroupStatus,
   WithdrawalRecordType,
+  timestampPreciseFromDb,
   timestampPreciseToDb,
 } from "./db.js";
 import { fetchFreshExchange } from "./exchanges.js";
@@ -79,6 +86,7 @@ import {
 import {
   TransitionInfo,
   constructTransactionIdentifier,
+  isUnsuccessfulTransaction,
   notifyTransition,
   parseTransactionIdentifier,
 } from "./transactions.js";
@@ -111,10 +119,103 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
     });
   }
 
+  /**
+   * Get the full transaction details for the transaction.
+   *
+   * Returns undefined if the transaction is in a state where we do not have a
+   * transaction item (e.g. if it was deleted).
+   */
+  async lookupFullTransaction(
+    tx: WalletDbAllStoresReadOnlyTransaction,
+  ): Promise<Transaction | undefined> {
+    const pushInc = await tx.peerPushCredit.get(this.peerPushCreditId);
+    if (!pushInc) {
+      return undefined;
+    }
+
+    let wg: WithdrawalGroupRecord | undefined = undefined;
+    let wgRetryRecord: OperationRetryRecord | undefined = undefined;
+    if (pushInc.withdrawalGroupId) {
+      wg = await tx.withdrawalGroups.get(pushInc.withdrawalGroupId);
+      if (wg) {
+        const withdrawalOpId = TaskIdentifiers.forWithdrawal(wg);
+        wgRetryRecord = await tx.operationRetries.get(withdrawalOpId);
+      }
+    }
+    const pushIncOpId = TaskIdentifiers.forPeerPushCredit(pushInc);
+    const pushRetryRecord = await tx.operationRetries.get(pushIncOpId);
+
+    const ct = await tx.contractTerms.get(pushInc.contractTermsHash);
+
+    if (!ct) {
+      throw Error("contract terms for P2P payment not found");
+    }
+
+    const peerContractTerms = ct.contractTermsRaw;
+
+    if (wg) {
+      if (wg.wgInfo.withdrawalType !== WithdrawalRecordType.PeerPushCredit) {
+        throw Error("invalid withdrawal group type for push payment credit");
+      }
+      checkDbInvariant(wg.instructedAmount !== undefined, "wg uninitialized");
+      checkDbInvariant(wg.denomsSel !== undefined, "wg uninitialized");
+      checkDbInvariant(wg.exchangeBaseUrl !== undefined, "wg uninitialized");
+
+      const txState = computePeerPushCreditTransactionState(pushInc);
+      return {
+        type: TransactionType.PeerPushCredit,
+        txState,
+        txActions: computePeerPushCreditTransactionActions(pushInc),
+        amountEffective: isUnsuccessfulTransaction(txState)
+          ? Amounts.stringify(Amounts.zeroOfAmount(wg.instructedAmount))
+          : Amounts.stringify(wg.denomsSel.totalCoinValue),
+        amountRaw: Amounts.stringify(wg.instructedAmount),
+        exchangeBaseUrl: wg.exchangeBaseUrl,
+        info: {
+          expiration: peerContractTerms.purse_expiration,
+          summary: peerContractTerms.summary,
+        },
+        timestamp: timestampPreciseFromDb(wg.timestampStart),
+        transactionId: constructTransactionIdentifier({
+          tag: TransactionType.PeerPushCredit,
+          peerPushCreditId: pushInc.peerPushCreditId,
+        }),
+        kycUrl: pushInc.kycUrl,
+        ...(wgRetryRecord?.lastError ? { error: wgRetryRecord.lastError } : {}),
+      };
+    }
+
+    const txState = computePeerPushCreditTransactionState(pushInc);
+    return {
+      type: TransactionType.PeerPushCredit,
+      txState,
+      txActions: computePeerPushCreditTransactionActions(pushInc),
+      amountEffective: isUnsuccessfulTransaction(txState)
+        ? Amounts.stringify(Amounts.zeroOfAmount(peerContractTerms.amount))
+        : // FIXME: This is wrong, needs to consider fees!
+          Amounts.stringify(peerContractTerms.amount),
+      amountRaw: Amounts.stringify(peerContractTerms.amount),
+      exchangeBaseUrl: pushInc.exchangeBaseUrl,
+      info: {
+        expiration: peerContractTerms.purse_expiration,
+        summary: peerContractTerms.summary,
+      },
+      kycUrl: pushInc.kycUrl,
+      timestamp: timestampPreciseFromDb(pushInc.timestamp),
+      transactionId: constructTransactionIdentifier({
+        tag: TransactionType.PeerPushCredit,
+        peerPushCreditId: pushInc.peerPushCreditId,
+      }),
+      ...(pushRetryRecord?.lastError
+        ? { error: pushRetryRecord.lastError }
+        : {}),
+    };
+  }
+
   async deleteTransaction(): Promise<void> {
     const { wex, peerPushCreditId } = this;
     await wex.db.runReadWriteTx(
-      ["withdrawalGroups", "peerPushCredit", "tombstones"],
+      { storeNames: ["withdrawalGroups", "peerPushCredit", "tombstones"] },
       async (tx) => {
         const pushInc = await tx.peerPushCredit.get(peerPushCreditId);
         if (!pushInc) {
@@ -143,7 +244,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
   async suspendTransaction(): Promise<void> {
     const { wex, peerPushCreditId, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPushCredit"],
+      { storeNames: ["peerPushCredit"] },
       async (tx) => {
         const pushCreditRec = await tx.peerPushCredit.get(peerPushCreditId);
         if (!pushCreditRec) {
@@ -197,7 +298,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
   async abortTransaction(): Promise<void> {
     const { wex, peerPushCreditId, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPushCredit"],
+      { storeNames: ["peerPushCredit"] },
       async (tx) => {
         const pushCreditRec = await tx.peerPushCredit.get(peerPushCreditId);
         if (!pushCreditRec) {
@@ -254,7 +355,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
   async resumeTransaction(): Promise<void> {
     const { wex, peerPushCreditId, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPushCredit"],
+      { storeNames: ["peerPushCredit"] },
       async (tx) => {
         const pushCreditRec = await tx.peerPushCredit.get(peerPushCreditId);
         if (!pushCreditRec) {
@@ -307,7 +408,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
   async failTransaction(): Promise<void> {
     const { wex, peerPushCreditId, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPushCredit"],
+      { storeNames: ["peerPushCredit"] },
       async (tx) => {
         const pushCreditRec = await tx.peerPushCredit.get(peerPushCreditId);
         if (!pushCreditRec) {
@@ -365,7 +466,7 @@ export async function preparePeerPushCredit(
   }
 
   const existing = await wex.db.runReadOnlyTx(
-    ["contractTerms", "peerPushCredit"],
+    { storeNames: ["contractTerms", "peerPushCredit"] },
     async (tx) => {
       const existingPushInc =
         await tx.peerPushCredit.indexes.byExchangeAndContractPriv.get([
@@ -407,8 +508,6 @@ export async function preparePeerPushCredit(
 
   const exchangeBaseUrl = uri.exchangeBaseUrl;
 
-  await fetchFreshExchange(wex, exchangeBaseUrl);
-
   const contractPriv = uri.contractPriv;
   const contractPub = encodeCrock(eddsaGetPublic(decodeCrock(contractPriv)));
 
@@ -459,8 +558,14 @@ export async function preparePeerPushCredit(
     undefined,
   );
 
+  if (wi.selectedDenoms.selectedDenoms.length === 0) {
+    throw Error(
+      `unable to prepare push credit from ${exchangeBaseUrl}, can't select denominations for instructed amount (${purseStatus.balance}`,
+    );
+  }
+
   const transitionInfo = await wex.db.runReadWriteTx(
-    ["contractTerms", "peerPushCredit"],
+    { storeNames: ["contractTerms", "peerPushCredit"] },
     async (tx) => {
       const rec: PeerPushPaymentIncomingRecord = {
         peerPushCreditId,
@@ -532,12 +637,19 @@ async function longpollKycStatus(
     `kyc-check/${kycInfo.requirementRow}/${kycInfo.paytoHash}/${userType}`,
     exchangeUrl,
   );
-  url.searchParams.set("timeout_ms", "30000");
   logger.info(`kyc url ${url.href}`);
-  const kycStatusRes = await wex.http.fetch(url.href, {
-    method: "GET",
-    cancellationToken: wex.cancellationToken,
-  });
+  const kycStatusRes = await wex.ws.runLongpollQueueing(
+    wex,
+    url.hostname,
+    async (timeoutMs) => {
+      url.searchParams.set("timeout_ms", `${timeoutMs}`);
+      return await wex.http.fetch(url.href, {
+        method: "GET",
+        cancellationToken: wex.cancellationToken,
+      });
+    },
+  );
+
   if (
     kycStatusRes.status === HttpStatusCode.Ok ||
     //FIXME: NoContent is not expected https://docs.taler.net/core/api-exchange.html#post--purses-$PURSE_PUB-merge
@@ -545,7 +657,7 @@ async function longpollKycStatus(
     kycStatusRes.status === HttpStatusCode.NoContent
   ) {
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPushCredit"],
+      { storeNames: ["peerPushCredit"] },
       async (tx) => {
         const peerInc = await tx.peerPushCredit.get(peerPushCreditId);
         if (!peerInc) {
@@ -606,7 +718,7 @@ async function processPeerPushCreditKycRequired(
     const kycStatus = await kycStatusRes.json();
     logger.info(`kyc status: ${j2s(kycStatus)}`);
     const { transitionInfo, result } = await wex.db.runReadWriteTx(
-      ["peerPushCredit"],
+      { storeNames: ["peerPushCredit"] },
       async (tx) => {
         const peerInc = await tx.peerPushCredit.get(peerPushCreditId);
         if (!peerInc) {
@@ -665,7 +777,11 @@ async function handlePendingMerge(
     exchangeBaseUrl: peerInc.exchangeBaseUrl,
   });
 
-  const mergeTimestamp = TalerProtocolTimestamp.now();
+  const timestamp = timestampPreciseFromDb(peerInc.timestamp);
+
+  const mergeTimestamp = AbsoluteTime.toProtocolTimestamp(
+    AbsoluteTime.fromPreciseTimestamp(timestamp),
+  );
 
   const reservePayto = talerPaytoFromExchangeReserve(
     peerInc.exchangeBaseUrl,
@@ -731,14 +847,16 @@ async function handlePendingMerge(
   });
 
   const txRes = await wex.db.runReadWriteTx(
-    [
-      "contractTerms",
-      "peerPushCredit",
-      "withdrawalGroups",
-      "reserves",
-      "exchanges",
-      "exchangeDetails",
-    ],
+    {
+      storeNames: [
+        "contractTerms",
+        "peerPushCredit",
+        "withdrawalGroups",
+        "reserves",
+        "exchanges",
+        "exchangeDetails",
+      ],
+    },
     async (tx) => {
       const peerInc = await tx.peerPushCredit.get(peerPushCreditId);
       if (!peerInc) {
@@ -798,7 +916,7 @@ async function handlePendingWithdrawing(
   const wgId = peerInc.withdrawalGroupId;
   let finished: boolean = false;
   const transitionInfo = await wex.db.runReadWriteTx(
-    ["peerPushCredit", "withdrawalGroups"],
+    { storeNames: ["peerPushCredit", "withdrawalGroups"] },
     async (tx) => {
       const ppi = await tx.peerPushCredit.get(peerInc.peerPushCreditId);
       if (!ppi) {
@@ -843,10 +961,14 @@ export async function processPeerPushCredit(
   wex: WalletExecutionContext,
   peerPushCreditId: string,
 ): Promise<TaskRunResult> {
+  if (!wex.ws.networkAvailable) {
+    return TaskRunResult.networkRequired();
+  }
+
   let peerInc: PeerPushPaymentIncomingRecord | undefined;
   let contractTerms: PeerContractTerms | undefined;
   await wex.db.runReadWriteTx(
-    ["contractTerms", "peerPushCredit"],
+    { storeNames: ["contractTerms", "peerPushCredit"] },
     async (tx) => {
       peerInc = await tx.peerPushCredit.get(peerPushCreditId);
       if (!peerInc) {
@@ -870,7 +992,10 @@ export async function processPeerPushCredit(
     `processing peerPushCredit in state ${peerInc.status.toString(16)}`,
   );
 
-  checkDbInvariant(!!contractTerms);
+  checkDbInvariant(
+    !!contractTerms,
+    `not contract terms for peer push ${peerPushCreditId}`,
+  );
 
   switch (peerInc.status) {
     case PeerPushCreditStatus.PendingMergeKycRequired: {
@@ -901,7 +1026,7 @@ export async function confirmPeerPushCredit(
   wex: WalletExecutionContext,
   req: ConfirmPeerPushCreditRequest,
 ): Promise<AcceptPeerPushPaymentResponse> {
-  let peerInc: PeerPushPaymentIncomingRecord | undefined;
+  // PeerPushPaymentIncomingRecord | undefined;
   let peerPushCreditId: string;
   const parsedTx = parseTransactionIdentifier(req.transactionId);
   if (!parsedTx) {
@@ -914,17 +1039,18 @@ export async function confirmPeerPushCredit(
 
   logger.trace(`confirming peer-push-credit ${peerPushCreditId}`);
 
-  await wex.db.runReadWriteTx(
-    ["contractTerms", "peerPushCredit"],
+  const peerInc = await wex.db.runReadWriteTx(
+    { storeNames: ["contractTerms", "peerPushCredit"] },
     async (tx) => {
-      peerInc = await tx.peerPushCredit.get(peerPushCreditId);
-      if (!peerInc) {
+      const rec = await tx.peerPushCredit.get(peerPushCreditId);
+      if (!rec) {
         return;
       }
-      if (peerInc.status === PeerPushCreditStatus.DialogProposed) {
-        peerInc.status = PeerPushCreditStatus.PendingMerge;
+      if (rec.status === PeerPushCreditStatus.DialogProposed) {
+        rec.status = PeerPushCreditStatus.PendingMerge;
       }
-      await tx.peerPushCredit.put(peerInc);
+      await tx.peerPushCredit.put(rec);
+      return rec;
     },
   );
 
@@ -934,6 +1060,9 @@ export async function confirmPeerPushCredit(
     );
   }
 
+  const exchange = await fetchFreshExchange(wex, peerInc.exchangeBaseUrl);
+  requireExchangeTosAcceptedOrThrow(exchange);
+
   const ctx = new PeerPushCreditTransactionContext(wex, peerPushCreditId);
 
   wex.taskScheduler.startShepherdTask(ctx.taskId);
@@ -1009,15 +1138,27 @@ export function computePeerPushCreditTransactionActions(
 ): TransactionAction[] {
   switch (pushCreditRecord.status) {
     case PeerPushCreditStatus.DialogProposed:
-      return [TransactionAction.Delete];
+      return [TransactionAction.Retry, TransactionAction.Delete];
     case PeerPushCreditStatus.PendingMerge:
-      return [TransactionAction.Abort, TransactionAction.Suspend];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Abort,
+        TransactionAction.Suspend,
+      ];
     case PeerPushCreditStatus.Done:
       return [TransactionAction.Delete];
     case PeerPushCreditStatus.PendingMergeKycRequired:
-      return [TransactionAction.Abort, TransactionAction.Suspend];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Abort,
+        TransactionAction.Suspend,
+      ];
     case PeerPushCreditStatus.PendingWithdrawing:
-      return [TransactionAction.Suspend, TransactionAction.Fail];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Suspend,
+        TransactionAction.Fail,
+      ];
     case PeerPushCreditStatus.SuspendedMerge:
       return [TransactionAction.Resume, TransactionAction.Abort];
     case PeerPushCreditStatus.SuspendedMergeKycRequired:
