@@ -33,6 +33,7 @@ import {
   TalerPreciseTimestamp,
   TalerProtocolTimestamp,
   TalerProtocolViolationError,
+  Transaction,
   TransactionAction,
   TransactionIdStr,
   TransactionMajorState,
@@ -45,13 +46,18 @@ import {
   encodeCrock,
   getRandomBytes,
   j2s,
+  stringifyPayPushUri,
 } from "@gnu-taler/taler-util";
 import {
   HttpResponse,
   readSuccessResponseJsonOrThrow,
   readTalerErrorResponse,
 } from "@gnu-taler/taler-util/http";
-import { PreviousPayCoins, selectPeerCoins } from "./coinSelection.js";
+import {
+  PreviousPayCoins,
+  selectPeerCoins,
+  selectPeerCoinsInTx,
+} from "./coinSelection.js";
 import {
   PendingTaskType,
   TaskIdStr,
@@ -59,6 +65,7 @@ import {
   TaskRunResultType,
   TransactionContext,
   constructTaskIdentifier,
+  runWithClientCancellation,
   spendCoins,
 } from "./common.js";
 import { EncryptContractRequest } from "./crypto/cryptoTypes.js";
@@ -66,6 +73,8 @@ import {
   PeerPushDebitRecord,
   PeerPushDebitStatus,
   RefreshOperationStatus,
+  WalletDbAllStoresReadOnlyTransaction,
+  timestampPreciseFromDb,
   timestampPreciseToDb,
   timestampProtocolFromDb,
   timestampProtocolToDb,
@@ -73,11 +82,13 @@ import {
 import {
   codecForExchangePurseStatus,
   getTotalPeerPaymentCost,
+  getTotalPeerPaymentCostInTx,
   queryCoinInfosForSelection,
 } from "./pay-peer-common.js";
 import { createRefreshGroup, waitRefreshFinal } from "./refresh.js";
 import {
   constructTransactionIdentifier,
+  isUnsuccessfulTransaction,
   notifyTransition,
 } from "./transactions.js";
 import { WalletExecutionContext } from "./wallet.js";
@@ -102,6 +113,59 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
     });
   }
 
+  /**
+   * Get the full transaction details for the transaction.
+   *
+   * Returns undefined if the transaction is in a state where we do not have a
+   * transaction item (e.g. if it was deleted).
+   */
+  async lookupFullTransaction(
+    tx: WalletDbAllStoresReadOnlyTransaction,
+  ): Promise<Transaction | undefined> {
+    const pushDebitRec = await tx.peerPushDebit.get(this.pursePub);
+    if (!pushDebitRec) {
+      return undefined;
+    }
+    const retryRec = await tx.operationRetries.get(this.taskId);
+
+    const ctRec = await tx.contractTerms.get(pushDebitRec.contractTermsHash);
+    checkDbInvariant(!!ctRec, `no contract terms for p2p push ${this.pursePub}`);
+
+    const contractTerms = ctRec.contractTermsRaw;
+
+    let talerUri: string | undefined = undefined;
+    switch (pushDebitRec.status) {
+      case PeerPushDebitStatus.PendingReady:
+      case PeerPushDebitStatus.SuspendedReady:
+        talerUri = stringifyPayPushUri({
+          exchangeBaseUrl: pushDebitRec.exchangeBaseUrl,
+          contractPriv: pushDebitRec.contractPriv,
+        });
+    }
+    const txState = computePeerPushDebitTransactionState(pushDebitRec);
+    return {
+      type: TransactionType.PeerPushDebit,
+      txState,
+      txActions: computePeerPushDebitTransactionActions(pushDebitRec),
+      amountEffective: isUnsuccessfulTransaction(txState)
+        ? Amounts.stringify(Amounts.zeroOfAmount(pushDebitRec.totalCost))
+        : pushDebitRec.totalCost,
+      amountRaw: pushDebitRec.amount,
+      exchangeBaseUrl: pushDebitRec.exchangeBaseUrl,
+      info: {
+        expiration: contractTerms.purse_expiration,
+        summary: contractTerms.summary,
+      },
+      timestamp: timestampPreciseFromDb(pushDebitRec.timestampCreated),
+      talerUri,
+      transactionId: constructTransactionIdentifier({
+        tag: TransactionType.PeerPushDebit,
+        pursePub: pushDebitRec.pursePub,
+      }),
+      ...(retryRec?.lastError ? { error: retryRec.lastError } : {}),
+    };
+  }
+
   async deleteTransaction(): Promise<void> {
     const { wex, pursePub, transactionId } = this;
     await wex.db.runReadWriteTx(
@@ -344,6 +408,18 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
 export async function checkPeerPushDebit(
   wex: WalletExecutionContext,
   req: CheckPeerPushDebitRequest,
+): Promise<CheckPeerPushDebitResponse> {
+  return runWithClientCancellation(
+    wex,
+    "checkPeerPushDebit",
+    req.clientCancellationId,
+    () => internalCheckPeerPushDebit(wex, req),
+  );
+}
+
+async function internalCheckPeerPushDebit(
+  wex: WalletExecutionContext,
+  req: CheckPeerPushDebitRequest,
 ): Promise<CheckPeerPushDebitResponse> {
   const instructedAmount = Amounts.parseOrThrow(req.amount);
   logger.trace(
@@ -406,7 +482,10 @@ async function handlePurseCreationConflict(
   const instructedAmount = Amounts.parseOrThrow(peerPushInitiation.amount);
   const sel = peerPushInitiation.coinSel;
 
-  checkDbInvariant(!!sel, `no coin selected for peer push initiation ${peerPushInitiation.pursePub}`);
+  checkDbInvariant(
+    !!sel,
+    `no coin selected for peer push initiation ${peerPushInitiation.pursePub}`,
+  );
 
   const repair: PreviousPayCoins = [];
 
@@ -508,14 +587,15 @@ async function processPeerPushDebitCreateReserve(
     const transitionDone = await wex.db.runReadWriteTx(
       {
         storeNames: [
-          "exchanges",
-          "contractTerms",
-          "coins",
           "coinAvailability",
+          "coinHistory",
+          "coins",
+          "contractTerms",
           "denominations",
+          "exchanges",
+          "peerPushDebit",
           "refreshGroups",
           "refreshSessions",
-          "peerPushDebit",
         ],
       },
       async (tx) => {
@@ -535,10 +615,7 @@ async function processPeerPushDebitCreateReserve(
         // we might want to mark the coins as used and spend them
         // after we've been able to create the purse.
         await spendCoins(wex, tx, {
-          allocationId: constructTransactionIdentifier({
-            tag: TransactionType.PeerPushDebit,
-            pursePub,
-          }),
+          transactionId: ctx.transactionId,
           coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
           contributions: coinSelRes.result.coins.map((x) =>
             Amounts.parseOrThrow(x.contribution),
@@ -727,12 +804,13 @@ async function processPeerPushDebitAbortingDeletePurse(
   const transitionInfo = await wex.db.runReadWriteTx(
     {
       storeNames: [
+        "coinAvailability",
+        "coinHistory",
+        "coins",
+        "denominations",
         "peerPushDebit",
         "refreshGroups",
         "refreshSessions",
-        "denominations",
-        "coinAvailability",
-        "coins",
       ],
     },
     async (tx) => {
@@ -937,12 +1015,18 @@ async function processPeerPushDebitReady(
     `purses/${pursePub}/merge`,
     peerPushInitiation.exchangeBaseUrl,
   );
-  mergeUrl.searchParams.set("timeout_ms", "30000");
-  logger.info(`long-polling on purse status at ${mergeUrl.href}`);
-  const resp = await wex.http.fetch(mergeUrl.href, {
-    // timeout: getReserveRequestTimeout(withdrawalGroup),
-    cancellationToken: wex.cancellationToken,
-  });
+  const resp = await wex.ws.runLongpollQueueing(
+    wex,
+    mergeUrl.hostname,
+    async (timeoutMs) => {
+      mergeUrl.searchParams.set("timeout_ms", `${timeoutMs}`);
+      logger.info(`long-polling on purse status at ${mergeUrl.href}`);
+      return await wex.http.fetch(mergeUrl.href, {
+        // timeout: getReserveRequestTimeout(withdrawalGroup),
+        cancellationToken: wex.cancellationToken,
+      });
+    },
+  );
   if (resp.status === HttpStatusCode.Ok) {
     const purseStatus = await readSuccessResponseJsonOrThrow(
       resp,
@@ -968,12 +1052,13 @@ async function processPeerPushDebitReady(
     const transitionInfo = await wex.db.runReadWriteTx(
       {
         storeNames: [
+          "coinAvailability",
+          "coinHistory",
+          "coins",
+          "denominations",
           "peerPushDebit",
           "refreshGroups",
           "refreshSessions",
-          "denominations",
-          "coinAvailability",
-          "coins",
         ],
       },
       async (tx) => {
@@ -1028,6 +1113,10 @@ export async function processPeerPushDebit(
   wex: WalletExecutionContext,
   pursePub: string,
 ): Promise<TaskRunResult> {
+  if (!wex.ws.networkAvailable) {
+    return TaskRunResult.networkRequired();
+  }
+
   const peerPushInitiation = await wex.db.runReadOnlyTx(
     { storeNames: ["peerPushDebit"] },
     async (tx) => {
@@ -1086,39 +1175,6 @@ export async function initiatePeerPushDebit(
 
   const contractKeyPair = await wex.cryptoApi.createEddsaKeypair({});
 
-  const coinSelRes = await selectPeerCoins(wex, {
-    instructedAmount,
-  });
-
-  let coins: SelectedProspectiveCoin[] | undefined = undefined;
-
-  switch (coinSelRes.type) {
-    case "failure":
-      throw TalerError.fromDetail(
-        TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
-        {
-          insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
-        },
-      );
-    case "prospective":
-      coins = coinSelRes.result.prospectiveCoins;
-      break;
-    case "success":
-      coins = coinSelRes.result.coins;
-      break;
-    default:
-      assertUnreachable(coinSelRes);
-  }
-
-  const sel = coinSelRes.result;
-
-  logger.info(`selected p2p coins (push):`);
-  logger.trace(`${j2s(coinSelRes)}`);
-
-  const totalAmount = await getTotalPeerPaymentCost(wex, coins);
-
-  logger.info(`computed total peer payment cost`);
-
   const pursePub = pursePair.pub;
 
   const ctx = new PeerPushDebitTransactionContext(wex, pursePub);
@@ -1127,20 +1183,49 @@ export async function initiatePeerPushDebit(
 
   const contractEncNonce = encodeCrock(getRandomBytes(24));
 
-  const transitionInfo = await wex.db.runReadWriteTx(
+  const res = await wex.db.runReadWriteTx(
     {
       storeNames: [
-        "exchanges",
-        "contractTerms",
-        "coins",
         "coinAvailability",
+        "coinHistory",
+        "coins",
+        "contractTerms",
         "denominations",
+        "exchangeDetails",
+        "exchanges",
+        "peerPushDebit",
         "refreshGroups",
         "refreshSessions",
-        "peerPushDebit",
       ],
     },
     async (tx) => {
+      const coinSelRes = await selectPeerCoinsInTx(wex, tx, {
+        instructedAmount,
+      });
+
+      let coins: SelectedProspectiveCoin[] | undefined = undefined;
+
+      switch (coinSelRes.type) {
+        case "failure":
+          throw TalerError.fromDetail(
+            TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
+            {
+              insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
+            },
+          );
+        case "prospective":
+          coins = coinSelRes.result.prospectiveCoins;
+          break;
+        case "success":
+          coins = coinSelRes.result.coins;
+          break;
+        default:
+          assertUnreachable(coinSelRes);
+      }
+
+      const sel = coinSelRes.result;
+
+      const totalAmount = await getTotalPeerPaymentCostInTx(wex, tx, coins);
       const ppi: PeerPushDebitRecord = {
         amount: Amounts.stringify(instructedAmount),
         contractPriv: contractKeyPair.priv,
@@ -1167,10 +1252,7 @@ export async function initiatePeerPushDebit(
         // we might want to mark the coins as used and spend them
         // after we've been able to create the purse.
         await spendCoins(wex, tx, {
-          allocationId: constructTransactionIdentifier({
-            tag: TransactionType.PeerPushDebit,
-            pursePub: pursePair.pub,
-          }),
+          transactionId: ctx.transactionId,
           coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
           contributions: coinSelRes.result.coins.map((x) =>
             Amounts.parseOrThrow(x.contribution),
@@ -1188,12 +1270,15 @@ export async function initiatePeerPushDebit(
 
       const newTxState = computePeerPushDebitTransactionState(ppi);
       return {
-        oldTxState: { major: TransactionMajorState.None },
-        newTxState,
+        transitionInfo: {
+          oldTxState: { major: TransactionMajorState.None },
+          newTxState,
+        },
+        exchangeBaseUrl: coinSelRes.result.exchangeBaseUrl,
       };
     },
   );
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, transactionId, res.transitionInfo);
   wex.ws.notify({
     type: NotificationType.BalanceChange,
     hintTransactionId: transactionId,
@@ -1205,7 +1290,7 @@ export async function initiatePeerPushDebit(
     contractPriv: contractKeyPair.priv,
     mergePriv: mergePair.priv,
     pursePub: pursePair.pub,
-    exchangeBaseUrl: coinSelRes.result.exchangeBaseUrl,
+    exchangeBaseUrl: res.exchangeBaseUrl,
     transactionId: constructTransactionIdentifier({
       tag: TransactionType.PeerPushDebit,
       pursePub: pursePair.pub,
@@ -1218,17 +1303,37 @@ export function computePeerPushDebitTransactionActions(
 ): TransactionAction[] {
   switch (ppiRecord.status) {
     case PeerPushDebitStatus.PendingCreatePurse:
-      return [TransactionAction.Abort, TransactionAction.Suspend];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Abort,
+        TransactionAction.Suspend,
+      ];
     case PeerPushDebitStatus.PendingReady:
-      return [TransactionAction.Abort, TransactionAction.Suspend];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Abort,
+        TransactionAction.Suspend,
+      ];
     case PeerPushDebitStatus.Aborted:
       return [TransactionAction.Delete];
     case PeerPushDebitStatus.AbortingDeletePurse:
-      return [TransactionAction.Suspend, TransactionAction.Fail];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Suspend,
+        TransactionAction.Fail,
+      ];
     case PeerPushDebitStatus.AbortingRefreshDeleted:
-      return [TransactionAction.Suspend, TransactionAction.Fail];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Suspend,
+        TransactionAction.Fail,
+      ];
     case PeerPushDebitStatus.AbortingRefreshExpired:
-      return [TransactionAction.Suspend, TransactionAction.Fail];
+      return [
+        TransactionAction.Retry,
+        TransactionAction.Suspend,
+        TransactionAction.Fail,
+      ];
     case PeerPushDebitStatus.SuspendedAbortingRefreshExpired:
       return [TransactionAction.Resume, TransactionAction.Fail];
     case PeerPushDebitStatus.SuspendedAbortingDeletePurse:
