@@ -30,6 +30,7 @@ import {
   Logger,
   RefreshReason,
   TalerPreciseTimestamp,
+  Transaction,
   TransactionIdStr,
   TransactionType,
   URL,
@@ -54,6 +55,7 @@ import {
   RecoupGroupRecord,
   RecoupOperationStatus,
   RefreshCoinSource,
+  WalletDbAllStoresReadOnlyTransaction,
   WalletDbReadWriteTransaction,
   WithdrawCoinSource,
   WithdrawalGroupStatus,
@@ -99,7 +101,7 @@ async function recoupRewardCoin(
   // Thus we just put the coin to sleep.
   // FIXME: somehow report this to the user
   await wex.db.runReadWriteTx(
-    ["recoupGroups", "denominations", "refreshGroups", "coins"],
+    { storeNames: ["recoupGroups", "denominations", "refreshGroups", "coins"] },
     async (tx) => {
       const recoupGroup = await tx.recoupGroups.get(recoupGroupId);
       if (!recoupGroup) {
@@ -121,7 +123,7 @@ async function recoupRefreshCoin(
   cs: RefreshCoinSource,
 ): Promise<void> {
   const d = await wex.db.runReadOnlyTx(
-    ["coins", "denominations"],
+    { storeNames: ["coins", "denominations"] },
     async (tx) => {
       const denomInfo = await getDenomInfo(
         wex,
@@ -168,7 +170,7 @@ async function recoupRefreshCoin(
   }
 
   await wex.db.runReadWriteTx(
-    ["coins", "denominations", "recoupGroups", "refreshGroups"],
+    { storeNames: ["coins", "denominations", "recoupGroups", "refreshGroups"] },
     async (tx) => {
       const recoupGroup = await tx.recoupGroups.get(recoupGroupId);
       if (!recoupGroup) {
@@ -199,24 +201,20 @@ async function recoupRefreshCoin(
         revokedCoin.exchangeBaseUrl,
         revokedCoin.denomPubHash,
       );
-      checkDbInvariant(!!oldCoinDenom);
-      checkDbInvariant(!!revokedCoinDenom);
+      checkDbInvariant(
+        !!oldCoinDenom,
+        `no denom for coin, hash ${oldCoin.denomPubHash}`,
+      );
+      checkDbInvariant(
+        !!revokedCoinDenom,
+        `no revoked denom for coin, hash ${revokedCoin.denomPubHash}`,
+      );
       revokedCoin.status = CoinStatus.Dormant;
-      if (!revokedCoin.spendAllocation) {
-        // We don't know what happened to this coin
-        logger.error(
-          `can't refresh-recoup coin ${revokedCoin.coinPub}, no spendAllocation known`,
-        );
-      } else {
-        let residualAmount = Amounts.sub(
-          revokedCoinDenom.value,
-          revokedCoin.spendAllocation.amount,
-        ).amount;
-        recoupGroup.scheduleRefreshCoins.push({
-          coinPub: oldCoin.coinPub,
-          amount: Amounts.stringify(residualAmount),
-        });
-      }
+      // FIXME: Schedule recoup for the sum of refreshes, based on the coin event history.
+      // recoupGroup.scheduleRefreshCoins.push({
+      //   coinPub: oldCoin.coinPub,
+      //   amount: Amounts.stringify(refreshAmount),
+      // });
       await tx.coins.put(revokedCoin);
       await tx.coins.put(oldCoin);
       await putGroupAsFinished(wex, tx, recoupGroup, coinIdx);
@@ -233,7 +231,7 @@ export async function recoupWithdrawCoin(
 ): Promise<void> {
   const reservePub = cs.reservePub;
   const denomInfo = await wex.db.runReadOnlyTx(
-    ["denominations"],
+    { storeNames: ["denominations"] },
     async (tx) => {
       const denomInfo = await getDenomInfo(
         wex,
@@ -276,7 +274,7 @@ export async function recoupWithdrawCoin(
 
   // FIXME: verify that our expectations about the amount match
   await wex.db.runReadWriteTx(
-    ["coins", "denominations", "recoupGroups", "refreshGroups"],
+    { storeNames: ["coins", "denominations", "recoupGroups", "refreshGroups"] },
     async (tx) => {
       const recoupGroup = await tx.recoupGroups.get(recoupGroupId);
       if (!recoupGroup) {
@@ -300,9 +298,16 @@ export async function processRecoupGroup(
   wex: WalletExecutionContext,
   recoupGroupId: string,
 ): Promise<TaskRunResult> {
-  let recoupGroup = await wex.db.runReadOnlyTx(["recoupGroups"], async (tx) => {
-    return tx.recoupGroups.get(recoupGroupId);
-  });
+  if (!wex.ws.networkAvailable) {
+    return TaskRunResult.networkRequired();
+  }
+
+  let recoupGroup = await wex.db.runReadOnlyTx(
+    { storeNames: ["recoupGroups"] },
+    async (tx) => {
+      return tx.recoupGroups.get(recoupGroupId);
+    },
+  );
   if (!recoupGroup) {
     return TaskRunResult.finished();
   }
@@ -320,9 +325,12 @@ export async function processRecoupGroup(
   });
   await Promise.all(ps);
 
-  recoupGroup = await wex.db.runReadOnlyTx(["recoupGroups"], async (tx) => {
-    return tx.recoupGroups.get(recoupGroupId);
-  });
+  recoupGroup = await wex.db.runReadOnlyTx(
+    { storeNames: ["recoupGroups"] },
+    async (tx) => {
+      return tx.recoupGroups.get(recoupGroupId);
+    },
+  );
   if (!recoupGroup) {
     return TaskRunResult.finished();
   }
@@ -339,22 +347,25 @@ export async function processRecoupGroup(
   const reservePrivMap: Record<string, string> = {};
   for (let i = 0; i < recoupGroup.coinPubs.length; i++) {
     const coinPub = recoupGroup.coinPubs[i];
-    await wex.db.runReadOnlyTx(["coins", "reserves"], async (tx) => {
-      const coin = await tx.coins.get(coinPub);
-      if (!coin) {
-        throw Error(`Coin ${coinPub} not found, can't request recoup`);
-      }
-      if (coin.coinSource.type === CoinSourceType.Withdraw) {
-        const reserve = await tx.reserves.indexes.byReservePub.get(
-          coin.coinSource.reservePub,
-        );
-        if (!reserve) {
-          return;
+    await wex.db.runReadOnlyTx(
+      { storeNames: ["coins", "reserves"] },
+      async (tx) => {
+        const coin = await tx.coins.get(coinPub);
+        if (!coin) {
+          throw Error(`Coin ${coinPub} not found, can't request recoup`);
         }
-        reserveSet.add(coin.coinSource.reservePub);
-        reservePrivMap[coin.coinSource.reservePub] = reserve.reservePriv;
-      }
-    });
+        if (coin.coinSource.type === CoinSourceType.Withdraw) {
+          const reserve = await tx.reserves.indexes.byReservePub.get(
+            coin.coinSource.reservePub,
+          );
+          if (!reserve) {
+            return;
+          }
+          reserveSet.add(coin.coinSource.reservePub);
+          reservePrivMap[coin.coinSource.reservePub] = reserve.reservePriv;
+        }
+      },
+    );
   }
 
   for (const reservePub of reserveSet) {
@@ -385,14 +396,17 @@ export async function processRecoupGroup(
   }
 
   await wex.db.runReadWriteTx(
-    [
-      "recoupGroups",
-      "coinAvailability",
-      "denominations",
-      "refreshGroups",
-      "refreshSessions",
-      "coins",
-    ],
+    {
+      storeNames: [
+        "coinAvailability",
+        "coinHistory",
+        "coins",
+        "denominations",
+        "recoupGroups",
+        "refreshGroups",
+        "refreshSessions",
+      ],
+    },
     async (tx) => {
       const rg2 = await tx.recoupGroups.get(recoupGroupId);
       if (!rg2) {
@@ -420,21 +434,6 @@ export async function processRecoupGroup(
 }
 
 export class RecoupTransactionContext implements TransactionContext {
-  abortTransaction(): Promise<void> {
-    throw new Error("Method not implemented.");
-  }
-  suspendTransaction(): Promise<void> {
-    throw new Error("Method not implemented.");
-  }
-  resumeTransaction(): Promise<void> {
-    throw new Error("Method not implemented.");
-  }
-  failTransaction(): Promise<void> {
-    throw new Error("Method not implemented.");
-  }
-  deleteTransaction(): Promise<void> {
-    throw new Error("Method not implemented.");
-  }
   public transactionId: TransactionIdStr;
   public taskId: TaskIdStr;
 
@@ -451,6 +450,32 @@ export class RecoupTransactionContext implements TransactionContext {
       recoupGroupId,
     });
   }
+
+  abortTransaction(): Promise<void> {
+    throw new Error("Method not implemented.");
+  }
+
+  suspendTransaction(): Promise<void> {
+    throw new Error("Method not implemented.");
+  }
+
+  resumeTransaction(): Promise<void> {
+    throw new Error("Method not implemented.");
+  }
+
+  failTransaction(): Promise<void> {
+    throw new Error("Method not implemented.");
+  }
+
+  deleteTransaction(): Promise<void> {
+    throw new Error("Method not implemented.");
+  }
+
+  lookupFullTransaction(
+    tx: WalletDbAllStoresReadOnlyTransaction,
+  ): Promise<Transaction | undefined> {
+    throw new Error("Method not implemented.");
+  }
 }
 
 export async function createRecoupGroup(
@@ -502,7 +527,7 @@ async function processRecoupForCoin(
   coinIdx: number,
 ): Promise<void> {
   const coin = await wex.db.runReadOnlyTx(
-    ["coins", "recoupGroups"],
+    { storeNames: ["coins", "recoupGroups"] },
     async (tx) => {
       const recoupGroup = await tx.recoupGroups.get(recoupGroupId);
       if (!recoupGroup) {
