@@ -29,7 +29,6 @@ import {
   Amounts,
   amountToPretty,
   assertUnreachable,
-  AsyncFlag,
   checkDbInvariant,
   codecForCoinHistoryResponse,
   codecForExchangeMeltResponse,
@@ -55,10 +54,10 @@ import {
   makeErrorDetail,
   NotificationType,
   RefreshReason,
-  TalerError,
   TalerErrorCode,
   TalerErrorDetail,
   TalerPreciseTimestamp,
+  Transaction,
   TransactionAction,
   TransactionIdStr,
   TransactionMajorState,
@@ -68,12 +67,14 @@ import {
   WalletNotification,
 } from "@gnu-taler/taler-util";
 import {
+  HttpResponse,
   readSuccessResponseJsonOrThrow,
   readTalerErrorResponse,
   throwUnexpectedRequestError,
 } from "@gnu-taler/taler-util/http";
 import {
   constructTaskIdentifier,
+  genericWaitForState,
   makeCoinsVisible,
   PendingTaskType,
   TaskIdStr,
@@ -92,6 +93,7 @@ import {
 import { CryptoApiStoppedError } from "./crypto/workers/crypto-dispatcher.js";
 import {
   CoinAvailabilityRecord,
+  CoinHistoryRecord,
   CoinRecord,
   CoinSourceType,
   DenominationRecord,
@@ -100,7 +102,9 @@ import {
   RefreshGroupRecord,
   RefreshOperationStatus,
   RefreshSessionRecord,
+  timestampPreciseFromDb,
   timestampPreciseToDb,
+  WalletDbAllStoresReadOnlyTransaction,
   WalletDbReadOnlyTransaction,
   WalletDbReadWriteTransaction,
   WalletDbStoresArr,
@@ -108,6 +112,7 @@ import {
 import { selectWithdrawalDenominations } from "./denomSelection.js";
 import {
   constructTransactionIdentifier,
+  isUnsuccessfulTransaction,
   notifyTransition,
   TransitionInfo,
 } from "./transactions.js";
@@ -129,7 +134,7 @@ async function updateRefreshTransaction(
   tx: WalletDbReadWriteTransaction<
     [
       "refreshGroups",
-      "transactions",
+      "transactionsMeta",
       "operationRetries",
       "exchanges",
       "exchangeDetails",
@@ -155,6 +160,52 @@ export class RefreshTransactionContext implements TransactionContext {
     });
   }
 
+  /**
+   * Get the full transaction details for the transaction.
+   *
+   * Returns undefined if the transaction is in a state where we do not have a
+   * transaction item (e.g. if it was deleted).
+   */
+  async lookupFullTransaction(
+    tx: WalletDbAllStoresReadOnlyTransaction,
+  ): Promise<Transaction | undefined> {
+    const refreshGroupRecord = await tx.refreshGroups.get(this.refreshGroupId);
+    if (!refreshGroupRecord) {
+      return undefined;
+    }
+    const ort = await tx.operationRetries.get(this.taskId);
+    const inputAmount = Amounts.sumOrZero(
+      refreshGroupRecord.currency,
+      refreshGroupRecord.inputPerCoin,
+    ).amount;
+    const outputAmount = Amounts.sumOrZero(
+      refreshGroupRecord.currency,
+      refreshGroupRecord.expectedOutputPerCoin,
+    ).amount;
+    const txState = computeRefreshTransactionState(refreshGroupRecord);
+    return {
+      type: TransactionType.Refresh,
+      txState,
+      txActions: computeRefreshTransactionActions(refreshGroupRecord),
+      refreshReason: refreshGroupRecord.reason,
+      amountEffective: isUnsuccessfulTransaction(txState)
+        ? Amounts.stringify(Amounts.zeroOfAmount(inputAmount))
+        : Amounts.stringify(Amounts.sub(outputAmount, inputAmount).amount),
+      amountRaw: Amounts.stringify(
+        Amounts.zeroOfCurrency(refreshGroupRecord.currency),
+      ),
+      refreshInputAmount: Amounts.stringify(inputAmount),
+      refreshOutputAmount: Amounts.stringify(outputAmount),
+      originatingTransactionId: refreshGroupRecord.originatingTransactionId,
+      timestamp: timestampPreciseFromDb(refreshGroupRecord.timestampCreated),
+      transactionId: constructTransactionIdentifier({
+        tag: TransactionType.Refresh,
+        refreshGroupId: refreshGroupRecord.refreshGroupId,
+      }),
+      ...(ort?.lastError ? { error: ort.lastError } : {}),
+    };
+  }
+
   /**
    * Transition a withdrawal transaction.
    * Extra object stores may be accessed during the transition.
@@ -166,7 +217,7 @@ export class RefreshTransactionContext implements TransactionContext {
       tx: WalletDbReadWriteTransaction<
         [
           "refreshGroups",
-          "transactions",
+          "transactionsMeta",
           "operationRetries",
           "exchanges",
           "exchangeDetails",
@@ -177,7 +228,7 @@ export class RefreshTransactionContext implements TransactionContext {
   ): Promise<TransitionInfo | undefined> {
     const baseStores = [
       "refreshGroups" as const,
-      "transactions" as const,
+      "transactionsMeta" as const,
       "operationRetries" as const,
       "exchanges" as const,
       "exchangeDetails" as const,
@@ -186,7 +237,7 @@ export class RefreshTransactionContext implements TransactionContext {
       ? [...baseStores, ...opts.extraStores]
       : baseStores;
     const transitionInfo = await this.wex.db.runReadWriteTx(
-      stores,
+      { storeNames: stores },
       async (tx) => {
         const wgRec = await tx.refreshGroups.get(this.refreshGroupId);
         let oldTxState: TransactionState;
@@ -386,7 +437,6 @@ async function getCoinAvailabilityForDenom(
   denom: DenominationInfo,
   ageRestriction: number,
 ): Promise<CoinAvailabilityRecord> {
-  checkDbInvariant(!!denom);
   let car = await tx.coinAvailability.get([
     denom.exchangeBaseUrl,
     denom.denomPubHash,
@@ -537,7 +587,10 @@ async function destroyRefreshSession(
       denom,
       oldCoin.maxAge,
     );
-    checkDbInvariant(car.pendingRefreshOutputCount != null);
+    checkDbInvariant(
+      car.pendingRefreshOutputCount != null,
+      `no pendingRefreshOutputCount for denom ${dph}`,
+    );
     car.pendingRefreshOutputCount =
       car.pendingRefreshOutputCount - refreshSession.newDenoms[i].count;
     await tx.coinAvailability.put(car);
@@ -546,7 +599,7 @@ async function destroyRefreshSession(
 
 function getRefreshRequestTimeout(rg: RefreshGroupRecord): Duration {
   return Duration.fromSpec({
-    seconds: 5,
+    seconds: 60,
   });
 }
 
@@ -565,7 +618,14 @@ async function refreshMelt(
 ): Promise<void> {
   const ctx = new RefreshTransactionContext(wex, refreshGroupId);
   const d = await wex.db.runReadWriteTx(
-    ["refreshGroups", "refreshSessions", "coins", "denominations"],
+    {
+      storeNames: [
+        "refreshGroups",
+        "refreshSessions",
+        "coins",
+        "denominations",
+      ],
+    },
     async (tx) => {
       const refreshGroup = await tx.refreshGroups.get(refreshGroupId);
       if (!refreshGroup) {
@@ -686,7 +746,7 @@ async function refreshMelt(
   switch (resp.status) {
     case HttpStatusCode.NotFound: {
       const errDetail = await readTalerErrorResponse(resp);
-      await handleRefreshMeltNotFound(ctx, coinIndex, errDetail);
+      await handleRefreshMeltNotFound(ctx, coinIndex, resp, errDetail);
       return;
     }
     case HttpStatusCode.Gone: {
@@ -723,7 +783,7 @@ async function refreshMelt(
   refreshSession.norevealIndex = norevealIndex;
 
   await wex.db.runReadWriteTx(
-    ["refreshGroups", "refreshSessions"],
+    { storeNames: ["refreshGroups", "refreshSessions"] },
     async (tx) => {
       const rg = await tx.refreshGroups.get(refreshGroupId);
       if (!rg) {
@@ -755,13 +815,15 @@ async function handleRefreshMeltGone(
   // FIXME: Validate signature.
 
   await ctx.wex.db.runReadWriteTx(
-    [
-      "refreshGroups",
-      "refreshSessions",
-      "coins",
-      "denominations",
-      "coinAvailability",
-    ],
+    {
+      storeNames: [
+        "refreshGroups",
+        "refreshSessions",
+        "coins",
+        "denominations",
+        "coinAvailability",
+      ],
+    },
     async (tx) => {
       const rg = await tx.refreshGroups.get(ctx.refreshGroupId);
       if (!rg) {
@@ -832,13 +894,15 @@ async function handleRefreshMeltConflict(
   // FIXME: If response seems wrong, report to auditor (in the future!);
 
   await ctx.wex.db.runReadWriteTx(
-    [
-      "refreshGroups",
-      "refreshSessions",
-      "denominations",
-      "coins",
-      "coinAvailability",
-    ],
+    {
+      storeNames: [
+        "refreshGroups",
+        "refreshSessions",
+        "denominations",
+        "coins",
+        "coinAvailability",
+      ],
+    },
     async (tx) => {
       const rg = await tx.refreshGroups.get(ctx.refreshGroupId);
       if (!rg) {
@@ -887,17 +951,28 @@ async function handleRefreshMeltConflict(
 async function handleRefreshMeltNotFound(
   ctx: RefreshTransactionContext,
   coinIndex: number,
+  resp: HttpResponse,
   errDetails: TalerErrorDetail,
 ): Promise<void> {
-  // FIXME: Validate the exchange's error response
+  // Make sure that we only act on a 404 that indicates a final problem
+  // with the coin.
+  switch (errDetails.code) {
+    case TalerErrorCode.EXCHANGE_GENERIC_COIN_UNKNOWN:
+    case TalerErrorCode.EXCHANGE_GENERIC_DENOMINATION_KEY_UNKNOWN:
+      break;
+    default:
+      throwUnexpectedRequestError(resp, errDetails);
+  }
   await ctx.wex.db.runReadWriteTx(
-    [
-      "refreshGroups",
-      "refreshSessions",
-      "coins",
-      "denominations",
-      "coinAvailability",
-    ],
+    {
+      storeNames: [
+        "refreshGroups",
+        "refreshSessions",
+        "coins",
+        "denominations",
+        "coinAvailability",
+      ],
+    },
     async (tx) => {
       const rg = await tx.refreshGroups.get(ctx.refreshGroupId);
       if (!rg) {
@@ -993,7 +1068,14 @@ async function refreshReveal(
   );
   const ctx = new RefreshTransactionContext(wex, refreshGroupId);
   const d = await wex.db.runReadOnlyTx(
-    ["refreshGroups", "refreshSessions", "coins", "denominations"],
+    {
+      storeNames: [
+        "refreshGroups",
+        "refreshSessions",
+        "coins",
+        "denominations",
+      ],
+    },
     async (tx) => {
       const refreshGroup = await tx.refreshGroups.get(refreshGroupId);
       if (!refreshGroup) {
@@ -1179,7 +1261,6 @@ async function refreshReveal(
         coinEvHash: pc.coinEvHash,
         maxAge: pc.maxAge,
         ageCommitmentProof: pc.ageCommitmentProof,
-        spendAllocation: undefined,
       };
 
       coins.push(coin);
@@ -1187,13 +1268,15 @@ async function refreshReveal(
   }
 
   await wex.db.runReadWriteTx(
-    [
-      "coins",
-      "denominations",
-      "coinAvailability",
-      "refreshGroups",
-      "refreshSessions",
-    ],
+    {
+      storeNames: [
+        "coins",
+        "denominations",
+        "coinAvailability",
+        "refreshGroups",
+        "refreshSessions",
+      ],
+    },
     async (tx) => {
       const rg = await tx.refreshGroups.get(refreshGroupId);
       if (!rg) {
@@ -1220,7 +1303,10 @@ async function refreshReveal(
           coin.exchangeBaseUrl,
           coin.denomPubHash,
         );
-        checkDbInvariant(!!denomInfo);
+        checkDbInvariant(
+          !!denomInfo,
+          `no denom with hash ${coin.denomPubHash}`,
+        );
         const car = await getCoinAvailabilityForDenom(
           wex,
           tx,
@@ -1230,6 +1316,7 @@ async function refreshReveal(
         checkDbInvariant(
           car.pendingRefreshOutputCount != null &&
             car.pendingRefreshOutputCount > 0,
+          `no pendingRefreshOutputCount for denom ${coin.denomPubHash} age ${coin.maxAge}`,
         );
         car.pendingRefreshOutputCount--;
         car.freshCoinCount++;
@@ -1247,13 +1334,15 @@ async function handleRefreshRevealError(
   errDetails: TalerErrorDetail,
 ): Promise<void> {
   await ctx.wex.db.runReadWriteTx(
-    [
-      "refreshGroups",
-      "refreshSessions",
-      "coins",
-      "denominations",
-      "coinAvailability",
-    ],
+    {
+      storeNames: [
+        "refreshGroups",
+        "refreshSessions",
+        "coins",
+        "denominations",
+        "coinAvailability",
+      ],
+    },
     async (tx) => {
       const rg = await tx.refreshGroups.get(ctx.refreshGroupId);
       if (!rg) {
@@ -1285,10 +1374,14 @@ export async function processRefreshGroup(
   wex: WalletExecutionContext,
   refreshGroupId: string,
 ): Promise<TaskRunResult> {
+  if (!wex.ws.networkAvailable) {
+    return TaskRunResult.networkRequired();
+  }
+
   logger.trace(`processing refresh group ${refreshGroupId}`);
 
   const refreshGroup = await wex.db.runReadOnlyTx(
-    ["refreshGroups"],
+    { storeNames: ["refreshGroups"] },
     async (tx) => tx.refreshGroups.get(refreshGroupId),
   );
   if (!refreshGroup) {
@@ -1305,35 +1398,36 @@ export async function processRefreshGroup(
     throw Error("refresh blocked");
   }
 
-  // Process refresh sessions of the group in parallel.
   logger.trace(
     `processing refresh sessions for ${refreshGroup.oldCoinPubs.length} old coins`,
   );
   let errors: TalerErrorDetail[] = [];
   let inShutdown = false;
-  const ps = refreshGroup.oldCoinPubs.map((x, i) =>
-    processRefreshSession(wex, refreshGroupId, i).catch((x) => {
+
+  // Process refresh sessions in sequence.
+  // In the future, we could parallelize request, in particular when multiple
+  // exchanges are involved.
+  // But we need to make sure that we write results to DB with high priority,
+  // otherwise we run into problems with very large refresh groups, where we'd first
+  // do many many network requests before even going to the DB.
+
+  for (let i = 0; i < refreshGroup.oldCoinPubs.length; i++) {
+    try {
+      await processRefreshSession(wex, refreshGroupId, i);
+    } catch (x) {
       if (x instanceof CryptoApiStoppedError) {
         inShutdown = true;
         logger.info(
           "crypto API stopped while processing refresh group, probably the wallet is currently shutting down.",
         );
-        return;
+        break;
       }
-      if (x instanceof TalerError) {
-        logger.warn("process refresh session got exception (TalerError)");
-        logger.warn(`exc ${x}`);
-        logger.warn(`exc stack ${x.stack}`);
-        logger.warn(`error detail: ${j2s(x.errorDetail)}`);
-      } else {
-        logger.warn("process refresh session got exception");
-        logger.warn(`exc ${x}`);
-        logger.warn(`exc stack ${x.stack}`);
-      }
-      errors.push(getErrorDetailFromException(x));
-    }),
-  );
-  await Promise.all(ps);
+      const err = getErrorDetailFromException(x);
+      logger.warn(`exception in refresh session: ${j2s(err)}`);
+      errors.push(err);
+    }
+  }
+
   if (inShutdown) {
     return TaskRunResult.finished();
   }
@@ -1344,7 +1438,7 @@ export async function processRefreshGroup(
   // status of the whole refresh group.
 
   const transitionInfo = await wex.db.runReadWriteTx(
-    ["coins", "coinAvailability", "refreshGroups"],
+    { storeNames: ["coins", "coinAvailability", "refreshGroups"] },
     async (tx) => {
       const rg = await tx.refreshGroups.get(refreshGroupId);
       if (!rg) {
@@ -1420,7 +1514,7 @@ async function processRefreshSession(
     `processing refresh session for coin ${coinIndex} of group ${refreshGroupId}`,
   );
   let { refreshGroup, refreshSession } = await wex.db.runReadOnlyTx(
-    ["refreshGroups", "refreshSessions"],
+    { storeNames: ["refreshGroups", "refreshSessions"] },
     async (tx) => {
       const rg = await tx.refreshGroups.get(refreshGroupId);
       const rs = await tx.refreshSessions.get([refreshGroupId, coinIndex]);
@@ -1507,7 +1601,13 @@ export async function calculateRefreshOutput(
 async function applyRefreshToOldCoins(
   wex: WalletExecutionContext,
   tx: WalletDbReadWriteTransaction<
-    ["denominations", "coins", "refreshGroups", "coinAvailability"]
+    [
+      "denominations",
+      "coins",
+      "coinHistory",
+      "refreshGroups",
+      "coinAvailability",
+    ]
   >,
   oldCoinPubs: CoinRefreshRequest[],
   refreshGroupId: string,
@@ -1535,9 +1635,22 @@ async function applyRefreshToOldCoins(
           coin.denomPubHash,
           coin.maxAge,
         ]);
-        checkDbInvariant(!!coinAv);
-        checkDbInvariant(coinAv.freshCoinCount > 0);
+        checkDbInvariant(
+          !!coinAv,
+          `no denom info for ${coin.denomPubHash} age ${coin.maxAge}`,
+        );
+        checkDbInvariant(
+          coinAv.freshCoinCount > 0,
+          `no fresh coins for ${coin.denomPubHash}`,
+        );
         coinAv.freshCoinCount--;
+        if (coin.visible) {
+          if (!coinAv.visibleCoinCount) {
+            logger.error("coin availability inconsistent");
+          } else {
+            coinAv.visibleCoinCount--;
+          }
+        }
         await tx.coinAvailability.put(coinAv);
         break;
       }
@@ -1552,16 +1665,24 @@ async function applyRefreshToOldCoins(
       default:
         assertUnreachable(coin.status);
     }
-    if (!coin.spendAllocation) {
-      coin.spendAllocation = {
-        amount: Amounts.stringify(ocp.amount),
-        // id: `txn:refresh:${refreshGroupId}`,
-        id: constructTransactionIdentifier({
-          tag: TransactionType.Refresh,
-          refreshGroupId,
-        }),
+    let histEntry: CoinHistoryRecord | undefined = await tx.coinHistory.get(
+      coin.coinPub,
+    );
+    if (!histEntry) {
+      histEntry = {
+        coinPub: coin.coinPub,
+        history: [],
       };
     }
+    histEntry.history.push({
+      type: "refresh",
+      transactionId: constructTransactionIdentifier({
+        tag: TransactionType.Refresh,
+        refreshGroupId,
+      }),
+      amount: Amounts.stringify(ocp.amount),
+    });
+    await tx.coinHistory.put(histEntry);
     await tx.coins.put(coin);
   }
 }
@@ -1586,6 +1707,7 @@ export async function createRefreshGroup(
     [
       "denominations",
       "coins",
+      "coinHistory",
       "refreshGroups",
       "refreshSessions",
       "coinAvailability",
@@ -1710,7 +1832,7 @@ export function getRefreshesForTransaction(
   wex: WalletExecutionContext,
   transactionId: string,
 ): Promise<string[]> {
-  return wex.db.runReadOnlyTx(["refreshGroups"], async (tx) => {
+  return wex.db.runReadOnlyTx({ storeNames: ["refreshGroups"] }, async (tx) => {
     const groups =
       await tx.refreshGroups.indexes.byOriginatingTransactionId.getAll(
         transactionId,
@@ -1736,15 +1858,18 @@ export async function forceRefresh(
     throw Error("refusing to create empty refresh group");
   }
   const res = await wex.db.runReadWriteTx(
-    [
-      "refreshGroups",
-      "coinAvailability",
-      "refreshSessions",
-      "denominations",
-      "coins",
-    ],
+    {
+      storeNames: [
+        "refreshGroups",
+        "coinAvailability",
+        "refreshSessions",
+        "denominations",
+        "coins",
+        "coinHistory",
+      ],
+    },
     async (tx) => {
-      let coinPubs: CoinRefreshRequest[] = [];
+      const coinPubs: CoinRefreshRequest[] = [];
       for (const c of req.refreshCoinSpecs) {
         const coin = await tx.coins.get(c.coinPub);
         if (!coin) {
@@ -1756,7 +1881,7 @@ export async function forceRefresh(
           coin.exchangeBaseUrl,
           coin.denomPubHash,
         );
-        checkDbInvariant(!!denom);
+        checkDbInvariant(!!denom, `no denom hash: ${coin.denomPubHash}`);
         coinPubs.push({
           coinPub: c.coinPub,
           amount: c.amount ?? denom.value,
@@ -1792,66 +1917,38 @@ export async function waitRefreshFinal(
   const ctx = new RefreshTransactionContext(wex, refreshGroupId);
   wex.taskScheduler.startShepherdTask(ctx.taskId);
 
-  // FIXME: Clean up using the new JS "using" / Symbol.dispose syntax.
-  const refreshNotifFlag = new AsyncFlag();
-  // Raise purchaseNotifFlag whenever we get a notification
-  // about our refresh.
-  const cancelNotif = wex.ws.addNotificationListener((notif) => {
-    if (
-      notif.type === NotificationType.TransactionStateTransition &&
-      notif.transactionId === ctx.transactionId
-    ) {
-      refreshNotifFlag.raise();
-    }
+  await genericWaitForState(wex, {
+    async checkState(): Promise<boolean> {
+      // Check if refresh is final
+      const res = await ctx.wex.db.runReadOnlyTx(
+        { storeNames: ["refreshGroups"] },
+        async (tx) => {
+          return {
+            rg: await tx.refreshGroups.get(ctx.refreshGroupId),
+          };
+        },
+      );
+      const { rg } = res;
+      if (!rg) {
+        // Must've been deleted, we consider that final.
+        return true;
+      }
+      switch (rg.operationStatus) {
+        case RefreshOperationStatus.Failed:
+        case RefreshOperationStatus.Finished:
+          // Transaction is final
+          return true;
+        case RefreshOperationStatus.Pending:
+        case RefreshOperationStatus.Suspended:
+          break;
+      }
+      return false;
+    },
+    filterNotification(notif): boolean {
+      return (
+        notif.type === NotificationType.TransactionStateTransition &&
+        notif.transactionId === ctx.transactionId
+      );
+    },
   });
-  const unregisterOnCancelled = wex.cancellationToken.onCancelled(() => {
-    cancelNotif();
-    refreshNotifFlag.raise();
-  });
-
-  try {
-    await internalWaitRefreshFinal(ctx, refreshNotifFlag);
-  } catch (e) {
-    unregisterOnCancelled();
-    cancelNotif();
-  }
-}
-
-async function internalWaitRefreshFinal(
-  ctx: RefreshTransactionContext,
-  flag: AsyncFlag,
-): Promise<void> {
-  while (true) {
-    if (ctx.wex.cancellationToken.isCancelled) {
-      throw Error("cancelled");
-    }
-
-    // Check if refresh is final
-    const res = await ctx.wex.db.runReadOnlyTx(
-      ["refreshGroups", "operationRetries"],
-      async (tx) => {
-        return {
-          rg: await tx.refreshGroups.get(ctx.refreshGroupId),
-        };
-      },
-    );
-    const { rg } = res;
-    if (!rg) {
-      // Must've been deleted, we consider that final.
-      return;
-    }
-    switch (rg.operationStatus) {
-      case RefreshOperationStatus.Failed:
-      case RefreshOperationStatus.Finished:
-        // Transaction is final
-        return;
-      case RefreshOperationStatus.Pending:
-      case RefreshOperationStatus.Suspended:
-        break;
-    }
-
-    // Wait for the next transition
-    await flag.wait();
-    flag.reset();
-  }
 }
