@@ -27,7 +27,6 @@ import {
   NotificationType,
   ObservabilityContext,
   ObservabilityEventType,
-  RetryLoopOpts,
   TalerErrorDetail,
   TaskThrottler,
   TransactionIdStr,
@@ -37,6 +36,7 @@ import {
   assertUnreachable,
   getErrorDetailFromException,
   j2s,
+  safeStringifyException,
 } from "@gnu-taler/taler-util";
 import { processBackupForProvider } from "./backup/index.js";
 import {
@@ -50,12 +50,13 @@ import {
   parseTaskIdentifier,
 } from "./common.js";
 import {
-  OPERATION_STATUS_ACTIVE_FIRST,
-  OPERATION_STATUS_ACTIVE_LAST,
+  OPERATION_STATUS_NONFINAL_FIRST,
+  OPERATION_STATUS_NONFINAL_LAST,
   OperationRetryRecord,
   WalletDbAllStoresReadOnlyTransaction,
   WalletDbReadOnlyTransaction,
   timestampAbsoluteFromDb,
+  timestampPreciseToDb,
 } from "./db.js";
 import {
   computeDepositTransactionStatus,
@@ -91,7 +92,6 @@ import {
   computeRefreshTransactionState,
   processRefreshGroup,
 } from "./refresh.js";
-import { computeRewardTransactionStatus } from "./reward.js";
 import {
   constructTransactionIdentifier,
   parseTransactionIdentifier,
@@ -114,6 +114,8 @@ const logger = new Logger("shepherd.ts");
  */
 interface ShepherdInfo {
   cts: CancellationToken.Source;
+  latch?: Promise<void>;
+  stopped: boolean;
 }
 
 /**
@@ -143,13 +145,22 @@ function taskGivesLiveness(taskId: string): boolean {
 }
 
 export interface TaskScheduler {
-  ensureRunning(): void;
-  run(opts?: RetryLoopOpts): Promise<void>;
+  /**
+   * Ensure that the task scheduler is running.
+   *
+   * If it is not running, start it, with previous
+   * tasks loaded from the database.
+   *
+   * Returns after the scheduler is running.
+   */
+  ensureRunning(): Promise<void>;
   startShepherdTask(taskId: TaskIdStr): void;
   stopShepherdTask(taskId: TaskIdStr): void;
   resetTaskRetries(taskId: TaskIdStr): Promise<void>;
-  reload(): void;
+  reload(): Promise<void>;
   getActiveTasks(): TaskIdStr[];
+  isIdle(): boolean;
+  shutdown(): Promise<void>;
 }
 
 export class TaskSchedulerImpl implements TaskScheduler {
@@ -177,58 +188,76 @@ export class TaskSchedulerImpl implements TaskScheduler {
     return [...this.sheps.keys()];
   }
 
-  ensureRunning(): void {
+  async shutdown(): Promise<void> {
+    const tasksIds = [...this.sheps.keys()];
+    logger.info(`Stopping task shepherd.`);
+    for (const taskId of tasksIds) {
+      this.stopShepherdTask(taskId);
+    }
+  }
+
+  /**
+   * @see TaskScheduler.ensureRunning
+   */
+  async ensureRunning(): Promise<void> {
     if (this.isRunning) {
       return;
     }
+    this.isRunning = true;
+    try {
+      await this.loadTasksFromDb();
+    } catch (e) {
+      this.isRunning = false;
+      throw e;
+    }
     this.run()
       .catch((e) => {
         logger.error("error running task loop");
         logger.error(`err: ${e}`);
       })
       .then(() => {
-        logger.info("done running task loop");
+        logger.trace("done running task loop");
+        this.isRunning = false;
       });
   }
 
-  async run(opts: RetryLoopOpts = {}): Promise<void> {
-    if (this.isRunning) {
-      throw Error("task loop already running");
-    }
-    logger.info("Running task loop.");
-    this.isRunning = true;
-    await this.loadTasksFromDb();
-    logger.info("loaded!");
-    logger.info(`sheps: ${this.sheps.size}`);
-    while (true) {
-      if (opts.stopWhenDone) {
-        let alive = false;
-        const taskIds = [...this.sheps.keys()];
-        logger.info(`current task IDs: ${j2s(taskIds)}`);
-        logger.info(`sheps: ${this.sheps.size}`);
-        for (const taskId of taskIds) {
-          if (taskGivesLiveness(taskId)) {
-            alive = true;
-            break;
-          }
-        }
-        if (!alive) {
-          logger.info("Breaking out of task loop (no more work).");
-          break;
-        }
-      }
-      if (this.ws.stopped) {
-        logger.info("Breaking out of task loop (wallet stopped).");
+  isIdle(): boolean {
+    let alive = false;
+    const taskIds = [...this.sheps.keys()];
+    for (const taskId of taskIds) {
+      if (taskGivesLiveness(taskId)) {
+        alive = true;
         break;
       }
+    }
+    // We're idle if no task is alive anymore.
+    return !alive;
+  }
+
+  private async run(): Promise<void> {
+    logger.trace("Running task loop.");
+    logger.trace(`sheps: ${this.sheps.size}`);
+    while (true) {
+      if (this.ws.stopped) {
+        logger.trace("Breaking out of task loop (wallet stopped).");
+        break;
+      }
+
+      if (this.isIdle()) {
+        this.ws.notify({
+          type: NotificationType.Idle,
+        });
+      }
+
       await this.iterCond.wait();
     }
-    this.isRunning = false;
-    logger.info("Done with task loop.");
+    logger.trace("Done with task loop.");
   }
 
   startShepherdTask(taskId: TaskIdStr): void {
-    this.ensureRunning();
+    this.ensureRunning().catch((e) => {
+      logger.error(`error running scheduler: ${safeStringifyException(e)}`);
+    });
     // Run in the background, no await!
     this.internalStartShepherdTask(taskId);
   }
@@ -238,10 +267,10 @@ export class TaskSchedulerImpl implements TaskScheduler {
    *
    * Mostly useful to interrupt all waits when time-travelling.
    */
-  reload() {
-    this.ensureRunning();
+  async reload(): Promise<void> {
+    await this.ensureRunning();
     const tasksIds = [...this.sheps.keys()];
-    logger.info(`reloading sheperd with ${tasksIds.length} tasks`);
+    logger.info(`reloading shepherd with ${tasksIds.length} tasks`);
     for (const taskId of tasksIds) {
       this.stopShepherdTask(taskId);
     }
@@ -254,16 +283,25 @@ export class TaskSchedulerImpl implements TaskScheduler {
     logger.trace(`Starting to shepherd task ${taskId}`);
     const oldShep = this.sheps.get(taskId);
     if (oldShep) {
-      logger.trace(`Already have a shepherd for ${taskId}`);
-      return;
+      if (!oldShep.stopped) {
+        logger.trace(`Already have a shepherd for ${taskId}`);
+        return;
+      }
+      logger.trace(
+        `Waiting for old task to complete the loop in cancel mode ${taskId}`,
+      );
+      await oldShep.latch;
+      logger.trace(`Old task ${taskId} completed in cancel mode`);
     }
     logger.trace(`Creating new shepherd for ${taskId}`);
     const newShep: ShepherdInfo = {
       cts: CancellationToken.create(),
+      stopped: false,
     };
     this.sheps.set(taskId, newShep);
     try {
-      await this.internalShepherdTask(taskId, newShep);
+      newShep.latch = this.internalShepherdTask(taskId, newShep);
+      await newShep.latch;
     } finally {
       logger.trace(`Done shepherding ${taskId}`);
       this.sheps.delete(taskId);
@@ -276,8 +314,8 @@ export class TaskSchedulerImpl implements TaskScheduler {
     const oldShep = this.sheps.get(taskId);
     if (oldShep) {
       logger.trace(`Cancelling old shepherd for ${taskId}`);
-      oldShep.cts.cancel();
-      this.sheps.delete(taskId);
+      oldShep.cts.cancel(`stopping task ${taskId}`);
+      oldShep.stopped = true;
       this.iterCond.trigger();
     }
   }
@@ -291,6 +329,7 @@ export class TaskSchedulerImpl implements TaskScheduler {
     const maybeNotification = await this.ws.db.runAllStoresReadWriteTx(
       {},
       async (tx) => {
+        logger.trace(`storing task [reset] for ${taskId}`);
         await tx.operationRetries.delete(taskId);
         return taskToRetryNotification(this.ws, tx, taskId, undefined);
       },
@@ -310,7 +349,13 @@ export class TaskSchedulerImpl implements TaskScheduler {
     try {
       await info.cts.token.racePromise(this.ws.timerGroup.resolveAfter(delay));
     } catch (e) {
-      logger.info(`waiting for ${taskId} interrupted`);
+      if (e instanceof CancellationToken.CancellationError) {
+        logger.info(
+          `waiting for ${taskId} interrupted: ${e.message} ${j2s(e.reason)}`,
+        );
+      } else {
+        logger.info(`waiting for ${taskId} interrupted: ${e}`);
+      }
     }
   }
 
@@ -348,17 +393,18 @@ export class TaskSchedulerImpl implements TaskScheduler {
       try {
         res = await callOperationHandlerForTaskId(wex, taskId);
       } catch (e) {
+        logger.trace(`Shepherd error ${taskId} saving response ${e}`);
         res = {
           type: TaskRunResultType.Error,
           errorDetail: getErrorDetailFromException(e),
         };
       }
       if (info.cts.token.isCancelled) {
-        logger.info("task cancelled, not processing result");
+        logger.trace(`task ${taskId} cancelled, not processing result`);
         return;
       }
       if (this.ws.stopped) {
-        logger.info("wallet stopped, not processing result");
+        logger.trace("wallet stopped, not processing result");
         return;
       }
       wex.oc.observe({
@@ -367,15 +413,16 @@ export class TaskSchedulerImpl implements TaskScheduler {
       });
       switch (res.type) {
         case TaskRunResultType.Error: {
-          logger.trace(`Shepherd for ${taskId} got error result.`);
+          logger.trace(
+            `Shepherd for ${taskId} got error result: ${j2s(res.errorDetail)}`,
+          );
           const retryRecord = await storePendingTaskError(
             this.ws,
             taskId,
             res.errorDetail,
           );
-          let delay: Duration;
           const t = timestampAbsoluteFromDb(retryRecord.retryInfo.nextRetry);
-          delay = AbsoluteTime.remaining(t);
+          const delay = AbsoluteTime.remaining(t);
           logger.trace(`Waiting for ${delay.d_ms} ms`);
           await this.wait(taskId, info, delay);
           break;
@@ -383,9 +430,8 @@ export class TaskSchedulerImpl implements TaskScheduler {
         case TaskRunResultType.Backoff: {
           logger.trace(`Shepherd for ${taskId} got backoff result.`);
           const retryRecord = await storePendingTaskPending(this.ws, taskId);
-          let delay: Duration;
           const t = timestampAbsoluteFromDb(retryRecord.retryInfo.nextRetry);
-          delay = AbsoluteTime.remaining(t);
+          const delay = AbsoluteTime.remaining(t);
           logger.trace(`Waiting for ${delay.d_ms} ms`);
           await this.wait(taskId, info, delay);
           break;
@@ -397,13 +443,19 @@ export class TaskSchedulerImpl implements TaskScheduler {
           await storeTaskProgress(this.ws, taskId);
           break;
         }
-        case TaskRunResultType.ScheduleLater:
+        case TaskRunResultType.ScheduleLater: {
           logger.trace(`Shepherd for ${taskId} got schedule-later result.`);
-          await storeTaskProgress(this.ws, taskId);
-          const delay = AbsoluteTime.remaining(res.runAt);
+          const retryRecord = await storePendingTaskPending(
+            this.ws,
+            taskId,
+            res.runAt,
+          );
+          const t = timestampAbsoluteFromDb(retryRecord.retryInfo.nextRetry);
+          const delay = AbsoluteTime.remaining(t);
           logger.trace(`Waiting for ${delay.d_ms} ms`);
           await this.wait(taskId, info, delay);
           break;
+        }
         case TaskRunResultType.Finished:
           logger.trace(`Shepherd for ${taskId} got finished result.`);
           await storePendingTaskFinished(this.ws, taskId);
@@ -425,6 +477,14 @@ export class TaskSchedulerImpl implements TaskScheduler {
           }
           break;
         }
+        case TaskRunResultType.NetworkRequired: {
+          logger.trace(`Shepherd for ${taskId} got network-required result.`);
+          await storePendingTaskPending(this.ws, taskId);
+          const delay = Duration.getForever();
+          logger.trace(`Not retrying task until network is restored.`);
+          await this.wait(taskId, info, delay);
+          break;
+        }
         default:
           assertUnreachable(res);
       }
@@ -437,7 +497,7 @@ async function storePendingTaskError(
   pendingTaskId: string,
   e: TalerErrorDetail,
 ): Promise<OperationRetryRecord> {
-  logger.info(`storing pending task error for ${pendingTaskId}`);
+  logger.trace(`storing task [pending] with ERROR for ${pendingTaskId}`);
   const res = await ws.db.runAllStoresReadWriteTx({}, async (tx) => {
     let retryRecord = await tx.operationRetries.get(pendingTaskId);
     if (!retryRecord) {
@@ -469,15 +529,21 @@ async function storeTaskProgress(
   ws: InternalWalletState,
   pendingTaskId: string,
 ): Promise<void> {
-  await ws.db.runReadWriteTx(["operationRetries"], async (tx) => {
-    await tx.operationRetries.delete(pendingTaskId);
-  });
+  logger.trace(`storing task [progress] for ${pendingTaskId}`);
+  await ws.db.runReadWriteTx(
+    { storeNames: ["operationRetries"] },
+    async (tx) => {
+      await tx.operationRetries.delete(pendingTaskId);
+    },
+  );
 }
 
 async function storePendingTaskPending(
   ws: InternalWalletState,
   pendingTaskId: string,
+  schedTime?: AbsoluteTime,
 ): Promise<OperationRetryRecord> {
+  logger.trace(`storing task [pending] for ${pendingTaskId}`);
   const res = await ws.db.runAllStoresReadWriteTx({}, async (tx) => {
     let retryRecord = await tx.operationRetries.get(pendingTaskId);
     let hadError = false;
@@ -493,6 +559,11 @@ async function storePendingTaskPending(
       delete retryRecord.lastError;
       retryRecord.retryInfo = DbRetryInfo.increment(retryRecord.retryInfo);
     }
+    if (schedTime) {
+      retryRecord.retryInfo.nextRetry = timestampPreciseToDb(
+        AbsoluteTime.toPreciseTimestamp(schedTime),
+      );
+    }
     await tx.operationRetries.put(retryRecord);
     let notification: WalletNotification | undefined = undefined;
     if (hadError) {
@@ -518,9 +589,13 @@ async function storePendingTaskFinished(
   ws: InternalWalletState,
   pendingTaskId: string,
 ): Promise<void> {
-  await ws.db.runReadWriteTx(["operationRetries"], async (tx) => {
-    await tx.operationRetries.delete(pendingTaskId);
-  });
+  logger.trace(`storing task [finished] for ${pendingTaskId}`);
+  await ws.db.runReadWriteTx(
+    { storeNames: ["operationRetries"] },
+    async (tx) => {
+      await tx.operationRetries.delete(pendingTaskId);
+    },
+  );
 }
 
 function getWalletExecutionContextForTask(
@@ -544,12 +619,17 @@ function getWalletExecutionContextForTask(
       },
     };
 
-    wex = getObservedWalletExecutionContext(ws, cancellationToken, oc);
+    wex = getObservedWalletExecutionContext(
+      ws,
+      cancellationToken,
+      undefined,
+      oc,
+    );
   } else {
     oc = {
       observe(evt) {},
     };
-    wex = getNormalWalletExecutionContext(ws, cancellationToken, oc);
+    wex = getNormalWalletExecutionContext(ws, cancellationToken, undefined, oc);
   }
   return wex;
 }
@@ -559,6 +639,15 @@ async function callOperationHandlerForTaskId(
   taskId: TaskIdStr,
 ): Promise<TaskRunResult> {
   const pending = parseTaskIdentifier(taskId);
+
+  const txId = convertTaskToTransactionId(taskId);
+  if (txId) {
+    wex.oc.observe({
+      type: ObservabilityEventType.DeclareConcernsTransaction,
+      transactionId: txId,
+    });
+  }
+
   switch (pending.tag) {
     case PendingTaskType.ExchangeUpdate:
       return await updateExchangeFromUrlHandler(wex, pending.exchangeBaseUrl);
@@ -713,13 +802,6 @@ async function getTransactionState(
       }
       return computeRefreshTransactionState(rec);
     }
-    case TransactionType.Reward: {
-      const rec = await tx.rewards.get(parsedTxId.walletRewardId);
-      if (!rec) {
-        return undefined;
-      }
-      return computeRewardTransactionStatus(rec);
-    }
     case TransactionType.Recoup:
       throw Error("not yet supported");
     case TransactionType.DenomLoss: {
@@ -796,93 +878,6 @@ async function makeExchangeRetryNotification(
   return notif;
 }
 
-export function listTaskForTransactionId(transactionId: string): TaskIdStr[] {
-  const tid = parseTransactionIdentifier(transactionId);
-  if (!tid) {
-    throw Error("invalid task ID");
-  }
-  switch (tid.tag) {
-    case TransactionType.Deposit:
-      return [
-        constructTaskIdentifier({
-          tag: PendingTaskType.Deposit,
-          depositGroupId: tid.depositGroupId,
-        }),
-      ];
-    case TransactionType.InternalWithdrawal:
-      return [
-        constructTaskIdentifier({
-          tag: PendingTaskType.Withdraw,
-          withdrawalGroupId: tid.withdrawalGroupId,
-        }),
-      ];
-    case TransactionType.Payment:
-      return [
-        constructTaskIdentifier({
-          tag: PendingTaskType.Purchase,
-          proposalId: tid.proposalId,
-        }),
-      ];
-    case TransactionType.PeerPullCredit:
-      return [
-        constructTaskIdentifier({
-          tag: PendingTaskType.PeerPullCredit,
-          pursePub: tid.pursePub,
-        }),
-      ];
-    case TransactionType.PeerPullDebit:
-      return [
-        constructTaskIdentifier({
-          tag: PendingTaskType.PeerPullDebit,
-          peerPullDebitId: tid.peerPullDebitId,
-        }),
-      ];
-    case TransactionType.PeerPushCredit:
-      return [
-        constructTaskIdentifier({
-          tag: PendingTaskType.PeerPullCredit,
-          pursePub: tid.peerPushCreditId,
-        }),
-      ];
-    case TransactionType.PeerPushDebit:
-      return [
-        constructTaskIdentifier({
-          tag: PendingTaskType.PeerPullCredit,
-          pursePub: tid.pursePub,
-        }),
-      ];
-    case TransactionType.Recoup:
-      return [
-        constructTaskIdentifier({
-          tag: PendingTaskType.Recoup,
-          recoupGroupId: tid.recoupGroupId,
-        }),
-      ];
-    case TransactionType.Refresh:
-      return [
-        constructTaskIdentifier({
-          tag: PendingTaskType.Refresh,
-          refreshGroupId: tid.refreshGroupId,
-        }),
-      ];
-    case TransactionType.Refund:
-      return [];
-    case TransactionType.Reward:
-      return [];
-    case TransactionType.Withdrawal:
-      return [
-        constructTaskIdentifier({
-          tag: PendingTaskType.Withdraw,
-          withdrawalGroupId: tid.withdrawalGroupId,
-        }),
-      ];
-    case TransactionType.DenomLoss:
-      return [];
-    default:
-      assertUnreachable(tid);
-  }
-}
-
 /**
  * Convert the task ID for a task that processes a transaction int
  * the ID for the transaction.
@@ -925,11 +920,6 @@ export function convertTaskToTransactionId(
         tag: TransactionType.Refresh,
         refreshGroupId: parsedTaskId.refreshGroupId,
       });
-    case PendingTaskType.RewardPickup:
-      return constructTransactionIdentifier({
-        tag: TransactionType.Reward,
-        walletRewardId: parsedTaskId.walletRewardId,
-      });
     case PendingTaskType.PeerPushDebit:
       return constructTransactionIdentifier({
         tag: TransactionType.PeerPushDebit,
@@ -956,22 +946,24 @@ export async function getActiveTaskIds(
     taskIds: [],
   };
   await ws.db.runReadWriteTx(
-    [
-      "exchanges",
-      "refreshGroups",
-      "withdrawalGroups",
-      "purchases",
-      "depositGroups",
-      "recoupGroups",
-      "peerPullCredit",
-      "peerPushDebit",
-      "peerPullDebit",
-      "peerPushCredit",
-    ],
+    {
+      storeNames: [
+        "exchanges",
+        "refreshGroups",
+        "withdrawalGroups",
+        "purchases",
+        "depositGroups",
+        "recoupGroups",
+        "peerPullCredit",
+        "peerPushDebit",
+        "peerPullDebit",
+        "peerPushCredit",
+      ],
+    },
     async (tx) => {
       const active = GlobalIDB.KeyRange.bound(
-        OPERATION_STATUS_ACTIVE_FIRST,
-        OPERATION_STATUS_ACTIVE_LAST,
+        OPERATION_STATUS_NONFINAL_FIRST,
+        OPERATION_STATUS_NONFINAL_LAST,
       );
 
       // Withdrawals
