@@ -39,8 +39,6 @@ import {
   j2s,
   Logger,
   NotificationType,
-  OpenedPromise,
-  openPromise,
   parsePaytoUri,
   PreparePayResultType,
   TalerCorebankApiClient,
@@ -58,8 +56,12 @@ import {
   readSuccessResponseJsonOrThrow,
 } from "@gnu-taler/taler-util/http";
 import { getBalances } from "./balance.js";
+import { genericWaitForState } from "./common.js";
 import { createDepositGroup } from "./deposits.js";
-import { fetchFreshExchange } from "./exchanges.js";
+import {
+  acceptExchangeTermsOfService,
+  fetchFreshExchange,
+} from "./exchanges.js";
 import {
   confirmPay,
   preparePayForUri,
@@ -78,7 +80,7 @@ import { initiatePeerPushDebit } from "./pay-peer-push-debit.js";
 import { getRefreshesForTransaction } from "./refresh.js";
 import { getTransactionById, getTransactions } from "./transactions.js";
 import type { WalletExecutionContext } from "./wallet.js";
-import { acceptWithdrawalFromUri } from "./withdraw.js";
+import { acceptBankIntegratedWithdrawal } from "./withdraw.js";
 
 const logger = new Logger("operations/testing.ts");
 
@@ -123,7 +125,10 @@ export async function withdrawTestBalance(
     amount,
   );
 
-  const acceptResp = await acceptWithdrawalFromUri(wex, {
+  await fetchFreshExchange(wex, req.exchangeBaseUrl);
+  await acceptExchangeTermsOfService(wex, req.exchangeBaseUrl);
+
+  const acceptResp = await acceptBankIntegratedWithdrawal(wex, {
     talerWithdrawUri: wresp.taler_withdraw_uri,
     selectedExchange: exchangeBaseUrl,
     forcedDenomSel: req.forcedDenomSel,
@@ -402,52 +407,58 @@ export async function waitUntilAllTransactionsFinal(
   wex: WalletExecutionContext,
 ): Promise<void> {
   logger.info("waiting until all transactions are in a final state");
-  wex.taskScheduler.ensureRunning();
-  let p: OpenedPromise<void> | undefined = undefined;
-  const cancelNotifs = wex.ws.addNotificationListener((notif) => {
-    if (!p) {
-      return;
-    }
-    if (notif.type === NotificationType.TransactionStateTransition) {
+  await wex.taskScheduler.ensureRunning();
+  await genericWaitForState(wex, {
+    filterNotification(notif) {
+      if (notif.type !== NotificationType.TransactionStateTransition) {
+        return false;
+      }
       switch (notif.newTxState.major) {
         case TransactionMajorState.Pending:
         case TransactionMajorState.Aborting:
-          break;
+        case TransactionMajorState.Finalizing:
+          return false;
         default:
-          p.resolve();
+          return true;
       }
-    }
+    },
+    async checkState() {
+      const txs = await getTransactions(wex, {
+        includeRefreshes: true,
+        filterByState: "nonfinal",
+      });
+      for (const tx of txs.transactions) {
+        switch (tx.txState.major) {
+          case TransactionMajorState.Pending:
+          case TransactionMajorState.Aborting:
+          case TransactionMajorState.Finalizing:
+          case TransactionMajorState.Suspended:
+          case TransactionMajorState.SuspendedAborting:
+            logger.info(
+              `continuing waiting, ${tx.transactionId} in ${tx.txState.major}(${tx.txState.minor})`,
+            );
+            return false;
+        }
+      }
+      return true;
+    },
   });
-  while (1) {
-    p = openPromise();
-    const txs = await getTransactions(wex, {
-      includeRefreshes: true,
-      filterByState: "nonfinal",
-    });
-    let finished = true;
-    for (const tx of txs.transactions) {
-      switch (tx.txState.major) {
-        case TransactionMajorState.Pending:
-        case TransactionMajorState.Aborting:
-        case TransactionMajorState.Suspended:
-        case TransactionMajorState.SuspendedAborting:
-          finished = false;
-          logger.info(
-            `continuing waiting, ${tx.transactionId} in ${tx.txState.major}(${tx.txState.minor})`,
-          );
-          break;
-      }
-    }
-    if (finished) {
-      break;
-    }
-    // Wait until transaction state changed
-    await p.promise;
-  }
-  cancelNotifs();
   logger.info("done waiting until all transactions are in a final state");
 }
 
+export async function waitTasksDone(
+  wex: WalletExecutionContext,
+): Promise<void> {
+  await genericWaitForState(wex, {
+    async checkState() {
+      return wex.taskScheduler.isIdle();
+    },
+    filterNotification(notif) {
+      return notif.type === NotificationType.Idle;
+    },
+  });
+}
+
 /**
  * Wait until all chosen transactions are in a final state.
  */
@@ -462,59 +473,52 @@ export async function waitUntilGivenTransactionsFinal(
   if (transactionIds.length === 0) {
     return;
   }
-  wex.taskScheduler.ensureRunning();
+
   const txIdSet = new Set(transactionIds);
-  let p: OpenedPromise<void> | undefined = undefined;
-  const cancelNotifs = wex.ws.addNotificationListener((notif) => {
-    if (!p) {
-      return;
-    }
-    if (notif.type === NotificationType.TransactionStateTransition) {
+
+  await genericWaitForState(wex, {
+    filterNotification(notif) {
+      if (notif.type !== NotificationType.TransactionStateTransition) {
+        return false;
+      }
       if (!txIdSet.has(notif.transactionId)) {
-        return;
+        return false;
       }
       switch (notif.newTxState.major) {
         case TransactionMajorState.Pending:
         case TransactionMajorState.Aborting:
         case TransactionMajorState.Suspended:
         case TransactionMajorState.SuspendedAborting:
-          break;
-        default:
-          p.resolve();
+          return false;
       }
-    }
+      return true;
+    },
+    async checkState() {
+      const txs = await getTransactions(wex, {
+        includeRefreshes: true,
+        filterByState: "nonfinal",
+      });
+      for (const tx of txs.transactions) {
+        if (!txIdSet.has(tx.transactionId)) {
+          // Don't look at this transaction, we're not interested in it.
+          continue;
+        }
+        switch (tx.txState.major) {
+          case TransactionMajorState.Pending:
+          case TransactionMajorState.Finalizing:
+          case TransactionMajorState.Aborting:
+          case TransactionMajorState.Suspended:
+          case TransactionMajorState.SuspendedAborting:
+            logger.info(
+              `continuing waiting, ${tx.transactionId} in ${tx.txState.major}(${tx.txState.minor})`,
+            );
+            return false;
+        }
+      }
+      // No transaction is pending, we're done waiting!
+      return true;
+    },
   });
-  while (1) {
-    p = openPromise();
-    const txs = await getTransactions(wex, {
-      includeRefreshes: true,
-      filterByState: "nonfinal",
-    });
-    let finished = true;
-    for (const tx of txs.transactions) {
-      if (!txIdSet.has(tx.transactionId)) {
-        // Don't look at this transaction, we're not interested in it.
-        continue;
-      }
-      switch (tx.txState.major) {
-        case TransactionMajorState.Pending:
-        case TransactionMajorState.Aborting:
-        case TransactionMajorState.Suspended:
-        case TransactionMajorState.SuspendedAborting:
-          finished = false;
-          logger.info(
-            `continuing waiting, ${tx.transactionId} in ${tx.txState.major}(${tx.txState.minor})`,
-          );
-          break;
-      }
-    }
-    if (finished) {
-      break;
-    }
-    // Wait until transaction state changed
-    await p.promise;
-  }
-  cancelNotifs();
   logger.info("done waiting until given transactions are in a final state");
 }
 
@@ -522,52 +526,44 @@ export async function waitUntilRefreshesDone(
   wex: WalletExecutionContext,
 ): Promise<void> {
   logger.info("waiting until all refresh transactions are in a final state");
-  wex.taskScheduler.ensureRunning();
-  let p: OpenedPromise<void> | undefined = undefined;
-  const cancelNotifs = wex.ws.addNotificationListener((notif) => {
-    if (!p) {
-      return;
-    }
-    if (notif.type === NotificationType.TransactionStateTransition) {
+
+  await genericWaitForState(wex, {
+    filterNotification(notif) {
+      if (notif.type !== NotificationType.TransactionStateTransition) {
+        return false;
+      }
       switch (notif.newTxState.major) {
         case TransactionMajorState.Pending:
         case TransactionMajorState.Aborting:
-          break;
+          return false;
         default:
-          p.resolve();
+          return true;
       }
-    }
+    },
+    async checkState() {
+      const txs = await getTransactions(wex, {
+        includeRefreshes: true,
+        filterByState: "nonfinal",
+      });
+      for (const tx of txs.transactions) {
+        if (tx.type !== TransactionType.Refresh) {
+          continue;
+        }
+        switch (tx.txState.major) {
+          case TransactionMajorState.Pending:
+          case TransactionMajorState.Finalizing:
+          case TransactionMajorState.Aborting:
+          case TransactionMajorState.Suspended:
+          case TransactionMajorState.SuspendedAborting:
+            logger.info(
+              `continuing waiting, ${tx.transactionId} in ${tx.txState.major}(${tx.txState.minor})`,
+            );
+            return false;
+        }
+      }
+      return true;
+    },
   });
-  while (1) {
-    p = openPromise();
-    const txs = await getTransactions(wex, {
-      includeRefreshes: true,
-      filterByState: "nonfinal",
-    });
-    let finished = true;
-    for (const tx of txs.transactions) {
-      if (tx.type !== TransactionType.Refresh) {
-        continue;
-      }
-      switch (tx.txState.major) {
-        case TransactionMajorState.Pending:
-        case TransactionMajorState.Aborting:
-        case TransactionMajorState.Suspended:
-        case TransactionMajorState.SuspendedAborting:
-          finished = false;
-          logger.info(
-            `continuing waiting, ${tx.transactionId} in ${tx.txState.major}(${tx.txState.minor})`,
-          );
-          break;
-      }
-    }
-    if (finished) {
-      break;
-    }
-    // Wait until transaction state changed
-    await p.promise;
-  }
-  cancelNotifs();
   logger.info("done waiting until all refreshes are in a final state");
 }
 
@@ -575,33 +571,10 @@ async function waitUntilTransactionPendingReady(
   wex: WalletExecutionContext,
   transactionId: string,
 ): Promise<void> {
-  logger.info(`starting waiting for ${transactionId} to be in pending(ready)`);
-  wex.taskScheduler.ensureRunning();
-  let p: OpenedPromise<void> | undefined = undefined;
-  const cancelNotifs = wex.ws.addNotificationListener((notif) => {
-    if (!p) {
-      return;
-    }
-    if (notif.type === NotificationType.TransactionStateTransition) {
-      p.resolve();
-    }
+  return await waitTransactionState(wex, transactionId, {
+    major: TransactionMajorState.Pending,
+    minor: TransactionMinorState.Ready,
   });
-  while (1) {
-    p = openPromise();
-    const tx = await getTransactionById(wex, {
-      transactionId,
-    });
-    if (
-      tx.txState.major == TransactionMajorState.Pending &&
-      tx.txState.minor === TransactionMinorState.Ready
-    ) {
-      break;
-    }
-    // Wait until transaction state changed
-    await p.promise;
-  }
-  logger.info(`done waiting for ${transactionId} to be in pending(ready)`);
-  cancelNotifs();
 }
 
 /**
@@ -617,34 +590,22 @@ export async function waitTransactionState(
       txState,
     )})`,
   );
-  wex.taskScheduler.ensureRunning();
-  let p: OpenedPromise<void> | undefined = undefined;
-  const cancelNotifs = wex.ws.addNotificationListener((notif) => {
-    if (!p) {
-      return;
-    }
-    if (notif.type === NotificationType.TransactionStateTransition) {
-      p.resolve();
-    }
+  await genericWaitForState(wex, {
+    async checkState() {
+      const tx = await getTransactionById(wex, {
+        transactionId,
+      });
+      return (
+        tx.txState.major === txState.major && tx.txState.minor === txState.minor
+      );
+    },
+    filterNotification(notif) {
+      return notif.type === NotificationType.TransactionStateTransition;
+    },
   });
-  while (1) {
-    p = openPromise();
-    const tx = await getTransactionById(wex, {
-      transactionId,
-    });
-    if (
-      tx.txState.major === txState.major &&
-      tx.txState.minor === txState.minor
-    ) {
-      break;
-    }
-    // Wait until transaction state changed
-    await p.promise;
-  }
   logger.info(
     `done waiting for ${transactionId} to be in ${JSON.stringify(txState)}`,
   );
-  cancelNotifs();
 }
 
 export async function waitUntilTransactionWithAssociatedRefreshesFinal(
@@ -669,7 +630,7 @@ export async function runIntegrationTest2(
   wex: WalletExecutionContext,
   args: IntegrationTestV2Args,
 ): Promise<void> {
-  wex.taskScheduler.ensureRunning();
+  await wex.taskScheduler.ensureRunning();
   logger.info("running test with arguments", args);
 
   const exchangeInfo = await fetchFreshExchange(wex, args.exchangeBaseUrl);
@@ -907,9 +868,12 @@ export async function testPay(
   if (r.type != ConfirmPayResultType.Done) {
     throw Error("payment not done");
   }
-  const purchase = await wex.db.runReadOnlyTx(["purchases"], async (tx) => {
-    return tx.purchases.get(result.proposalId);
-  });
+  const purchase = await wex.db.runReadOnlyTx(
+    { storeNames: ["purchases"] },
+    async (tx) => {
+      return tx.purchases.get(result.proposalId);
+    },
+  );
   checkLogicInvariant(!!purchase);
   return {
     numCoins: purchase.payInfo?.payCoinSelection?.coinContributions.length ?? 0,
