@@ -15,51 +15,59 @@
  */
 
 import { ExchangeTosStatus } from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, h, VNode } from "preact";
 import { CheckboxOutlined } from "../../components/CheckboxOutlined.js";
 import { ExchangeXmlTos } from "../../components/ExchangeToS.js";
 import {
+  Input,
   LinkSuccess,
-  TermsOfService,
+  TermsOfServiceStyle,
   WarningBox,
-  WarningText,
 } from "../../components/styled/index.js";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Button } from "../../mui/Button.js";
 import { State } from "./index.js";
+import { SelectList } from "../SelectList.js";
+import { EnabledBySettings } from "../EnabledBySettings.js";
 
 export function ShowButtonsAcceptedTosView({
   termsAccepted,
   showingTermsOfService,
+  children,
 }: State.ShowButtonsAccepted): VNode {
   const { i18n } = useTranslationContext();
-  const ableToReviewTermsOfService =
-    showingTermsOfService.button.onClick !== undefined;
 
   return (
     <Fragment>
-      {ableToReviewTermsOfService && (
-        <section style={{ justifyContent: "space-around", display: "flex" }}>
-          <LinkSuccess
-            upperCased
-            onClick={showingTermsOfService.button.onClick}
-          >
-            <i18n.Translate>Show terms of service</i18n.Translate>
-          </LinkSuccess>
-        </section>
+      {showingTermsOfService.button.onClick !== undefined && (
+        <Fragment>
+          <section style={{ justifyContent: "space-around", display: "flex" }}>
+            <LinkSuccess
+              upperCased
+              onClick={showingTermsOfService.button.onClick}
+            >
+              <i18n.Translate>Show terms of service</i18n.Translate>
+            </LinkSuccess>
+          </section>
+          {termsAccepted.button.onClick !== undefined && (
+            <section
+              style={{ justifyContent: "space-around", display: "flex" }}
+            >
+              <CheckboxOutlined
+                name="terms"
+                enabled={termsAccepted.value}
+                label={
+                  <i18n.Translate>
+                    I accept the exchange terms of service
+                  </i18n.Translate>
+                }
+                onToggle={termsAccepted.button.onClick}
+              />
+            </section>
+          )}
+        </Fragment>
       )}
-      <section style={{ justifyContent: "space-around", display: "flex" }}>
-        <CheckboxOutlined
-          name="terms"
-          enabled={termsAccepted.value}
-          label={
-            <i18n.Translate>
-              I accept the exchange terms of service
-            </i18n.Translate>
-          }
-          onToggle={termsAccepted.button.onClick}
-        />
-      </section>
+      {children}
     </Fragment>
   );
 }
@@ -69,45 +77,18 @@ export function ShowButtonsNonAcceptedTosView({
   terms,
 }: State.ShowButtonsNotAccepted): VNode {
   const { i18n } = useTranslationContext();
-  // const ableToReviewTermsOfService =
-  //   showingTermsOfService.button.onClick !== undefined;
-
-  // if (!ableToReviewTermsOfService) {
-  //   return (
-  //     <Fragment>
-  //       {terms.status === ExchangeTosStatus.Pending && (
-  //         <section style={{ justifyContent: "space-around", display: "flex" }}>
-  //           <WarningText>
-  //             <i18n.Translate>
-  //               Exchange doesn&apos;t have terms of service
-  //             </i18n.Translate>
-  //           </WarningText>
-  //         </section>
-  //       )}
-  //     </Fragment>
-  //   );
-  // }
 
   return (
     <Fragment>
-      {/* {terms.status === ExchangeTosStatus.NotFound && (
-        <section style={{ justifyContent: "space-around", display: "flex" }}>
-          <WarningText>
-            <i18n.Translate>
-              Exchange doesn&apos;t have terms of service
-            </i18n.Translate>
-          </WarningText>
-        </section>
-      )} */}
-        <section style={{ justifyContent: "space-around", display: "flex" }}>
-          <Button
-            variant="contained"
-            color="success"
-            onClick={showingTermsOfService.button.onClick}
-          >
-            <i18n.Translate>Review exchange terms of service</i18n.Translate>
-          </Button>
-        </section>
+      <section style={{ justifyContent: "space-around", display: "flex" }}>
+        <Button
+          variant="contained"
+          color="success"
+          onClick={showingTermsOfService.button.onClick}
+        >
+          <i18n.Translate>Review exchange terms of service</i18n.Translate>
+        </Button>
+      </section>
     </Fragment>
   );
 }
@@ -116,13 +97,58 @@ export function ShowTosContentView({
   termsAccepted,
   showingTermsOfService,
   terms,
+  tosLang,
+  tosFormat,
+  skipTos,
 }: State.ShowContent): VNode {
   const { i18n } = useTranslationContext();
-  const ableToReviewTermsOfService =
-    showingTermsOfService?.button.onClick !== undefined;
+  const ableToReviewTermsOfService = termsAccepted.button.onClick !== undefined;
 
+  if (terms.status === ExchangeTosStatus.MissingTos) {
+    return (
+      <section>
+        <section style={{ justifyContent: "space-around", display: "flex" }}>
+          <WarningBox>
+            <i18n.Translate>
+              The exchange doesn't have a terms of service.
+            </i18n.Translate>
+          </WarningBox>
+        </section>
+
+        <section style={{ justifyContent: "space-around", display: "flex" }}>
+          <Button
+            variant="contained"
+            color="success"
+            disabled={!skipTos.onClick}
+            onClick={skipTos.onClick}
+          >
+            <i18n.Translate>Skip it for now.</i18n.Translate>
+          </Button>
+        </section>
+      </section>
+    );
+  }
   return (
-    <Fragment>
+    <section>
+      <Input style={{ display: "flex", justifyContent: "end" }}>
+        <EnabledBySettings name="selectTosFormat">
+          <SelectList
+            label={i18n.str`Format`}
+            list={tosFormat.list}
+            name="format"
+            value={tosFormat.value}
+            onChange={tosFormat.onChange}
+          />
+        </EnabledBySettings>
+        <SelectList
+          label={i18n.str`Language`}
+          list={tosLang.list}
+          name="lang"
+          value={tosLang.value}
+          onChange={tosLang.onChange}
+        />
+      </Input>
+
       {!terms.content && (
         <section style={{ justifyContent: "space-around", display: "flex" }}>
           <WarningBox>
@@ -143,9 +169,9 @@ export function ShowTosContentView({
                 </i18n.Translate>
               </WarningBox>
             ) : (
-              <TermsOfService>
+              <TermsOfServiceStyle>
                 <ExchangeXmlTos doc={terms.content.document} />
-              </TermsOfService>
+              </TermsOfServiceStyle>
             ))}
           {terms.content.type === "plain" &&
             (!terms.content.content ? (
@@ -160,7 +186,7 @@ export function ShowTosContentView({
               </div>
             ))}
           {terms.content.type === "html" && (
-            <iframe src={terms.content.href.toString()} />
+            <iframe style={{ width: "100%" }} srcDoc={terms.content.html} />
           )}
           {terms.content.type === "pdf" && (
             <a href={terms.content.location.toString()} download="tos.pdf">
@@ -179,20 +205,21 @@ export function ShowTosContentView({
           </LinkSuccess>
         </section>
       )}
-      {termsAccepted && terms.status !== ExchangeTosStatus.Accepted && (
-        <section style={{ justifyContent: "space-around", display: "flex" }}>
-          <CheckboxOutlined
-            name="terms"
-            enabled={termsAccepted.value}
-            label={
-              <i18n.Translate>
-                I accept the exchange terms of service
-              </i18n.Translate>
-            }
-            onToggle={termsAccepted.button.onClick}
-          />
-        </section>
-      )}
-    </Fragment>
+      {termsAccepted.button.onClick &&
+        terms.status !== ExchangeTosStatus.Accepted && (
+          <section style={{ justifyContent: "space-around", display: "flex" }}>
+            <CheckboxOutlined
+              name="terms"
+              enabled={termsAccepted.value}
+              label={
+                <i18n.Translate>
+                  I accept the exchange terms of service
+                </i18n.Translate>
+              }
+              onToggle={termsAccepted.button.onClick}
+            />
+          </section>
+        )}
+    </section>
   );
 }
