@@ -690,6 +690,16 @@ export const SmallBoldText = styled.div`
   font-weight: bold;
 `;
 
+export const AgeSign = styled.div<{size:number}>`
+  display: inline-block;
+  border: red solid 1px;
+  border-radius: 100%;
+  width: ${({ size }: {size:number}) => (`${size}px`)};
+  height: ${({ size }: {size:number}) => (`${size}px`)};
+  line-height: ${({ size }: {size:number}) => (`${size}px`)};
+  padding: 3px;
+`;
+
 export const LargeText = styled.div`
   font-size: large;
 `;
