@@ -20,14 +20,14 @@
  */
 
 import { Amounts } from "@gnu-taler/taler-util";
-import { createExample } from "../../test-utils.js";
+import * as tests from "@gnu-taler/web-util/testing";
 import { ReadyView } from "./views.js";
 
 export default {
-  title: "cta/deposit",
+  title: "deposit",
 };
 
-export const Ready = createExample(ReadyView, {
+export const Ready = tests.createExample(ReadyView, {
   status: "ready",
   confirm: {},
   cost: Amounts.parseOrThrow("EUR:1.2"),
