@@ -19,19 +19,23 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { createExample } from "../../test-utils.js";
+import * as tests from "@gnu-taler/web-util/testing";
+import { nullFunction } from "../../mui/handlers.js";
 import { ReadyView } from "./views.js";
 
 export default {
-  title: "wallet/invoice create",
+  title: "invoice create",
 };
 
-export const Ready = createExample(ReadyView, {
-  chosenAmount: {
+export const Ready = tests.createExample(ReadyView, {
+  requestAmount: {
     currency: "ARS",
     value: 1,
     fraction: 0,
   },
+  expiration: {
+    value: "2/12/12",
+  },
   cancel: {},
   toBeReceived: {
     currency: "ARS",
@@ -42,9 +46,7 @@ export const Ready = createExample(ReadyView, {
   exchangeUrl: "https://exchange.taler.ar",
   subject: {
     value: "some subject",
-    onInput: async () => {
-      null;
-    },
+    onInput: nullFunction,
   },
   create: {},
 });
