@@ -19,28 +19,38 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { PreparePayResult, PreparePayResultType } from "@gnu-taler/taler-util";
-import { createExample } from "../../test-utils.js";
+import {
+  AbsoluteTime,
+  PreparePayResult,
+  PreparePayResultType,
+} from "@gnu-taler/taler-util";
+import * as tests from "@gnu-taler/web-util/testing";
 import { ReadyView } from "./views.js";
 
 export default {
-  title: "wallet/invoice payment",
+  title: "invoice payment",
 };
 
-export const Ready = createExample(ReadyView, {
-  amount: {
+export const Ready = tests.createExample(ReadyView, {
+  effective: {
+    currency: "ARS",
+    value: 1,
+    fraction: 0,
+  },
+  raw: {
     currency: "ARS",
     value: 1,
     fraction: 0,
   },
   summary: "some subject",
+  uri: "taler://pay/merchant.ar/123",
   payStatus: {
     status: PreparePayResultType.PaymentPossible,
     amountEffective: "ARS:1",
   } as PreparePayResult,
-  expiration: {
-    t_ms: new Date().getTime() + 1000 * 60 * 60,
-  },
+  expiration: AbsoluteTime.fromMilliseconds(
+    new Date().getTime() + 1000 * 60 * 60,
+  ),
   accept: {},
   cancel: {},
 });
