@@ -84,7 +84,8 @@ export function useComponentState({
     return {
       status: "error",
       error: alertFromError(
-        i18n.str`Could not load the status of the term of service`,
+        i18n,
+        i18n.str`Could not load the payment and balance status`,
         hook,
       ),
     };
