@@ -29,6 +29,7 @@ import {
   PreparePayResultPaymentPossible,
   PreparePayResultType,
   ScopeType,
+  TransactionMajorState,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { expect } from "chai";
@@ -549,8 +550,13 @@ describe("Payment CTA states", () => {
           // expect(r.totalFees).deep.equal(Amounts.parseOrThrow("USD:1"));
           expect(state.payHandler.onClick).not.undefined;
 
-          handler.notifyEventFromWallet(
-            NotificationType.TransactionStateTransition,
+          handler.notifyEventFromWallet({
+            type: NotificationType.TransactionStateTransition,
+            newTxState: {} as any,
+            oldTxState:  {} as any,
+            transactionId: "123",
+          }
+            
           );
         },
         (state) => {
