@@ -18,6 +18,7 @@ import {
   AbsoluteTime,
   Amounts,
   MerchantContractTerms as ContractTerms,
+  Duration,
   PreparePayResultType,
   TranslatedString,
 } from "@gnu-taler/taler-util";
@@ -27,7 +28,11 @@ import { Part } from "../../components/Part.js";
 import { PaymentButtons } from "../../components/PaymentButtons.js";
 import { ShowFullContractTermPopup } from "../../components/ShowFullContractTermPopup.js";
 import { Time } from "../../components/Time.js";
-import { SuccessBox, WarningBox } from "../../components/styled/index.js";
+import {
+  AgeSign,
+  SuccessBox,
+  WarningBox,
+} from "../../components/styled/index.js";
 import { MerchantDetails } from "../../wallet/Transaction.js";
 import { State } from "./index.js";
 import { EnabledBySettings } from "../../components/EnabledBySettings.js";
@@ -50,13 +55,39 @@ export function BaseView(state: SupportedStates): VNode {
         : Amounts.zeroOfCurrency(state.amount.currency)
       : state.amount;
 
+  const expiration = !contractTerms.pay_deadline
+    ? undefined
+    : AbsoluteTime.fromProtocolTimestamp(contractTerms.pay_deadline);
+  const inFiveMinutes = AbsoluteTime.addDuration(
+    AbsoluteTime.now(),
+    Duration.fromSpec({ minutes: 5 }),
+  );
+  const willExpireSoon =
+    !expiration || expiration.t_ms === "never"
+      ? undefined
+      : AbsoluteTime.cmp(expiration, inFiveMinutes) === -1;
   return (
     <Fragment>
       <ShowImportantMessage state={state} />
 
       <section style={{ textAlign: "left" }}>
         <Part
-          title={i18n.str`Purchase`}
+          title={
+            contractTerms.minimum_age ? (
+              <Fragment>
+                <i18n.Translate>Purchase</i18n.Translate>
+                &nbsp;
+                <AgeSign
+                  size={20}
+                  title={i18n.str`This purchase is age restricted.`}
+                >
+                  {contractTerms.minimum_age}+
+                </AgeSign>
+              </Fragment>
+            ) : (
+              <i18n.Translate>Purchase</i18n.Translate>
+            )
+          }
           text={contractTerms.summary as TranslatedString}
           kind="neutral"
         />
@@ -65,17 +96,10 @@ export function BaseView(state: SupportedStates): VNode {
           text={<MerchantDetails merchant={contractTerms.merchant} />}
           kind="neutral"
         />
-        {contractTerms.pay_deadline && (
+        {willExpireSoon && (
           <Part
-            title={i18n.str`Valid until`}
-            text={
-              <Time
-                timestamp={AbsoluteTime.fromProtocolTimestamp(
-                  contractTerms.pay_deadline,
-                )}
-                format="dd MMMM yyyy, HH:mm"
-              />
-            }
+            title={i18n.str`Expires at`}
+            text={<Time timestamp={expiration} format="HH:mm" />}
             kind="neutral"
           />
         )}
