@@ -20,9 +20,9 @@
  */
 
 import { Amounts } from "@gnu-taler/taler-util";
-import { createExample } from "../../test-utils.js";
+import * as tests from "@gnu-taler/web-util/testing";
 import { ReadyView } from "./views.js";
 
 export default {
-  title: "cta/recovery",
+  title: "recovery",
 };
