@@ -14,7 +14,11 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { AmountJson, AmountString, TalerErrorDetail } from "@gnu-taler/taler-util";
+import {
+  AmountJson,
+  AmountString,
+  TalerErrorDetail,
+} from "@gnu-taler/taler-util";
 import { ErrorAlertView } from "../../components/CurrentAlerts.js";
 import { Loading } from "../../components/Loading.js";
 import { ErrorAlert } from "../../context/alert.js";
@@ -39,6 +43,7 @@ export namespace State {
 
   export interface LoadingUriError {
     status: "error";
+    retry: ButtonHandler;
     error: ErrorAlert;
   }
 
