@@ -17,19 +17,18 @@
 import {
   AmountString,
   Amounts,
-  TalerError,
   TalerErrorCode,
   TalerProtocolTimestamp,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { isFuture, parse } from "date-fns";
-import { useEffect, useState } from "preact/hooks";
+import { useState } from "preact/hooks";
 import { alertFromError, useAlertContext } from "../../context/alert.js";
 import { useBackendContext } from "../../context/backend.js";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { useAsyncAsHook } from "../../hooks/useAsyncAsHook.js";
-import { Props, State } from "./index.js";
 import { BackgroundError, WxApiType } from "../../wxApi.js";
+import { Props, State } from "./index.js";
 
 export function useComponentState({
   amount: amountStr,
@@ -58,8 +57,14 @@ export function useComponentState({
   if (hook.hasError) {
     return {
       status: "error",
+      retry: {
+        onClick: pushAlertOnError(async () => {
+          hook.retry();
+        }),
+      },
       error: alertFromError(
-        i18n.str`Could not load the status of the term of service`,
+        i18n,
+        i18n.str`Could not load the max amount to transfer`,
         hook,
       ),
     };
@@ -163,11 +168,14 @@ async function checkPeerPushDebitAndCheckMax(
     const material = Amounts.parseOrThrow(
       e.errorDetail.insufficientBalanceDetails.balanceMaterial,
     );
-    const gap = Amounts.parseOrThrow(
-      e.errorDetail.insufficientBalanceDetails.feeGapEstimate,
-    );
-    const newAmount = Amounts.sub(material, gap).amount;
     const amount = Amounts.parseOrThrow(amountState);
+    const gap = Amounts.sub(
+      amount,
+      Amounts.parseOrThrow(
+        e.errorDetail.insufficientBalanceDetails.maxEffectiveSpendAmount,
+      ),
+    ).amount;
+    const newAmount = Amounts.sub(material, gap).amount;
     if (Amounts.cmp(newAmount, amount) === 0) {
       //insufficient balance and the exception didn't give
       //a good response that allow us to try again
