@@ -19,19 +19,23 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { createExample } from "../../test-utils.js";
+import * as tests from "@gnu-taler/web-util/testing";
+import { nullFunction } from "../../mui/handlers.js";
 import { ReadyView } from "./views.js";
 
 export default {
-  title: "wallet/transfer create",
+  title: "transfer create",
 };
 
-export const Ready = createExample(ReadyView, {
-  chosenAmount: {
+export const Ready = tests.createExample(ReadyView, {
+  debitAmount: {
     currency: "ARS",
     value: 1,
     fraction: 0,
   },
+  expiration: {
+    value: "20/1/2022",
+  },
   create: {},
   cancel: {},
   toBeReceived: {
@@ -41,8 +45,6 @@ export const Ready = createExample(ReadyView, {
   },
   subject: {
     value: "the subject",
-    onInput: async () => {
-      null;
-    },
+    onInput: nullFunction,
   },
 });
