@@ -49,6 +49,11 @@ export function useComponentState({
   if (hook.hasError) {
     return {
       status: "error",
+      retry: {
+        onClick: pushAlertOnError(async () => {
+          hook.retry();
+        }),
+      },
       error: alertFromError(
         i18n,
         i18n.str`Could not load the invoice payment status`,
