@@ -18,7 +18,7 @@ import {
   AmountJson,
   AmountString,
   CurrencySpecification,
-  ExchangeListItem
+  ExchangeListItem,
 } from "@gnu-taler/taler-util";
 import { Loading } from "../../components/Loading.js";
 import { State as SelectExchangeState } from "../../hooks/useSelectedExchange.js";
@@ -85,7 +85,7 @@ export namespace State {
     operationState: "confirmed" | "aborted" | "selected";
     thisWallet: boolean;
     redirectToTx: () => void;
-    confirmTransferUrl?: string,
+    confirmTransferUrl?: string;
     error: undefined;
   }
 
@@ -96,20 +96,25 @@ export namespace State {
     currentExchange: ExchangeListItem;
 
     amount: AmountFieldHandler;
+    editableAmount: boolean;
 
-    withdrawalFee: AmountJson;
+    bankFee: AmountJson;
     toBeReceived: AmountJson;
+    toBeSent: AmountJson;
 
     doWithdrawal: ButtonHandler;
     doSelectExchange: ButtonHandler;
+    editableExchange: boolean;
 
     chooseCurrencies: string[];
     selectedCurrency: string;
     changeCurrency: (s: string) => void;
-    conversionInfo: {
-      spec: CurrencySpecification,
-      amount: AmountJson,
-    } | undefined;
+    conversionInfo:
+      | {
+          spec: CurrencySpecification;
+          amount: AmountJson;
+        }
+      | undefined;
 
     ageRestriction?: SelectFieldHandler;
 
