@@ -14,14 +14,14 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-/* eslint-disable react-hooks/rules-of-hooks */
 import {
   AmountJson,
+  AmountString,
   Amounts,
   ExchangeFullDetails,
   ExchangeListItem,
   NotificationType,
-  TalerError,
+  TransactionMajorState,
   parseWithdrawExchangeUri,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
@@ -44,6 +44,7 @@ export function useComponentStateFromParams({
   const api = useBackendContext();
   const { i18n } = useTranslationContext();
   const paramsAmount = amount ? Amounts.parse(amount) : undefined;
+  const [updatedExchangeByUser, setUpdatedExchangeByUser] = useState<string>();
   const uriInfoHook = useAsyncAsHook(async () => {
     const exchanges = await api.wallet.call(
       WalletApiOperation.ListExchanges,
@@ -52,12 +53,12 @@ export function useComponentStateFromParams({
     const uri = maybeTalerUri
       ? parseWithdrawExchangeUri(maybeTalerUri)
       : undefined;
-    const exchangeByTalerUri = uri?.exchangeBaseUrl;
+    const exchangeByTalerUri = updatedExchangeByUser ?? uri?.exchangeBaseUrl;
+
     let ex: ExchangeFullDetails | undefined;
     if (exchangeByTalerUri) {
       await api.wallet.call(WalletApiOperation.AddExchange, {
         exchangeBaseUrl: exchangeByTalerUri,
-        masterPub: uri.exchangePub,
       });
       const info = await api.wallet.call(
         WalletApiOperation.GetExchangeDetailedInfo,
@@ -141,8 +142,8 @@ export function useComponentStateFromParams({
         confirm: {
           onClick: isValid
             ? pushAlertOnError(async () => {
-              onAmountChanged(Amounts.stringify(amount));
-            })
+                onAmountChanged(Amounts.stringify(amount));
+              })
             : undefined,
         },
         amount: {
@@ -159,6 +160,7 @@ export function useComponentStateFromParams({
   async function doManualWithdraw(
     exchange: string,
     ageRestricted: number | undefined,
+    amount: AmountString,
   ): Promise<{
     transactionId: string;
     confirmTransferUrl: string | undefined;
@@ -167,7 +169,7 @@ export function useComponentStateFromParams({
       WalletApiOperation.AcceptManualWithdrawal,
       {
         exchangeBaseUrl: exchange,
-        amount: Amounts.stringify(chosenAmount),
+        amount,
         restrictAge: ageRestricted,
       },
     );
@@ -183,9 +185,17 @@ export function useComponentStateFromParams({
       cancel,
       onSuccess,
       undefined,
-      chosenAmount,
-      exchangeList,
-      exchangeByTalerUri,
+      {
+        amount: chosenAmount,
+        currency: chosenAmount.currency,
+        maxAmount: Amounts.zeroOfCurrency(chosenAmount.currency),
+        bankFee: Amounts.zeroOfCurrency(chosenAmount.currency),
+        editableAmount: true,
+        editableExchange: true,
+        exchange: exchangeByTalerUri,
+        exchangeList: exchangeList,
+      },
+      setUpdatedExchangeByUser,
     );
 }
 
@@ -196,6 +206,8 @@ export function useComponentStateFromURI({
 }: PropsFromURI): RecursiveState<State> {
   const api = useBackendContext();
   const { i18n } = useTranslationContext();
+
+  const [updatedExchangeByUser, setUpdatedExchangeByUser] = useState<string>();
   /**
    * Ask the wallet about the withdraw URI
    */
@@ -206,46 +218,42 @@ export function useComponentStateFromURI({
       : maybeTalerUri;
 
     const uriInfo = await api.wallet.call(
-      WalletApiOperation.GetWithdrawalDetailsForUri,
-      {
-        talerWithdrawUri,
-        notifyChangeFromPendingTimeoutMs: 30 * 1000,
-      },
-    );
-    const {
-      amount,
-      defaultExchangeBaseUrl,
-      possibleExchanges,
-      operationId,
-      confirmTransferUrl,
-      status,
-    } = uriInfo;
-    const transaction = await api.wallet.call(
-      WalletApiOperation.GetWithdrawalTransactionByUri,
+      WalletApiOperation.PrepareBankIntegratedWithdrawal,
       { talerWithdrawUri },
     );
+    const { status } = uriInfo.info;
+    const txInfo = await api.wallet.call(
+      WalletApiOperation.GetTransactionById,
+      {
+        transactionId: uriInfo.transactionId,
+      },
+    );
     return {
       talerWithdrawUri,
-      operationId,
       status,
-      transaction,
-      confirmTransferUrl,
-      amount: Amounts.parseOrThrow(amount),
-      thisExchange: defaultExchangeBaseUrl,
-      exchanges: possibleExchanges,
+      transactionId: uriInfo.transactionId,
+      bankWithdrawalInfo: uriInfo.info,
+      txInfo: txInfo,
     };
   });
 
   const readyToListen = uriInfoHook && !uriInfoHook.hasError;
 
   useEffect(() => {
-    if (!uriInfoHook) {
+    if (!uriInfoHook || uriInfoHook.hasError) {
       return;
     }
+    const txId = uriInfoHook.response.transactionId;
+
     return api.listener.onUpdateNotification(
-      [NotificationType.WithdrawalOperationTransition],
-      (asd) => {
-        uriInfoHook.retry();
+      [NotificationType.TransactionStateTransition],
+      (notif) => {
+        if (
+          notif.type === NotificationType.TransactionStateTransition &&
+          notif.transactionId === txId
+        ) {
+          uriInfoHook.retry();
+        }
       },
     );
   }, [readyToListen]);
@@ -264,39 +272,55 @@ export function useComponentStateFromURI({
   }
 
   const uri = uriInfoHook.response.talerWithdrawUri;
-  const chosenAmount = uriInfoHook.response.amount;
-  const defaultExchange = uriInfoHook.response.thisExchange;
-  const exchangeList = uriInfoHook.response.exchanges;
+  const txId = uriInfoHook.response.transactionId;
+  const bwi = uriInfoHook.response.bankWithdrawalInfo;
+
+  const amount =
+    bwi.amount === undefined
+      ? Amounts.zeroOfCurrency(bwi.currency)
+      : Amounts.parseOrThrow(bwi.amount);
+
+  const maxAmount =
+    bwi.maxAmount === undefined
+      ? Amounts.zeroOfCurrency(bwi.currency)
+      : Amounts.parseOrThrow(bwi.maxAmount);
+
+  const bankFee =
+    bwi.wireFee === undefined
+      ? Amounts.zeroOfCurrency(bwi.currency)
+      : Amounts.parseOrThrow(bwi.wireFee);
 
   async function doManagedWithdraw(
     exchange: string,
     ageRestricted: number | undefined,
+    amount: AmountString,
   ): Promise<{
     transactionId: string;
     confirmTransferUrl: string | undefined;
   }> {
-    const res = await api.wallet.call(
-      WalletApiOperation.AcceptBankIntegratedWithdrawal,
-      {
-        exchangeBaseUrl: exchange,
-        talerWithdrawUri: uri,
-        restrictAge: ageRestricted,
-      },
-    );
+    const res = await api.wallet.call(WalletApiOperation.ConfirmWithdrawal, {
+      exchangeBaseUrl: exchange,
+      amount,
+      restrictAge: ageRestricted,
+      transactionId: txId,
+    });
     return {
       confirmTransferUrl: res.confirmTransferUrl,
       transactionId: res.transactionId,
     };
   }
 
-  if (uriInfoHook.response.status !== "pending") {
-    if (uriInfoHook.response.transaction) {
-      onSuccess(uriInfoHook.response.transaction.transactionId);
-    }
+  if (
+    uriInfoHook.response.txInfo &&
+    uriInfoHook.response.status !== "pending"
+  ) {
+    const info = uriInfoHook.response.txInfo;
     return {
       status: "already-completed",
       operationState: uriInfoHook.response.status,
-      confirmTransferUrl: uriInfoHook.response.confirmTransferUrl,
+      confirmTransferUrl: bwi.confirmTransferUrl,
+      thisWallet: info.txState.major === TransactionMajorState.Pending,
+      redirectToTx: () => onSuccess(info.transactionId),
       error: undefined,
     };
   }
@@ -307,16 +331,36 @@ export function useComponentStateFromURI({
       cancel,
       onSuccess,
       uri,
-      chosenAmount,
-      exchangeList,
-      defaultExchange,
+      {
+        amount,
+        bankFee,
+        maxAmount,
+        currency: bwi.currency,
+        editableAmount: bwi.editableAmount,
+        editableExchange: bwi.editableExchange,
+        exchange: bwi.defaultExchangeBaseUrl,
+        exchangeList: bwi.possibleExchanges,
+      },
+      setUpdatedExchangeByUser,
     );
   }, []);
 }
 
+type WithdrawalInfo = {
+  currency: string;
+  amount: AmountJson;
+  bankFee: AmountJson;
+  maxAmount: AmountJson;
+  editableAmount: boolean;
+  exchange: string | undefined;
+  editableExchange: boolean;
+  exchangeList: ExchangeListItem[];
+};
+
 type ManualOrManagedWithdrawFunction = (
   exchange: string,
   ageRestricted: number | undefined,
+  amount: AmountString,
 ) => Promise<{ transactionId: string; confirmTransferUrl: string | undefined }>;
 
 function exchangeSelectionState(
@@ -324,17 +368,30 @@ function exchangeSelectionState(
   cancel: () => Promise<void>,
   onSuccess: (txid: string) => Promise<void>,
   talerWithdrawUri: string | undefined,
-  chosenAmount: AmountJson,
-  exchangeList: ExchangeListItem[],
-  exchangeSuggestedByTheBank: string | undefined,
+  wInfo: WithdrawalInfo,
+  onExchangeUpdated: (ex: string) => void,
 ): RecursiveState<State> {
   const api = useBackendContext();
   const selectedExchange = useSelectedExchange({
-    currency: chosenAmount.currency,
-    defaultExchange: exchangeSuggestedByTheBank,
-    list: exchangeList,
+    currency: wInfo.currency,
+    defaultExchange: wInfo.exchange,
+    list: wInfo.exchangeList,
   });
 
+  const current =
+    selectedExchange.status !== "ready"
+      ? undefined
+      : selectedExchange.selected.exchangeBaseUrl;
+  useEffect(() => {
+    if (current) {
+      onExchangeUpdated(current);
+    }
+  }, [current]);
+
+  const safeAmount = wInfo.amount
+    ? wInfo.amount
+    : Amounts.zeroOfCurrency(wInfo.currency);
+
   if (selectedExchange.status !== "ready") {
     return selectedExchange;
   }
@@ -345,11 +402,12 @@ function exchangeSelectionState(
     | State.Loading => {
     const { i18n } = useTranslationContext();
     const { pushAlertOnError } = useAlertContext();
+
+    const [choosenAmount, setChoosenAmount] = useState(safeAmount);
     const [ageRestricted, setAgeRestricted] = useState(0);
-    const currentExchange = selectedExchange.selected;
 
     const [selectedCurrency, setSelectedCurrency] = useState<string>(
-      chosenAmount.currency,
+      wInfo.currency,
     );
     /**
      * With the exchange and amount, ask the wallet the information
@@ -359,8 +417,8 @@ function exchangeSelectionState(
       const info = await api.wallet.call(
         WalletApiOperation.GetWithdrawalDetailsForAmount,
         {
-          exchangeBaseUrl: currentExchange.exchangeBaseUrl,
-          amount: Amounts.stringify(chosenAmount),
+          exchangeBaseUrl: selectedExchange.selected.exchangeBaseUrl,
+          amount: Amounts.stringify(choosenAmount),
           restrictAge: ageRestricted,
         },
       );
@@ -372,36 +430,14 @@ function exchangeSelectionState(
 
       return {
         amount: withdrawAmount,
+        currentExchange: selectedExchange.selected,
         ageRestrictionOptions: info.ageRestrictionOptions,
         accounts: info.withdrawalAccountsList,
       };
-    }, []);
+    }, [choosenAmount, selectedExchange.selected, ageRestricted]);
 
-    const [withdrawError, setWithdrawError] = useState<TalerError | undefined>(
-      undefined,
-    );
     const [doingWithdraw, setDoingWithdraw] = useState<boolean>(false);
 
-    async function doWithdrawAndCheckError(): Promise<void> {
-      try {
-        setDoingWithdraw(true);
-        const res = await doWithdraw(
-          currentExchange.exchangeBaseUrl,
-          !ageRestricted ? undefined : ageRestricted,
-        );
-        if (res.confirmTransferUrl) {
-          document.location.href = res.confirmTransferUrl;
-        } else {
-          onSuccess(res.transactionId);
-        }
-      } catch (e) {
-        if (e instanceof TalerError) {
-          setWithdrawError(e);
-        }
-      }
-      setDoingWithdraw(false);
-    }
-
     if (!amountHook) {
       return { status: "loading", error: undefined };
     }
@@ -419,15 +455,37 @@ function exchangeSelectionState(
       return { status: "loading", error: undefined };
     }
 
-    const withdrawalFee = Amounts.sub(
-      amountHook.response.amount.raw,
-      amountHook.response.amount.effective,
-    ).amount;
+    const currentExchange = amountHook.response.currentExchange;
+
+    async function doWithdrawAndCheckError(): Promise<void> {
+      try {
+        setDoingWithdraw(true);
+        const res = await doWithdraw(
+          currentExchange.exchangeBaseUrl,
+          !ageRestricted ? undefined : ageRestricted,
+          Amounts.stringify(choosenAmount),
+        );
+        if (res.confirmTransferUrl) {
+          document.location.href = res.confirmTransferUrl;
+        } else {
+          onSuccess(res.transactionId);
+        }
+      } catch (e) {
+        console.error(e);
+        // if (e instanceof TalerError) {
+        // }
+      }
+      setDoingWithdraw(false);
+    }
+
+    const toBeSent = amountHook.response.amount.raw;
     const toBeReceived = amountHook.response.amount.effective;
 
+    const bankFee = wInfo.bankFee;
+
     const ageRestrictionOptions =
       amountHook.response.ageRestrictionOptions?.reduce(
-        (p, c) => ({ ...p, [c]: `under ${c}` }),
+        (p, c) => ({ ...p, [c]: i18n.str`under ${c}` }),
         {} as Record<string, string>,
       );
 
@@ -439,12 +497,12 @@ function exchangeSelectionState(
     //TODO: calculate based on exchange info
     const ageRestriction = ageRestrictionEnabled
       ? {
-        list: ageRestrictionOptions,
-        value: String(ageRestricted),
-        onChange: pushAlertOnError(async (v: string) =>
-          setAgeRestricted(parseInt(v, 10)),
-        ),
-      }
+          list: ageRestrictionOptions,
+          value: String(ageRestricted),
+          onChange: pushAlertOnError(async (v: string) =>
+            setAgeRestricted(parseInt(v, 10)),
+          ),
+        }
       : undefined;
 
     const altCurrencies = amountHook.response.accounts
@@ -464,32 +522,54 @@ function exchangeSelectionState(
     const conversionInfo = !convAccount
       ? undefined
       : {
-        spec: convAccount.currencySpecification!,
-        amount: Amounts.parseOrThrow(convAccount.transferAmount!),
-      };
+          spec: convAccount.currencySpecification!,
+          amount: Amounts.parseOrThrow(convAccount.transferAmount!),
+        };
+
+    const amountError = Amounts.isZero(choosenAmount)
+      ? i18n.str`should be greater than zero`
+      : Amounts.cmp(choosenAmount, wInfo.maxAmount) === 1
+        ? i18n.str`you can withdra`
+        : undefined;
 
     return {
       status: "success",
       error: undefined,
-      doSelectExchange: selectedExchange.doSelect,
+      doSelectExchange: {
+        onClick: wInfo.editableExchange
+          ? selectedExchange.doSelect.onClick
+          : undefined,
+      },
+      editableAmount: wInfo.editableAmount,
+      editableExchange: wInfo.editableExchange,
       currentExchange,
       toBeReceived,
+      toBeSent,
       chooseCurrencies,
+      bankFee,
       selectedCurrency,
       changeCurrency: (s) => {
         setSelectedCurrency(s);
       },
       conversionInfo,
-      withdrawalFee,
-      chosenAmount,
+      amount: {
+        value: choosenAmount,
+        onInput: wInfo.editableAmount
+          ? pushAlertOnError(async (v) => {
+              setChoosenAmount(v);
+            })
+          : undefined,
+        error: amountError,
+      },
       talerWithdrawUri,
       ageRestriction,
       doWithdrawal: {
-        onClick: doingWithdraw
-          ? undefined
-          : pushAlertOnError(doWithdrawAndCheckError),
+        onClick:
+          doingWithdraw || amountError
+            ? undefined
+            : pushAlertOnError(doWithdrawAndCheckError),
       },
       cancel,
     };
-  }, []);
+  }, [selectedExchange.selected]);
 }
