@@ -26,9 +26,9 @@ import {
   purple,
   red,
   // eslint-disable-next-line import/extensions
-} from "./colors/constants";
+} from "./colors/constants.js";
 // eslint-disable-next-line import/extensions
-import { getContrastRatio } from "./colors/manipulation";
+import { getContrastRatio } from "./colors/manipulation.js";
 
 export type Colors =
   | "primary"
@@ -56,8 +56,6 @@ export interface Spacing {
   (top: number, right: number, bottom: number, left: number): string;
 }
 
-export const theme = createTheme();
-
 const zIndex = {
   mobileStepper: 1000,
   speedDial: 1050,
@@ -68,6 +66,8 @@ const zIndex = {
   tooltip: 1500,
 };
 
+export const theme = createTheme();
+
 export const ripple = css`
   background-position: center;
 
