@@ -53,7 +53,7 @@ const api: BackgroundPlatformAPI & ForegroundPlatformAPI = {
   redirectTabToWalletPage,
   registerAllIncomingConnections,
   registerOnInstalled,
-  listenToAllChannels ,
+  listenToAllChannels,
   registerReloadOnNewVersion,
   sendMessageToAllChannels,
   openNewURLFromPopup,
@@ -276,18 +276,20 @@ async function sendMessageToBackground<
   Op extends WalletOperations | BackgroundOperations,
 >(message: MessageFromFrontend<Op>): Promise<MessageResponse> {
   nextMessageIndex = (nextMessageIndex + 1) % (Number.MAX_SAFE_INTEGER - 100);
-  const messageWithId = { ...message, id: `id_${nextMessageIndex}` };
+  const messageWithId = { ...message, id: `fg:${nextMessageIndex}` };
 
   return new Promise<MessageResponse>((resolve, reject) => {
     logger.trace("send operation to the wallet background", message);
     let timedout = false;
     const timerId = setTimeout(() => {
       timedout = true;
-      reject(TalerError.fromDetail(TalerErrorCode.GENERIC_TIMEOUT, {
-        requestMethod: "wallet",
-        requestUrl: message.operation,
-        timeoutMs: 20 * 1000,
-      }));
+      reject(
+        TalerError.fromDetail(TalerErrorCode.GENERIC_TIMEOUT, {
+          requestMethod: "wallet",
+          requestUrl: message.operation,
+          timeoutMs: 20 * 1000,
+        }),
+      );
     }, 20 * 1000);
     chrome.runtime.sendMessage(messageWithId, (backgroundResponse) => {
       if (timedout) {
@@ -309,7 +311,9 @@ async function sendMessageToBackground<
  * To be used by the foreground
  */
 let notificationPort: chrome.runtime.Port | undefined;
-function listenToWalletBackground(listener: (message: MessageFromBackend) => void): () => void {
+function listenToWalletBackground(
+  listener: (message: MessageFromBackend) => void,
+): () => void {
   if (notificationPort === undefined) {
     notificationPort = chrome.runtime.connect({ name: "notifications" });
   }
@@ -380,13 +384,18 @@ function registerAllIncomingConnections(): void {
   });
 }
 
+function createTabId(tab: chrome.tabs.Tab | undefined) {
+  return !tab ? "popup" : `${tab.windowId}:${tab.id}`;
+}
+
 function listenToAllChannels(
   notifyNewMessage: <Op extends WalletOperations | BackgroundOperations>(
     message: MessageFromFrontend<Op> & { id: string },
+    id: string,
   ) => Promise<MessageResponse>,
 ): void {
   chrome.runtime.onMessage.addListener((message, sender, reply) => {
-    notifyNewMessage(message)
+    notifyNewMessage(message, createTabId(sender.tab))
       .then((apiResponse) => {
         try {
           reply(apiResponse);
@@ -483,26 +492,26 @@ function setAlertedIcon(): void {
 
 interface OffscreenCanvasRenderingContext2D
   extends CanvasState,
-  CanvasTransform,
-  CanvasCompositing,
-  CanvasImageSmoothing,
-  CanvasFillStrokeStyles,
-  CanvasShadowStyles,
-  CanvasFilters,
-  CanvasRect,
-  CanvasDrawPath,
-  CanvasUserInterface,
-  CanvasText,
-  CanvasDrawImage,
-  CanvasImageData,
-  CanvasPathDrawingStyles,
-  CanvasTextDrawingStyles,
-  CanvasPath {
+    CanvasTransform,
+    CanvasCompositing,
+    CanvasImageSmoothing,
+    CanvasFillStrokeStyles,
+    CanvasShadowStyles,
+    CanvasFilters,
+    CanvasRect,
+    CanvasDrawPath,
+    CanvasUserInterface,
+    CanvasText,
+    CanvasDrawImage,
+    CanvasImageData,
+    CanvasPathDrawingStyles,
+    CanvasTextDrawingStyles,
+    CanvasPath {
   readonly canvas: OffscreenCanvas;
 }
 declare const OffscreenCanvasRenderingContext2D: {
   prototype: OffscreenCanvasRenderingContext2D;
-  new(): OffscreenCanvasRenderingContext2D;
+  new (): OffscreenCanvasRenderingContext2D;
 };
 
 interface OffscreenCanvas extends EventTarget {
@@ -515,7 +524,7 @@ interface OffscreenCanvas extends EventTarget {
 }
 declare const OffscreenCanvas: {
   prototype: OffscreenCanvas;
-  new(width: number, height: number): OffscreenCanvas;
+  new (width: number, height: number): OffscreenCanvas;
 };
 
 function createCanvas(size: number): OffscreenCanvas {
@@ -760,7 +769,6 @@ function listenNetworkConnectionState(
   };
 }
 
-
 function runningOnPrivateMode(): boolean {
   return chrome.extension.inIncognitoContext;
 }
