@@ -26,9 +26,6 @@ import chromePlatform, {
   containsClipboardPermissions as chromeClipContains,
   removeClipboardPermissions as chromeClipRemove,
   requestClipboardPermissions as chromeClipRequest,
-  // containsHostPermissions as chromeHostContains,
-  // requestHostPermissions as chromeHostRequest,
-  // removeHostPermissions as chromeHostRemove,
 } from "./chrome.js";
 
 const api: BackgroundPlatformAPI & ForegroundPlatformAPI = {
@@ -47,16 +44,8 @@ function isFirefox(): boolean {
   return true;
 }
 
-function addPermissionsListener(callback: (p: Permissions) => void): void {
-  throw Error("addPermissionListener is not supported for Firefox");
-}
-
 function getPermissionsApi(): CrossBrowserPermissionsApi {
   return {
-    addPermissionsListener,
-    // containsHostPermissions: chromeHostContains,
-    // requestHostPermissions: chromeHostRequest,
-    // removeHostPermissions: chromeHostRemove,
     containsClipboardPermissions: chromeClipContains,
     removeClipboardPermissions: chromeClipRemove,
     requestClipboardPermissions: chromeClipRequest,
