@@ -77,6 +77,17 @@ function ContentByUriType({
           </Button>
         </div>
       );
+    case TalerUriAction.AddExchange:
+      return (
+        <div>
+          <p>
+            <i18n.Translate>This page has a add exchange action.</i18n.Translate>
+          </p>
+          <Button variant="contained" color="success" onClick={onConfirm}>
+            <i18n.Translate>Open add exchange page</i18n.Translate>
+          </Button>
+        </div>
+      );
 
     case TalerUriAction.DevExperiment:
     case TalerUriAction.PayPull:
