@@ -66,8 +66,6 @@ export function AddNewActionView({ onCancel }: Props): VNode {
                   return <i18n.Translate>Open pay page</i18n.Translate>;
                 case TalerUriAction.Refund:
                   return <i18n.Translate>Open refund page</i18n.Translate>;
-                case TalerUriAction.Reward:
-                  return <i18n.Translate>Open tip page</i18n.Translate>;
                 case TalerUriAction.Withdraw:
                   return <i18n.Translate>Open withdraw page</i18n.Translate>;
               }
