@@ -177,6 +177,27 @@ export function Application(): VNode {
               </WalletTemplate>
             )}
           />
+          <Route
+            path={Pages.searchHistory.pattern}
+            component={({ currency }: { currency?: string }) => (
+              <WalletTemplate path="balance" goToTransaction={redirectToTxInfo} goToURL={redirectToURL}>
+                <HistoryPage
+                  currency={currency}
+                  search
+                  goToWalletDeposit={(currency: string) =>
+                    redirectTo(Pages.sendCash({ amount: `${currency}:0` }))
+                  }
+                  goToWalletManualWithdraw={(currency?: string) =>
+                    redirectTo(
+                      Pages.receiveCash({
+                        amount: !currency ? undefined : `${currency}:0`,
+                      }),
+                    )
+                  }
+                />
+              </WalletTemplate>
+            )}
+          />
           <Route
             path={Pages.sendCash.pattern}
             component={({ amount }: { amount?: string }) => (
@@ -568,17 +589,17 @@ function Redirect({ to }: { to: string }): null {
   return null;
 }
 
-function matchesRoute(url: string, route: string): boolean {
-  type MatcherFunc = (
-    url: string,
-    route: string,
-    opts: any,
-  ) => Record<string, string> | false;
+// function matchesRoute(url: string, route: string): boolean {
+//   type MatcherFunc = (
+//     url: string,
+//     route: string,
+//     opts: any,
+//   ) => Record<string, string> | false;
 
-  const internalPreactMatcher: MatcherFunc = (Router as any).exec;
-  const result = internalPreactMatcher(url, route, {});
-  return !result ? false : true;
-}
+//   const internalPreactMatcher: MatcherFunc = (Router as any).exec;
+//   const result = internalPreactMatcher(url, route, {});
+//   return !result ? false : true;
+// }
 
 function CallToActionTemplate({
   title,
