@@ -94,9 +94,9 @@ export namespace State {
 
     currentAccount: PaytoUri;
     totalFee: AmountJson;
-    totalToDeposit: AmountJson;
 
     amount: AmountFieldHandler;
+    totalToDeposit: AmountFieldHandler;
     account: SelectFieldHandler;
     cancelHandler: ButtonHandler;
     depositHandler: ButtonHandler;
