@@ -20,17 +20,16 @@
  */
 
 import {
+  AmountResponse,
   Amounts,
   AmountString,
-  DepositGroupFees,
   parsePaytoUri,
-  PrepareDepositResponse,
   ScopeType,
-  stringifyPaytoUri,
+  stringifyPaytoUri
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { expect } from "chai";
 import * as tests from "@gnu-taler/web-util/testing";
+import { expect } from "chai";
 import { nullFunction } from "../../mui/handlers.js";
 import { createWalletApiMock } from "../../test-utils.js";
 
@@ -38,24 +37,14 @@ import { useComponentState } from "./state.js";
 
 const currency = "EUR";
 const amount = `${currency}:0`;
-const withoutFee = (): PrepareDepositResponse => ({
-  effectiveDepositAmount: `${currency}:5` as AmountString,
-  totalDepositCost: `${currency}:5` as AmountString,
-  fees: {
-    coin: Amounts.stringify(`${currency}:0`),
-    wire: Amounts.stringify(`${currency}:0`),
-    refresh: Amounts.stringify(`${currency}:0`),
-  },
+const withoutFee = (value: number): AmountResponse => ({
+  effectiveAmount: `${currency}:${value}` as AmountString,
+  rawAmount: `${currency}:${value}` as AmountString,
 });
 
-const withSomeFee = (): PrepareDepositResponse => ({
-  effectiveDepositAmount: `${currency}:5` as AmountString,
-  totalDepositCost: `${currency}:5` as AmountString,
-  fees: {
-    coin: Amounts.stringify(`${currency}:1`),
-    wire: Amounts.stringify(`${currency}:1`),
-    refresh: Amounts.stringify(`${currency}:1`),
-  },
+const withSomeFee = (value: number, fee: number): AmountResponse => ({
+  effectiveAmount: `${currency}:${value}` as AmountString,
+  rawAmount: `${currency}:${value - fee}` as AmountString,
 });
 
 describe("DepositPage states", () => {
@@ -195,9 +184,9 @@ describe("DepositPage states", () => {
       },
     );
     handler.addWalletCallResponse(
-      WalletApiOperation.PrepareDeposit,
+      WalletApiOperation.ConvertDepositAmount,
       undefined,
-      withoutFee(),
+      withoutFee(0),
     );
 
     const hookBehavior = await tests.hookBehaveLikeThis(
@@ -255,15 +244,15 @@ describe("DepositPage states", () => {
       },
     );
     handler.addWalletCallResponse(
-      WalletApiOperation.PrepareDeposit,
+      WalletApiOperation.ConvertDepositAmount,
       undefined,
-      withoutFee(),
+      withoutFee(0),
     );
 
     handler.addWalletCallResponse(
-      WalletApiOperation.PrepareDeposit,
+      WalletApiOperation.ConvertDepositAmount,
       undefined,
-      withoutFee(),
+      withoutFee(0),
     );
 
     const accountSelected = stringifyPaytoUri(ibanPayto.uri);
@@ -345,19 +334,19 @@ describe("DepositPage states", () => {
       },
     );
     handler.addWalletCallResponse(
-      WalletApiOperation.PrepareDeposit,
+      WalletApiOperation.ConvertDepositAmount,
       undefined,
-      withoutFee(),
+      withoutFee(0),
     );
     handler.addWalletCallResponse(
-      WalletApiOperation.PrepareDeposit,
+      WalletApiOperation.ConvertDepositAmount,
       undefined,
-      withSomeFee(),
+      withSomeFee(10,3),
     );
     handler.addWalletCallResponse(
-      WalletApiOperation.PrepareDeposit,
+      WalletApiOperation.ConvertDepositAmount,
       undefined,
-      withSomeFee(),
+      withSomeFee(10,3),
     );
 
     const accountSelected = stringifyPaytoUri(ibanPayto.uri);
@@ -404,7 +393,7 @@ describe("DepositPage states", () => {
           expect(state.account.value).eq(accountSelected);
           expect(state.amount.value).deep.eq(Amounts.parseOrThrow("EUR:10"));
           expect(state.totalFee).deep.eq(Amounts.parseOrThrow(`${currency}:3`));
-          expect(state.totalToDeposit).deep.eq(
+          expect(state.totalToDeposit.value).deep.eq(
             Amounts.parseOrThrow(`${currency}:7`),
           );
           expect(state.depositHandler.onClick).not.undefined;
@@ -416,7 +405,7 @@ describe("DepositPage states", () => {
           expect(state.account.value).eq(accountSelected);
           expect(state.amount.value).deep.eq(Amounts.parseOrThrow("EUR:10"));
           expect(state.totalFee).deep.eq(Amounts.parseOrThrow(`${currency}:3`));
-          expect(state.totalToDeposit).deep.eq(
+          expect(state.totalToDeposit.value).deep.eq(
             Amounts.parseOrThrow(`${currency}:7`),
           );
           expect(state.depositHandler.onClick).not.undefined;
