@@ -25,6 +25,7 @@ import {
   ExchangeListItem,
   ExchangeTosStatus,
   ExchangeUpdateStatus,
+  ScopeType,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import * as tests from "@gnu-taler/web-util/testing";
@@ -36,14 +37,20 @@ import { useComponentState } from "./state.js";
 const exchangeArs: ExchangeListItem = {
   currency: "ARS",
   exchangeBaseUrl: "http://",
-  masterPub:"123qwe123",
-  scopeInfo: undefined,
+  masterPub: "123qwe123",
+  scopeInfo: {
+    currency: "ARS",
+    type: ScopeType.Exchange,
+    url: "http://",
+  },
   tosStatus: ExchangeTosStatus.Accepted,
   exchangeEntryStatus: ExchangeEntryStatus.Used,
   exchangeUpdateStatus: ExchangeUpdateStatus.Initial,
   paytoUris: [],
   ageRestrictionOptions: [],
   lastUpdateTimestamp: undefined,
+  noFees: false,
+  peerPaymentsDisabled: false,
 };
 
 describe("Destination selection states", () => {
