@@ -169,7 +169,9 @@ const CircleDiv = styled.div`
   text-align: center;
   text-decoration: none;
   text-transform: uppercase;
-  transition: background-color 0.15s ease, border-color 0.15s ease,
+  transition:
+    background-color 0.15s ease,
+    border-color 0.15s ease,
     color 0.15s ease;
   font-size: 16px;
   background-color: #86a7bd1a;
@@ -275,6 +277,16 @@ export function ReadyGetView({
               </Button>
             </Paper>
           </Grid>
+          <Grid item xs={1}>
+            <Paper style={{ padding: 8 }}>
+              <p>
+                <i18n.Translate>From a <pre style={{display:"inline"}}>taler://peer-push-credit</pre> URI</i18n.Translate>
+              </p>
+              <a href={Pages.qr}>
+                <i18n.Translate>Enter URI here</i18n.Translate>
+              </a>
+            </Paper>
+          </Grid>
         </Grid>
       </Grid>
     </Container>
