@@ -22,7 +22,7 @@ import {
   LogLevel,
   NotificationType,
   ScopeType,
-  stringifyWithdrawExchange
+  stringifyWithdrawExchange,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
@@ -112,21 +112,21 @@ export function DeveloperPage(): VNode {
   const currencies: { [ex: string]: string } = {};
   const money_by_exchange = coins.reduce(
     (prev, cur) => {
-      const denom = Amounts.parseOrThrow(cur.denom_value);
-      if (!prev[cur.exchange_base_url]) {
-        prev[cur.exchange_base_url] = [];
-        currencies[cur.exchange_base_url] = denom.currency;
+      const denom = Amounts.parseOrThrow(cur.denomValue);
+      if (!prev[cur.exchangeBaseUrl]) {
+        prev[cur.exchangeBaseUrl] = [];
+        currencies[cur.exchangeBaseUrl] = denom.currency;
       }
-      prev[cur.exchange_base_url].push({
+      prev[cur.exchangeBaseUrl].push({
         // ageKeysCount: cur.ageCommitmentProof?.proof.privateKeys.length,
         denom_value: denom.value,
         denom_fraction: denom.fraction,
         // remain_value: parseFloat(
         //   Amounts.stringifyValue(Amounts.parseOrThrow(cur.remaining_value)),
         // ),
-        status: cur.coin_status,
-        from_refresh: cur.refresh_parent_coin_pub !== undefined,
-        id: cur.coin_pub,
+        status: cur.coinStatus,
+        from_refresh: cur.refreshParentCoinPub !== undefined,
+        id: cur.coinPub,
       });
       return prev;
     },
@@ -351,7 +351,7 @@ export function DeveloperPage(): VNode {
                       <a
                         href={new URL(`/keys`, e.exchangeBaseUrl).href}
                         target="_blank"
-                        rel="noreferrer" 
+                        rel="noreferrer"
                       >
                         {e.exchangeBaseUrl}
                       </a>
