@@ -135,7 +135,9 @@ export function TosContentView({
       <Button variant="outlined" onClick={onClose.onClick}>
         <i18n.Translate>Close</i18n.Translate>
       </Button>
-      <TermsOfService exchangeUrl={exchangeUrl} />
+      <TermsOfService exchangeUrl={exchangeUrl} readOnly >
+        s
+      </TermsOfService>
     </div>
   );
 }
