@@ -35,7 +35,7 @@ import {
   CenteredBoldText,
   CenteredText,
   DateSeparator,
-  NiceSelect
+  NiceSelect,
 } from "../components/styled/index.js";
 import { alertFromError, useAlertContext } from "../context/alert.js";
 import { useBackendContext } from "../context/backend.js";
@@ -45,32 +45,39 @@ import { Button } from "../mui/Button.js";
 import { NoBalanceHelp } from "../popup/NoBalanceHelp.js";
 import DownloadIcon from "../svg/download_24px.inline.svg";
 import UploadIcon from "../svg/upload_24px.inline.svg";
+import { TextField } from "../mui/TextField.js";
+import { TextFieldHandler } from "../mui/handlers.js";
 
 interface Props {
   currency?: string;
+  search?: boolean;
   goToWalletDeposit: (currency: string) => Promise<void>;
   goToWalletManualWithdraw: (currency?: string) => Promise<void>;
 }
 export function HistoryPage({
   currency: _c,
+  search: showSearch,
   goToWalletManualWithdraw,
   goToWalletDeposit,
 }: Props): VNode {
   const { i18n } = useTranslationContext();
   const api = useBackendContext();
   const [balanceIndex, setBalanceIndex] = useState<number>(0);
+  const [search, setSearch] = useState<string>();
+
   const [settings] = useSettings();
   const state = useAsyncAsHook(async () => {
     const b = await api.wallet.call(WalletApiOperation.GetBalances, {});
     const balance =
       b.balances.length > 0 ? b.balances[balanceIndex] : undefined;
     const tx = await api.wallet.call(WalletApiOperation.GetTransactions, {
-      scopeInfo: balance?.scopeInfo,
+      scopeInfo: showSearch ? undefined : balance?.scopeInfo,
       sort: "descending",
       includeRefreshes: settings.showRefeshTransactions,
+      search,
     });
     return { b, tx };
-  }, [balanceIndex]);
+  }, [balanceIndex, search]);
 
   useEffect(() => {
     return api.listener.onUpdateNotification(
@@ -105,6 +112,40 @@ export function HistoryPage({
       />
     );
   }
+
+  const byDate = state.response.tx.transactions.reduce(
+    (rv, x) => {
+      const startDay =
+        x.timestamp.t_s === "never"
+          ? 0
+          : startOfDay(x.timestamp.t_s * 1000).getTime();
+      if (startDay) {
+        if (!rv[startDay]) {
+          rv[startDay] = [];
+          // datesWithTransaction.push(String(startDay));
+        }
+        rv[startDay].push(x);
+      }
+
+      return rv;
+    },
+    {} as { [x: string]: Transaction[] },
+  );
+
+  if (showSearch) {
+    return (
+      <FilteredHistoryView
+        search={{
+          value: search ?? "",
+          onInput: pushAlertOnError(async (d: string) => {
+            setSearch(d);
+          }),
+        }}
+        transactionsByDate={byDate}
+      />
+    );
+  }
+
   return (
     <HistoryView
       balanceIndex={balanceIndex}
@@ -112,7 +153,7 @@ export function HistoryPage({
       balances={state.response.b.balances}
       goToWalletManualWithdraw={goToWalletManualWithdraw}
       goToWalletDeposit={goToWalletDeposit}
-      transactions={state.response.tx.transactions}
+      transactionsByDate={byDate}
     />
   );
 }
@@ -121,7 +162,7 @@ export function HistoryView({
   balances,
   balanceIndex,
   changeBalanceIndex,
-  transactions,
+  transactionsByDate,
   goToWalletManualWithdraw,
   goToWalletDeposit,
 }: {
@@ -129,7 +170,7 @@ export function HistoryView({
   changeBalanceIndex: (s: number) => void;
   goToWalletDeposit: (currency: string) => Promise<void>;
   goToWalletManualWithdraw: (currency?: string) => Promise<void>;
-  transactions: Transaction[];
+  transactionsByDate: Record<string, Transaction[]>;
   balances: WalletBalance[];
 }): VNode {
   const { i18n } = useTranslationContext();
@@ -140,25 +181,7 @@ export function HistoryView({
     ? Amounts.jsonifyAmount(balance.available)
     : undefined;
 
-  const datesWithTransaction: string[] = [];
-  const byDate = transactions.reduce(
-    (rv, x) => {
-      const startDay =
-        x.timestamp.t_s === "never"
-          ? 0
-          : startOfDay(x.timestamp.t_s * 1000).getTime();
-      if (startDay) {
-        if (!rv[startDay]) {
-          rv[startDay] = [];
-          datesWithTransaction.push(String(startDay));
-        }
-        rv[startDay].push(x);
-      }
-
-      return rv;
-    },
-    {} as { [x: string]: Transaction[] },
-  );
+  const datesWithTransaction: string[] = Object.keys(transactionsByDate);
 
   return (
     <Fragment>
@@ -195,8 +218,8 @@ export function HistoryView({
               </Button>
             )}
           </div>
-          <div style={{display:"flex", flexDirection:"column"}}>
-            <h3 style={{marginBottom: 0}}>Balance</h3>
+          <div style={{ display: "flex", flexDirection: "column" }}>
+            <h3 style={{ marginBottom: 0 }}>Balance</h3>
             <div
               style={{
                 width: "fit-content",
@@ -270,7 +293,72 @@ export function HistoryView({
                     format="dd MMMM yyyy"
                   />
                 </DateSeparator>
-                {byDate[d].map((tx, i) => (
+                {transactionsByDate[d].map((tx, i) => (
+                  <HistoryItem key={i} tx={tx} />
+                ))}
+              </Fragment>
+            );
+          })}
+        </section>
+      )}
+    </Fragment>
+  );
+}
+
+export function FilteredHistoryView({
+  search,
+  transactionsByDate,
+}: {
+  search: TextFieldHandler;
+  transactionsByDate: Record<string, Transaction[]>;
+}): VNode {
+  const { i18n } = useTranslationContext();
+
+  const datesWithTransaction: string[] = Object.keys(transactionsByDate);
+
+  return (
+    <Fragment>
+      <section>
+        <div
+          style={{
+            display: "flex",
+            flexWrap: "wrap",
+            alignItems: "center",
+            justifyContent: "space-between",
+            marginRight: 20,
+          }}
+        >
+          <TextField
+            label="Search"
+            variant="filled"
+            error={search.error}
+            required
+            fullWidth
+            value={search.value}
+            onChange={search.onInput}
+          />
+        </div>
+      </section>
+      {datesWithTransaction.length === 0 ? (
+        <section>
+          <i18n.Translate>
+            Your transaction history is empty for this currency.
+          </i18n.Translate>
+        </section>
+      ) : (
+        <section>
+          {datesWithTransaction.map((d, i) => {
+            return (
+              <Fragment key={i}>
+                <DateSeparator>
+                  <Time
+                    timestamp={AbsoluteTime.fromMilliseconds(
+                      Number.parseInt(d, 10),
+                    )}
+                    format="dd MMMM yyyy"
+                  />
+                </DateSeparator>
+                {transactionsByDate[d].map((tx, i) => (
                   <HistoryItem key={i} tx={tx} />
                 ))}
               </Fragment>
