@@ -61,7 +61,10 @@ export function useComponentState({
   if (hook.hasError) {
     return {
       status: "error",
-      error: alertFromError(i18n.str`Could not load known bank accounts`, hook),
+      error: alertFromError(
+        i18n,
+        i18n.str`Could not load known bank accounts`,
+        hook),
     };
   }
 
