@@ -38,11 +38,10 @@ import {
   TransactionPeerPushDebit,
   TransactionRefresh,
   TransactionRefund,
-  TransactionReward,
   TransactionType,
   TransactionWithdrawal,
   WithdrawalDetails,
-  WithdrawalType,
+  WithdrawalType
 } from "@gnu-taler/taler-util";
 import * as tests from "@gnu-taler/web-util/testing";
 import beer from "../../static-dev/beer.png";
@@ -137,17 +136,6 @@ const exampleData = {
     exchangeBaseUrl: "http://exchange.taler",
     refreshReason: RefreshReason.Manual,
   } as TransactionRefresh,
-  tip: {
-    ...commonTransaction,
-    type: TransactionType.Reward,
-    // merchant: {
-    //   name: "the merchant",
-    //   logo: merchantIcon,
-    //   website: "https://www.themerchant.taler",
-    //   email: "contact@merchant.taler",
-    // },
-    merchantBaseUrl: "http://merchant.taler",
-  } as TransactionReward,
   refund: {
     ...commonTransaction,
     type: TransactionType.Refund,
@@ -293,6 +281,12 @@ export const WithdrawPendingManual = tests.createExample(
         type: WithdrawalType.ManualTransfer,
         exchangePaytoUris: ["payto://iban/ES8877998399652238"],
         reservePub: "A05AJGMFNSK4Q62NXR2FKNDB1J4EXTYQTE7VA4M9GZQ4TR06YBNG",
+        exchangeCreditAccountDetails: [{
+          paytoUri: "payto://IBAN/1231231231",
+        },
+        {
+          paytoUri: "payto://IBAN/2342342342",
+        }],
       } as WithdrawalDetails,
       txState: {
         major: TransactionMajorState.Pending,
@@ -578,26 +572,6 @@ export const RefreshError = tests.createExample(TestedComponent, {
   },
 });
 
-export const Tip = tests.createExample(TestedComponent, {
-  transaction: exampleData.tip,
-});
-
-export const TipError = tests.createExample(TestedComponent, {
-  transaction: {
-    ...exampleData.tip,
-    error: transactionError,
-  },
-});
-
-export const TipPending = tests.createExample(TestedComponent, {
-  transaction: {
-    ...exampleData.tip,
-    txState: {
-      major: TransactionMajorState.Pending,
-    },
-  },
-});
-
 export const Refund = tests.createExample(TestedComponent, {
   transaction: exampleData.refund,
 });
