@@ -1416,9 +1416,11 @@ export function TransferPickupDetails({
 export function WithdrawDetails({
   conversion,
   amount,
+  bankFee,
 }: {
   conversion?: AmountJson;
   amount: AmountWithFee;
+  bankFee?: AmountJson;
 }): VNode {
   const { i18n } = useTranslationContext();
 
@@ -1481,6 +1483,16 @@ export function WithdrawDetails({
           </tr>
         </Fragment>
       )}
+      {!bankFee || Amounts.isZero(bankFee) ? undefined : (
+        <tr>
+          <td>
+            <i18n.Translate>Bank fee</i18n.Translate>
+          </td>
+          <td>
+            <Amount value={bankFee} maxFracSize={amount.maxFrac} />
+          </td>
+        </tr>
+      )}
     </PurchaseDetailsTable>
   );
 }
