@@ -27,33 +27,37 @@ import {
   AbsoluteTime,
   LogLevel,
   Logger,
+  NotificationType,
   OpenedPromise,
   SetTimeoutTimerAPI,
   TalerError,
   TalerErrorCode,
   TalerErrorDetail,
+  TransactionMinorState,
+  WalletNotification,
   getErrorDetailFromException,
   makeErrorDetail,
   openPromise,
   setGlobalLogLevelFromString,
-  setLogLevelFromString
+  setLogLevelFromString,
 } from "@gnu-taler/taler-util";
 import { HttpRequestLibrary } from "@gnu-taler/taler-util/http";
 import {
   DbAccess,
   SynchronousCryptoWorkerFactoryPlain,
   Wallet,
+  WalletApiOperation,
   WalletOperations,
   WalletStoresV1,
   deleteTalerDatabase,
   exportDb,
   importDb,
 } from "@gnu-taler/taler-wallet-core";
+import { BrowserFetchHttpLib } from "@gnu-taler/web-util/browser";
 import { MessageFromFrontend, MessageResponse } from "./platform/api.js";
 import { platform } from "./platform/background.js";
 import { ExtensionOperations } from "./taler-wallet-interaction-loader.js";
-import { BackgroundOperations, WalletEvent } from "./wxApi.js";
-import { BrowserFetchHttpLib } from "@gnu-taler/web-util/browser";
+import { BackgroundOperations } from "./wxApi.js";
 
 /**
  * Currently active wallet instance.  Might be unloaded and
@@ -65,11 +69,6 @@ let currentWallet: Wallet | undefined;
 
 let currentDatabase: DbAccess<typeof WalletStoresV1> | undefined;
 
-/**
- * Last version of an outdated DB, if applicable.
- */
-let outdatedDbVersion: number | undefined;
-
 const walletInit: OpenedPromise<void> = openPromise<void>();
 
 const logger = new Logger("wxBackend.ts");
@@ -91,17 +90,172 @@ async function resetDb(): Promise<void> {
   await reinitWallet();
 }
 
+export type WalletActivityTrack = {
+  // id: number;
+  events: (WalletNotification & { when: AbsoluteTime })[];
+  start: AbsoluteTime;
+  type: NotificationType;
+  end: AbsoluteTime;
+  groupId: string;
+  requestId?: string; //only for request event
+};
+
 //FIXME: maybe circular buffer
-const notifications: WalletEvent[] = []
-async function getNotifications(): Promise<WalletEvent[]> {
-  return notifications
+const activity: WalletActivityTrack[] = [];
+
+function convertWalletActivityNotification(
+  knownEvents: WalletActivityTrack[],
+  event: WalletNotification & {
+    when: AbsoluteTime;
+  },
+): WalletActivityTrack | undefined {
+  switch (event.type) {
+    case NotificationType.BalanceChange: {
+      const groupId = `${event.type}:${event.hintTransactionId}`;
+      const found = knownEvents.find((a) => a.groupId === groupId);
+      if (found) {
+        found.end = event.when;
+        found.events.unshift(event);
+        return undefined;
+      }
+      return {
+        type: event.type,
+        start: event.when,
+        end: AbsoluteTime.never(),
+        events: [event],
+        groupId,
+      };
+    }
+    case NotificationType.BackupOperationError: {
+      const groupId = "";
+      return {
+        type: event.type,
+        start: event.when,
+        end: AbsoluteTime.never(),
+        events: [event],
+        groupId,
+      };
+    }
+    case NotificationType.TransactionStateTransition: {
+      const groupId = `${event.type}:${event.transactionId}`;
+      const found = knownEvents.find((a) => a.groupId === groupId);
+      if (found) {
+        found.end = event.when;
+        found.events.unshift(event);
+        return undefined;
+      }
+      return {
+        type: event.type,
+        start: event.when,
+        end: AbsoluteTime.never(),
+        events: [event],
+        groupId,
+      };
+    }
+    case NotificationType.WithdrawalOperationTransition: {
+      return undefined;
+    }
+    case NotificationType.ExchangeStateTransition: {
+      const groupId = `${event.type}:${event.exchangeBaseUrl}`;
+      const found = knownEvents.find((a) => a.groupId === groupId);
+      if (found) {
+        found.end = event.when;
+        found.events.unshift(event);
+        return undefined;
+      }
+      return {
+        type: event.type,
+        start: event.when,
+        end: AbsoluteTime.never(),
+        events: [event],
+        groupId,
+      };
+    }
+    case NotificationType.Idle: {
+      const groupId = "";
+      return {
+        type: event.type,
+        start: event.when,
+        end: AbsoluteTime.never(),
+        events: [event],
+        groupId,
+      };
+    }
+    case NotificationType.TaskObservabilityEvent: {
+      const groupId = `${event.type}:${event.taskId}`;
+      const found = knownEvents.find((a) => a.groupId === groupId);
+      if (found) {
+        found.end = event.when;
+        found.events.unshift(event);
+        return undefined;
+      }
+      return {
+        type: event.type,
+        start: event.when,
+        end: AbsoluteTime.never(),
+        events: [event],
+        groupId,
+      };
+    }
+    case NotificationType.RequestObservabilityEvent: {
+      const groupId = `${event.type}:${event.operation}:${event.requestId}`;
+      const found = knownEvents.find((a) => a.groupId === groupId);
+      if (found) {
+        found.end = event.when;
+        found.events.unshift(event);
+        return undefined;
+      }
+      return {
+        type: event.type,
+        start: event.when,
+        end: AbsoluteTime.never(),
+        events: [event],
+        groupId,
+        requestId: event.requestId,
+      };
+    }
+  }
+}
+
+function addNewWalletActivityNotification(
+  list: WalletActivityTrack[],
+  n: WalletNotification,
+) {
+  const start = AbsoluteTime.now();
+  const ev = { ...n, when: start };
+  const activity = convertWalletActivityNotification(list, ev);
+  if (activity) {
+    list.unshift(activity); // insert at start
+  }
+}
+
+async function getNotifications({
+  filter,
+  operationFrom,
+}: {
+  filter: string;
+  operationFrom?: string;
+}): Promise<WalletActivityTrack[]> {
+  const rg = new RegExp(`.*${filter}.*`);
+  return activity.filter((event) => {
+    if (operationFrom) {
+      if (event.type !== NotificationType.RequestObservabilityEvent) {
+        return false;
+      }
+      if (event.requestId) {
+        return event.requestId.startsWith(operationFrom);
+      }
+    }
+    if (!filter) return true;
+    const testFilter = rg.test(event.groupId.toLowerCase());
+    return testFilter;
+  });
 }
 
 async function clearNotifications(): Promise<void> {
-  notifications.splice(0, notifications.length)
+  activity.splice(0, activity.length);
 }
 
-
 async function runGarbageCollector(): Promise<void> {
   const dbBeforeGc = currentDatabase;
   if (!dbBeforeGc) {
@@ -164,7 +318,10 @@ let nextMessageIndex = 0;
 
 async function dispatch<
   Op extends WalletOperations | BackgroundOperations | ExtensionOperations,
->(req: MessageFromFrontend<Op> & { id: string }): Promise<MessageResponse> {
+>(
+  req: MessageFromFrontend<Op> & { id: string },
+  from: string,
+): Promise<MessageResponse> {
   nextMessageIndex = (nextMessageIndex + 1) % (Number.MAX_SAFE_INTEGER - 100);
 
   switch (req.channel) {
@@ -229,8 +386,10 @@ async function dispatch<
     case "wallet": {
       const w = currentWallet;
       if (!w) {
-        const lastError: TalerErrorDetail = walletInit.lastError instanceof TalerError ?
-          walletInit.lastError.errorDetail : undefined
+        const lastError: TalerErrorDetail =
+          walletInit.lastError instanceof TalerError
+            ? walletInit.lastError.errorDetail
+            : undefined;
 
         return {
           type: "error",
@@ -239,16 +398,22 @@ async function dispatch<
           error: makeErrorDetail(
             TalerErrorCode.WALLET_CORE_NOT_AVAILABLE,
             { lastError },
-            `wallet core not available${!lastError ? "" : `,last error: ${lastError.hint}`}`,
+            `wallet core not available${
+              !lastError ? "" : `,last error: ${lastError.hint}`
+            }`,
           ),
         };
       }
       //multiple client can create the same id, send the wallet an unique key
-      const newId = `${req.id}_${nextMessageIndex}`
-      const resp = await w.handleCoreApiRequest(req.operation, newId, req.payload);
+      const newId = `${from}:${req.id}`;
+      const resp = await w.handleCoreApiRequest(
+        req.operation,
+        newId,
+        req.payload,
+      );
       //return to the client the original id
-      resp.id = req.id
-      return resp
+      resp.id = req.id;
+      return resp;
     }
   }
 
@@ -267,7 +432,7 @@ async function dispatch<
 
 async function reinitWallet(): Promise<void> {
   if (currentWallet) {
-    currentWallet.stop();
+    await currentWallet.client.call(WalletApiOperation.Shutdown, {});
     currentWallet = undefined;
   }
   currentDatabase = undefined;
@@ -310,7 +475,7 @@ async function reinitWallet(): Promise<void> {
         features: {
           allowHttp: settings.walletAllowHttp,
         },
-      }
+      },
     });
   } catch (e) {
     logger.error("could not initialize wallet", e);
@@ -319,28 +484,23 @@ async function reinitWallet(): Promise<void> {
   }
   wallet.addNotificationListener((message) => {
     if (settings.showWalletActivity) {
-      notifications.push({
-        notification: message,
-        when: AbsoluteTime.now()
-      })
+      addNewWalletActivityNotification(activity, message);
     }
 
+    processWalletNotification(message);
+
     platform.sendMessageToAllChannels({
       type: "wallet",
       notification: message,
     });
   });
 
-  platform.keepAlive(() => {
-    return wallet.runTaskLoop().catch((e) => {
-      logger.error("error during wallet task loop", e);
-    });
-  });
   // Useful for debugging in the background page.
   if (typeof window !== "undefined") {
     (window as any).talerWallet = wallet;
   }
   currentWallet = wallet;
+  updateIconBasedOnBalance();
   return walletInit.resolve();
 }
 
@@ -359,10 +519,10 @@ export async function wxMain(): Promise<void> {
   // Handlers for messages coming directly from the content
   // script on the page
   logger.trace("listen all channels");
-  platform.listenToAllChannels(async (message) => {
+  platform.listenToAllChannels(async (message, from) => {
     //wait until wallet is initialized
     await afterWalletIsInitialized;
-    const result = await dispatch(message);
+    const result = await dispatch(message, from);
     return result;
   });
 
@@ -378,3 +538,46 @@ export async function wxMain(): Promise<void> {
     console.error(e);
   }
 }
+
+async function updateIconBasedOnBalance() {
+  const balance = await currentWallet?.client.call(
+    WalletApiOperation.GetBalances,
+    {},
+  );
+  if (balance) {
+    let showAlert = false;
+    for (const b of balance.balances) {
+      if (b.flags.length > 0) {
+        console.log("b.flags", JSON.stringify(b.flags));
+        showAlert = true;
+        break;
+      }
+    }
+
+    if (showAlert) {
+      platform.setAlertedIcon();
+    } else {
+      platform.setNormalIcon();
+    }
+  }
+}
+
+/**
+ * All the actions triggered by notification that need to be
+ * run in the background.
+ *
+ * @param message
+ */
+async function processWalletNotification(message: WalletNotification) {
+  if (
+    message.type === NotificationType.TransactionStateTransition &&
+    (message.newTxState.minor === TransactionMinorState.KycRequired ||
+      message.oldTxState.minor === TransactionMinorState.KycRequired ||
+      message.newTxState.minor === TransactionMinorState.AmlRequired ||
+      message.oldTxState.minor === TransactionMinorState.AmlRequired ||
+      message.newTxState.minor === TransactionMinorState.BankConfirmTransfer ||
+      message.oldTxState.minor === TransactionMinorState.BankConfirmTransfer)
+  ) {
+    await updateIconBasedOnBalance();
+  }
+}
