@@ -9,7 +9,7 @@ export function LocalNotificationBanner({ notification, showDebug }: { notificat
       return <div class="relative">
         <div class="fixed top-0 left-0 right-0 z-20 w-full p-4">
           <Attention type="danger" title={notification.message.title} onClose={() => {
-            notification.remove()
+            notification.acknowledge()
           }}>
             {notification.message.description &&
               <div class="mt-2 text-sm text-red-700">
@@ -26,7 +26,7 @@ export function LocalNotificationBanner({ notification, showDebug }: { notificat
       return <div class="relative">
         <div class="fixed top-0 left-0 right-0 z-20 w-full p-4">
           <Attention type="success" title={notification.message.title} onClose={() => {
-            notification.remove();
+            notification.acknowledge();
           }} /></div></div>
   }
 }
