@@ -5,6 +5,8 @@ export {
   useTranslationContext
 } from "./translation.js";
 export * from "./bank-api.js";
+export * from "./challenger-api.js";
 export * from "./merchant-api.js";
+export * from "./exchange-api.js";
 export * from "./navigation.js";
 export * from "./wallet-integration.js";
