@@ -22,6 +22,7 @@ import {
   Location,
   findMatch,
   RouteDefinition,
+  LocationNotFound,
 } from "../utils/route.js";
 
 /**
@@ -44,7 +45,7 @@ export const useNavigationContext = (): Type => useContext(Context);
 // eslint-disable-next-line @typescript-eslint/no-explicit-any
 export function useCurrentLocation<T extends ObjectOf<RouteDefinition<any>>>(
   pagesMap: T,
-): Location<T> | undefined {
+): Location<T> | LocationNotFound<T> {
   const pageList = Object.keys(pagesMap as object) as Array<keyof T>;
   const { path, params } = useNavigationContext();
 
