@@ -22,7 +22,7 @@
 import { TranslatedString } from "@gnu-taler/taler-util";
 import * as tests from "@gnu-taler/web-util/testing";
 import {
-  FlexibleForm,
+  FlexibleForm_Deprecated,
   DefaultForm as TestedComponent,
 } from "./DefaultForm.js";
 
@@ -38,12 +38,12 @@ const initial: TargetObject = {
   age: 5,
 }
 
-const form: FlexibleForm<TargetObject> = {
+const form: FlexibleForm_Deprecated<TargetObject> = {
   design: [{
     title: "this is a simple form" as TranslatedString,
     fields: [{
       type: "integer",
-      props: {
+      properties: {
         label: "label of the field" as TranslatedString,
         name: "age",
         tooltip: "just numbers" as TranslatedString,
