@@ -1,5 +1,6 @@
 import { VNode, h } from "preact";
-import { InputLine, UIFormProps } from "./InputLine.js";
+import { InputLine } from "./InputLine.js";
+import { UIFormProps } from "./FormProvider.js";
 
 export function InputInteger<T extends object, K extends keyof T>(
   props: UIFormProps<T, K>,
