@@ -1,5 +1,5 @@
 export { useLang } from "./useLang.js";
-export { useLocalStorage, buildStorageKey } from "./useLocalStorage.js";
+export { useLocalStorage, buildStorageKey, StorageKey, StorageState } from "./useLocalStorage.js";
 export { useMemoryStorage } from "./useMemoryStorage.js";
 export * from "./useNotifications.js";
 export {
